/**
 * @author kostenko
 *
 * @deprecated DO NOT USE
 *
 * @constructor
 *
 * @param config the configuration object.
 *               Note that you must provide one and only one field from the following list:
 *               requests, requestsWithUsers, requestsWithUsersRpcMethod.
 *
 * @config [requests] - an array of request objects
 * @config [requestsWithUsers] - an array of objects which contain request and userInfo fields
 * @config [requestsWithUsersRpcMethod] - JSON-RPC method to use to obtain the data.
 *              Data format is the same as with requestsWithUsers,
 *              but wrapped in DataTables response structure and then in JSON-RPC response structure.
 * @config renderTo - table element to render to
 * @config [showEmployeeColumn] - whether to show Employee column or not (false by default)
 * @config [showDeleteColumn] - whether to show Delete column or not (false by default)
 * @config [showSelectColumn] - whether to show Select column or not (false by default)
 * @config [paginate] - whether to use pager or not (false by default)
 * @config [enableUserGroupFilter] - whether to enable user group filter (false by default)
 *              (only server-side filtering is supported, so the requestsWithUsersRpcMethod is required)
 * @config [enableLeaveTypeFilter] - whether to enable leave type filter (false by default)
 *              (only server-side filtering is supported, so the requestsWithUsersRpcMethod is required)
 * @config [enableStatusFilter] - whether to enable status filter (false by default)
 *              (only server-side filtering is supported, so the requestsWithUsersRpcMethod is required)
 * @config [showRejected] - whether to show rejected requests or not (false by default)
 *              (only server-side filtering is supported, so the requestsWithUsersRpcMethod is required)
 * @config onRequestLeaveTypeEditClick - function(request, button) - invoked when the user clicks on a request leave type editing button
 * @config onRequestPeriodEditClick - function(request, button) - invoked when the user clicks on a request period editing button
 * @config [onRequestDeleteClick] - function(request, button) - invoked when the user clicks on a request deletion button
 *              (mandatory if showDeleteColumn is true)
 * @config [onVisibleSelectionChanged] - function(selectedRequests) - invoked when visible request selection is changed
 *              (it actually might be invoked in some cases even if selection didn't actually change,
 *              but it is guaranteed to be invoked when selection changes)
 * @config [userGroups] - an array of user groups, ordered as needed for presentation.
 *                            Required if enableUserGroupFilter is true.
 *                            The required fields of an user group object are: id, name.
 *                            (Note that the filter may create additional fields as needed for its functionality.)
 * @config ptoHoursPerDay - the number of hours in PTO day
 */
function LeaveRequestTable(config)
{
    /**
     * Must be called if the user cancels request editing, deletion or revocation.
     */
    this.cancelRowAction = function()
    {
        if ( !table.locked )
            table.find('tr.active').removeClass('active');
    };

    this.lockTable = function ( isLocked ) {
        table.locked = isLocked;
        table.toggleClass('locked', isLocked);
    };

    /**
     * Returns the requests which are selected by user, but only those which are currently visible
     * (located on the current page and not filtered out).
     */
    this.getVisibleSelectedRequests = function()
    {
        var rows = table.find('td.selectCell :checked').parents('tr.requestRow');
        var requests = [];

        rows.each( function() {
            requests.push( table.fnGetData(this).request );
        });

        return requests;
    };

    /**
     * Causes the table to redraw. In case of JSON-RPC source, the new data will be fetched from the server.
     */
    this.redraw = function()
    {
        table.fnDraw(false);
    };

    /**
     * Same as 'redraw' method, but also deselects all the requests (on all pages).
     */
    this.redrawAndDeselectAll = function()
    {
        //noinspection ReuseOfLocalVariableJS
        allSelectedRequestIdsSet = {};
        table.find('.columnCheckbox').removeAttr('checked');
        this.redraw();
    };

    /**
     * Destroys the table, leaving the DOM in the same state as before creating.
     */
    this.destroy = function()
    {
        table.fnDestroy();
    };

    this.getBalanceTypeFilter = function()
    {
        return balanceTypeFilter;
    };


    // === private ===

    var table;
    var daysFormatter;
    var oneDayFormatter;
    var hoursFormatter;
    var ptoFormatter;
    var sickFormatter;
    var balanceTypeFilter;
    var isBalanceColumnEnabled;
    var showPastApprovedRejectedRequestInPast;
    var self = this;

    // A set of ids of all selected requests, including invisible (filtered out or located on other page)
    var allSelectedRequestIdsSet = {};

    var statusFilterButton;
    var leaveTypeSelector;
    var userGroupSelector;

    var statusTextMap = {};
    statusTextMap[LeaveTimeRequestStatus.REQUESTED] = messageResource.getMessage('actiplans.request_status.waiting_for_approval');
    statusTextMap[LeaveTimeRequestStatus.APPROVED] = messageResource.getMessage('actiplans.request_status.approved');
    statusTextMap[LeaveTimeRequestStatus.REJECTED] = messageResource.getMessage('actiplans.request_status.rejected');

    var wasAtIntegratedTooltip = new LeaveTimeRequestsPageTooltip($("#wasAtIntegratedTooltip"));
    var cannotEditPastRequestTooltip = new LeaveTimeRequestsPageTooltip($("#cannotEditPastRequestTooltip"));

    init();

    function Employee(userInfo)
    {
        if (userInfo)
            this.fullName = UserNameFormat.formatLFM( userInfo.lastName, userInfo.firstName, userInfo.middleInitial );

        // Presentation for displaying and client-side sorting.
        this.toString = function()
        {
            return this.fullName ? this.fullName : '';
        }
    }

    function Period(request)
    {
        this.firstDate = request.firstDate;
        this.lastDate = request.lastDate;
        this.clickable = true; //request.status != LeaveTimeRequestStatus.REJECTED;
        this.past = request.past;

        // Presentation for client-side sorting. Assuming that dates are in Y-m-d format (see date_js.jsp).
        this.toString = function()
        {
            return this.firstDate + ' ' + this.lastDate;
        }
    }

    function Status(request)
    {

        $.extend(this,  LeaveTimeRequestUtils.getStatusFromRequestWithChanges(request));

        this.firstDate = request.firstDate;
        this.past = request.past;
        this.statusDate = request.statusDate;
        // Presentation suitable for client-side sorting.
        this.toString = function()
        {
            if (this.code == LeaveTimeRequestStatus.REQUESTED && !this.waitingDeletion)
                return "1";
            else if (this.waitingDeletion)
                return "2";
            else if (this.code == LeaveTimeRequestStatus.APPROVED)
                return "3";
            else if (this.code == LeaveTimeRequestStatus.REJECTED)
                return "4";
            else
                return "5";
        }
    }

    function RequestedTime(request)
    {
        var leaveTimeTotal =  LeaveTimeRequestUtils.getLeaveTimeTotalValueFromRequestWithChanges(request);
        this.days = leaveTimeTotal.days;
        this.hours = leaveTimeTotal.hours;
    }

    function Balance(request, userInfo)
    {
        return { request: request, userInfo: userInfo };
    }

    function LeaveTypeWithComment(request)
    {
        if(request.requestChange)
            request = request.requestChange;
        this.leaveType = LEAVE_TYPES[request.leaveTypeId] ? LEAVE_TYPES[request.leaveTypeId] : LEAVE_TYPES[LEAVE_TYPES_ORDER[0]];
        this.comment = request.comment ? request.comment : "";
    }

    function period_render(o, val)
    {
        var firstDate = val.firstDate.parseJSONDate();
        var lastDate = val.lastDate.parseJSONDate();

        var text;

        if (firstDate.isSameDate(lastDate))
            text = firstDate.format("M d, Y");
        else if (firstDate.getFullYear() == lastDate.getFullYear())
            text = firstDate.format("M d") + " &ndash; " + lastDate.format("M d, Y");
        else if (config.showFullPeriod)
            text = firstDate.format("M d, Y") + " &ndash; " + lastDate.format("M d, Y");
        else
            text = firstDate.format("M d") + " &ndash; " + lastDate.format("M d, Y");

        var content;
        if (val.clickable)
            content = createSpan('editButton', text);
        else
            content = text;

        if(val.past)
        {
            content = createDiv("pastText", messageResource.getMessage("requests_table.period_cell.past_text")) + content;
        }

        return content;

    }

    function getStatusText(statusCode)
    {
        return statusTextMap[ statusCode ];
    }

    function status_render(o, val)
    {
        var byWhom = val.approverInfo ?
                     messageResource.getMessage('actiplans.request_status.by_user',
                                                [UserNameFormat.formatFML(val.approverInfo.firstName, val.approverInfo.lastName, val.approverInfo.middleInitial)]) :
                     val.code == LeaveTimeRequestStatus.APPROVED && !val.waitingDeletion ?
                     messageResource.getMessage('actiplans.request_status.automatic_approval') : '';

        var commentsButton = '';
        var statusMessage = '';
        var moireForCommentButton = '';
        if (val.statusMessage)
        {
            statusMessage = ' - ' + escapeHtmlSymbols(val.statusMessage);
            commentsButton = createDiv('commentsButton', '');
            moireForCommentButton = createDiv('moireForCommentButtonContainer', createDiv('moireForCommentButton', '&nbsp;'));
        }
        var detailsPart = byWhom ? createDiv('details', escapeHtmlSymbols(byWhom) + statusMessage + moireForCommentButton) : '';


        var status = val.waitingDeletion ? messageResource.getMessage("actiplans.request_status.waiting_for_deletion_approval") : getStatusText(val.code);
        var when = val.statusDate.parseJSONDate().format("M d, Y");
        var className;
        if (val.waitingDeletion)
        {
            className = "waitingDeletion"
        }
        else
        {
            switch (val.code)
            {
                case LeaveTimeRequestStatus.REQUESTED:
                    className = 'requested';
                    break;
                case LeaveTimeRequestStatus.APPROVED:
                    className = 'approved';
                    break;
                case LeaveTimeRequestStatus.REJECTED:
                    className = 'rejected';
                    break;
            }
        }
        var statusDiv = createDiv('statusWrapper ' + className,  status + createDiv('when', when) + detailsPart + commentsButton);

        return createDiv('statusContent',
                            "<table><tr><td class='statusIconContainer'><div class='statusIcon'></div>" +
                            "           </td><td>"+ statusDiv +"</td>" +
                            "</tr></table>");

    }

    function leaveType_render(o, val)
    {
        var divForPrint = createDiv('for-print-only', createImg(val.leaveType.getPrintIconPath()));
        var commentsButton = '';
        var moireForCommentButton = '';

        if (val.comment)
        {
            commentsButton = createDiv('commentsButton', '');
            moireForCommentButton = createDiv('moireForCommentButtonContainer', createDiv('moireForCommentButton', '&nbsp;'));
        }

        return createDiv('leaveTypeInfo',
                         createImg(val.leaveType.getIconPath())
                             + divForPrint
                             + createSpan('leaveTypeName', escapeHtmlSymbols(val.leaveType.name))
                             + (val.comment != "" ? createDiv('comment', escapeHtmlSymbols(val.comment) + moireForCommentButton) : "")
                             + commentsButton
        );
    }

    function leaveTime_render(o, val)
    {
        var formatter = val.days == 1 ? oneDayFormatter : daysFormatter;
        return createSpan( 'timeRequested', formatter.format( val.days ) + ' (' + hoursFormatter.format( val.hours ) + ')' );
    }


    function balance_render(o, val)
    {
        var request = val.request;
        var userInfo = val.userInfo;
        var isPtoOn = config.ptoCalculationEnabled && userInfo.ptoEnabled;
        var isSickDaysOn = config.sickCalculationEnabled && userInfo.sickEnabled;
        var balanceValue = LeaveTimeRequestUtils.getBalanceValueFromRequestWithChanges(request, userInfo);
        var balanceView = LeaveTimeRequestUtils.viewBalanceAsType(request, isPtoOn, isSickDaysOn, balanceValue.endBeforeHireDate);
        switch (balanceView)
        {
            case LeaveTimeRequestUtils.ViewBalanceAs.DOES_NOT_AFFECT_BALANCE:
            case LeaveTimeRequestUtils.ViewBalanceAs.BALANCE_ON_START_END:
            case LeaveTimeRequestUtils.ViewBalanceAs.DO_NOT_VIEW:
                return getBalancesOnStartEndData(o, balanceValue, isPtoOn, isSickDaysOn);
            case LeaveTimeRequestUtils.ViewBalanceAs.CURRENT_BALANCE_BEFORE_AFTER_APPROVAL:
            case LeaveTimeRequestUtils.ViewBalanceAs.DOES_NOT_AFFECT_CURRENT_BALANCE:
                return getLeaveBalanceTodayChangedData(balanceValue.changedTodayBalance, isPtoOn, isSickDaysOn, ptoFormatter, sickFormatter);
        }
    }

    function getBalancesOnStartEndData(o, val, isPtoOn, isSickDaysOn)
    {
        switch (LEAVE_TYPES[o.aData.request.leaveTypeId].balanceType)
        {
            case BalanceType.PTO:
                if (isPtoOn)
                {
                    return getLeaveBalanceData(val, ptoFormatter, val.startBeforeHireDate);
                }
                else
                {
                    return getAsNotApplicable();
                }
                break;
            case BalanceType.SICK:
                if (isSickDaysOn)
                {
                    return getLeaveBalanceData(val, sickFormatter, val.startBeforeHireDate) +
                           (val.endBeforeHireDate ? "" : createDiv("sickSpan", "Sick Days Balance"));
                }
                else
                {
                    return getAsNotApplicable();
                }
                break;
            default:
                return getAsNotApplicable();
        }
    }

    function getLeaveBalanceData( val, formatter, beforeHireDate )
    {
        var before = val.before;
        var after = val.after;
        if (beforeHireDate)
        {
            if (!before) before = 0;
            if (!after) after = 0;
        }

        var hasBefore = val.before != null;
        var hasAfter = val.after != null;
        if ( hasBefore || hasAfter )
        {
            var beforeClasses = 'before' + ( hasBefore ? (formatter.isNegativePTO(val.before) ? ' negative' : '') : " beforeNA");
            var afterClasses = 'after' + ( hasAfter && formatter.isNegativePTO(val.after) ? ' negative' : '');

            return createSpan(beforeClasses, hasBefore ? formatter.format(val.before) : "\u2013") +
                   createSpan(afterClasses, hasAfter ? formatter.format(val.after)  : "\u2013") ;
        }
        else
        {
            return getAsNotApplicable();
        }
    }

    function getLeaveBalanceTodayChangedData(val, isPtoEnabled, isSickEnabled, ptoFormatter, sickFormatter)
    {
        return LeaveTimeRequestUtils.getChangeTodayBalanceCellContentForLeaveRequestTables(val, isPtoEnabled, isSickEnabled, ptoFormatter, sickFormatter);
    }

    function getAsNotApplicable()
    {
        return createSpan('na', "\u2013") + createSpan('afterNa', "\u2013");
    }

    function delete_render(o, val)
    {
        if (val.past && !config.canEditPastRequests)
            return "";
        else
        {
            var cls, titleKey;
            if (val.waitingDeletion)
            {
                cls = "undoButton";
                titleKey = "my_schedule.page.leave_requests_table.cancel_deletion";
            }
            else if (val.changedRequest)
            {
                cls = "undoButton";
                titleKey = "my_schedule.page.leave_requests_table.cancel_modification";
            }
            else
            {
                cls = "deleteButton";
                titleKey = "my_schedule.page.leave_requests_table.delete";
            }

            return "<div class='" + cls + "' title='" + messageResource.getMessage(titleKey) + "'></div>";
        }
    }

    function select_render(o, val)
    {
        return '<input type="checkbox">';
    }

    function select_createdCell(nTd, sData, oData, iRow, iCol)
    {
        var row = $(nTd).parent();
        var requestId = oData.request.id;
        var checkbox = $(nTd).find(':checkbox');

        if (allSelectedRequestIdsSet[requestId])
        {
            row.addClass('selected');
            checkbox.attr('checked', true);
        }

        checkbox.click( function()
        {
            if (this.checked)
            {
                allSelectedRequestIdsSet[requestId] = true;
                row.addClass('selected');
            }
            else
            {
                delete allSelectedRequestIdsSet[requestId];
                row.removeClass('selected');
            }

            fireVisibleSelectionChanged();
        });
    }

    /**
     * Prevents buttons placed in table header from triggering sorting controls.
     * @param buttonElement
     */
    function headerButtonShallNotTriggerSorting(buttonElement)
    {
        $(buttonElement)
                .click( function(event) { event.stopPropagation(); } )    // this prevents sorting change
                .mousedown( function(event) { event.preventDefault(); }); // this prevents focusing the column header
    }

    function createSelector( panelConstructor, data, renderTo )
    {
        var selector = new panelConstructor({
            data: data,
            showResetButton: true
        });

        var button = new AT.SelectorDropdownButton({
            renderTo: $(renderTo)[0],
            selectorPanel: selector,
            additionalMenuCls: "ap-selector-menu"
        });

        button.on("menuhide", function() {
            table.fnDraw();
            selector.setInitialValuesFromCurrent();
        });

        headerButtonShallNotTriggerSorting(button.el.dom);

        return selector;
    }

    function getFilterValue( selector )
    {
        if (!selector)
            return null; // filter is not enabled

        if (!selector.isCustomOptionSelected())
            return null; // 'all' option selected

        var selection = selector.getSelection();
        var selectedIds = [];

        $.each(selection, function() {
            selectedIds.push(this.id);
        });

        return selectedIds;
    }

    function initUserGroupFilter()
    {
        userGroupSelector = createSelector( AT.GroupsSelectorPanel,
                                                config.userGroups,
                                                'th.employeeCell .filterButton' );
    }

    function getCurrentUserGroupFilter()
    {
        return getFilterValue(userGroupSelector);
    }

    function initLeaveTypeFilter()
    {
        leaveTypeSelector = createSelector( AT.APLeaveTypesSelectorPanel, LEAVE_TYPES_ORDERED,
                                            'th.leaveTypeCell .filterButton' );
    }

    function getCurrentLeaveTypeFilter()
    {
        return getFilterValue(leaveTypeSelector);
    }

    function initStatusFilter()
    {
        var options = '';

        function addOption(value, text)
        {
            options += '<option value="' + value + '">' + text + '</option>';
        }

        var statuses = [LeaveTimeRequestStatus.REQUESTED, LeaveTimeRequestStatus.APPROVED];

        if (config.showRejected)
            statuses.push(LeaveTimeRequestStatus.REJECTED);

        addOption(statuses.toString(), 'All statuses'); // TODO: take this string from resources

        for (var i = 0; i < statuses.length; i++)
            addOption(statuses[i], getStatusText(i));

        var select = table.find('th.statusCell select').html(options); // TODO: do we really need to create select element first?

        statusFilterButton = new AT.DropdownListButton({ applyTo: select[0] });
        statusFilterButton.addListener('change', function() { table.fnDraw() });

        headerButtonShallNotTriggerSorting(statusFilterButton.el.dom);
    }

    function getCurrentStatusFilter()
    {
        if (!statusFilterButton)
            return null; // filter is not enabled

        var selected = statusFilterButton.getValue();
        return selected ? selected.split(',') : null;
    }

    function fireVisibleSelectionChanged()
    {
        if (config.onVisibleSelectionChanged)
            config.onVisibleSelectionChanged( self.getVisibleSelectedRequests() );
    }

    function BalanceTypeFilter( renderTo, onChange, balanceTypeFilter )
    {
        var leaveBalanceFilterTypes = [];

        $.each( LeaveBalanceFilter, function() {
            leaveBalanceFilterTypes.push( this );
        } );

        var leaveBalanceFilterTexts = {
            ALL: messageResource.getMessage( "actiplans.balance_type_filter.all" ),
            PTO: messageResource.getMessage( "actiplans.balance_type_filter.pto" ),
            SICK: messageResource.getMessage( "actiplans.balance_type_filter.sick" )
        };

        $.each( leaveBalanceFilterTypes, function() {
            renderTo.append( $('<option/>').val( this ).text( leaveBalanceFilterTexts[this] ) );
        });

        var button = new AT.DropdownListButton({ applyTo: renderTo[0] });

        function balanceTypeFilterToggleClass(value){
            $(".balanceTypeFilterBlock").toggleClass("active", value != LeaveBalanceFilter.ALL);
        }
        balanceTypeFilterToggleClass(balanceTypeFilter);
        this.button = button;

        button.addListener('change', function() {
            var value = button.getValue();
            onChange( value );
            balanceTypeFilterToggleClass(value);
        });

        this.setActiveItemByValue = function ( value, suppressOnChangeEvent ){
            this.button.setActiveItemByValue( value, suppressOnChangeEvent );
        };

        this.setActiveItemByValue(balanceTypeFilter, true);
    }

    function initPastRequestsFilter()
    {
        var $filterPastRequestsCheckbox = $("#filterPastRequestsCheckbox").off();
        var prevValue = $.cookie("actiPLANS-mySchedule-requests-showPastApprovedRejectedRequests");
        prevValue = _.isUndefined(prevValue) ? $filterPastRequestsCheckbox.is(":checked") : (prevValue == "true");
        showPastApprovedRejectedRequestInPast = prevValue;

        $filterPastRequestsCheckbox
                .prop("checked", prevValue)
                .on("change", function()
                {
                    showPastApprovedRejectedRequestInPast = this.checked;
                    $.cookie("actiPLANS-mySchedule-requests-showPastApprovedRejectedRequests", this.checked);
                    table.fnDraw();
                });
    }

    function init()
    {
        table = $(config.renderTo);
        var isPTO = config.ptoCalculationEnabled && config.isUserPTOControlOn;
        var isSick = config.sickCalculationEnabled && config.isUserSickControlOn;
        isBalanceColumnEnabled = isPTO || isSick;
        if ( isPTO && isSick )
        {
            table.find( ".balanceColumnHeaderContainer" ).hide();
            table.find( ".balanceBeforeAfterSpan" ).show();
            table.find( ".balanceTypeFilterBlock" ).show();
            table.find( "thead > tr" ).addClass( "withBalanceTypeFilter" );
            var balanceFilter = $.cookie( "actiPLANS-mySchedule-requests-balanceTypeFilter" );
            balanceFilter = balanceFilter ? balanceFilter : LeaveBalanceFilter.ALL;
            var balanceTypeFilterCallback = function( value )
            {
                table.fnDraw();
                $.cookie( "actiPLANS-mySchedule-requests-balanceTypeFilter", value );
            };
            balanceTypeFilter = config.balanceTypeFilter ?
                                config.balanceTypeFilter :
                                new BalanceTypeFilter( $( ".balanceTypeFilter" ), balanceTypeFilterCallback, balanceFilter );

        }
        else if ( isBalanceColumnEnabled )
        {
            var textKey = isPTO ? "my_schedule.page.leave_requests_table.pto" : "my_schedule.page.leave_requests_table.sick";
            table.find( ".balanceColumnHeaderContainer" ).text( messageResource.getMessage( textKey ) + " " )
                    .append( $( createSpan( "beforeAfterSpan", messageResource.getMessage( "my_schedule.page.leave_requests_table.before_after_req" ) ) ) ).show();
            table.find( ".balanceBeforeAfterSpan" ).hide();
            table.find( ".balanceTypeFilterBlock" ).hide();
            balanceTypeFilter = null;
        }
        initPastRequestsFilter();

        daysFormatter =
            new NumberFormatter(SystemSettings.decimalSeparator, SystemSettings.digitGroupingSymbol, 1, true, '', ' days');
        oneDayFormatter =
            new NumberFormatter(SystemSettings.decimalSeparator, SystemSettings.digitGroupingSymbol, 1, true, '', ' day');
        hoursFormatter =
            new NumberFormatter(SystemSettings.decimalSeparator, SystemSettings.digitGroupingSymbol, 2, true, '', 'h');
        ptoFormatter = new PTOFormatter().init( config.ptoHoursPerDay );
        sickFormatter = new SickFormatter().init( config.ptoHoursPerDay );

        // 'leave-type' type sorting plugin
        $.extend( $.fn.dataTableExt.oSort,
                  {
                      'leave-type-asc': function (a, b) {
                          return a.leaveType.position - b.leaveType.position;
                      },
                      'leave-type-desc': function (a, b) {
                          return b.leaveType.position - a.leaveType.position;
                      },
                      'status-cell-asc': function (a, b) {
                          var e = a.toString() + ' ' + a.firstDate;
                          var h = b.toString() + ' ' + b.firstDate;
                          return ( e < h ) ? -1 :
                                 ( e > h ) ?  1 : 0;
                      },
                      'status-cell-desc': function (a, b) {
                          // changed order on date sub-sorting
                          var e = a.toString() + ' ' + b.firstDate;
                          var h = b.toString() + ' ' + a.firstDate;
                          return ( e < h ) ?  1 :
                                 ( e > h ) ? -1 : 0;
                      }
                  });

        // this function is using for filtering rows
        $.fn.dataTableExt.afnFiltering = [];
        $.fn.dataTableExt.afnFiltering.push(
                function(oSettings, aData, iDataIndex)
                {

                    var requestStatus = aData[5];
                    var isPastRequestFilteredByStatus = false;
                    // filter past approved and rejected requests
                    if (requestStatus.past && !requestStatus.waitingDeletion)
                    {
                        if(requestStatus.code == LeaveTimeRequestStatus.APPROVED || requestStatus.code == LeaveTimeRequestStatus.REJECTED)
                        {
                            // check if need to show spproved or rejected request up to left allowed date
                            isPastRequestFilteredByStatus = showPastApprovedRejectedRequestInPast ?
                                                            requestStatus.statusDate.parseJSONDate() <= config.leftDateToShowPastFilteredRequests : true;
                        }
                    }
                    if(isPastRequestFilteredByStatus) {
                        return false;
                    }

                    if (!isBalanceColumnEnabled || !balanceTypeFilter)
                        return true;
                    if (balanceTypeFilter)
                    {
                        var balanceType = aData[1].leaveType.balanceType;
                        switch (balanceTypeFilter.button.getValue())
                        {
                            case LeaveBalanceFilter.PTO:
                                return balanceType == BalanceType.PTO && aData[4].request.balanceData;
                            case LeaveBalanceFilter.SICK:
                                return balanceType == BalanceType.SICK && aData[4].request.balanceData;
                        }
                    }
                    return true;
                }
        );

        var data = [];
        for( var i = 0; i < config.requests.length; i++ )
        {
           data.push({request: config.requests[i]});
        }

        var leaveRequestsTableTexts = DataTablesHelpers.defaultTexts;
        leaveRequestsTableTexts.sEmptyTable = messageResource.getMessage('my_schedule.js.empty.table.tooltip');

        var employeeColumn         = 0;
        var leaveTypeColumn        = 1;
        var periodColumn           = 2;
        var leaveTimeColumn        = 3;
        var balanceColumn          = 4;
        var statusColumn           = 5;
        var deleteColumn           = 6;
        var selectColumn           = 7;
        var userGroupColumn    = 8; // invisible

        var tableSource = {
            aaData: data,
            sAjaxSource: null, // no need to specify source url, because we use our makeRpcCall method below,
                               // which already knows the url
            bServerSide: config.requestsWithUsersRpcMethod ? true : false,

            // TODO: this function must be reusable & more readable
            fnServerData: function (sSource, aoData, fnCallback, oSettings)
            {
                oSettings.jqXHR = makeRpcCall( config.requestsWithUsersRpcMethod,
                                               [(function()
                                               {
                                                   var result = {};
                                                   for (var i = 0; i < aoData.length; i++)
                                                       result[aoData[i].name] = aoData[i].value;

                                                   // this is specific for leave requests table
                                                   result.statusFilter = getCurrentStatusFilter();
                                                   result.leaveTypeFilter = getCurrentLeaveTypeFilter();
                                                   result.userGroupFilter = getCurrentUserGroupFilter();

                                                   return result;
                                               })()],
                                               fnCallback,
                                               null );
            },
            aaSorting: [[periodColumn,'asc']],
            aoColumnDefs: [
                { aTargets: [employeeColumn],
                    sName: 'employee',
                    mData: function (source) { return new Employee(source.userInfo) },
                    sClass: 'employeeCell',
                    bVisible: config.showEmployeeColumn },
                { aTargets: [leaveTypeColumn],
                    sName: 'leaveType',
                    mData: function (source) { return new LeaveTypeWithComment(source.request) }, // the page will be refreshed
                    fnRender: leaveType_render,
                    bUseRendered: false,
                    sType: 'leave-type',
                    sClass: 'leaveTypeCell' },
                { aTargets: [periodColumn],
                    sName: 'period',
                    mData: function (source) { return new Period(source.request) },
                    fnRender: period_render,
                    bUseRendered: false,
                    sClass: 'periodCell' },
                { aTargets: [leaveTimeColumn],
                    sName: 'leaveTime',
                    mData: function (source) { return new RequestedTime(source.request) },
                    fnRender: leaveTime_render,
                    bUseRendered: false,
                    sClass: 'leaveTimeCell',
                    bSortable: false },
                { aTargets: [balanceColumn],
                    sName: 'balance',
                    mData: function (source) { return new Balance(source.request, config.userInfo) },
                    fnRender: balance_render,
                    bUseRendered: false,
                    sClass: 'balanceCell',
                    bSortable: false,
                    bVisible: isBalanceColumnEnabled },
                { aTargets: [statusColumn],
                    sName: 'status',
                    mData: function (source) { return new Status(source.request) },
                    sType: 'status-cell',
                    fnRender: status_render,
                    bUseRendered: false,
                    sClass: 'statusCell' },
                { aTargets: [deleteColumn],
                    sName: 'delete',
                    mData: function (source) { return {past: source.request.past, waitingDeletion: source.request.waitingDeletion, changedRequest: source.request.requestChange }},
                    fnRender: delete_render,
                    bUseRendered: false,
                    sClass: 'deleteCell',
                    bSortable: false,
                    bVisible: config.showDeleteColumn },
                { aTargets: [selectColumn],
                    sName: 'select',
                    mData: null,
                    fnRender: select_render,
                    fnCreatedCell: select_createdCell,
                    bUseRendered: false,
                    sClass: 'selectCell',
                    bSortable: false,
                    bVisible: config.showSelectColumn },
                { aTargets: [userGroupColumn],
                    sName: 'userGroup',
                    mData: null,
                    sClass: 'agCell',
                    bSortable: true,
                    bVisible: true // We hide it later. We need DataTables to attach click handler to it,
                                   // which we reattach to the element in the 'employee' column.
                }
            ],
            bPaginate: Boolean(config.paginate),
            sPaginationType: 'full_numbers',
            sDom: config.paginate ? 'tilp' : 't',
            oLanguage: leaveRequestsTableTexts,
            fnInfoCallback: DataTablesHelpers.createInfoStringRenderer(
                                                    'leave_time_planning.1_request', 'leave_time_planning.n_requests'),
            bStateSave: true,
            iCookieDuration: 400000000,

            bFilter: true,

            // auto-width is not needed, it only introduces glitches when re-creating the table
            bAutoWidth: false,

            // this is needed to prevent RPC call on table initialization, because filters are not ready yet
            // the RPC call will be performed later with fnDraw
            iDeferLoading: 0,

            fnCreatedRow: function(nRow, aData, iDataIndex)
            {
                var $row = $(nRow);
                $row.addClass('requestRow')
                        .find('.leaveTypeCell, .periodCell');

                if(aData.request.past)
                {
                    $row.addClass("pastRequest");

                    if (aData.request.waitingDeletion)
                    {
                        $row.addClass("waitingDeletion")
                    }
                }

                var editable = config.canEditPastRequests || !aData.request.past;
                var notEditableTooltip = null;
                if(aData.request.past)
                {
                    if (config.wasATIntegrated && config.userInfo.activeInAT)
                        notEditableTooltip = wasAtIntegratedTooltip;
                    if (!config.canEditPastRequests && !config.wasATIntegrated || (config.wasATIntegrated && !config.userInfo.activeInAT))
                        notEditableTooltip = cannotEditPastRequestTooltip;
                }
                if(notEditableTooltip != null)
                {
                    var leaveTypeCell = $row.find('.leaveTypeCell');
                    var periodCell = $row.find('.periodCell');
                    notEditableTooltip.bind([leaveTypeCell, periodCell]);
                }
                if (editable)
                {
                    $row.addClass("editable");

                    $row.find('.leaveTypeCell, .periodCell').click( function() { if ( !table.locked ) $row.addClass('active'); } );

                    $row.find('.leaveTypeCell')
                            .mouseenter( function() { if ( !table.locked ) $row.find(".leaveTypeCell" ).addClass('hover') } )
                            .mouseleave( function() { if ( !table.locked ) $row.find(".leaveTypeCell" ).removeClass('hover') } )
                            .click( function() {
                                if ( !table.locked )
                                {
                                    var cell = $( this );
                                    cell.addClass("dialogSelected");
                                    config.onRequestLeaveTypeEditClick(LeaveTimeRequestUtils.mergeChangesToRequest(aData.request), this, function() { cell.removeClass('dialogSelected'); } );
                                }
                            } );

                    $row.find('.periodCell')
                            .mouseenter( function() { if ( !table.locked ) $row.find(".periodCell" ).addClass('hover') } )
                            .mouseleave( function() { if ( !table.locked ) $row.find(".periodCell" ).removeClass('hover') } )
                            .click( function() {
                                if ( !table.locked )
                                {
                                    var cell = $( this );
                                    cell.addClass('dialogSelected');
                                    config.onRequestPeriodEditClick( LeaveTimeRequestUtils.mergeChangesToRequest(aData.request), this, function() { cell.removeClass('dialogSelected'); } );
                                }
                            } );

                    $row.find('.deleteCell')
                            .mouseenter( function() { if ( !table.locked ) $row.addClass('active') } )
                            .mouseleave( function() { if ( !table.locked ) $row.removeClass('active') } )
                            .click( function() {
                                if ( !table.locked )
                                    config.onRequestDeleteClick(aData.request, this);
                            } );

                    $row.find('.commentsButton')
                            .click( function() {
                                if ( table.locked )
                                    return;

                                $row.addClass('active');
                            });
                }

                $row.find('.statusCell .commentsButton').click( function() {
                    new DropdownDialog({
                            wrapperClass: 'statusInfoTooltip commentPopup leaveRequestTableTooltip',
                            dialogDiv: createDiv('commentsDialog', createDiv('commentsDialogText', escapeHtmlSymbols(aData.request.statusMessage))),
                            hideButtons: true,
                            isWithBeak: true,
                            hideOnResize: true,
                            onClose: function() {
                                $row.removeClass('active');
                            }
                        }).showAt(this, {offset: "30px 10px", my: "right top", at: "left bottom", collision: "none"});
                });

                $row.find('.leaveTypeCell .commentsButton').click( function(event) {
                    new DropdownDialog({
                        wrapperClass: 'leaveTypeTooltip commentPopup leaveRequestTableTooltip',
                        dialogDiv: createDiv('commentsDialog', createDiv('commentsDialogText', escapeHtmlSymbols(aData.request.comment))),
                        hideButtons: true,
                        isWithBeak: true,
                        hideOnResize: true,
                        onClose: function() {
                            $row.removeClass('active');
                        }
                    }
                    ).showAt(this, {offset: "-17px 10px", my: "left top", at: "left bottom", collision: "none"});
                    event.stopPropagation();
                });
            },

            fnDrawCallback: function()
            {
                table.find('tr.requestRow.lastRow').removeClass('lastRow');
                var lastRow = table.find('tr.requestRow:last').addClass('lastRow');
                lastRow.find('td:first').addClass("firstCell");
                lastRow.find('td:last').addClass("lastCell");
                // although selection might remain unchanged, I currently see no reason to detect that
                fireVisibleSelectionChanged();
            },

            fnHeaderCallback: function( nHead )
            {
                // update AG sorting indicator in the 'employee' header cell

                var agSortIndicator = $(nHead).find("th.employeeCell .sortingIndicator_alt");
                var sorting = table.fnSettings().aaSorting;

                if( sorting.length > 0 )
                {
                    // currently only single-column sort is supported, although DataTables can handle more
                    agSortIndicator.toggleClass( 'asc',  sorting[0][0] == userGroupColumn && sorting[0][1] == 'asc' );
                    agSortIndicator.toggleClass( 'desc', sorting[0][0] == userGroupColumn && sorting[0][1] == 'desc' );
                }
                else
                {
                    // no sorting at all (user clicked with shift key on the currently sorted column)
                    // (probably we'll disable this behavior later, but for now we should handle it)
                    agSortIndicator.removeClass( 'asc' );
                    agSortIndicator.toggleClass( 'desc' );
                }
            },

            fnStateLoadParams: function(oSettings, oData)
            {
                // do not load column visibility flags from cookies, column visibility is determined by the config
                oData.abVisCols = [];
            },

            fnStateSaveParams: function(oSettings, oData)
            {
                // do not store column visibility flags to cookies
                oData.abVisCols = [];
            }
        };

        table.dataTable(tableSource);

        // reattach invisible 'user group' column header cell click handler to the label in the 'employee' cell
        var agHeaderCell = table.find("th.agCell");
        table.find("th.employeeCell .sortingLink_alt").click( agHeaderCell.data("events").click[0].handler );
        agHeaderCell.off("click");

        // now we may hide 'user group' column
        table.fnSetColumnVis( userGroupColumn, false, false );

        DataTablesHelpers.narrowColumnHeaderClickAreas.call( table );

        if (config.enableStatusFilter)
            initStatusFilter();

        if (config.enableLeaveTypeFilter)
            initLeaveTypeFilter();

        if (config.enableUserGroupFilter)
            initUserGroupFilter();

        table.find('.columnCheckbox').click( function() {
            if (this.checked) {
                table.find("tr.requestRow").addClass('selected');
                table.find('td.selectCell :checkbox').attr('checked', true);
            }
            else {
                table.find('td.selectCell :checked').removeAttr('checked');
                table.find('tr.requestRow').removeClass('selected');
                allSelectedRequestIdsSet = {};
            }

            // although selection might remain unchanged, I currently see no reason to detect that
            fireVisibleSelectionChanged();
        });

        table.fnDraw();
    }
}