/**
 * Note: methods and fields which starts from '_' must not be used from the outside
 *
 * @constructor
 */

function LeaveTableUpdateController()
{
    this._lastScrollingDirection = LeaveTableUpdateController._NO_SCROLLING_DIRECTION;
    this._updateSchedulesRequestObject = null;
    this._loadingSchedules = false;
    this._updateDelayTimer = null;
}

LeaveTableUpdateController._NO_SCROLLING_DIRECTION = 1;
LeaveTableUpdateController._NEXT_WEEK_SCROLLING = 2;
LeaveTableUpdateController._PREVIOUS_WEEK_SCROLLING = 3;

/**
 * @param dateRangeCalculator object of DateRangeCalculator
 * @param tryUpdateTablesFunc function( dateFrom ), which return:
 *         true if tables was rendered
 *              (only if visible date range (dateFrom, dateFrom + visibleDays)
 *               contained in (pageData.dateFrom, pageData.dateTo)),
 *         false otherwise
 * @param rpcMethod rpc method name
 * @param rpcParamsFunc function( dateFrom, dateRange ), which return rpc method params
 * @param onSuccessRequestCallback function( data ), which calls when request is success
 * @param stateController object of StateController
 *
 * @return this object
 */
LeaveTableUpdateController.prototype.init = function ( dateRangeCalculator, tryUpdateTablesFunc, rpcMethod, rpcParamsFunc, onSuccessRequestCallback, stateController )
{
    this._dateRangeCalculator = dateRangeCalculator;
    this._tryUpdateTablesFunc = tryUpdateTablesFunc;

    this._rpcMethod = rpcMethod;
    this._rpcParamsFunc = rpcParamsFunc;
    this._onSuccessRequestCallback = onSuccessRequestCallback;
    this._stateController = stateController;
    this._renderWhileLoading = true;

    this._spinnerController = new SpinnerController();

    return this;
};

LeaveTableUpdateController.prototype.abortUpdateSchedulesRequest = function ()
{
    this._abortUpdateSchedulesRequest( false );
};

LeaveTableUpdateController.prototype._abortUpdateSchedulesRequest = function ( isBeforeSchedulesUpdate )
{
    if( this._updateDelayTimer != null )
    {
        clearTimeout( this._updateDelayTimer );
        this._updateDelayTimer = null;
        if( !isBeforeSchedulesUpdate )
        {
            this._renderWhileLoading = true;
            this._stateController.loadState();
        }
    }

    this._loadingSchedules = false;
    if( this._updateSchedulesRequestObject != null )
        if( isBeforeSchedulesUpdate )
            this._updateSchedulesRequestObject.abort( 'abortBeforeSchedulesUpdate' );
        else
            this._updateSchedulesRequestObject.abort();
};

LeaveTableUpdateController.prototype.loadPreviousWeek = function ( lastVisibleDateFrom )
{
    var dateFrom = lastVisibleDateFrom.getNDaysBefore( 7 );
    this._onDateRangeChanged( dateFrom,
                              this._dateRangeCalculator.getDateRangeWithLeftScrolling( dateFrom ),
                              'spinnerPlaceholderPrevWeek',
                              LeaveTableUpdateController._PREVIOUS_WEEK_SCROLLING
    );
};

LeaveTableUpdateController.prototype.loadNextWeek = function ( lastVisibleDateFrom )
{
    var dateFrom = lastVisibleDateFrom.getNDaysAfter( 7 );
    this._onDateRangeChanged(
            dateFrom,
            this._dateRangeCalculator.getDateRangeWithRightScrolling( dateFrom ),
            'spinnerPlaceholderNextWeek',
            LeaveTableUpdateController._NEXT_WEEK_SCROLLING
    );
};

LeaveTableUpdateController.prototype.loadCustomWeek = function ( date )
{
    var dateFrom = new Date( date );
    this._onDateRangeChanged(
            dateFrom,
            this._dateRangeCalculator.getDateRangeWithoutScrolling( dateFrom ),
            'spinnerPlaceholderNextWeek',
            LeaveTableUpdateController._NO_SCROLLING_DIRECTION
    );
};

LeaveTableUpdateController.prototype.simpleUpdateSchedules = function ( visibleDateFrom, spinnerContainerId )
{
    this._renderWhileLoading = false;
    this._updateSchedules( visibleDateFrom,
                           this._dateRangeCalculator.getDateRangeWithoutScrolling( visibleDateFrom ),
                           spinnerContainerId,
                           LeaveTableUpdateController._NO_SCROLLING_DIRECTION );
};

LeaveTableUpdateController.prototype.updateSchedulesWithDelay = function ( visibleDateFrom, spinnerContainerId, delay )
{
    this._abortUpdateSchedulesRequest( true );
    RpcControlManager.cancelAllAJAXRequests();
    this._renderWhileLoading = false;
    var that = this;
    // Note: if timer will be stopped, then it must be set to null
    this._updateDelayTimer = setTimeout( function ()
                                         {
                                             that.simpleUpdateSchedules( visibleDateFrom, spinnerContainerId );
                                             that._updateDelayTimer = null;
                                         }, delay );
};


LeaveTableUpdateController.prototype._onDateRangeChanged = function ( dateFrom, dateRange, spinnerContainerId, scrollingDirection )
{
    this._updateSchedules( dateFrom, dateRange, spinnerContainerId, scrollingDirection );
};

/**
 * !!!private method!!!
 *
 * Note: this._renderWhileLoading - if true then tables will be rendered if data was loaded earlier
 *                                  otherwise spinner will be started while data loading from server and when it is happened tables will be rendered
 *                                  must be true only if scrolling occurred and not started request with changed filters,
 *                                  in other cases (filters or pager was changed) old data not suitable
 *
 * @param dateFrom first visible day in the tables
 * @param dateRange date range for loading data - extended date range (see DateRangeCalculator)
 * @param spinnerContainerId container id for rendering spinner while data loading
 * @param scrollingDirection if scrollingDirection is equals to previous scrolling direction, then query will not be made, but tables will be try rendered
 */
LeaveTableUpdateController.prototype._updateSchedules = function ( dateFrom, dateRange, spinnerContainerId, scrollingDirection )
{
    RpcControlManager.cancelAllAJAXRequests(); // close active save/edit/delete/... requests before loading new data

    var isRendered = false;
    var needLoadData = true;
    if( this._lastScrollingDirection != LeaveTableUpdateController._NO_SCROLLING_DIRECTION && this._lastScrollingDirection === scrollingDirection )
    {
        if( this._loadingSchedules )
        {
            // if last scrolling direction is equals to current direction and data is loading, then no need to load data
            needLoadData = false;
        }
        // if can render old data then it can be need to load new data
        if( this._renderWhileLoading && this._tryUpdateTablesFunc( dateFrom ) ) // try to render on old data
        {
            isRendered = true;
        }
    }
    this._lastScrollingDirection = scrollingDirection;

    if( needLoadData )
    {
        this._abortUpdateSchedulesRequest( true );
        this._loadingSchedules = true;
        var that = this;
        var finallyFunc = function ()
        {
            that._loadingSchedules = false;
            that._spinnerController.stop();
        };
        this._updateSchedulesRequestObject = makeRpcCall(
                this._rpcMethod,
                this._rpcParamsFunc( dateFrom, dateRange ),
                function ( data )
                {
                    that._onSuccessRequestCallback( data );
                    if( !isRendered )
                    {
                        that._tryUpdateTablesFunc( dateFrom ); // render on new data
                    }
                    finallyFunc();

                    that._renderWhileLoading = true;
                    that._stateController.saveState();
                },
                function ( jqXHR, textStatus, errorThrown )
                {
                    finallyFunc();

                    if( textStatus == 'abortBeforeSchedulesUpdate' )
                    {
                        return true;
                    }
                    else
                    {
                        that._renderWhileLoading = true;
                        that._stateController.loadState();
                    }
                }
        );
        if( this._renderWhileLoading && !isRendered && this._tryUpdateTablesFunc( dateFrom ) ) // try to render on old data
        {
            isRendered = true;
        }
    }
    if( !isRendered )
    {
        // in any case if we can not render, we need to show a spinner
        this._spinnerController.spin( spinnerContainerId );
    }
};
