/**
 *
 * @param userInfo
 * @param requests
 * @param statuses
 * @param today
 * @param workdayDurationsSlice ArraySlice object, where workdayDurationsSlice.get(0) matches to dateFrom day
 * @param dailyPTOSlice ArraySlice object, where dailyPTOSlice.get(0) matches to dateFrom day
 * @param dailySickSlice ArraySlice object, where dailySickSlice.get(0) matches to dateFrom day
 * @constructor
 */
function UserLeaveRow( userInfo, requests, statuses, today, workdayDurationsSlice, dailyPTOSlice, dailySickSlice, currentBalanceInfo )
{
    this.userInfo = userInfo;
    LeaveRow.call( this, today, workdayDurationsSlice, this.userInfo ? this.userInfo.hireDate.parseJSONDate() : null );

    if (userInfo) this.userId = userInfo.id;
    this.requests = requests;
    this.statuses = statuses;
    this.defaultLeaveCellRenderer = new LeaveRequestCellRenderer();
    this.dailyPTOSlice = dailyPTOSlice;
    this.dailySickSlice = dailySickSlice;
    this.currentBalanceInfo = currentBalanceInfo;

    this.createOnBeforePrintHandler();
}

UserLeaveRow.prototype = _.create(LeaveRow.prototype);
UserLeaveRow.superclass = LeaveRow.prototype;

UserLeaveRow.prototype.getRequests = function()
{
    return this.requests;
};

UserLeaveRow.prototype.getUserInfo = function()
{
    return this.userInfo;
};

UserLeaveRow.prototype.getUserId = function()
{
    return this.userId;
};

UserLeaveRow.prototype.getDailyPTOSlice = function ()
{
    return this.dailyPTOSlice;
};

UserLeaveRow.prototype.isPTOUser = function ()
{
    return this.dailyPTOSlice.getArray() ? true : false;
};

UserLeaveRow.prototype.getDailySickSlice = function ()
{
    return this.dailySickSlice;
};

UserLeaveRow.prototype.isSickUser = function ()
{
    return this.dailySickSlice.getArray() ? true : false;
};

//noinspection JSUnusedLocalSymbols
UserLeaveRow.prototype.createCellRenderer = function( cell )
{
    return this.getDefaultLeaveCellRenderer();
};

UserLeaveRow.prototype.getDefaultLeaveCellRenderer = function()
{
    return this.defaultLeaveCellRenderer;
};

UserLeaveRow.prototype.isStatusCell = function(cell)
{
    return !_.isNull(cell.getValue()) && !_.isNull(cell.getValue().statusData);
};

UserLeaveRow.prototype.addLeaveCells = function ( dateFrom, dateTo, corporateScheduleSlice, ptoHoursPerDay )
{
    var from = dateFrom;

    var requestIndex = 0;
    var statusIndex = 0;
    var prevRequest = null;

    // Merge arrays of requests and statuses into row cells
    while (requestIndex < this.requests.length || statusIndex < this.statuses.length)
    {
        var request = this.requests[requestIndex];
        var status = this.statuses[statusIndex];

        var requestFirstDate = dateTo.nextDay();
        var requestLastDate = dateTo.nextDay();
        var statusDate = dateTo.nextDay();
        if (request)
        {
            requestFirstDate = request.firstDate.parseJSONDate();
            requestLastDate = request.lastDate.parseJSONDate();
        }
        if (status)
            statusDate = status.date.parseJSONDate();

        if (request && requestFirstDate - statusDate <= 0)
        {
            // Handle leave time request

            requestIndex++;

            // Skip overlapping requests
            if (prevRequest)
            {
                var prevRequestLastDate = prevRequest.lastDate.parseJSONDate();
                if (requestFirstDate <= prevRequestLastDate)
                {
                    continue;
                }
            }
            if (request.status == LeaveTimeRequestStatus.REJECTED)
                continue;
            if (requestLastDate < dateFrom)
                continue;
            if (requestFirstDate > dateTo)
            {
                // Fast-forward to requests array end
                requestIndex = this.requests.length;
                continue;
            }
            this.addEmptyDayCells(from, requestFirstDate.prevDay());
            this.addDayCellsForRequest(request, dateFrom, dateTo, this.getWorkdayDurationsSlice(), corporateScheduleSlice, ptoHoursPerDay);
            from = requestLastDate.nextDay();

            prevRequest = request;
        }
        else if (status)
        {
            // Handle leave status
            statusIndex++;

            if (statusDate < from)
                continue;
            if (statusDate > dateTo)
            {
                // Fast-forward to statuses array end
                statusIndex = this.statuses.length;
                continue;
            }

            this.addEmptyDayCells(from, statusDate.prevDay());
            this.addDayCellsForStatus(status, dateFrom);
            from = statusDate.nextDay();
        }
        else break;
    }

    if( from <= dateTo )
        this.addEmptyDayCells( from, dateTo );
};


UserLeaveRow.prototype.addEmptyDayCells = function(dateFrom, dateTo)
{
    //noinspection JSUnresolvedFunction
    for( var d = new DatesIterator( dateFrom, dateTo ); d.hasNext(); d.next() )
        this.addCell( null );
};

UserLeaveRow.prototype.addDayCell = function( requestedDuration, userWorkdayDuration, corporateWorkdayDuration, ptoHoursPerDay, request, status )
{
    var percentDuration = requestedDuration < 0 ? userWorkdayDuration : Math.min( requestedDuration, userWorkdayDuration );

    // leave percent for non-working days is calculated from the pto workday duration.
    // Otherwise all non-working days won't be covered with leave image
    var workdayDuration = userWorkdayDuration;
    if( workdayDuration <= 0 )
        workdayDuration = corporateWorkdayDuration;
    if( workdayDuration <= 0 )
        workdayDuration = ptoHoursPerDay * 60;

    if( percentDuration == 0 )
        percentDuration = requestedDuration < 0 ? workdayDuration : Math.min( requestedDuration, workdayDuration );

    var leavePercent = percentDuration / workdayDuration;

    this.addCell( new LeaveRowDayInfo( requestedDuration, leavePercent, request, status ) );
};

UserLeaveRow.prototype.addDayCellsForRequest = function ( request, dateFrom, dateTo, workdayDurationsSlice, corporateScheduleSlice, ptoHoursPerDay )
{
    var periodLength = dateTo.offsetInDaysFrom(dateFrom) + 1; // TODO: This can be calculated only once per table
    var firstDate = request.firstDate.parseJSONDate();
    var firstDateOffset = firstDate.offsetInDaysFrom(dateFrom);
    var i = Math.max( 0, firstDateOffset );
    var stopIdx = Math.min( request.length + firstDateOffset , periodLength );

    var requestDurationsSource = request.requestChange || request;

    if( i - firstDateOffset == 0 )
    {
        this.addDayCell( requestDurationsSource.firstDayDuration, workdayDurationsSlice.get( i ), corporateScheduleSlice.get( i ), ptoHoursPerDay, request, null );
        i++;
    }

    for( ; i < stopIdx - 1; i++ )
        this.addDayCell( requestDurationsSource.middleDayDuration, workdayDurationsSlice.get( i ), corporateScheduleSlice.get( i ), ptoHoursPerDay, request, null );

    if( i < stopIdx && request.length > 1 )
        this.addDayCell( requestDurationsSource.lastDayDuration, workdayDurationsSlice.get( i ), corporateScheduleSlice.get( i ), ptoHoursPerDay, request, null );
};

UserLeaveRow.prototype.addDayCellsForStatus = function (status, dateFrom)
{
    var date = status.date.parseJSONDate();
    var dateOffset = date.offsetInDaysFrom(dateFrom);
    if (dateOffset >= 0)
        this.addCell(new LeaveRowDayInfo(null, null, null, status));
};

UserLeaveRow.prototype.createOnBeforePrintHandler = function()
{
    this.printEventHandled = false;

    var beforePrintHandler = (function(caller) {
        return function() {
            if (!caller.printEventHandled) {
                caller.generateContentForPrinting.apply(caller, arguments);
                caller.printEventHandled = true;
            }
        }
    })(this);

    if (window.onbeforeprint !== undefined)
        $(window).on('beforeprint', beforePrintHandler);
    else if (window.matchMedia)
    {
        window.matchMedia('print').addListener(function(media) {
            if (media.matches) {
                beforePrintHandler();
            }
        });
    }
}

UserLeaveRow.prototype.generateContentForPrinting = function()
{
    var cells = this.getCells();
    for( var j = 0; j < cells.length; j++ )
    {
        this.defaultLeaveCellRenderer.generateContentForPrinting(cells[j]);
    }
}

UserLeaveRow.prototype.setLeaveTypeNotFoundCallback = function(leaveTypeNotFoundCallback)
{
    this.defaultLeaveCellRenderer.setLeaveTypeNotFoundCallback(leaveTypeNotFoundCallback);
}

/**
 * Override method for set oncontextmenu handler that will prevent the showing 'right button context menu in browser'
 * @param data for day cell
 */
UserLeaveRow.prototype.addCell = function( value ) {
    UserLeaveRow.superclass.addCell.apply(this, arguments );
    var prevHandler = this.getLastCell().getCellEl().oncontextmenu;
    this.getLastCell().getCellEl().oncontextmenu = function( evt ) {
        prevHandler.apply(this, arguments);
        return false;
    };
};
/*

UserLeaveRow.prototype.accept = function( visitor, cell )
{
    if( visitor.visitUserLeaveRow )
        visitor.visitUserLeaveRow( cell );
};
*/
