/**
 *
 * @param containerEl Container element in DOM
 * @param dateFrom
 * @param dateTo
 * @param corporateScheduleSlice ArraySlice object, where corporateScheduleSlice.get(0) matches to dateFrom day
 * @param today
 * @param groupsData might be null if groups disabled
 * @param userSchedules
 * @param schedulesDataOffset offset, where userData[i].workdayDurations[schedulesDataOffset] matches to dateFrom day
 * @param leaveTypeNotFoundCallback
 * @param headerRowEl
 * @param tokensHighlighter
 * @param hasUsers
 * @param groupedBy
 * @param ptoHoursPerDay used to convert days to hours when displaying PTO balance
 * @param canEditPastRequests Indicates if editing past requests is allowed
 * @param wasAtIntegrated Indicates if actiTIME was ever integrated
 * @constructor
 */
function LeaveTimePlanningTable(
        containerEl,
        dateFrom,
        dateTo,
        corporateScheduleSlice,
        today,
        groupsData,
        userSchedules,
        schedulesDataOffset,
        leaveTypeNotFoundCallback,
        headerRowEl,
        tokensHighlighter,
        hasUsers,
        groupedBy,
        ptoHoursPerDay,
        canEditPastRequests,
        wasAtIntegrated,
        approvableUsers,
        editableNotApprovableUsers,
        isCurrentUserActiveInAT
        )
{
    this.userSchedules = userSchedules;
    this.groupsDataMap = groupsData ? makeMapForEntitiesArray(groupsData) : null;
    this.userRowMap = [];
    this._schedulesDataOffset = schedulesDataOffset;
    this.leaveTypeNotFoundCallback = leaveTypeNotFoundCallback;
    this._tokensHighlighter = tokensHighlighter;
    this._hasUsers = hasUsers;
    this._approvableUsers = approvableUsers;
    this._editableNotApprovableUsers = editableNotApprovableUsers;
    this._groupedBy = groupedBy;

    var monthsRowEl = document.createElement("tr");
    var weeksRowEl = document.createElement("tr");
    weeksRowEl.className = "weeksRow";

    LeaveTable.call( this, containerEl, dateFrom, dateTo, corporateScheduleSlice, today, monthsRowEl, weeksRowEl, headerRowEl, ptoHoursPerDay, canEditPastRequests, wasAtIntegrated, null, isCurrentUserActiveInAT );
    this.tableEl.id = "leaveTimePlanningTable";
}

LeaveTimePlanningTable.prototype = _.create(LeaveTable.prototype);

LeaveTimePlanningTable.prototype.createTooltip = function()
{
    return new LeaveTimePlanningTooltip(this.ptoHoursPerDay).init();
};

LeaveTimePlanningTable.numberFormatter = new NumberFormatter (
        SystemSettings.decimalSeparator,
        SystemSettings.digitGroupingSymbol,
        1,
        true
);

LeaveTimePlanningTable.prototype.createModelRows = function()
{
    var groupedByUserGroup = this._groupedBy == LeaveTimePlanningPageConstants.SORT_BY_USER_GROUP;
    var getRowsCount = groupedByUserGroup ? this._getRowsCountForUserGroupCell : this._getRowsCountForTimeZoneGroupCell;
    for( var i = 0; i < this.userSchedules.length; i++ )
    {
        var userData = this.userSchedules[i];
        var userRow = new LeaveTimePlanningUserRow(
                userData.userInfo,
                userData.requests,
                userData.statuses,
                userData.totalLeaveTime,
                userData.today.parseJSONDate(),
                userData.currentBalanceInfo,
                new ArraySlice( userData.workdayDurations, this._schedulesDataOffset ),
                new ArraySlice( userData.dailyPTOBalance, this._schedulesDataOffset ),
                new ArraySlice( userData.dailySickBalance, this._schedulesDataOffset ),
                this._groupedBy,
                this.groupsDataMap ? getRowsCount.call(this, i) : 0,
                this.groupsDataMap ? this.groupsDataMap[groupedByUserGroup ? userData.userInfo.userGroupId : userData.userInfo.timeZoneGroupId] : null,
                this._tokensHighlighter,
                $.inArray( userData.userInfo.id, this._approvableUsers ) >= 0,
                $.inArray( userData.userInfo.id, this._editableNotApprovableUsers ) >= 0
        );
        userRow.addLeaveCells( this.dateFrom, this.dateTo, this.getCorporateScheduleSlice(), this.ptoHoursPerDay );
        userRow.setLeaveTypeNotFoundCallback(this.leaveTypeNotFoundCallback);
        this.addRow( userRow );

        this.userRowMap[userData.userInfo.id] = {
            rowIdx:userRow.getRowIdx()
        }
    }
    if( this._isEmptyTable() )
    {
        this.addRow( new LeaveTimePlanningNoUsersRow(
                this.getToday(),
                this.dateFrom,
                this.dateTo,
                this.getCorporateScheduleSlice(),
                this._hasUsers
        ) );
    }
};

LeaveTimePlanningTable.prototype._getRowsCountForUserGroupCell = function ( userIdx )
{
    if( userIdx > 0 && this.userSchedules[userIdx].userInfo.userGroupId == this.userSchedules[userIdx - 1].userInfo.userGroupId )
    {
        return 0;
    }
    var res = 1;
    for( var i = userIdx + 1; i < this.userSchedules.length; i++ )
    {
        if( this.userSchedules[userIdx].userInfo.userGroupId == this.userSchedules[i].userInfo.userGroupId )
        {
            res++;
        }
    }
    return res;
};

LeaveTimePlanningTable.prototype._getRowsCountForTimeZoneGroupCell = function ( userIdx )
{
    if (userIdx > 0 && this.userSchedules[userIdx].userInfo.timeZoneGroupId == this.userSchedules[userIdx - 1].userInfo.timeZoneGroupId)
    {
        return 0;
    }
    var res = 1;
    for (var i = userIdx + 1; i < this.userSchedules.length; i++)
    {
        if (this.userSchedules[userIdx].userInfo.timeZoneGroupId == this.userSchedules[i].userInfo.timeZoneGroupId)
        {
            res++;
        }
    }
    return res;
};

LeaveTimePlanningTable.prototype.getBottomRowHeaderCellsNumber = function()
{
    return 4;
};

LeaveTimePlanningTable.prototype.updateUserSchedule = function( userScheduleData, scheduleDataOffset )
{
    var userIndexes = this.userRowMap[ userScheduleData.userInfo.id ];

    var oldUserRow = this.getRow( userIndexes.rowIdx );
    var newUserRow = new LeaveTimePlanningUserRow(
            userScheduleData.userInfo,
            userScheduleData.requests,
            userScheduleData.statuses,
            userScheduleData.totalLeaveTime,
            userScheduleData.today.parseJSONDate(),
            userScheduleData.currentBalanceInfo,
            new ArraySlice( userScheduleData.workdayDurations, scheduleDataOffset ),
            new ArraySlice( userScheduleData.dailyPTOBalance, scheduleDataOffset ),
            new ArraySlice( userScheduleData.dailySickBalance, scheduleDataOffset ),
            this._groupedBy,
            oldUserRow.getGroupRowsCount(),
            oldUserRow.getGroupInfo(), // if group is changed, row must not jump or error happen
            this._tokensHighlighter,
            $.inArray( userScheduleData.userInfo.id, this._approvableUsers ) >= 0,
            $.inArray( userScheduleData.userInfo.id, this._editableNotApprovableUsers ) >= 0
    );

    newUserRow.addLeaveCells( this.dateFrom, this.dateTo, this.getCorporateScheduleSlice(), this.ptoHoursPerDay );
    newUserRow.setLeaveTypeNotFoundCallback(this.leaveTypeNotFoundCallback);

    this.replaceRowAndUpdateDOM( userIndexes.rowIdx, newUserRow, true );
};

LeaveTimePlanningTable.prototype._isEmptyTable = function(){
    return this.userSchedules.length == 0;
};

LeaveTimePlanningTable.prototype.enableUserActivities = function ()
{
    if( this._isEmptyTable() )
    {
        LeaveTable.prototype.disableUserActivities.call( this );
    }
    else
    {
        LeaveTable.prototype.enableUserActivities.call( this );
    }
};

/*LeaveTimePlanningTable.prototype.updateUserGroupRequests = function( userGroupRequests )
{
    // update userGroup
    var deptIdx = this.deptRowMap[ userGroupRequests.userGroup.id ].deptIdx;
    this.userGroupRequestsArray[ deptIdx ].userGroup = userGroupRequests.userGroup;
    this.userGroupRequestsArray[ deptIdx ].leaveAndWorkingTotals = userGroupRequests.leaveAndWorkingTotals;

    // update users
    for( var j = 0; j < userGroupRequests.usersLeaveRequestsWithTotalsArray.length; j++ )
    {
        var userId = userGroupRequests.usersLeaveRequestsWithTotalsArray[j].userNames.id;
        if( this.userRowMap[ userId ] )
        {
            var userIdx = this.userRowMap[ userId ].userIdx;
            var userRowIdx = this.userRowMap[ userId ].rowIdx;

            this.userGroupRequestsArray[ deptIdx ].usersLeaveRequestsWithTotalsArray[ userIdx ] = userGroupRequests.usersLeaveRequestsWithTotalsArray[j];

            var userRow = new LeaveTimePlanningUserRow(
                    this.dateFrom,
                    this.dateTo,
                    userGroupRequests.usersLeaveRequestsWithTotalsArray[j],
                    userIdx == 0
            );

            this.replaceRowAndUpdateDOM( userRowIdx, userRow );
        }
    }

    var userGroupRow = new LeaveTimePlanningDepartmentRow(
            this.dateFrom,
            this.dateTo,
            this.userGroupRequestsArray[ deptIdx ].userGroup,
            this.userGroupRequestsArray[ deptIdx ].leaveAndWorkingTotals,
            this.userGroupRequestsArray[ deptIdx ].usersLeaveRequestsWithTotalsArray.length
    );

    var deptRowIdx = this.deptRowMap[ userGroupRequests.userGroup.id ].rowIdx;

    this.replaceRowAndUpdateDOM( deptRowIdx, userGroupRow );

};*/