/**
 * Iterates an array in chunks, making "pauses" between chunks.
 * This gives browser a chance to render something and to handle user interaction before array is fully iterated.
 *
 * Triggers following events:
 *   "progress" - after each chunk is processed, has single number parameter, which is how many array items were processed.
 *   "completed" - after the entire array is processed.
 *
 * @param array Array to iterate.
 * @param chunkSize Amount of array items, which will be iterated in chunk without pause.
 * @param callback Iterator function with array item and index as arguments.
 * @param context Context to bind the iterator function.
 * @constructor
 */
function ChunkIterator(array, chunkSize, callback, context)
{
    this.array = array;
    this.chunkSize = chunkSize;
    this.callback = callback;
    this.context = context;
}

ChunkIterator.prototype = {
    /**
     * Starts processing, putting next chunk iterations into queue.
     * @returns {ChunkIterator}
     */
    start: function()
    {
        this.bindCallback = _.bind(this.callback, this.context);
        this.bindIterateChunk = _.bind(this._iterateChunk, this);
        this.i = 0;
        _.defer(this.bindIterateChunk);

        return this;
    },

    /**
     * Pauses processing, the next queued chunk will not be processed.
     */
    pause: function()
    {
        this.paused = true;
    },

    /**
     * Resumes processing from the same chunk we left off last time.
     */
    resume: function()
    {
        if (this.paused && this.i < this.array.length)
        {
            this.paused = false;
            _.defer(this.bindIterateChunk);
        }
    },

    /**
     * Chunk iterator function.
     * @private
     */
    _iterateChunk: function()
    {
        if (this.paused)
            return;

        var length = this.array.length;

        for (var j = 0; this.i < length && j < this.chunkSize; this.i++, j++)
            this.bindCallback(this.array[this.i], this.i);

        $(this).trigger("progress", this.i);

        if (this.i < length)
            _.defer(this.bindIterateChunk);
        else
            $(this).trigger("completed");
    }
};