/**
 * Represents customer/project selection, which is widely used in reports and filters
 * @param {String} [qualifier] Selection qualifier, can be "ALL", "ALL_ACTIVE" or "SELECTED" (see CustomersProjectsSelectionQualifier enum). Defaults to "ALL".
 * @param {Array} [customers] IDs of selected customers. Ignored if 'qualifier' param is "ALL" or "ALL_ACTIVE".
 * @param {Array} [projects] IDs of selected projects. Ignored if 'qualifier' param is "ALL" or "ALL_ACTIVE".
 * @constructor
 */
function CustomerProjectSelection(qualifier, customers, projects, activeOnly)
{
    this.qualifier = qualifier || CustomersProjectsSelectionQualifier.ALL;

    this._all = this.qualifier === CustomersProjectsSelectionQualifier.ALL || this.qualifier === CustomersProjectsSelectionQualifier.ALL_ACTIVE;
    this.activeOnly = this.qualifier === CustomersProjectsSelectionQualifier.ALL_ACTIVE || !!activeOnly;

    if (!this._all)
    {
        this.customers = customers || [];
        this.projects = projects || [];
    }
    else
    {
        this.customers = [];
        this.projects = [];
    }

    this._customersSet = arrayToSet(this.customers);
    this._projectsSet = arrayToSet(this.projects);
}

CustomerProjectSelection.prototype = {
    /**
     * Determines if customer is included in this selection.
     * @param {Customer} customer
     * @param {boolean} [transitive]
     * @returns {boolean}
     */
    includesCustomer: function(customer, transitive)
    {
        if (!customer)
            return false;
        if (this.activeOnly && customer.archived)
            return false;
        else if (this._all)
            return true;
        else
            return !!this._customersSet[customer.id] || transitive && _.any(ProjectStore.getByCustomer(customer.id), function(p) { return this.includesProject(p) }, this);
    },

    /**
     * Determines if project is included in this selection.
     * @param {Project} project
     * @returns {boolean}
     */
    includesProject: function(project)
    {
        if (!project)
            return false;
        if (this.activeOnly && project.archived)
            return false;
        else if (this._all)
            return true;
        else
            return !!this._projectsSet[project.id] || !!this._customersSet[project.customerId];
    },

    /**
     * Determines if this selection is empty and thus guaranteed to not include any customer/project
     * (i.e. has "SELECTED" qualifier and doesn't specify any customer/project ID).
     * @returns {boolean}
     */
    isEmpty: function()
    {
        return !this._all && this.customers.length === 0 && this.projects.length === 0;
    },

    /**
     * Determines if this selection is equal to another.
     * @param {CustomerProjectSelection} otherSelection
     * @return {boolean}
     */
    equals: function(otherSelection)
    {
        if (otherSelection && this.qualifier === otherSelection.qualifier)
        {
            var self = this;
            return this === otherSelection ||
                   this._all ||
                   this.customers.length === otherSelection.customers.length &&
                   _.all(otherSelection.customers, function(c) { return self._customersSet[c] }) &&
                   this.projects.length === otherSelection.projects.length &&
                   _.all(otherSelection.projects, function(p) { return self._projectsSet[p] });
        }

        return false;
    },

    /**
     * Intersects this selection with other, returning selection, which only includes customers/projects from both.
     * @param {CustomerProjectSelection} otherSelection
     * @returns {CustomerProjectSelection}
     */
    intersect: function(otherSelection)
    {
        var self = this;

        var intersectionQualifier;
        if (this.qualifier === CustomersProjectsSelectionQualifier.ALL)
        {
            intersectionQualifier = otherSelection.qualifier;
        }
        else if (this.qualifier === CustomersProjectsSelectionQualifier.ALL_ACTIVE)
        {
            if (otherSelection.qualifier === CustomersProjectsSelectionQualifier.ALL)
                intersectionQualifier = this.qualifier;
            else
                intersectionQualifier = otherSelection.qualifier;
        }
        else if (this.qualifier === CustomersProjectsSelectionQualifier.SELECTED)
        {
            intersectionQualifier = this.qualifier;
        }

        var intersectionCustomersSet = {};
        _.each(this.customers, function(customerId)
        {
            var customer = CustomerStore.get(customerId);
            if (customer && otherSelection.includesProject(customer))
                intersectionCustomersSet[customer.id] = true;
        });
        _.each(otherSelection.customers, function(customerId)
        {
            var customer = CustomerStore.get(customerId);
            if (customer && self.includesCustomer(customer))
                intersectionCustomersSet[customer.id] = true;
        });
        var intersectionCustomers = _.keys(intersectionCustomersSet);

        var intersectionProjectsSet = {};
        _.each(this.projects, function(projectId)
        {
            var project = ProjectStore.get(projectId);
            if (project && otherSelection.includesProject(project))
                intersectionProjectsSet[projectId] = true;
        });
        _.each(otherSelection.projects, function(projectId)
        {
            var project = ProjectStore.get(projectId);
            if (project && self.includesProject(project))
                intersectionProjectsSet[projectId] = true;
        });
        var intersectionProjects = _.keys(intersectionProjectsSet);

        var intersectionActiveOnly = this.activeOnly || otherSelection.activeOnly;

        return new CustomerProjectSelection(intersectionQualifier, intersectionCustomers, intersectionProjects, intersectionActiveOnly);
    }
};

/**
 * Instance of customer/project selection with "ALL" qualifier.
 * @type {CustomerProjectSelection}
 */
CustomerProjectSelection.ALL = new CustomerProjectSelection();

/**
 * Instance of customer/project selection with "ALL_ACTIVE" qualifier.
 * @type {CustomerProjectSelection}
 */
CustomerProjectSelection.ALL_ACTIVE = new CustomerProjectSelection(CustomersProjectsSelectionQualifier.ALL_ACTIVE);

/**
 * Makes selection instance converting it from plain object form, commonly used in project.
 * @param {Object} cpSelectionObject
 * @param {String} cpSelectionObject.qualifier Selection qualifier.
 * @param {Array} cpSelectionObject.customers IDs of selected customers.
 * @param {Array} cpSelectionObject.projects IDs of selected projects.
 * @returns {CustomerProjectSelection}
 */
CustomerProjectSelection.fromPlainObject = function(cpSelectionObject)
{
    return new CustomerProjectSelection(cpSelectionObject.qualifier, cpSelectionObject.customers, cpSelectionObject.projects);
};