/**
 * Implementation of View class, which uses Underscore or plain HTML strings as templates for rendering.
 * @constructor
 */
function TemplateView($containerEl)
{
    this.$containerEl = $containerEl;
}

TemplateView.prototype = _.create(View.prototype, {
    /**
     * HTML template or just plain markup used to render the view.
     */
    html: null,

    /**
     * Key, which will be used to identify the template.
     * TemplateView instances with identical template key will reuse the same Underscore template.
     */
    templateKey: null,

    /**
     * Defines if a view should use Underscore templates functionality.
     * If template HTML is static, it would be beneficial to set this to false.
     */
    useUnderscoreTemplate: true,

    render: function()
    {
        if (!this.$el)
            this.$el = $(this.toHtml());
        this.appendToContainer();
        return this;
    },

    appendToContainer: function()
    {
        this.$containerEl.append(this.$el);
    },

    toHtml: function()
    {
        if (!this._htmlCache)
            this._htmlCache = this._toHtml();
        return this._htmlCache;
    },

    _toHtml: function()
    {
        var html;
        if (this.useUnderscoreTemplate)
        {
            if (this.templateKey)
            {
                var cachedTemplate = TemplateView.TemplateCache[this.templateKey];
                if (!cachedTemplate)
                {
                    cachedTemplate = _.template(this.html);
                    TemplateView.TemplateCache[this.templateKey] = cachedTemplate;
                }

                this._underscoreTemplate = cachedTemplate;
            }

            if (!this._underscoreTemplate)
                this._underscoreTemplate = _.template(this.html);

            var model = this.templateModel();
            var messageKeys = this.messageKeys();
            _.each(_.keys(messageKeys), function(modelKey)
            {
                model[modelKey] = messageResource.getMessage(messageKeys[modelKey]);
            });
            html = this._underscoreTemplate(model);
        }
        else
            html = this.html;

        return html;
    },

    /**
     * When TemplateView is rendered using Underscore template,
     * this function's returned object will be used as a model.
     * @returns {Object}
     */
    templateModel: function()
    {
        return this;
    },

    /**
     * Works similarly to .templateModel() method, but interprets object keys as messageKeys
     * for messageResource, which will be exchanged to actual message strings during render.
     * @returns {Object}
     */
    messageKeys: function()
    {
        return {};
    }
});

TemplateView.TemplateCache = {};