/**
 * Base class for custom views. Contains base
 * @param [$containerEl] jQuery selector for element, which is going to be view's container.
 *                       It's not necessary to specify it in constructor, it can can done in .init() method.
 * @constructor
 */
function View($containerEl)
{
    this.$containerEl = $containerEl;
}

View.prototype = {
    /**
     * Defines if .update() method will be called once on .init()
     */
    updateOnInit: true,

    /**
     * Renders the view, calls before and after hooks and optionally
     * @param [$containerEl] jQuery selector for element, which is going to be view's container.
     *                       This is alternative to passing container element in the constructor,
     *                       which is useful for postponed render.
     * @returns {View}
     */
    init: function($containerEl)
    {
        if ($containerEl)
            this.$containerEl = $containerEl;

        this.beforeRender();
        this.render();
        this.rendered = true;
        this.afterRender();

        if (this.updateOnInit)
            this.update();

        return this;
    },

    /**
     * Abstract render function, should put the view into container.
     */
    render: function()
    {
        // Abstract
    },

    /**
     * Before-render hook.
     */
    beforeRender: function()
    {

    },

    /**
     * After-render hook.
     */
    afterRender: function()
    {

    },

    /**
     * Abstract update function, which should render any dynamic changes in the view.
     */
    update: function()
    {

    },

    /**
     * Shortcut to jQuery $.on() function for event binding.
     * @param eventName {String} Event name.
     * @param handler {Function} Event handler.
     */
    on: function(eventName, handler)
    {
        $(this).on(eventName, handler);
    },

    /**
     * Shortcut to jQuery $.trigger() function for event binding.
     * @param eventName {String} Event name.
     * @param data {Function} Event data.
     */
    trigger: function(eventName, data)
    {
        $(this).trigger(eventName, data);
    },

    /**
     * Shortcut for binding a function to view's instance. Useful when you want some callback to be a class method.
     * @param f {Function} Function to bind.
     * @returns {Function}
     */
    bind: function(f)
    {
        return _.bind(f, this);
    },

    /**
     * Shortcut for finding view's sub-elements using jQuery.
     * @param selector jQuery selector.
     */
    find: function(selector)
    {
        return this.$el.find(selector);
    },

    /**
     * Shortcut for finding view's sub-elements using jQuery and binding a click handler.
     * @param selector jQuery selector.
     * @param handler {Function} Click handler.
     */
    findAndBindClick: function(selector, handler)
    {
        return this.find(selector).click(this.bind(handler));
    },

    /**
     * Shortcut for binding a click handler to the entire view.
     * @param handler {Function} Click handler.
     */
    bindClick: function(handler)
    {
        return this.$el.click(this.bind(handler));
    },

    /**
     * Shows view.
     */
    show: function()
    {
        this.$el.show();
    },

    /**
     * Hides view.
     */
    hide: function()
    {
        this.$el.hide();
    },

    /**
     * Toggles view visibility, calling .show() if parameter is true and .hide() if false.
     * @param {Boolean} show
     */
    toggle: function(show)
    {
        if (show)
            this.show();
        else
            this.hide();
    },

    /**
     * Disables re-renders and updates for the view (only those initiated by standard functions).
     * Rendering is disabled for the rest of view's lifespan, which should be short.
     * This can be useful to prevent unwanted effects just before the view will be destroyed.
     */
    disableRendering: function()
    {
        this.render = _.noop;
        this.beforeRender = _.noop;
        this.afterRender = _.noop;
        this.update = _.noop;
    }
};