function RatesTableRow()
{
}

RatesTableRow.prototype = new TabularDataWithEffectiveDatesTableRow();

RatesTableRow.prototype.init = function( ratesObj, effectiveDateCellRenderer, regularRateCellRenderer, overtimeRateCellRenderer, leaveRatesCellRenderer, removeRowCellRenderer, effectiveDateCellValidator, rateCellValidator, leaveRatesCellValidator, overtimeRateCoefficient, activeLeaveTypes )
{
    TabularDataWithEffectiveDatesTableRow.prototype.init.call( this );
    this.renderers = [effectiveDateCellRenderer, regularRateCellRenderer];
    this.dateCell = this.createDateCell( ratesObj, effectiveDateCellValidator );
    this.addCellObj( this.dateCell );
    this.regularRateCell = new TabularDataTableCell().init( ratesObj.regularRate, rateCellValidator, this );
    this.addCellObj( this.regularRateCell );

    if( overtimeRateCellRenderer )
    {
        this.renderers.push( overtimeRateCellRenderer );
        this.overtimeRateCell = new TabularDataTableCell().init( ratesObj.overtimeRate, rateCellValidator, this );
        this.addCellObj( this.overtimeRateCell );
    }

    if( leaveRatesCellRenderer )
    {
        this.renderers.push( leaveRatesCellRenderer );

        // append empty leave rates for newly created leave types
        var foundLeaveTypeIdsSet = {};
        for( var r = 0; r < ratesObj.leaveRates.length; r++ )
            foundLeaveTypeIdsSet[ratesObj.leaveRates[r].leaveTypeId] = true;
        for( var l = 0; l < activeLeaveTypes.length; l++ )
        {
            var leaveType = activeLeaveTypes[l];
            if( !foundLeaveTypeIdsSet[ leaveType.id ] )
                ratesObj.leaveRates.push( { leaveTypeId: leaveType.id, rate:"" } );
        }

        this.leaveRatesCell = new TabularDataTableCell().init( ratesObj.leaveRates, leaveRatesCellValidator, this );
        this.addCellObj( this.leaveRatesCell );
    }else{
        var leaveCell = document.createElement("td");
        leaveCell.className="leaveRatesCell";
        this.getRowEl().appendChild( leaveCell);
    }

    this.removeRowCell = new TableCell( null, this );
    this.renderers.push( removeRowCellRenderer );
    this.addCellObj( this.removeRowCell );

    var self = this;
    this.regularRateCell.onChange( function()
    {
        var regularRateValue = self.getRegularRateValue();
        if( !_.isNumber(regularRateValue) )
        {
            if( self.leaveRatesCell )
            {
                self.leaveRatesCell.validate(false);
                self.leaveRatesCell.updateUI();
            }
            return;
        }

        if( self.overtimeRateCell && self.overtimeRateCell.isEmpty() && overtimeRateCoefficient )
        {
            self.overtimeRateCell.setValue( regularRateValue * overtimeRateCoefficient  );
            self.overtimeRateCell.updateUI();
        }

        if( self.leaveRatesCell && self.leaveRatesCell.isEmpty() && activeLeaveTypes )
        {
            var leaveRatesArray = [];
            for( var i = 0; i < activeLeaveTypes.length; i++ )
            {
                var leaveType = activeLeaveTypes[i];
                leaveRatesArray.push({
                    leaveTypeId: leaveType.id,
                    rate: regularRateValue * leaveType.rateCoefficient
                });
            }

            self.leaveRatesCell.setValue( leaveRatesArray );
            self.leaveRatesCell.updateUI();
        }

    });

    this.rateId = ratesObj.rateId;

    return this;
};

RatesTableRow.prototype.getRateCellValue = function( rateCell )
{
    if( rateCell.isEmpty() || !rateCell.isValid() )
        return null;

    return this.getTable().getFloatNumberValidator().parseFloat( rateCell.getValue() );
};


RatesTableRow.prototype.getRegularRateValue = function()
{
    return  this.getRateCellValue( this.regularRateCell );
};

RatesTableRow.prototype.createCellRenderer = function( cell )
{
    return this.renderers[ cell.getColIdx() ];
};


RatesTableRow.prototype.createDateCell = function( ratesObj, validator )
{
    var effectiveDate = ratesObj.effectiveDate ? ratesObj.effectiveDate.parseJSONDate() : null;
    var cell = new TabularDataTableCell().init( effectiveDate, validator, this );
    cell.onChange( function() {
        cell.getTable().markRowsWithEqualEffectiveDatesAsInvalid();
        cell.getTable().updateEffectiveRow();
    } );

    return cell;
};


RatesTableRow.prototype.isEmpty = function()
{
    if( !this.regularRateCell.isEmpty() )
        return false;
    if( this.overtimeRateCell && this.getTable().isOvertimeEnabled() && !this.overtimeRateCell.isEmpty() )
        return false;
    //noinspection RedundantIfStatementJS
    if( this.leaveRatesCell && !this.leaveRatesCell.isEmpty() )
        return false;
    return true;
};

RatesTableRow.prototype.canBeDeletedPermanently = function()
{
    return this.isEmpty();
};

RatesTableRow.prototype.validate = function()
{
    this.dateCell.validate( false );
    this.regularRateCell.validate(  false );
    if( this.overtimeRateCell )
        this.overtimeRateCell.validate( false );
    if( this.leaveRatesCell )
        this.leaveRatesCell.validate( false );


    if( this.isEmpty() )
    {
        this.dateCell.setValid();
        this.regularRateCell.setValid();
        if( this.overtimeRateCell )
            this.overtimeRateCell.setValid();
        if( this.leaveRatesCell )
            this.leaveRatesCell.setValid();

        return true;
    }

    if( !this.dateCell.isValid() )
        return false;

    if( !this.regularRateCell.isValid() )
        return false;

    if( this.getTable().isOvertimeEnabled() && this.overtimeRateCell && !this.overtimeRateCell.isValid() )
        return false;

    //noinspection RedundantIfStatementJS
    if( this.leaveRatesCell && !this.leaveRatesCell.isValid() )
        return false;

    return true;
};

RatesTableRow.prototype.getRates = function()
{
    if( this.isMarkedToRemove() || this.isEmpty() )
        return null;

    var ratesObj = {rateId: this.rateId};

    if( this.dateCell.isEmpty() || !this.dateCell.isValid() )
        return null;
    ratesObj.effectiveDate = this.dateCell.getValue().toJSON();

    ratesObj.regularRate = this.getRegularRateValue();
    if( ratesObj.regularRate === null || ratesObj.regularRate === undefined )
        return null;

    if( this.getTable().isOvertimeEnabled() && this.overtimeRateCell )
    {
        ratesObj.overtimeRate = this.getRateCellValue( this.overtimeRateCell );
        if( ratesObj.overtimeRate === null || ratesObj.overtimeRate === undefined )
            return null;
    }

    if( this.leaveRatesCell )
    {
        if ( this.leaveRatesCell.isEmpty() || !this.leaveRatesCell.isValid() )
            return null;
        ratesObj.leaveRates = this.leaveRatesCell.getValue();
        for( var i = 0; i < ratesObj.leaveRates.length; i++ )
        {
            var leaveRatesObj = ratesObj.leaveRates[i];
            leaveRatesObj.rate = this.getTable().getFloatNumberValidator().parseFloat( leaveRatesObj.rate );
        }
    }

    return ratesObj;
};

RatesTableRow.prototype.hasErrors = function()
{
    if( this.markedToRemove )
        return false;

    if( !this.dateCell.isValid() )
        return true;

    if( !this.regularRateCell.isValid() )
        return true;

    if( this.overtimeRateCell && this.getTable().isOvertimeEnabled() && !this.overtimeRateCell.isValid() )
        return true;

    //noinspection RedundantIfStatementJS
    if( this.leaveRatesCell && !this.leaveRatesCell.isValid() )
        return true;

    return false;
};
