/**
 * Animation util functions.
 */
var AtAnimation = {
    /**
     * Gets browser-specific event name given name-by-styles map.
     * @param namesByStyle Map of possible event names by supported CSS key.
     */
    browserEventName: function(namesByStyle)
    {
        // Fabricate element and search for supported CSS
        var div = document.createElement("div");
        for (var style in namesByStyle)
        {
            if (div.style[style] !== undefined)
            {
                return namesByStyle[style];
            }
        }
    },

    /**
     * Returns browser specific prefixes, which are necessary for several styles and functions.
     */
    browserVendorPrefixes: function()
    {
        var pre = null;
        if (!_.isUndefined(window.getComputedStyle))
        {
            var styles = window.getComputedStyle(document.documentElement, '');
            pre = ((_.unique(styles)).join("").match(/-(moz|webkit|ms)-/) || (styles.OLink === '' && ['', 'o']))[1];
        }
        return {
            css: pre ? ('-' + pre + '-') : '',
            js: pre ? (pre[0].toUpperCase() + pre.substr(1)) : ''
        };
    },

    /**
     * Indicates if browser supports CSS animation.
     */
    isAnimationSupported: function()
    {
        AtAnimation._init();
        return !_.isUndefined(this._animationEndEventName);
    },

    /**
     * Indicates if browser supports CSS transitions.
     */
    isTransitionSupported: function()
    {
        AtAnimation._init();
        return !_.isUndefined(this._transitionEndEventName);
    },

    /**
     * Gets browser-specific animation end event name.
     */
    getAnimationEndEventName: function()
    {
        AtAnimation._init();
        return this._animationEndEventName;
    },

    /**
     * Gets browser-specific transition end event name.
     */
    getTransitionEndEventName: function()
    {
        AtAnimation._init();
        return this._transitionEndEventName;
    },

    /**
     * Converts JavaScript name to browser-specific.
     * @param jsName JavaScript name
     */
    getVendorPrefixedJSName: function(jsName)
    {
        AtAnimation._init();
        return this._vendorPrefixes.js + jsName;
    },

    /**
     * Converts CSS name to browser-specific.
     * @param cssName CSS name
     */
    getVendorPrefixedCssName: function(cssName)
    {
        AtAnimation._init();
        return this._vendorPrefixes.css + cssName;
    },

    /**
     * Initializes animation related properties. Runs only one time.
     * @private
     */
    _init: function()
    {
        if (!AtAnimation._initialized)
        {
            AtAnimation._vendorPrefixes = AtAnimation.browserVendorPrefixes();

            AtAnimation._transitionEndEventName = AtAnimation.browserEventName({
                                                                                   'transition': 'transitionend',
                                                                                   'OTransition': 'oTransitionEnd',
                                                                                   'MozTransition': 'transitionend',
                                                                                   'WebkitTransition': 'webkitTransitionEnd'
                                                                               });

            AtAnimation._animationEndEventName = AtAnimation.browserEventName({
                                                                                  'animation': 'animationend',
                                                                                  'OAnimation': 'oAnimationEnd',
                                                                                  'MozAnimation': 'animationend',
                                                                                  'WebkitAnimation': 'webkitAnimationEnd'
                                                                              });

            AtAnimation._initialized = true;
        }
    }
};

/**
 * Wrapper to provide jQuery element with animation functions.
 * @param e jQuery element
 * @constructor
 */
function AtAnimationJqueryPlugin(e)
{
    this.e = e;
}

AtAnimationJqueryPlugin.prototype = _.create(Object.prototype, {
    /**
     * Shows element and runs standard fade-in animation.
     * @param cb Callback
     */
    fadeIn: function(cb)
    {
        return this.showAnimated("anim-fade-in", cb);
    },

    /**
     * Hides element while running standard fade-out animation.
     * @param cb Callback
     */
    fadeOut: function(cb)
    {
        return this.hideAnimated("anim-fade-out", cb);
    },

    /**
     * Shows element and runs standard fade-in-with-scale animation.
     * @param cb Callback
     */
    fadeInWithScale: function(cb)
    {
        return this.showAnimated("anim-fade-in-with-scale", cb);
    },

    /**
     * Hides element while running standard fade-out-with-scale animation.
     * @param cb Callback
     */
    fadeOutWithScale: function(cb)
    {
        return this.hideAnimated("anim-fade-out-with-scale", cb);
    },

    /**
     * Prematurely stops element animation.
     */
    stopAnimation: function()
    {
        this.oneAnimationEnd(function() {});
    },

    /**
     * Prematurely stops element transition.
     */
    stopTransition: function()
    {
        this.oneTransitionEnd(function() {});
    },

    /**
     * Subscribes to animation end event. Previously subscribed callback will be fired prematurely.
     * @param cb Callback
     */
    oneAnimationEnd: function(cb)
    {
        AtAnimation._init();
        return this._oneEvent(AtAnimation._animationEndEventName, "_atAnimationEndCb", cb);
    },

    /**
     * Subscribes to transition end event. Previously subscribed callback will be fired prematurely.
     * @param cb Callback
     */
    oneTransitionEnd: function(cb)
    {
        AtAnimation._init();
        return this._oneEvent(AtAnimation._transitionEndEventName, "_atTransitionEndCb", cb);
    },

    /**
     * Shows element and runs specified animation.
     * @param animationClass CSS class, containing animation properties.
     * @param cb Callback
     */
    showAnimated: function(animationClass, cb)
    {
        AtAnimation._init();

        // Show element and immediately add CSS class now and remove class on animation end
        var e = this.e;
        this.oneAnimationEnd(function()
                             {
                                 e.show().removeClass(animationClass);
                                 if (cb) cb();
                             });
        e.show().addClass(animationClass);

        return e;
    },

    /**
     * Hides element running specified animation.
     * @param animationClass CSS class, containing animation properties.
     * @param cb Callback
     */
    hideAnimated: function(animationClass, cb)
    {
        AtAnimation._init();

        // Only run animation on visible elements
        var e = this.e;
        if (e.is(":visible"))
        {
            // Add CSS class now and remove it on animation end, also completely hide the element in the end
            this.oneAnimationEnd(function()
                                 {
                                     e.hide().removeClass(animationClass);
                                     if (cb) cb();
                                 });
            e.addClass(animationClass);
        }
        else
        {
            e.hide();
            if (cb)
            {
                cb();
            }
        }

        return e;
    },

    /**
     * Subscribes to animation event.
     * Previously subscribed callback will be fired prematurely.
     * @param eventName Event name
     * @param cbName Name of the property to store callback
     * @param cb Callback
     * @private
     */
    _oneEvent: function(eventName, cbName, cb)
    {
        if (cb)
        {
            if (eventName)
            {
                // Set callback
                this._fireAndReplaceElementCallback(cbName, cb);
                var self = this;
                this.e.one(eventName, function(event)
                {
                    // Handle event and set callback to null
                    self._fireAndReplaceElementCallback(cbName, null, event);
                });
            }
            else
            {
                // Empty event name means event is unsupported. Fire callback immediately.
                cb();
            }
        }

        return this.e;
    },

    /**
     * Fires event callback if it exists and sets new callback.
     * @param cbName Name of the property to store callback.
     * @param newCb New callback.
     * @param event Handled event object
     * @private
     */
    _fireAndReplaceElementCallback: function(cbName, newCb, event)
    {
        if (this.e[0][cbName])
        {
            this.e[0][cbName](event);
        }
        this.e[0][cbName] = newCb;
    }
});