//NOTE: should include rpc_control_manager.js to use this js

/*
 * Manager class for cell controls having only two possible states. Based on RpcControlManager.
 */
function BinaryStateCellManager(cell, checkedTooltip, uncheckedTooltip, checkingErrorMsg, uncheckingErrorMsg)
{
    this.base = RpcControlManager;
    this.base();
    this.cell = cell;
    this.checkedTooltip = checkedTooltip;
    this.uncheckedTooltip = uncheckedTooltip;
    this.checkingErrorMsg = checkingErrorMsg;
    this.uncheckingErrorMsg = uncheckingErrorMsg;
    this.titleBeforeSavingState = "";
    this.savingCaption = messageResource.getMessage("common.saving");
    this.savingBackgroundColor = DHTMLColors.beingSavedColor;
}

BinaryStateCellManager.prototype = new RpcControlManager();

BinaryStateCellManager.prototype.init = function()
{
    this.initialState = this.isChecked();
};

BinaryStateCellManager.prototype.beginSavingData = function()
{
    this.removeChild();
    this.cell.className = 'listtblcell';
    this.titleBeforeSavingState = this.cell.title;
    this.cell.title = "";
    this.appendText(this.savingCaption);
    this.cell.style.backgroundColor = this.savingBackgroundColor;
};

BinaryStateCellManager.prototype.removeChild = function()
{
    if (this.cell.firstChild)
    {
        this.cell.removeChild(this.cell.firstChild);
    }
};

BinaryStateCellManager.prototype.appendText = function(text)
{
    this.cell.appendChild(document.createTextNode(text));
};

BinaryStateCellManager.prototype.getTooltip = function(state)
{
    return state ? this.checkedTooltip : this.uncheckedTooltip;
};

BinaryStateCellManager.prototype.getRpcErrorMsg = function()
{
    return this.initialState ? this.uncheckingErrorMsg : this.checkingErrorMsg;
};

BinaryStateCellManager.prototype.endSavingData = function(success)
{
    this.cell.title = this.titleBeforeSavingState;
    this.updateCell(success ? !this.initialState : this.initialState);
};


/*
 * Provides methods for text cell content managment.
 */
function TextCellManager(cell, checkedTooltip, uncheckedTooltip,
                               checkingErrorMsg, uncheckingErrorMsg,
                               checkedText, uncheckedText) {
    this.base = BinaryStateCellManager;
    this.base(cell, checkedTooltip, uncheckedTooltip, checkingErrorMsg, uncheckingErrorMsg);

    this.checkedText = checkedText;
    this.uncheckedText = uncheckedText;
}

TextCellManager.prototype = new BinaryStateCellManager();

TextCellManager.prototype.enabledCellClassName = 'listtblcellenabled';
TextCellManager.prototype.disabledCellClassName = 'listtblcelldisabled';

TextCellManager.prototype.updateCell = function(state)
{
    this.removeChild();
    if (state)
    {
        this.appendText(this.checkedText);
        this.cell.className = this.enabledCellClassName;
    }
    else
    {
        this.appendText(this.uncheckedText);
        this.cell.className = this.disabledCellClassName;
    }
    this.cell.title = this.getTooltip(state);
    removeHighlighting(this.cell);
};

TextCellManager.prototype.isChecked = function()
{
    return this.cell.innerText.substring(0, this.checkedText.length) === this.checkedText;
};

/*
 * Provides methods for management of a single choice cell with a link (when not selected).
 */
function SingleChoiceLinkCellManager(cell, checkedTooltip, uncheckedTooltip, disabledTooltip,
                               checkingErrorMsg, uncheckingErrorMsg,
                               checkedText, uncheckedText, disabledText,
                               onClick) {
    this.base = BinaryStateCellManager;
    this.base(cell, checkedTooltip, uncheckedTooltip, checkingErrorMsg, uncheckingErrorMsg);

    this.checkedText = checkedText;
    this.uncheckedText = uncheckedText;
    this.disabledTooltip = disabledTooltip;
    this.disabledText = disabledText;
    this.onClick = onClick;
}

SingleChoiceLinkCellManager.prototype = new BinaryStateCellManager();

SingleChoiceLinkCellManager.prototype.updateCell = function(state) {
    this.removeChild();

    this.cell.title = this.getTooltip(state);
    if (state) {
        this.appendText(this.checkedText);
        var current = this.getCurrentChoiceManager();
        if (current) {
            current.updateCell(false);
        }
        this.setCurrentChoiceManager(this);
    } else {
        var a = document.createElement('a');
        a.href = 'javascript:;';
        a.appendChild(document.createTextNode(this.uncheckedText));
        a.onclick = new Function(this.onClick);
        this.cell.appendChild(a);
    }
    removeHighlighting(this.cell);
};

SingleChoiceLinkCellManager.prototype.disableCell = function() {
    if (this.isChecked()) {
        this.setCurrentChoiceManager(null);
    }
    this.removeChild();
    var span = document.createElement("span");
    span.className = "textinactive";
    span.appendChild(document.createTextNode(this.disabledText));
    this.cell.appendChild(span);
    this.cell.title = this.disabledTooltip;
};

SingleChoiceLinkCellManager.prototype.isChecked = function() {
    var current = this.getCurrentChoiceManager();
    return current && this.cell === current.cell;
};

/**
 * The two methods provide access to a global variable
 * that references currently selected cell's manager.
 * Should be overridden in subclass.
**/
SingleChoiceLinkCellManager.prototype.getCurrentChoiceManager = function() { return null; };
SingleChoiceLinkCellManager.prototype.setCurrentChoiceManager = function(manager) { };

/*
 * Provides methods for icon cell content management.
 */
function IconCellManager(cell, checkedTooltip, uncheckedTooltip,
                               checkingErrorMsg, uncheckingErrorMsg,
                               checkedImage, uncheckedImage)
{
    this.base = BinaryStateCellManager;
    this.base(cell, checkedTooltip, uncheckedTooltip, checkingErrorMsg, uncheckingErrorMsg);

    this.checkedImage = checkedImage;
    this.uncheckedImage = uncheckedImage;
}

IconCellManager.prototype = new BinaryStateCellManager();

IconCellManager.prototype.updateCell = function(state, delayHighlighting)
{
    this.removeChild();
    var img = document.createElement('img');
    img.src = this.getCellImage(state);
    img.width = 14;
    img.height = 13;
    img.title = this.getTooltip(state);
    this.cell.appendChild(img);
    this.cell.title = this.getTooltip(state);

    var cell = this.cell;
    var removeCellHighlightingFunc = function() {
            removeHighlighting(cell);
    };
    if (delayHighlighting) {
        window.setTimeout(removeCellHighlightingFunc, 1000);
    } else {
        removeCellHighlightingFunc();
    }

};

IconCellManager.prototype.getCellImage = function(state)
{
    if (state)
        return this.checkedImage;
    else
        return this.uncheckedImage;
};

IconCellManager.prototype.isChecked = function()
{
    return this.cell.firstChild &&
           this.cell.firstChild.nodeType == 1 &&
           this.cell.firstChild.src.lastIndexOf(this.checkedImage) > 0;
};

function highlightCell(cell)
{
    if (RpcControlManager.isRpcBusy())
        return;
    cell.style.backgroundColor = DHTMLColors.mouseOverHighlightColor;
    try { cell.style.cursor = 'pointer'; } catch (e) {}
}

function removeHighlighting(cell)
{
    cell.style.backgroundColor = "";
}
