/**
 *   require /js/util/AtMap.js
 */

/**
 * This class implements Listbox element with groups.
 * functionality:
 *  - show grouped list of items
 *  - sort groups in list and objects in groups
 *  - change color and background for any item
 *  - create data which sends to server
 *  - add new items to the list
 *  - remove items from list
 *  - collect statistics about all elements in list.
 * @param itemKeyGenerator generator which use to generate unique keys for list objects and groups (Class KeyGenerator)
 * @param objectGroupKeyGenerator generator which use to generate unique keys for group of object (Class KeyGenerator)
 *  Must be agreed with itemKeyGenerator for groups
 * @param groupsArray array of groups (Class KeyGenerator)
 * @param objectPresentationController configured visibility Controller for operate
 *  presentation of listbox objects (Class PresentationController)
 * @param groupPresentationController configured visibility Controller for operate
 *  presentation of listbox groups (Class GroupPresentationController)
 * @param contentController  configured content Controller for operate
 *  consistence of listbox (Class ContentController)
 * @param outputDataController configured Output data controller for operate
 *  data which sends to server.(Class OutputDataController)
 * @param statisticsController used for collect any available statistics about object
 * in listbox. (Class StatisticsController)
 */
function GroupListController( itemKeyGenerator, objectGroupKeyGenerator, groupsArray, objectPresentationController, groupPresentationController, contentController, outputDataController, statisticsController )
{
    this.base = ListController;
    this.base( itemKeyGenerator, objectPresentationController, contentController, outputDataController, statisticsController );

    this.groups = new AtMap();

    this.objectGroupKeyGenerator = objectGroupKeyGenerator;
    this.groupPresentationController = groupPresentationController;

    function initGroups(groupsArray, groups){
        for( var i = 0; i < groupsArray.length; i++ )
        {
            var group = {};
            group.id = groupsArray[i].id;
            group.name = groupsArray[i].name;
            group.objects = new AtMap();
            groups.put( itemKeyGenerator.createKey( groupsArray[i] ), group );
            group.unremovableGroupObjectsCount = 0;
            group.visibleGroupObjectsCount = 0;
        }
    }
    initGroups(groupsArray, this.groups);
}
GroupListController.prototype = new ListController();

GroupListController.prototype.setObjectsMap = function ( objectsMap )
{
    ListController.prototype.setObjectsMap.call(this,objectsMap);
    var objectsList = objectsMap.getValues();
    for( var i = 0; i < objectsList.length; i++ )
    {
        this.add(objectsList[i]);
    }
};

// clear all selection flags of objects and groups
GroupListController.prototype.unselectObjects = function ()
{
    ListController.prototype.unselectObjects.call(this);
    this.unselectItems(this.groups.getValues());
};

/**
 * this function added object to list.
 * @param object object
 */
GroupListController.prototype.add = function ( object )
{
    ListController.prototype.add.call( this, object );
    var group = this.groups.get( this.objectGroupKeyGenerator.createKey( object ) );
    if (this.objectPresentationController.isVisible(object))
    {
        group.visibleGroupObjectsCount++;
        if ( this.contentController.isRemovable(object) )
        {
            group.unremovableGroupObjectsCount++;
        }
    }
    group.objects.put( this.itemKeyGenerator.createKey( object ), object );
};

/**
 * This function delete object from list(if object is Removable)
 * @param object object to delete
 * @return deleted object or null if object are not present in list.
 */
GroupListController.prototype.remove = function ( object )
{
    var retObject = ListController.prototype.remove.call( this, object );
    if( retObject !== null )
    {
        var group = this.groups.get( this.objectGroupKeyGenerator.createKey( retObject ) );
        if (this.objectPresentationController.isVisible(retObject))
        {
            group.visibleGroupObjectsCount--;
            if ( this.contentController.isRemovable(retObject) )
            {
                group.unremovableGroupObjectsCount--;
            }
        }
        group.objects.remove( this.itemKeyGenerator.createKey( retObject ) );
    }
    return retObject;
};

/**
 * update list without rerendering. Used when need only remove one item from list
 * and update statistics.
 */
GroupListController.prototype.lazyUpdateList = function ( object )
{
    ListController.prototype.lazyUpdateList.call( this, object );
    var groupKey = this.objectGroupKeyGenerator.createKey( object );
    var group = this.groups.get( groupKey );
    if( !this.groupPresentationController.isVisible( group ) )
    {
        this.groupPresentationController.getElement()
                .remove(group.option.index);
    }
    else if( this.groupPresentationController.isDisable( group ) )
    {
        group.option.disabled = true;
        group.option.selected = false;
        this.updateOptionGroupStyle( group, this.groupPresentationController );
    }
};

/**
 * This function recalc group counters, such as unremovableGroupObjectsCount and visibleGroupObjectsCount
 */
GroupListController.prototype.updateGroupCounters = function ( group )
{
    group.unremovableGroupObjectsCount = 0;
    group.visibleGroupObjectsCount = 0;
    var objectsList = group.objects.getValues();
    for( var i = 0; i < objectsList.length; i++ )
    {
        if( this.objectPresentationController.isVisible( objectsList[i] ) )
        {
            group.visibleGroupObjectsCount++;
            if( this.contentController.isRemovable( objectsList[i] ) )
            {
                group.unremovableGroupObjectsCount++;
            }
        }
    }
};

/**
 * This function show listbox with needed elements, update statistics and
 * data which sends to server.
 */
GroupListController.prototype.updateList = function ()
{
    var groupsToShow = this.groups.getValues();
    this.outputDataController.clearElement();
    this.objectPresentationController.clearElement();
    this.groupPresentationController.clearElement();
    this.statisticsController.clearStatistics();
    groupsToShow.sort( this.groupPresentationController.compareFunction );
    for( var i = 0; i < groupsToShow.length; i++ )
    {
        var group = groupsToShow[i];
        this.updateGroupCounters(group);
        var objectsToShow = group.objects.getValues();
        var isVisibleGroup = this.groupPresentationController.isVisible( group );
        if( isVisibleGroup )
        {
            this.showGroup( group );
            objectsToShow.sort( this.objectPresentationController.compareFunction );
        }
        for( var j = 0; j < objectsToShow.length; j++ )
        {
            var object = objectsToShow[j];
            this.statisticsController.addObjectStatistics( object );
            if( isVisibleGroup && this.objectPresentationController.isVisible( object ) )
            {
                this.showObject( object );
            }
            if( this.outputDataController.isEnabled() )
            {
                this.outputDataController.addObjectToOutputData( object );
            }
        }
        if( isVisibleGroup )
        {
            group.option.disabled = group.unremovableGroupObjectsCount === 0;
            group.option.selected = false;
            group.selected = false;
            this.updateOptionGroupStyle( group, this.groupPresentationController );
        }
    }

    this.updateSelection();
    this.scrollToFirstSelected();
    this.scrollToVisibleArea();
    this.showEmptyListIfNeeded();
};

GroupListController.prototype.showObject = function ( object )
{
    ListController.prototype.showObject.call(this,object);
    this.groupPresentationController.filterObjectOptionAttributes( object );
};

//this function used for show allowed groups.
GroupListController.prototype.showGroup = function ( group )
{
    this.createItemOption(group, this.itemKeyGenerator, this.groupPresentationController);
    this.updateOptionGroupStyle(group, this.groupPresentationController);
};

// this function select or unselect some options or groups in listbox, basing on old and current state
GroupListController.prototype.updateSelection = function ()
{
    ListController.prototype.updateSelection.call(this);
    var groupsList = [];
    var objectsMap = [];
    var list = this.groupPresentationController.getElement();
    for( var i = 0; i < list.length; i++ )
    {
        if( list.options[i].className === this.groupPresentationController.ITEM_CLASS )
        {
            var group = this.groups.get( list.options[i].value );
            group.option = list.options[i];
            groupsList.push( group );
        }
        else if( list.options[i].className === this.objectPresentationController.ITEM_CLASS )
        {
            var object = this.objects.get( list.options[i].value );
            object.option = list.options[i];
            objectsMap[this.itemKeyGenerator.createKey( object )] = true;
        }
    }
    for( var i = 0; i < groupsList.length; i++ )
    {
        var group = groupsList[i];
        var objectsList = groupsList[i].objects.getValues();
        if( (group.option.selected === false && group.selected === true)
                || (group.option.selected === true && group.selected === false) )
        {
            var isNeedChange = true;
            if( !group.option.selected )
            {
                for( var j = 0; j < objectsList.length; j++ )
                {
                    if( (this.itemKeyGenerator.createKey( objectsList[j] ) in objectsMap)
                            && this.contentController.isRemovable( objectsList[j] ) )
                    {
                        if( objectsList[j].option.selected !== objectsList[j].selected )
                        {
                            isNeedChange = false;
                            break;
                        }
                    }
                }
            }
            if( isNeedChange || (group.option.selected && !group.selected) )
            {
                for( var j = 0; j < objectsList.length; j++ )
                {
                    if( (this.itemKeyGenerator.createKey( objectsList[j] ) in objectsMap)
                            && this.contentController.isRemovable( objectsList[j] ) )
                    {
                        objectsList[j].option.selected = group.option.selected;
                    }
                }
            }
            group.selected = group.option.selected;
        }

        var isSelect = group.unremovableGroupObjectsCount > 0;
        for( var j = 0; j < objectsList.length; j++ )
        {
            var object = objectsList[j];
            if( this.objectPresentationController.isVisible( object )
                    && this.contentController.isRemovable( object ) )
            {
                if( object.option.selected === false )
                {
                    isSelect = false;
                }
                object.selected = object.option.selected;
            }
        }
        group.option.selected = isSelect;
        group.selected = isSelect;
    }
};

// end GroupListController definition
// -------------------------------------------------------------------------------------------------

/**
 * abstract class for visibility rules of ListBox with groups.
 * @param listElementId id of select element,
 */
function GroupPresentationController(listElementId, isVisibleEmptyGroup)
{
    this.base = PresentationController;
    this.base(listElementId);
    this.ITEM_CLASS = "listGroup";
    this.visibleEmptyGroup = isVisibleEmptyGroup;
}
GroupPresentationController.prototype = new PresentationController();

GroupPresentationController.prototype.isVisibleEmptyGroup = function()
{
    return this.visibleEmptyGroup;
};

/**
 * This method filtrate of option attributes of listbox object if it is needed
 */
GroupPresentationController.prototype.filterObjectOptionAttributes = function(option){};

/**
 * This method return true if group must be disable.
 */
GroupPresentationController.prototype.isDisable = function (group){};

// end GroupPresentationController definition
// -------------------------------------------------------------------------------------------------
