/**
 *   require /js/util/AtMap.js
 */

/**
 * This class implements Listbox element.
 * functionality:
 *  - show list of items.
 *  - sort items in list
 *  - change color and background for any item.
 *  - create data which sends to server
 *  - add new items to the list
 *  - remove items from list
 *  - collect statistics about all elements in list.
 * @param itemKeyGenerator generator which use to generate unique keys for list objects.(Class KeyGenerator)
 * @param objectPresentationController configured visibility Controller for operate
 *  presentation of listbox (Class PresentationController)
 * @param contentController  configured content Controller for operate
 *  consistence of listbox (Class ContentController)
 * @param outputDataController configured Output data controller for operate
 *  data which sends to server.(Class OutputDataController)
 * @param statisticsController used for collect any available statistics about object
 * in listbox. (Class StatisticsController)
 */
function ListController(itemKeyGenerator, objectPresentationController, contentController, outputDataController, statisticsController)
{
    this.objects = new AtMap();
    this.objectPresentationController = objectPresentationController;
    this.itemKeyGenerator = itemKeyGenerator;
    this.contentController = contentController;
    this.outputDataController = outputDataController;
    this.statisticsController = statisticsController;
}

ListController.prototype.setObjectsMap = function(objectsMap)
{
    this.objects = objectsMap;
};

/**
 *
 * @return all selected objects in list.
 * NDF: if you want reshow list but save selection simply call this method and then call updateElement.
 */
ListController.prototype.getSelectedObjects = function ()
{
    var arrayToReturn = [];
    var list = this.objectPresentationController.getElement();
    for (var i = 0; i < list.length; i++ )
    {
        var option = list.options[i];
        if (option.selected && option.className === this.objectPresentationController.ITEM_CLASS)
        {
            var key = option.value;
            var object = this.objects.get(key);
            object.selected = true;
            object.option = option;
            arrayToReturn.push(object);
        }
    }
    return arrayToReturn;
};


ListController.prototype.scrollToFirstSelected = function()
{
    var selectElement = this.objectPresentationController.getElement();
    if( selectElement.selectedIndex != -1 )
    {
        var optionsHeight = selectElement.scrollHeight / selectElement.options.length;
        var selectPosition = selectElement.selectedIndex * optionsHeight;
        if ( selectPosition < selectElement.scrollTop ||
             selectPosition > selectElement.scrollTop + (selectElement.size - 2) * optionsHeight )
        {
            selectElement.scrollTop = ( selectElement.selectedIndex - selectElement.size / 3 ) * optionsHeight;
        }
    }
};

ListController.prototype.scrollToVisibleArea = function()
{
    var selectElement = this.objectPresentationController.getElement();
    var optionsHeight = selectElement.scrollHeight / selectElement.options.length;
    selectElement.scrollTop = Math.min(
            selectElement.scrollTop,
            (selectElement.options.length - selectElement.size + 1) * optionsHeight
    );
};

/**
 * Check if visible list is empty and show message for empty list if needed.
 */
ListController.prototype.showEmptyListIfNeeded = function()
{
    if (this.objectPresentationController.isListEmpty())
    {
        this.objectPresentationController.showEmptyList();
    }
};

ListController.prototype.updateOptionGroupStyle = function ( group , presentationController )
{
    var groupStyle = presentationController.getItemStyle(group);
    group.option.style.background = groupStyle.background;
    group.option.style.color = groupStyle.color;
};

/**
 * add new item option to select of presentationController
 */
ListController.prototype.createItemOption = function (item, itemKeyGenerator, presentationController)
{
    var list = presentationController.getElement();
    var index = list.options.length;
    item.option = list.options[index] = new Option();

    item.option.value = itemKeyGenerator.createKey(item);
    item.option.className = presentationController.ITEM_CLASS;
    item.option.selected = item.selected;
    $(item.option).text(presentationController.getItemStringPresentation(item));
    //force update option view
    var bug_54518 = list.options[index].text;
    item.option.title = presentationController.getItemTitle(item);
};

// clear all selection flags
ListController.prototype.unselectItems = function(itemsList)
{
    for ( var i = 0; i < itemsList.length ; i++ )
    {
        itemsList[i].selected = false;
        if (itemsList[i].option)
        {
            itemsList[i].option.selected = false;
        }
    }
};

// clear all selection flags of objects
ListController.prototype.unselectObjects = function()
{
    this.unselectItems(this.objects.getValues());
};

/**
 * this function added object to list.
 * @param object object
 */
ListController.prototype.add = function (object)
{
    if (object != null)
    {
        this.objects.put(this.itemKeyGenerator.createKey(object), object);
    }
};

/**
 * This function delete object from list(if object is Removable)
 * @param object object to delete
 * @return deleted object or null if object are not present in list.
 */
ListController.prototype.remove = function (object)
{
    if (this.contentController.isRemovable(object))
    {
        return this.objects.remove(this.itemKeyGenerator.createKey(object));
    }
    else
    {
        object.selected = false;
    }
    return null;
};

/**
 * update list without rerendering. Used when need only remove one item from list
 * and update statistics.
 */
ListController.prototype.lazyUpdateList = function(object)
{
    var index = object.option.index;
    var value = object.option.value;
    this.objectPresentationController.getElement().remove(index);

    // we cannot rely on the fact, that presentation controller and output data controller have the same indexing -
    // for example, in users_selector.jsp output data controller also include disabled users, while presentation controller don't.
    // so we need to search the element by value
    var idx = -1;
    var options = this.outputDataController.getElement().options;
    for (var i = 0; i < options.length; i++)
    {
        if (options[i].value == value) {
            idx = options[i].index;
        }
    }

    if (idx >= 0) this.outputDataController.getElement().remove(idx);
    this.statisticsController.removeObjectStatistics(object);
};

/**
 * This function show listbox with needed elements, update statistics and
 * data which sends to server.
 */
ListController.prototype.updateList = function ()
{
    var arrayToShow = this.objects.getValues();
    this.outputDataController.clearElement();
    this.objectPresentationController.clearElement();
    this.statisticsController.clearStatistics();
    arrayToShow.sort(this.objectPresentationController.compareFunction);
    for (var i = 0; i < arrayToShow.length; i++)
    {
        var object = arrayToShow[i];
        this.statisticsController.addObjectStatistics(object);
        if (this.objectPresentationController.isVisible(object))
        {
            this.showObject(object);
        }

        if (this.outputDataController.isEnabled())
        {
            this.outputDataController.addObjectToOutputData(object);
        }
    }

    this.updateSelection();
    this.scrollToFirstSelected();
    this.scrollToVisibleArea();
    this.showEmptyListIfNeeded();
};

//this function used for show allowed objects.
ListController.prototype.showObject = function(object)
{
    this.createItemOption(object, this.itemKeyGenerator, this.objectPresentationController);
    object.option.disabled = (!this.contentController.isRemovable(object));
    this.updateOptionGroupStyle(object, this.objectPresentationController);
};

// this function execute additional actions for select or unselect some options in listbox
ListController.prototype.updateSelection = function(){};

// end ListController definition
// -------------------------------------------------------------------------------------------------

/**
 * Base Class for element Controlers.
 * @param ElementId id attribute of document element.
 */
function ElementController(ElementId)
{
    this.elementId = ElementId;
}

ElementController.prototype.getElement = function()
{
    return document.getElementById(this.elementId);
};

ElementController.prototype.getElementContainer = function()
{
    return document.getElementById(this.elementId + "Container");
};

ElementController.prototype.clearElement = function()
{
    var element = this.getElement();
    element.length = 0;
    element.disabled = false;
    this.customClear();
};

/**
 * if you should custom clear anything use this method.
 */
ElementController.prototype.customClear = function (){};

// end ElementController definition
// -------------------------------------------------------------------------------------------------

/**
 * abstract class for visibility rules of ListBox.
 * @param listElementId id of select element,
 */
function PresentationController(listElementId)
{
    this.base = ElementController;
    this.base(listElementId);
    this.ITEM_CLASS = "listObject";
}
PresentationController.prototype = new ElementController();

/**
 * Return true if Listbox is empty.
 */
PresentationController.prototype.isListEmpty = function()
{
    return this.getElement().length == 0;
};

/**
 * This method create String presentation of item
 */
PresentationController.prototype.getItemStringPresentation = function(item){};

/**
 * This method which get style for item
 */
PresentationController.prototype.getItemStyle = function(item){};

/**
 * This method create title for item
 */
PresentationController.prototype.getItemTitle = function(item)
{
    return "";
};

/**
 * This method rules item visibility.
 */
PresentationController.prototype.isVisible = function(item){};

/**
 * This methods call when list is Empty.
 */
PresentationController.prototype.showEmptyList = function(){};

/**
 * This method return function which use to compare two elements of array for sorting
 */
PresentationController.prototype.compareFunction = function(item1, item2){};

// end PresentationController definition
// -------------------------------------------------------------------------------------------------


/**
 * abstract Class for content rules of listBox
 */
function ContentController()
{
}

/**
 * This method retrurn true if object may remove from list.
 */
ContentController.prototype.isRemovable = function(object){};

// end ContentController definition
// -------------------------------------------------------------------------------------------------

/**
 *
 * @param isOutputDataPresents true if data in list box should be send to server flse otherwise.
 * @param outputDataElementId id of output element.
 */
function OutputDataController(isOutputDataPresents, outputDataElementId)
{
    this.base = ElementController;
    this.base(outputDataElementId);
    this.isEnabledOutput = isOutputDataPresents;
}
OutputDataController.prototype = new ElementController();

OutputDataController.prototype.isEnabled = function ()
{
    return this.isEnabledOutput;
};

/**
 * This method create output data which send to server.
 */
OutputDataController.prototype.getOutputDataForObject = function (object){};

/**
 * This method return true if object should send to server false otherwise.
 */
OutputDataController.prototype.isObjectIncludedToOutput = function (object){};

//this function create data which sends to server.
OutputDataController.prototype.addObjectToOutputData = function(object)
{
    var list = this.getElement();
    var index = list.options.length;
    var option = new Option();
    option.value = this.getOutputDataForObject(object);
    option.text = this.getOutputDataForObject(object);
    option.selected = true;
    list.options[index] = option;
};

// end OutputDataController definition
// -------------------------------------------------------------------------------------------------

/**
 * Class used for generate keys from objects.
 */
function KeyGenerator()
{
    this.createKey = function(object){};
}

/**
 * Class used to collect statistics data of controller.
 */
function StatisticsController()
{
}
/**
 * On update ListController calls this method for each contained object
 */
StatisticsController.prototype.addObjectStatistics = function(object){};

/**
 *remove Statistics
 */
StatisticsController.prototype.removeObjectStatistics = function(object){};

/**
 * Before update ListController call this method to clear all statistics data.
 */
StatisticsController.prototype.clearStatistics = function (){};

// end StatisticsController definition
// -------------------------------------------------------------------------------------------------

/**
 * Class container for item style.
 * @param color color of items.
 * @param background background color of item.
 */
function ItemStyle(color, background)
{
    this.color = color;
    this.background = background;
}