function ApproversDialog()
{
    this.constructor = ApproversDialog;
}

/**
 * @param callBacks is an object {
 *                    submit  = function( dialogInfo, isAutoApproved, selectedApproverIds )
 *                            function for save data to server (will be called, when user selected approvers)
 *
 *                    loadData = function( dialogInfo, dataLoadedCallback, loadingFailedCallback )
 *                            function for load data from server or other store
 *                            where dataLoadedCallback = function ( allApprovers, isUserAutoApproved, selectedApproverIds ),
 *                                      must be called from loadData( ... ), when all data prepared
 *                                  loadingFailedCallback = function() must be called from loadData( ... ) for hiding dialog, when data loading failed
 *
 *                    abortDataLoading = function()
 *                                     function must stop loading data, processing in loadData( ... )
 *                  }
 *
 * @param width
 */
ApproversDialog.prototype.init = function ( callBacks, width )
{

    this._callBacks = callBacks;
    this._treeController = null;
    this._activeFilter = null;

    this._emptyMessageBuilder = new EmptyMessageBuilder();

    this._spinnerController = new SpinnerController();

    var self = this;

    this._dialogDiv = $( ".approversDialog" );
    this._selectUsersManuallyRadio = $( "#approversDialog_selectUsersManuallyRadio" );
    this._userAutoApprovedRadio = $( "#approversDialog_userAutoApprovedRadio" );
    var selectAllUsersLink = $( "#approversDialog_selectAllLink" );
    var deselectAllUsersLink = $( "#approversDialog_deselectAllUsers" );
    this._treeContainer = $( "#approversDialog_treeContainer" );
    var resetSelectionLink = $( "#approversDialog_resetSelectionLink" );
    var selectionControlButton = $("#approversDialog_topSection").find(".doubleButton");
    this._submitButton = $( "#approversDialog_submitButton" );
    this._cancelButton = $( "#approversDialog_cancelButton" );

    this._dialog = this._dialogDiv.dialog( {
                                               autoOpen: false,
                                               draggable: false,
                                               resizable: false,
                                               modal: true, // this is needed for overlay to appear, so we can handle 'outside' clicks to close the dialog
                                               close: function ()
                                               {
                                                   if( self._onClose != null )
                                                   {
                                                       self._onClose();
                                                       self._onClose = null;
                                                   }
                                                   self._callBacks.abortDataLoading();
                                               },
                                               width: width,
                                               dialogClass: "approversDialog"
                                           } );


    resetSelectionLink.click( function ()
                              {
                                  self._treeController.resetNodes();
                                  self._updateRadioButtonsState( self._isInitiallyAutoApproved );
                              } );

    selectionControlButton.hover( function ()
                              {
                                  $(this).addClass("x-btn-over");
                              },
                              function ()
                              {
                                  $(this).removeClass("x-btn-over");
                              });

    selectAllUsersLink.click( function ()
                              {
                                  self._treeController.selectAll();

                                  var filteredNodesSet = self._activeFilter.getResults();

                                  //todo it would be better to have a method "selectAllExcept" instead of this
                                  for( var level in filteredNodesSet.map )
                                  {
                                      //noinspection JSUnfilteredForInLoop
                                      var levelObj = filteredNodesSet.map[level];
                                      for( var id in levelObj )
                                      {
                                          //noinspection JSUnfilteredForInLoop
                                          self._treeController.getSelectionController().markDeselected( levelObj[id] );
                                      }
                                  }

                                  self._selectionChangedCallBack();
                              } );

    this._userAutoApprovedRadio.change( function ()
                                        {

                                            self._treeContainer.addClass( "allUsersOptions" );

                                        } );

    this._selectUsersManuallyRadio.change( function ()
                                           {

                                               self._treeContainer.removeClass( "allUsersOptions" );

                                           } );


    deselectAllUsersLink.click( function ()
                                {
                                    self._treeController.deselectAll();
                                    self._selectionChangedCallBack();
                                } );

    this._submitButton.click( function ()
                              {
                                  if( !$( this ).hasClass( 'buttonIsDisabled' ) )
                                  {
                                      self._switchDialogInSavingState();
                                      self._save();
                                  }
                              } );

    this._cancelButton.click( this._self( this._close ) );

    return this;
};

ApproversDialog.prototype._save = function ()
{
    var isAutoApproved = this._userAutoApprovedRadio.attr( "checked" ) == "checked";
    var selectedApproverIds = [];
    var selectedApproverNodeSet = this._treeController.getSelectionController().getSelectedNodeSet();

    var selectedApproverIdToNodeMap = selectedApproverNodeSet.getLevelMap( 1 );

    for( var id in selectedApproverIdToNodeMap )
    {
        if( selectedApproverIdToNodeMap.hasOwnProperty( id ) )
        {
            selectedApproverIds.push( selectedApproverIdToNodeMap[id].value.id );
        }
    }
    selectedApproverIds = selectedApproverIds.length > 0 ? selectedApproverIds : null;

    this._callBacks.submit( this._dialogInfo, isAutoApproved, selectedApproverIds );
};

ApproversDialog.prototype._close = function ()
{
    this._dialog.dialog( 'close' );
};

ApproversDialog.prototype.hideAfterSaving = function ()
{
    this._dialog.off( "dialogbeforeclose" );
    this._close();
};

ApproversDialog.prototype._selectionChangedCallBack = function ()
{
    this._updateRadioButtonsState( false );
};

ApproversDialog.prototype._updateRadioButtonsState = function ( isUserAutoApproved )
{
    if( isUserAutoApproved )
    {
        this._userAutoApprovedRadio.attr( 'checked', 'checked' );
        this._treeContainer.addClass( "allUsersOptions" );
    }
    else
    {
        this._selectUsersManuallyRadio.attr( 'checked', 'checked' );
        this._treeContainer.removeClass( "allUsersOptions" );
    }
};

ApproversDialog.prototype._self = function ( func )
{
    var self = this;
    return function ()
    {
        func.apply( self, arguments );
    }
};

/**
 *  Note: callback will be run only once!
 */
ApproversDialog.prototype.setOnCloseCallback = function ( callback )
{
    this._onClose = callback;
};

ApproversDialog.prototype.showAt = function ( dialogInfo )
{
    this._dialogInfo = dialogInfo;
    this._open();
    this._switchDialogInLoadingState();
    this._callBacks.loadData( dialogInfo, this._self( this._dataLoadedCallback ), this._self( this._close ) );
};

ApproversDialog.prototype.update = function ( allApprovers, isUserAutoApproved, selectedApproverIds )
{
    if( this._dialogInfo == null ) return;
    this._dataLoadedCallback( allApprovers, isUserAutoApproved, selectedApproverIds );
};

ApproversDialog.prototype._open = function ()
{
    this._dialog.dialog( "open" );
    // when the dialog is created, it is wrapped into a container, that's why we use parent()
    this._dialog.parent().position( {
                                        of: $( this._dialogInfo.element ),
                                        my: "left top",
                                        at: "left bottom",
                                        offset: this._dialogInfo.offset,
                                        collision: "flip"
                                    } );
    this._initCloseEvents();

    this._dialog.focus();
};

ApproversDialog.prototype._initCloseEvents = function(){
    // close the dialog when user clicks outside
    doOnceOn( $( '.ui-widget-overlay' ), 'click', this._self( this._close ) );
    doOnceOnWindowResize( this._self( this._close ) );
};

ApproversDialog.prototype._switchDialogInSavingState = function ()
{
    this._dialogDiv.removeClass( 'loadingState' ).addClass( 'savingState' );
    this._dialog.on( "dialogbeforeclose", false );
    this._submitButton.addClass( 'buttonIsDisabled' );
    this._cancelButton.addClass( 'buttonIsDisabled' );
};

ApproversDialog.prototype._switchDialogInLoadingState = function ()
{
    this._selectUsersManuallyRadio.attr( 'checked', false );
    this._userAutoApprovedRadio.attr( 'checked', false );
    this._dialogDiv.removeClass( 'savingState' ).addClass( 'loadingState' );
    this._dialogDiv.removeClass( 'noLeaveTimeManagers' );
    this._dialog.off( "dialogbeforeclose" );
    this._submitButton.addClass( 'buttonIsDisabled' );
    this._cancelButton.removeClass( 'buttonIsDisabled' );
    this._spinnerController.spin( 'approversDialog_spinnerPlaceholder' );

    //it is needed because while data was saving and close was inactive, window may be resized
    this._initCloseEvents();
};

ApproversDialog.prototype._switchDialogInReadyState = function ()
{
    this._dialogDiv.removeClass( 'savingState loadingState' );
    this._dialog.off( "dialogbeforeclose" );
    this._submitButton.removeClass( 'buttonIsDisabled' );
    this._cancelButton.removeClass( 'buttonIsDisabled' );
    this._spinnerController.stop();

    //it is needed because while data was saving and close was inactive, window may be resized
    this._initCloseEvents();
};

ApproversDialog.prototype._updateSelectUsersManuallyState = function ( thereIsNoLeaveTimeManagers )
{
    this._dialogDiv.toggleClass( 'noLeaveTimeManagers', thereIsNoLeaveTimeManagers );
    this._selectUsersManuallyRadio.attr( "disabled", thereIsNoLeaveTimeManagers );
};

ApproversDialog.prototype._dataLoadedCallback = function ( allApprovers, isUserAutoApproved, selectedApproverIds )
{
    this._switchDialogInReadyState();
    this._updateTreeController( allApprovers );

    this._activeFilter.setUserIdToHide( this._dialogInfo.userId );

    var nodeIdToNodeMap = this._approverNodesMaker.getMap();
    var selectionNodeSet = new TreeNodeSet();
    for( var i = 0; i < selectedApproverIds.length; i++ )
    {
        var node = nodeIdToNodeMap[ selectedApproverIds[i] ];
        if( node != null )
        {
            selectionNodeSet.add( node );
        }
    }

    this._treeController.treeShownCallBack( this._treeContainer[0], this._activeFilter );
    this._treeController.initializeByNodeSet( selectionNodeSet );

    this._isInitiallyAutoApproved = isUserAutoApproved;
    this._updateRadioButtonsState( isUserAutoApproved );
    this._updateSelectUsersManuallyState( allApprovers.length == 0 );
};

ApproversDialog.prototype._updateTreeController = function ( allApprovers )
{
    if( this._isApproversChanged( allApprovers ) )
    {
        this._oldListOfAllApprovers = allApprovers;

        this._approverNodesMaker = new OneLevelTreeNodesHierarchyMaker().init( allApprovers );
        this._approverNodesMaker.compile(function(treeNode) { return treeNode.value.id; }, UserTreeNode);

        var rootNode = this._approverNodesMaker.getRootNode();
        this._treeController = new LazyTree( rootNode, this._emptyMessageBuilder );
        this._activeFilter = new CertainUserFilter( rootNode );
        var nodeRenderers = {};
        nodeRenderers[TreeNode.TYPE_USER] = new ApproverNodeRenderer( this._treeController, this._activeFilter, null );
        this._treeController.setNodeRenderers( nodeRenderers );
        this._treeController.getSelectionController().addListener( {
                                                                       nodeDeselected: this._self( this._selectionChangedCallBack ),
                                                                       nodeSelected: this._self( this._selectionChangedCallBack )
                                                                   } );
    }
};

/**
 * Note: order sensitive!
 */
ApproversDialog.prototype._isApproversChanged = function ( allApprovers )
{
    if( !this._oldListOfAllApprovers || this._oldListOfAllApprovers.length != allApprovers.length )
        return true;
    for( var i = 0; i < allApprovers.length; i++ )
        if( this._areApproversEqual( this._oldListOfAllApprovers[i], allApprovers[i] ) )
            return true;
    return false;
};

ApproversDialog.prototype._areApproversEqual = function ( oldApprover, newApprover )
{
    return oldApprover.id != newApprover.id ||
           oldApprover.firstName != newApprover.firstName ||
           oldApprover.lastName != newApprover.lastName ||
           oldApprover.userName != newApprover.userName ||
           oldApprover.middleInitial != newApprover.middleInitial ||
           oldApprover.canManageAllUsers != newApprover.canManageAllUsers;
};