// requires /js/controls/gui_control.js
// requires /js/util.js


/**
 * CheckListbox item.
 *  value - array contains strings with parts of name
 *  isCheckBox - true if the item has a checkbox
 *  checked - is item checked? only makes sense if isCheckBox == true
 *  hidden - is item invisible?
 *  children - array of item children (of CheckListboxItem type)
 *  tooltip - tooltip for the item
 */
function CheckListboxItem(value, isCheckBox, checked, hidden, children, tooltip)
{
    this.id = null;
    this.parent = null;
    this.value = value;
    this.isCheckBox = isCheckBox;
    this.checked = checked;
    this.hidden = hidden;
    this.children = children;
    this.tooltip = tooltip;
}


/**
 * Hierarchical listbox with checkboxes.
 *  id - listbox identifier, will be used as element id
 */
function CheckListbox(id)
{
    this.base = GUIControl;
    this.base(id);

    this.rootItem = null;
    this.itemsMap = null;
    this.grayed = false;
    this.selectionChangedHandler = null;

    this.normalClassName = "check-listbox";
    this.grayedClassName = "check-listbox grayed";
    this.uncheckedCheckboxLabelClassName = "text";
    this.checkedCheckboxLabelClassName = "text checked";
}
CheckListbox.prototype = new GUIControl;


/**
 * Sets the listbox items.
 *  rootItem - listbox root item. This item itself won't be shown, but its children will.
 * This method performs items syncronization, see syncAll() for details.
 */
CheckListbox.prototype.setItems = function(rootItem)
{
    this.rootItem = rootItem;
    this.itemsMap = [];
    this.syncAll(this.rootItem, this.getId() + "_item", null);
}


/**
 * Synchronizes the states of the items that are not leafs.
 * (unchecks items which have at least one unchecked child)
 *  item - root item of the subtree to synchronize.
 */
CheckListbox.prototype.syncNodeStates = function(item)
{
    for (var i = 0; i < item.children.length; i++)
    {
        this.syncNodeStates(item.children[i]);
    }

    if (item.children == null || item.children.length == 0)
    {
        this.syncItemParentState(item);
    }
}


/**
 * Overridden.
 */
CheckListbox.prototype.getHTML = function()
{
    return "<div id = '" + this.getContentsContainerId() + "' " +
                "class='" + this.getClassNameForCurrentState() + "' >" +
                 this.getContentsHTML() +
               "</div>";
}


/**
 * Sets all items' state to 'checked'.
 */
CheckListbox.prototype.selectAll = function()
{
    this.setAll(true);
    this.updateContents();

    if (this.selectionChangedHandler)
        this.selectionChangedHandler();
}


/**
 * Sets all items' state to 'unchecked'.
 */
CheckListbox.prototype.selectNone = function()
{
    this.setAll(false);
    this.updateContents();

    if (this.selectionChangedHandler)
        this.selectionChangedHandler();
}


/**
 * Makes listbox shown grayed.
 */
CheckListbox.prototype.makeGray = function()
{
    if (!this.grayed)
    {
        this.grayed = true;
        this.updateCssClass();
    }
}


/**
 * Makes listbox shown ungrayed.
 */
CheckListbox.prototype.unGray = function()
{
    if (this.grayed)
    {
        this.grayed = false;
        this.updateCssClass();
    }
}


/**
 * Sets the handler for the selection changed event.
 */
CheckListbox.prototype.setSelectionChangedHandler = function(handler)
{
    this.selectionChangedHandler = handler;
}


/**
 * Default item onClick event handler.
 * Updates parent class, syncs parent and children states.
 */
CheckListbox.prototype.onItemClick = function(element, isDerivative)
{
    if (!element)
    {
        return;
    }
    var isChecked = element.checked;
    var item = this.itemsMap[element.id];
    item.checked = isChecked;

    this.updateCheckboxLabelClass(element.id, isChecked);

    for (var i = 0; i < item.children.length; i++)
    {
        var child = document.getElementById(item.children[i].id);
        this.setCheckboxState(child, isChecked);
        this.onItemClick(child, true);
    }

    if (!isDerivative)
    {
        this.syncItemParentState(item);
        if (this.selectionChangedHandler)
            this.selectionChangedHandler();
    }
}

CheckListbox.prototype.onItemDblclick = function(elementId, isDerivative)
{
    var item = this.itemsMap[elementId];

    var allItemsWereChecked = true;
    for (var i = 0; i < item.children.length; i++)
    {
        var child = document.getElementById(item.children[i].id);
        if (!child.checked)
        {
            allItemsWereChecked = false;
            break;
        }
    }

    for (var i = 0; i < item.children.length; i++)
    {
        var child = document.getElementById(item.children[i].id);
        if (allItemsWereChecked || !child.checked)
        {
            this.setCheckboxState(child, !allItemsWereChecked);
            this.onItemClick(child, true);
        }
    }

    if (item.isCheckBox)
    {
        var cbx = document.getElementById(elementId);
        cbx.checked = !allItemsWereChecked;

        this.setCheckboxState(item, !allItemsWereChecked);
        this.onItemClick(child, true);
    }

    if (!isDerivative)
    {
        this.syncItemParentState(item);
        if (this.selectionChangedHandler)
            this.selectionChangedHandler();
    }
}

/**
 * Private helper method.
 * Recursively synchronizes the items.
 * 1. sets 'parent' property for all items
 * 2. unchecks items which have at least one unchecked child
 * 3. creates html element ids for all items.
 *
 * item - root item of the subtree to synchronize.
 * itemId - item's html element id (descendants' ids will be created by adding postfixes to this id)
 * itemParent - item's parent (or null for the listbox root item)
 */
CheckListbox.prototype.syncAll = function(item, itemId, itemParent)
{
    item.parent = itemParent;
    item.id = itemId;
    this.itemsMap[itemId] = item;

    for (var i = 0; i < item.children.length; i++)
    {
        this.syncAll(item.children[i], itemId + "_" + i, item);
    }

    if (item.children == null || item.children.length == 0)
    {
        this.syncItemParentState(item);
    }
}


/**
 * Private helper method.
 * Recursively synchronizes item's ancestors' states
 * (unchecks items which have at least one unchecked child).
 */
CheckListbox.prototype.syncItemParentState = function(item)
{
    var parent = item.parent;

    if (!parent)
        return;

    var checkedCount = 0;

    for (var i = 0; i < parent.children.length; i++)
    {
        if (parent.children[i].checked)
            checkedCount++;
    }

    var parentElement = document.getElementById(parent.id);

    if (checkedCount != parent.children.length)
    {
        parent.checked = false;
        this.setCheckboxState(parentElement, false);
    }

    this.syncItemParentState(parent);
}


/**
 * Private helper method.
 * Generates HTML for the item.
 */
CheckListbox.prototype.getHtmlForItem = function(item, level)
{
    var parts = [];

    if (level != 0)
    {
        // We use array instead of strings concatenation to improve performance under IE.
        parts.push("<tr style='");
        if (item.hidden)
        {
            parts.push("display: none;");
        }
        parts.push("' class='check-listbox-level-");
        parts.push(level);
        parts.push("'><td style='padding-left: ");
        parts.push(level-1);
        parts.push("em;' ");
        parts.push("title='");
        parts.push(item.tooltip);
        parts.push("' ondblclick='");
        parts.push(this.getId());
        parts.push(".onItemDblclick(\"" + item.id + "\")");
        parts.push("'>");

        if (item.isCheckBox)
        {
            parts.push("<input class='check-listbox-checkbox' type='checkbox' ");
            parts.push(item.checked ? "checked " : "");
            parts.push("id='");
            parts.push(item.id);
            parts.push("' onclick='");
            parts.push(this.getId());
            parts.push(".onItemClick(this)' >");
        }
        else
        {
            parts.push("&nbsp;");
        }

        parts.push("<label id='");
        parts.push(item.id);
        parts.push(".label' for='");
        parts.push(item.id);
        parts.push("' class='");
        parts.push(item.checked ? this.checkedCheckboxLabelClassName : this.uncheckedCheckboxLabelClassName);
        parts.push("'>");

        for (var n=0;n<item.value.length;n++)
        {
            parts.push("<span class='check-listbox-caption check-listbox-caption-part-");
            parts.push(n + 1);
            parts.push("'>");
            parts.push(escapeHtmlSymbols(item.value[n]));
            parts.push("</span> ");
        }

        parts.push("</label></td></tr>");
    }

    for (var i = 0; i < item.children.length; i++)
    {
        parts.push(this.getHtmlForItem(item.children[i], level + 1));
    }

    return parts.join("");
}


/**
 * Overridden.
 */
CheckListbox.prototype.getContentsHTML = function()
{
    return "<table cellspacing='0' cellpadding='0' border='0'>" +
            this.getHtmlForItem(this.rootItem, 0) +
            "<tr><td height='5'></td></tr>" +
           "</table>";
}


/**
 * Private helper method.
 */
CheckListbox.prototype.updateCheckboxLabelClass = function(checkboxId, checked)
{
    document.getElementById(checkboxId + ".label").className =
                (checked ? this.checkedCheckboxLabelClassName : this.uncheckedCheckboxLabelClassName);
}


/**
 * Private helper method.
 */
CheckListbox.prototype.setCheckboxState = function(checkbox, checked)
{
    if (checkbox)
    {
        checkbox.checked = checked;
        this.updateCheckboxLabelClass(checkbox.id, checked);
    }
}


/**
 * Private helper method.
 * Sets all items state.
 * Doesn't redraw, so do not forget to call updateContents() after this.
 */
CheckListbox.prototype.setAll = function(isChecked, root)
{
    if (!root)
        root = this.rootItem;

    if (root.isCheckBox)
        root.checked = isChecked;
    else
        root.checked = false;

    for (var i = 0; i < root.children.length; i++)
    {
        this.setAll(isChecked, root.children[i]);
    }
}


/**
 * Private helper method.
 */
CheckListbox.prototype.getClassNameForCurrentState = function()
{
    if (this.grayed)
        return this.grayedClassName;
    else
        return this.normalClassName;
}


/**
 * Private helper method.
 */
CheckListbox.prototype.updateCssClass = function()
{
    var element = document.getElementById(this.getContentsContainerId());
    if (element)
        element.className = this.getClassNameForCurrentState();
}
