Ext.namespace("AT");


/**
 * Base class for date range dropdown menu items.
 * @constructor
 * @cfg {Function} describe function that returns item description when evaluated in its scope.
 * @cfg {String} id range type identifier (required).
 * @cfg {String} title item title (optional).
 * If <code>title</code> is not specified, it is obtained with <code>id</code> used as a message key.
 * @cfg {String} style item style attribute value.
 * @cfg {Function} clickHandler item click handler. Menu item is passed as argument.
 * @cfg {Object} handlerScope scope to run the clickHandler in.
 */
AT.DateRangeSelectorState = Ext.extend(Object, {
   constructor: function(config) {
       Ext.apply(this, config);
       if (!this.title) {
           this.title = messageResource.getMessage(this.id);
       }
   },

   itemTextTemplate: new Ext.Template(
        '<span class="rangeItem" style="{2}">',
            '<div class="rangeItemComment">{1}</div>',
            '<div class="rangeItemTitle">{0}</div>',
        '</span>'),

   createItem: function(selectedState) {
       var title = this.title ? this.title : "";
       var comment = this.describe ? this.describe() : "";
       var style = this.style ? this.style : "";
       var item = new Ext.menu.CheckItem({
           text: this.itemTextTemplate.apply([title, comment, style]),
           checked: this === selectedState,
           group: this.radioGroup,
           state: this
       });
       item.on("click", this.clickHandler, this.handlerScope);
       return item;
   }
});

/**
 * Date range menu item with a background image. Also see the base class' parameters.
 * @constructor
 * @cfg {String} imgSrc url of the image file (required).
 * @see AT.DateRangeSelectorState
 */
AT.DateRangeSelectorImageMenuItem = Ext.extend(AT.DateRangeSelectorState, {
    constructor: function(config) {
        this.style = "background-image: url(" + atImgPath(this.imgSrc) + ");";
        AT.DateRangeSelectorState.call(this, config);
    }
});

/**
 * Date range menu separator item.
 */
AT.DateRangeSelectorSeparator = {
    createItem: function() { return new Ext.menu.Separator(); }
};


var daysFmt = new DateFormat("MMMM dd");
var daysYearFmt = new DateFormat("MMMM dd, yyyy");
var monthFmt = new DateFormat("MMMM, yyyy");
var shortMonthFmt = new DateFormat("MMMM");

var describeDate = function() {
    return daysFmt.formatDate(this.range.getToDate());
};
var describeDaysRange = function() {
    return daysFmt.formatDate(this.range.getFromDate()) + " &ndash; " + daysFmt.formatDate(this.range.getToDate());
};
var describeMonthRange = function() {
    return monthFmt.formatDate(this.range.getFromDate());
};
var describeYearRange = function() {
    return this.range.getFromDate().getFullYear();
};
var describeTwoMonthsRange = function() {
    return shortMonthFmt.formatDate(this.range.getFromDate()) + ", " + shortMonthFmt.formatDate(this.range.getToDate());
};

var describeDaysYearRange = function() {
    return daysFmt.formatDate(this.range.getFromDate()) + " &ndash; " + daysYearFmt.formatDate(this.range.getToDate());
};


AT.CurrentMonthMenuItem = Ext.extend(AT.DateRangeSelectorState,
{
    range: new AT.CurrentMonthDateRangeState(),
    describe: describeMonthRange,
    id: DateRangeStateUtil.CURRENT_MONTH,
    value: DateRangeSelectionQualifier.CURRENT_MONTH
});

AT.CurrentMonthTillTodayMenuItem = Ext.extend(AT.DateRangeSelectorState,
{
    range: new AT.CurrentMonthTillTodayDateRangeState(),
    describe: describeDaysRange,
    id: DateRangeStateUtil.CURRENT_MONTH_TILL_TODAY,
    value: DateRangeSelectionQualifier.CURRENT_MONTH_TILL_TODAY
});

AT.NextMonthMenuItem = Ext.extend(AT.DateRangeSelectorState,
{
    range: new AT.NextMonthDateRangeState(),
    describe: describeMonthRange,
    id: DateRangeStateUtil.NEXT_MONTH,
    value: DateRangeSelectionQualifier.NEXT_MONTH
});

AT.PreviousMonthMenuItem = Ext.extend(AT.DateRangeSelectorState,
{
    range: new AT.PreviousMonthDateRangeState(),
    describe: describeMonthRange,
    id: DateRangeStateUtil.PREVIOUS_MONTH,
    value: DateRangeSelectionQualifier.PREVIOUS_MONTH
});

AT.CurrentWeekMenuItem = Ext.extend(AT.DateRangeSelectorImageMenuItem,
{
    range: new AT.CurrentWeekDateRangeState(),
    imgSrc: "date_range_selector/current-week.gif",
    id: DateRangeStateUtil.CURRENT_WEEK,
    value: DateRangeSelectionQualifier.CURRENT_WEEK
});

AT.CurrentWeekTillTodayMenuItem = Ext.extend(AT.DateRangeSelectorState,
{
    range: new AT.CurrentWeekTillTodayDateRangeState(),
    describe: describeDaysRange,
    id: DateRangeStateUtil.CURRENT_WEEK_TILL_TODAY,
    value: DateRangeSelectionQualifier.CURRENT_WEEK_TILL_TODAY
});

AT.NextWeekMenuItem = Ext.extend(AT.DateRangeSelectorImageMenuItem,
{
    range: new AT.NextWeekDateRangeState(),
    imgSrc: "date_range_selector/next-week.gif",
    id: DateRangeStateUtil.NEXT_WEEK,
    value: DateRangeSelectionQualifier.NEXT_WEEK
});

AT.PreviousWeekMenuItem = Ext.extend(AT.DateRangeSelectorImageMenuItem,
{
    range: new AT.PreviousWeekDateRangeState(),
    imgSrc: "date_range_selector/previous-week-wide.gif",
    id: DateRangeStateUtil.PREVIOUS_WEEK,
    value: DateRangeSelectionQualifier.PREVIOUS_WEEK
});

AT.CurrentAndPreviousWeeksMenuItem = Ext.extend(AT.DateRangeSelectorImageMenuItem,
{
    range: new AT.CurrentAndPreviousWeeksDateRangeState(),
    imgSrc: "date_range_selector/current-and-previous-weeks.gif",
    id: DateRangeStateUtil.CURRENT_AND_PREVIOUS_WEEK,
    value: DateRangeSelectionQualifier.CURRENT_AND_PREVIOUS_WEEK
});

AT.CurrentAndTwoPreviousWeeksMenuItem = Ext.extend(AT.DateRangeSelectorImageMenuItem,
{
    range: new AT.CurrentAndTwoPreviousWeeksDateRangeState(),
    imgSrc: "date_range_selector/current-and-two-previous-weeks.gif",
    id: DateRangeStateUtil.CURRENT_AND_TWO_PREVIOUS_WEEKS,
    value: DateRangeSelectionQualifier.CURRENT_AND_TWO_PREVIOUS_WEEKS
});

AT.FirstHalfOfCurrentMonthMenuItem = Ext.extend(AT.DateRangeSelectorState,
{
    range: new AT.FirstHalfOfCurrentMonthState(),
    describe: describeDaysRange,
    id: DateRangeStateUtil.FIRST_HALF_OF_CURRENT_MONTH,
    value: null
});

AT.SecondHalfOfCurrentMonthMenuItem = Ext.extend(AT.DateRangeSelectorState,
{
    range: new AT.SecondHalfOfCurrentMonthState(),
    describe: describeDaysRange,
    id: DateRangeStateUtil.SECOND_HALF_OF_CURRENT_MONTH,
    value: null
});

AT.SecondHalfOfPreviousMonthMenuItem = Ext.extend(AT.DateRangeSelectorState,
{
    range: new AT.SecondHalfOfPreviousMonthState(),
    describe: describeDaysRange,
    id: DateRangeStateUtil.SECOND_HALF_OF_PREVIOUS_MONTH,
    value: null
});

AT.Last7DaysMenuItem = Ext.extend(AT.DateRangeSelectorState,
{
    range: new AT.Last7DaysDateRangeState(),
    describe: describeDaysRange,
    id: DateRangeStateUtil.LAST_7_DAYS,
    value: DateRangeSelectionQualifier.LAST_7_DAYS
});

AT.Last14DaysMenuItem = Ext.extend(AT.DateRangeSelectorState,
{
    range: new AT.Last14DaysDateRangeState(),
    describe: describeDaysRange,
    id: DateRangeStateUtil.LAST_14_DAYS,
    value: DateRangeSelectionQualifier.LAST_14_DAYS
});

AT.Last21DaysMenuItem = Ext.extend(AT.DateRangeSelectorState,
{
    range: new AT.Last21DaysDateRangeState(),
    describe: describeDaysRange,
    id: DateRangeStateUtil.LAST_21_DAYS,
    value: DateRangeSelectionQualifier.LAST_21_DAYS
});

AT.CurrentYearMenuItem = Ext.extend(AT.DateRangeSelectorState,
{
    range: new AT.CurrentYearDateRangeState(),
    describe: describeYearRange,
    id: DateRangeStateUtil.CURRENT_YEAR,
    value: DateRangeSelectionQualifier.CURRENT_YEAR
});

AT.CurrentYearTillTodayMenuItem = Ext.extend(AT.DateRangeSelectorState,
{
    range: new AT.CurrentYearTillTodayDateRangeState(),
    describe: describeDaysRange,
    id: DateRangeStateUtil.CURRENT_YEAR_TILL_TODAY,
    value: DateRangeSelectionQualifier.CURRENT_YEAR_TILL_TODAY
});

AT.PreviousYearMenuItem = Ext.extend(AT.DateRangeSelectorState,
{
    range: new AT.PreviousYearDateRangeState(),
    describe: describeYearRange,
    id: DateRangeStateUtil.PREVIOUS_YEAR,
    value: DateRangeSelectionQualifier.PREVIOUS_YEAR
});

AT.TodayMenuItem = Ext.extend(AT.DateRangeSelectorState,
{
    range: new AT.TodayDateState(),
    describe: describeDate,
    id: DateType.TODAY,
    value: EvaDateQualifier.TODAY
});

AT.YesterdayMenuItem = Ext.extend(AT.DateRangeSelectorState,
{
    range: new AT.YesterdayDateState(),
    describe: describeDate,
    id: DateType.YESTERDAY,
    value: EvaDateQualifier.YESTERDAY
});

AT.LastMondayMenuItem = Ext.extend(AT.DateRangeSelectorState,
{
    range: new AT.LastMondayDateState(),
    describe: describeDate,
    id: DateType.LAST_MONDAY,
    value: EvaDateQualifier.LAST_MONDAY
});

AT.LastTuesdayMenuItem = Ext.extend(AT.DateRangeSelectorState,
{
    range: new AT.LastTuesdayDateState(),
    describe: describeDate,
    id: DateType.LAST_TUESDAY,
    value: EvaDateQualifier.LAST_TUESDAY
});

AT.LastWednesdayMenuItem = Ext.extend(AT.DateRangeSelectorState,
{
    range: new AT.LastWednesdayDateState(),
    describe: describeDate,
    id: DateType.LAST_WEDNESDAY,
    value: EvaDateQualifier.LAST_WEDNESDAY
});

AT.LastThursdayMenuItem = Ext.extend(AT.DateRangeSelectorState,
{
    range: new AT.LastThursdayDateState(),
    describe: describeDate,
    id: DateType.LAST_THURSDAY,
    value: EvaDateQualifier.LAST_THURSDAY
});

AT.LastFridayMenuItem = Ext.extend(AT.DateRangeSelectorState,
{
    range: new AT.LastFridayDateState(),
    describe: describeDate,
    id: DateType.LAST_FRIDAY,
    value: EvaDateQualifier.LAST_FRIDAY
});

AT.LastSaturdayMenuItem = Ext.extend(AT.DateRangeSelectorState,
{
    range: new AT.LastSaturdayDateState(),
    describe: describeDate,
    id: DateType.LAST_SATURDAY,
    value: EvaDateQualifier.LAST_SATURDAY
});

AT.LastSundayMenuItem = Ext.extend(AT.DateRangeSelectorState,
{
    range: new AT.LastSundayDateState(),
    describe: describeDate,
    id: DateType.LAST_SUNDAY,
    value: EvaDateQualifier.LAST_SUNDAY
});

AT.EndOfThePreviousMonthMenuItem = Ext.extend(AT.DateRangeSelectorState,
{
    range: new AT.EndOfThePreviousMonthDateState(),
    describe: describeDate,
    id: DateType.END_OF_THE_PREVIOUS_MONTH,
    value: EvaDateQualifier.END_OF_PREVIOUS_MONTH
});

AT.BeginningOfTheCurrentMonthMenuItem = Ext.extend(AT.DateRangeSelectorState,
{
    range: new AT.BeginningOfTheCurrentMonthDateState(),
    describe: describeDate,
    id: DateType.BEGINNING_OF_THE_CURRENT_MONTH,
    value: EvaDateQualifier.BEGINNING_OF_CURRENT_MONTH
});

//------------------------- actiPLANS states -----------------------------

AT.CurrentAndNextMonthMenuItem = Ext.extend(AT.DateRangeSelectorState,
{
    range: new AT.CurrentAndNextMonthDateRangeState(),
    describe: describeTwoMonthsRange,
    id: DateRangeStateUtil.CURRENT_AND_NEXT_MONTH,
    value: null
});

AT.NextTwoMonthsMenuItem = Ext.extend(AT.DateRangeSelectorState,
{
    range: new AT.NextTwoMonthsDateRangeState(),
    describe: describeTwoMonthsRange,
    id: DateRangeStateUtil.NEXT_TWO_MONTHS,
    value: null
});

AT.EightWeeksAheadMenuItem = Ext.extend(AT.DateRangeSelectorState,
{
    range: new AT.SomeWeeksAheadDateRangeState(
        {countOfWeeks: 8}
    ),
    describe: describeDaysRange,
    id: DateRangeStateUtil.EIGHT_WEEKS_AHEAD,
    value: null
});

AT.TenWeeksAheadMenuItem = Ext.extend(AT.DateRangeSelectorState,
{
    range: new AT.SomeWeeksAheadDateRangeState(
        {countOfWeeks: 10}
    ),
    describe: describeDaysRange,
    id: DateRangeStateUtil.TEN_WEEKS_AHEAD,
    value: null
});

AT.TwelveWeeksAheadMenuItem = Ext.extend(AT.DateRangeSelectorState,
{
    range: new AT.SomeWeeksAheadDateRangeState(
        {countOfWeeks: 12}
    ),
    describe: describeDaysRange,
    id: DateRangeStateUtil.TWELVE_WEEKS_AHEAD,
    value: null
});

//------------------------------------------------------------------------

/*
 * This menu item differs from CustomDateMenuItem by id
 * and should be used in dateRange menuItems list like date_range_items.jsp
 */
AT.CustomDateRangeMenuItem = Ext.extend(AT.DateRangeSelectorState, {
    describe: function() { return "" },
    custom: true,
    id: DateRangeStateUtil.CUSTOM,
    value: DateRangeSelectionQualifier.CUSTOM
});

/*
 * This menu item differs from CustomDateRangeMenuItem by id
 * and should be used in single day menuItems list like date_type_items.jsp
 */
AT.CustomDateMenuItem = Ext.extend(AT.DateRangeSelectorState, {
    describe: function() { return "" },
    custom: true,
    id: DateType.CUSTOM_DATE,
    value: EvaDateQualifier.CUSTOM
});

AT.CurrentQuarterMenuItem = Ext.extend(
        AT.DateRangeSelectorState,
        {
            range: new AT.CurrentQuarterDateRangeState(),
            describe: describeDaysYearRange,
            id: DateRangeStateUtil.CURRENT_QUARTER,
            value: DateRangeSelectionQualifier.CURRENT_QUARTER
        });

AT.PreviousQuarterMenuItem = Ext.extend(
        AT.DateRangeSelectorState,
        {
            range: new AT.PreviousQuarterDateRangeState(),
            describe: describeDaysYearRange,
            id: DateRangeStateUtil.PREVIOUS_QUARTER,
            value: DateRangeSelectionQualifier.PREVIOUS_QUARTER
        });

AT.CurrentWeekNoImgMenuItem = Ext.extend(
        AT.DateRangeSelectorState,
        {
            range: new AT.CurrentWeekDateRangeState(),
            describe: describeDaysYearRange,
            id: DateRangeStateUtil.CURRENT_WEEK,
            value: DateRangeSelectionQualifier.CURRENT_WEEK
        });

AT.PreviousWeekNoImgMenuItem = Ext.extend(
        AT.DateRangeSelectorState,
        {
            range: new AT.PreviousWeekDateRangeState(),
            describe: describeDaysYearRange,
            id: DateRangeStateUtil.PREVIOUS_WEEK,
            value: DateRangeSelectionQualifier.PREVIOUS_WEEK
        });