Ext.namespace("AT");

/**
 * Date range selection component.
 * Has two date buttons, status label, and dropdown menu.
 *
 * Depends on:
 * <code>/js/at.js</code>
 * <code>/js/date_format.js</code>
 * <code>/js/date_js.jsp</code>
 * <code>/js/system_settings_js.jsp</code>
 * <code>/js/controls/at_rounded_box.js</code>
 * <code>/js/controls/dateControls/at_date_button.js</code>
 * <code>/js/message_resources.js</code> with prefixes "date_range_selector" and "range.state"
 *
 * Default styles in:
 * <code>/css/extjs/at-css/at-date-range-selector.css.jsp</code>
 *
 * @constructor
 * @cfg {Date} fromDate current selected 'from' date (required).
 * @cfg {Date} toDate current selected 'to' date (required).
 * @cfg {String} rangeTypeId current range type identifier (required).
 * @cfg {Array} menuStates array of {@link AT.DateRangeSelectorState} objects that contains the dropdown menu elements.
 * @cfg {Number} rangeMinDays minimal length of date range in days (defaults to 7). If equals -1,
 *               then no verification is performed.
 * @cfg {Number} rangeMaxDays maximal length of date range in days (defaults to 62). If equals -1,
 *               then no verification is performed.
 * @cfg {String} rangeReverseMessageKey key for message to show when 'to' date is before 'from' date.
 * @cfg {String} rangeTooSmallMessageKey key for message to show when date range is too small.
 * @cfg {String} rangeTooLargeMessageKey key for message to show when date range is too large.
 * @cfg {DateFormat} dateFormat format for parsing and returning {@link String} values.
 * @cfg {Function} onChangeCallBack a function to be called on selector state is changed (default Ext.emptyFn)
 * @cfg {Function} onRenderCallBack a function to be called on selector is rendered (default Ext.emptyFn)
 */
AT.DateRangeSelector = Ext.extend(Ext.Container, {
    autoEl: "div",
    layout: "table",
    calendarIcoSrc: "calendar_small_ico.gif",
    buttonTextFormat: Date.convertSmartToExtDateFormat(SystemSettings.dateFormat),
    onChangeCallBack: Ext.emptyFn,
    onRenderCallBack: Ext.emptyFn,
    hideFromDateButton: false,

    fromDate: null,
    toDate: null,
    rangeTypeId: null,
    menuStates: [],
    radioGroup: "dateRange",
    buttonsPaneCls: "",
    dropdownButtonCls: "",
    statusCls: "",
    rangeMinDays: 7,
    rangeMaxDays: 62,
    rangeReverseMessageKey: "date_range_selector.range_error.reverse.short",
    rangeTooSmallMessageKey: "date_range_selector.range_error.shorter_than_7_days",
    rangeTooLongMessageKey: "date_range_selector.range_error.longer_than_2_months",
    dateTooOldMessageKey: "date_range_selector.range_error.too_old",
    dateFormat: new DateFormat(TimeUtils.UNIFORM_DATE_FORMAT),
    errorMessageKey: null,
    scheduleCalculator: null,

    initComponent: function() {
        AT.DateRangeSelector.superclass.initComponent.call(this);

        this.dateRangeInitState = {
            fromDate : this.fromDate,
            toDate : this.toDate,
            dateRangeStateId : this.rangeTypeId
        };

        this.createStatusElement();

        this.fromDateButton = this.createDateButton("fromDate", this.scheduleCalculator);
        this.toDateButton = this.createDateButton("toDate", this.scheduleCalculator);
        this.updateButtons( this.fromDate, this.toDate );

        this.buttonsPane = this.getButtonsPane();
        this.add(this.buttonsPane);

        this.createDropdown();
        this.addEvents("change");
    },

    createDropdown: function() {
        this.dropdownCtn = this.getDropdownButton();
        this.add(this.dropdownCtn);
    },

    createStatusElement: function() {
        this.statusElement = new Ext.BoxComponent({
            autoEl: {
                tag: "span",
                cls: this.statusCls,
                html: ""
            }
        });
    },

    defineDropdownEvents: function() {
        this.dropdownCtn.on("render", function() { // deferring until the component is rendered
        this.dropdownCtn.el.on("mouseover", this.clickableAreaOver, this);
        this.dropdownCtn.el.on("mouseout", this.clickableAreaOut, this);
        this.dropdownCtn.el.on("mousedown", this.toggleMenu, this);
        this.dropdownCtn.el.dom.style.cursor = "pointer";
        }, this);
    },

    defineStatusElementEvents: function() {
        this.statusElement.on("render", function()
        {
            this.dropdownCtn.el.dom.style.cursor = "pointer";
        }, this);
    },

    defineButtonsPaneEvents: function() {
        this.buttonsPane.on("render", function() {
            this.buttonsPane.el.on("mouseover", this.clickableAreaOver, this);
            this.buttonsPane.el.on("mouseout", this.clickableAreaOut, this);
            this.buttonsPane.el.on("mousedown", this.toggleMenu, this);
            this.buttonsPane.el.dom.style.cursor = "pointer";
        }, this);
    },

    afterRender: function() {
        this.overClickableArea = false;
        this.overDate = false;
        this.defineStatusElementEvents();
        this.fromDateButton.on("render", function() {
            this.fromDateButton.el.on("mouseover", this.dateOver, this);
            this.fromDateButton.el.on("mouseout", this.dateOut, this);
        }, this);
        this.toDateButton.on("render", function() {
            this.toDateButton.el.on("mouseover", this.dateOver, this);
            this.toDateButton.el.on("mouseout", this.dateOut, this);
        }, this);
        this.defineButtonsPaneEvents();
        this.defineDropdownEvents();

        for (var i = 0; i < this.menuStates.length; ++i) {
            var state = this.menuStates[i];
            state.radioGroup = this.radioGroup;
            state.clickHandler = this.itemClicked;
            state.handlerScope = this;
            if (state.custom) {
                this.customState = state;
            }
            if (this.rangeTypeId == state.id) {
                var st = state;
                this.statusElement.on("render", function() { // deferring until the component is rendered
                    if (st.custom) {
                        this.updateState(st);
                        this.updateButtons(this.fromDate, this.toDate);
                    } else {
                        this.selectState(st);
                    }
                    this.onRenderCallBack.call(this);
                }, this);
            }
        }

        this.menuShown = false;

        function onDateChanged() {
            this.updateState(this.customState);
            this.verifyRangeSize();
            this.fireChangeEvent();
            this.onChangeCallBack();
        }

        var controller = this;

        function pickerOpened(button) {
            button.on("change", onDateChanged, controller);
        }

        function pickerClosed(button) {
            button.un("change", onDateChanged, controller);
        }

        function prepareButton(button, hoverMessageKey) {
            button.on("menushow", pickerOpened);
            button.on("menuhide", pickerClosed);
            button.show();
        }

        prepareButton(this.fromDateButton);
        prepareButton(this.toDateButton);
    },

    getBaseButtonItems: function() {
        var buttonItems = new Array();
        if( !this.hideFromDateButton )
        {
            buttonItems.push( this.fromDateButton,
            {
                xtype: "box",
                autoEl: {
                    tag: "span",
                    html: "&ndash;",
                    style: "text-align: center; width: 15px; display: inline-block; margin-left: 5px;"
                }
            });
        }
        buttonItems.push( this.toDateButton );
        return buttonItems;
    },

    /**
     * Creates the date buttons pane.
     * @private
     */
    getButtonsPane: function() {
        return new AT.RoundedBox({
            cls: this.buttonsPaneCls,
            straightBorder: "right",
            items: [
                this.statusElement,
                {
                    xtype: "container",
                    autoEl: "div",
                    layout: "table",
                    items: this.getBaseButtonItems()
                }]
        });
    },

    /**
     * Creates the dropdown button.
     * @private
     */
    getDropdownButton: function() {
        return new AT.RoundedBox({
            cls: this.dropdownButtonCls,
            straightBorder: "left"
        });
    },

    /**
     * Date button creation helper.
     * @private
     */
    createDateButton: function(name, scheduleCalculator) {
        return new AT.DateButton({
            format: this.buttonTextFormat,
            minValue: new Date(1970, 0, 1),
            weekStartDay: SystemSettings.weekStartDay - 1,
            scheduleCalculator: scheduleCalculator,
            icoSrc: imgPath(this.calendarIcoSrc),
            name: name
        });
    },

    /**
     * @private
     */
    fireChangeEvent: function() {
        this.fireEvent("change", this);
    },

    /**
     * Selects the provided state.
     * @private
     * @param {AT.DateRangeSelectorState} state state to select
     */
    selectState: function(state) {
        this.updateState(state);
        if (!state.custom) {
            this.updateButtons(state.range.getFromDate(), state.range.getToDate(), state.title);
        }
    },

    /**
     * Selects state by the provided stateId.
     * @private
     * @param {AT.DateRangeSelectorState} state state to select
     */
    selectStateById: function(stateId) {
        this.selectState( this.getRangeTypeById(stateId) );
    },

    /**
     * Recreates the dropdown menu.
     * @private
     */
    refreshMenu: function() {
        var menuItems = [];
        for (var i = 0; i < this.menuStates.length; ++i) {
            var state = this.menuStates[i];
            menuItems.push(state.createItem(this.currentState))
        }

        this.menu = new Ext.menu.Menu({
            items: menuItems,
            cls: "at-dropdown-list-btn-menu rangesListMenu"
        });
        this.menu.on("hide", this.hideMenu, this);
    },

    /**
     * Updates the 'state' title on the page.
     * @private
     * @param {AT.DateRangeSelectorState} state state to use
     */
    updateState: function(state) {
        this.statusElement.removeClass("errorStatus");
        this.statusElement.el.dom.innerHTML = state.title;
        this.currentState = state;
        this.errorMessageKey = null;
    },

    /**
     * Changes the 'state' title to show error text.
     * @private
     * @param {String} messageKey message key
     */
    errorState: function(messageKey) {
        this.statusElement.addClass("errorStatus");
        this.statusElement.el.dom.innerHTML = messageResource.getMessage(messageKey);
        this.errorMessageKey = messageKey;
    },

    getErrorMessageKey: function() {
        return this.errorMessageKey;
    },

    /**
     * Updates the date controls on page.
     * @private
     * @param {Date} fromDate 'from' date
     * @param {Date} toDate 'to' date
     */
    updateButtons: function(fromDate, toDate) {
        this.fromDateButton.setValue(fromDate);
        this.toDateButton.setValue(toDate);
    },

    /**
     * Flashes date controls with custom background color.
     */
    animateCustom: function() {
        if (!this.hideFromDateButton) {
            var fromTextEl = this.fromDateButton.el;
        }
        var toTextEl = this.toDateButton.el;

        var that = this;

        var cls = "buttonAnimated";
        (function() {
            if (!that.hideFromDateButton) {
                fromTextEl.toggleClass(cls);
            }
            toTextEl.toggleClass(cls);
        }).defer(500);

        (function() {
            if (!that.hideFromDateButton) {
                fromTextEl.toggleClass(cls);
            }
            toTextEl.toggleClass(cls);
        }).defer(1250);
    },

    /**
     * Dropdown menu item click handler.
     * @private
     */
    itemClicked: function (item) {
        if (item.state.custom) {
            this.animateCustom();
        }
        if (item.state !== this.currentState) {
            this.selectState(item.state);
            this.fireChangeEvent();
            this.onChangeCallBack();
        }
    },

    /**
     * Date controls mouseover handler.
     * @private
     */
    dateOver: function()
    {
        this.mouseOut();
        this.overDate = true;
    },

    /**
     * Date controls mouseout handler.
     * @private
     */
    dateOut: function()
    {
        this.overDate = false;
    },


    /**
     * mouseover handler.
     * @private
     */
    mouseOver: function()
    {
        if (!this.overDate)
            this.addClass("active");
    },

    /**
     * mouseout handler.
     * @private
     */
    mouseOut: function()
    {
        if (!this.menuShown)
        {
            this.removeClass("active");
        }
    },

    /**
     * Clickable area mouseover handler.
     * @private
     */
    clickableAreaOver: function() {
        this.mouseOver();
        this.overClickableArea = true;
    },

    /**
     * Clickable area mouseout handler.
     * @private
     */
    clickableAreaOut: function(event) {
        //only if new mouse target is not clickable area too
        if (!this.isClickableArea(event.getRelatedTarget()))
        {
            this.overClickableArea = false;
            this.mouseOut();
        }
    }, 

    isClickableArea: function (elem)
    {
        return isSameOrParentOf(elem, this.buttonsPane.el.dom)
            || isSameOrParentOf(elem, this.dropdownCtn.el.dom);
    },

    getMenuAlign: function() {
        return "t-b?";
    },

    getElementToShowMenu: function() {
        return this.el.child("table:first");
    },

    /**
     * Dropdown button click handler.
     * @private
     */
    toggleMenu: function() {
        if (!this.overDate || this.menuShown)
        {
            this.menuShown = !this.menuShown;
            if (this.menuShown) {
                this.refreshMenu(); // updating dates in the descriptions in case they changed
                this.menu.show(this.getElementToShowMenu(), this.getMenuAlign(), null);
            }
            if (!this.overClickableArea) {
                this.mouseOut();
            }
        }
    },

    /**
     * Dropdown menu hide handler.
     * @private
     */
    hideMenu: function() {
        this.menuShown = false;
        if (!this.overClickableArea) {
            this.mouseOut();
        }        
    },

    /**
     * Returns selected range type's identifier.
     * @return {String} range type's identifier
     */
    getRangeTypeId: function() {
        return this.currentState.id;
    },

    /**
     * Returns selected range type's title.
     * @return {String} range type's title
     */
    getRangeType: function() {
        return this.currentState;
    },

    /**
     * Returns dateRangeType with specified <code>id</code>
     * @param id of dateRangeType to return
     */
     getRangeTypeById: function( id ) {
        for( var i = 0; i < this.menuStates.length; i++ )
            if( this.menuStates[i].id == id )
                return this.menuStates[i];
    },

    /**
     * @return {Date} selected 'from' date.
     */
    getFromDate: function() {
        return this.fromDateButton.getValue();
    },

    /**
     * @return {Date} selected 'to' date.
     */
    getToDate: function() {
        return this.toDateButton.getValue();
    },

    /**
     * @return {String} selected 'from' date formatted to string.
     */
    getFormattedFromDate: function() {
        return this.dateFormat.formatDate(this.fromDateButton.getValue());
    },

    /**
     * @return {String} selected 'to' date formatted to string.
     */
    getFormattedToDate: function() {
        return this.dateFormat.formatDate(this.toDateButton.getValue());
    },

    /**
     * Checks that selected range is of proper size. Shows error message if it isn't.
     * @return {Boolean} true if it is, false otherwise
     */
    verifyRangeSize: function() {
        var messageKey = null;
        var fromDate = this.getFromDate().clearTime();
        var toDate = this.getToDate().clearTime();
        if (fromDate > toDate) {
            messageKey = this.rangeReverseMessageKey;
        } else if ( fromDate && fromDate.getFullYear() < 1970 ||
                    toDate && toDate.getFullYear() < 1970 ) {
            messageKey = this.dateTooOldMessageKey;
        } else if ( (this.rangeMinDays != -1) &&
                    fromDate.add(Date.DAY, this.rangeMinDays - 1).clearTime() > toDate ) {
            messageKey = this.rangeTooSmallMessageKey;
        } else if ( (this.rangeMaxDays != -1) &&
                    fromDate.add(Date.DAY, this.rangeMaxDays - 1).clearTime() < toDate ) {
            messageKey = this.rangeTooLongMessageKey;
        }
        if (messageKey) {
            this.errorState(messageKey);
            return false;
        } else {
            return true;
        }
    },

    /**
     * Checks that 'to' and 'from' dates correspond to the selected range type.
     * Falls back to 'custom' otherwise.
     */
    adjustRangeType: function() {
        if (this.currentState !== this.customState &&
            (!areDatesEqual(this.currentState.range.getFromDate(), this.getFromDate()) ||
             !areDatesEqual(this.currentState.range.getToDate(), this.getToDate()))) {
            this.currentState = this.customState;
        }
    },

    getDateRangeSelectorState : function( ) {
        return {
            fromDate : this.getFromDate(),
            toDate : this.getToDate(),
            dateRangeStateId : this.getRangeTypeId()
        };
    },

    isDateRangeChanged : function() {
        return !areDatesEqual(this.dateRangeInitState.toDate, this.getToDate()) ||
               (!this.hideFromDateButton && !areDatesEqual(this.dateRangeInitState.fromDate, this.getFromDate()))
    }
});

Ext.reg("at_date_range_selector", AT.DateRangeSelector);

/**
 * Returns Date object based on given data, or null if one or more of arguments are negative
*/
AT.DateRangeSelector.makeDate = function(day, month, year) {
    if (day < 0 || month < 0 || year < 0) {
        return null;
    } else {
        return createDate(day, month, year);
    }
};