/**
 * @author kostenko
 *
 * Contents of this file:
 *  - SmartField - general purpose smart text field
 *  - DecimalField - decimal number editing field (uses SmartField)
 */

/**
 * Smart field functionality consists of 3 parts:
 * 1. Allows only specific characters to be typed (still, any characters can be pasted). Optionally limits max length.
 * 2. On the field 'change' event (usually when the field loses focus), trims the text and validates it.
 *    Applies the 'invalid' CSS class and adds the 'title' attribute with hint if validation fails.
 * 3. Re-formats the entered text if validation succeeds and removes the 'invalid' class and 'title' attribute
 *    if they were applied before.
 *
 * @param fieldElement HTML input element (can be wrapped into jQuery)
 * @param charFilter function(charCode) - returns boolean whether the given character is allowed in this field
 * @param validator function(text) - returns boolean - whether the given text is valid for this field.
 *        (Empty string is never passed through validator; its correctness is determined by allowEmpty param.)
 * @param formatter function(text) - returns string - the formatted text.
 *        (Empty string is never passed through formatter).
 * @param allowEmpty boolean - whether an empty string is treated as a valid value
 * @param maxLength - maximum number of characters that can be entered. If not specified, then there's no limit.
 * @param [invalidHint] - text to display as tooltip when the field is in 'invalid' state
 * @param [maxValue] - maximum value for this field (value should be < than maxValue)
 */
function SmartField(fieldElement, charFilter, validator, formatter, allowEmpty, maxLength, invalidHint, maxValue)
{
    /**
     * @return string the current value
     */
    this.getValue = function()
    {
        return fieldElement.val();
    };

    /**
     * Sets the new value to the field.
     * Note that this will trigger the 'change' event handler which will trim, validate and re-format the given value.
     * @param value new value to set
     */
    this.setValue = function (value)
    {
        fieldElement.val(value);
        onChange();
    };

    /**
     * Trims, validates and re-formats the current value.
     * This function is invoked automatically on 'change' event, but sometimes it is useful to invoke it manually.
     */
    this.process = function ()
    {
        fieldElement.val(self.getValue().trim());

        if (!self.isValid())
        {
            setInvalidStatus();
        }
        else
        {
            unsetInvalidStatus();

            if (!self.isEmpty())
                fieldElement.val(formatter(self.getValue()));
        }
    };

    /**
     * @return boolean whether the current value is valid
     */
    this.isValid = function()
    {
        return self.isEmpty() ? Boolean(allowEmpty) : validator(self.getValue());
    };

    /**
     * @return boolean whether the current value is an empty string
     */
    this.isEmpty = function()
    {
        return self.getValue().length == 0;
    };

    /**
     * Removes the 'invalid' field status, if set, even if the text is still invalid.
     * This effectively removes the CSS 'invalid' class from the field and removes tooltip.
     */
    this.clearValidationResult = function()
    {
        unsetInvalidStatus();
    };

    this.focus = function()
    {
        fieldElement.focus();
    };

    /**
     * Sets new invalid hint
     * @param newInvalidHint new hint to set
     */
    this.setInvalidHint = function( newInvalidHint )
    {
        invalidHint = newInvalidHint;
        if (!self.isValid())
            setInvalidStatus();
    };

    // ---------- private ----------

    fieldElement = $(fieldElement);
    var self = this;

    function setInvalidStatus()
    {
        fieldElement.addClass('invalid');
        fieldElement.attr('title', invalidHint);
    }

    function unsetInvalidStatus()
    {
        fieldElement.removeClass('invalid');
        fieldElement.removeAttr('title');
    }

    function onKeyPress(event)
    {
        // always allow special keys such as arrows, backspace, etc. (fired under FireFox & Opera only)
        if (event.which < 32)
            return;

        // FireFox, Opera & Safari fire this event even when using modifier keys.
        // In this case we must not prevent the default behavior.
        if (event.ctrlKey || event.altKey || event.metaKey)
            return;

        // apply filter
        if (!charFilter(event.which))
            event.preventDefault();
    }

    function onChange()
    {
        self.process();
    }

    fieldElement.keypress(onKeyPress);
    fieldElement.change(onChange);

    if (maxLength != null)
        fieldElement.attr('maxlength', maxLength);
}


/**
 * Smart field for editing decimal numbers.
 *
 * @param fieldElement HTML input element (can be wrapped into jQuery)
 * @param allowEmpty boolean - whether an empty string is treated as a valid value
 * @param maxLength - maximum number of characters that can be entered. If not specified, then there's no limit.
 * @param maxFractionDigits - maximum number of digits in fractional part.
 *        If entered value contains more fractional digits, it will be rounded during re-formatting.
 * @param [decimalSeparator] - symbol to use as decimal separator when formatting and parsing. By default '.' (dot).
 *                             NB! When parsing, '.' and ',' are always allowed as decimal separators, unless one of
 *                             them is set as digit grouping symbol.
 * @param [allowNegative] - whether to allow entering negative values. True by default
 * @param [digitGroupingSymbol] - digit grouping symbol to use when formatting and parsing. Default is empty string.
 * @param [invalidHint] - text to display as tooltip when the field is in 'invalid' state
 * @param [allowZero] - whether to allow entering zero value. True by default
 * @param [maxValue] - maximum value for this field (value should be < than maxValue)
 *
 * TODO: allow/disallow fractions? limit max/min?
 */
function DecimalField(fieldElement, allowEmpty, maxLength, maxFractionDigits, decimalSeparator, allowNegative, digitGroupingSymbol,
                      invalidHint, allowZero, maxValue)
{
    /**
     * @return string the current value
     */
    this.getValue = function()
    {
        return smartField.getValue();
    };

    /**
     * @return Number the current value as number
     */
    this.getValueAsNumber = function()
    {
        return convertToNumber( this.getValue() );
    };

    /**
     * Sets the new value to the field.
     * This method treats the given parameter as a string, as if the user entered the value from keyboard.
     * Note that this will trigger the 'change' event handler which will trim, validate and re-format the given value.
     * @param value string new value to set
     */
    this.setValue = function (value)
    {
        smartField.setValue(value);
    };

    /**
     * Sets the new value to the field.
     * This method treats the given parameter as a javascript number, and formats it as string using the rules specified
     * in the constructor.
     * Note that the validation will also occur, because the given number may be negative while negatives may be disallowed.
     * @param value number
     */
    this.setValueAsNumber = function (value)
    {
        this.setValue( formatNumber(value) );
    };

    /**
     * Trims, validates and re-formats the current value.
     * This function is invoked automatically on 'change' event, but sometimes it is useful to invoke it manually.
     */
    this.process = function ()
    {
        smartField.process();
    };

    /**
     * @return boolean whether the current value is valid
     */
    this.isValid = function()
    {
        return smartField.isValid();
    };

    /**
     * @return boolean whether the current value is an empty string
     */
    this.isEmpty = function()
    {
        return smartField.isEmpty();
    };

    /**
     * Removes the 'invalid' field status, if set, even if the text is still invalid.
     * This effectively removes the CSS 'invalid' class from the field.
     */
    this.clearValidationResult = function()
    {
        smartField.clearValidationResult();
    };

    this.focus = function()
    {
        smartField.focus();
    };

    /**
     * Sets decimal separator and then re-formats value if the field is valid
     * @param newDecimalSeparator
     */
    this.setDecimalSeparator = function( newDecimalSeparator )
    {
        var wasValid = this.isValid();
        var value;
        if ( wasValid )
            value = this.getValueAsNumber();

        decimalSeparator = newDecimalSeparator;
        numberFormatter = new NumberFormatter(decimalSeparator, digitGroupingSymbol, maxFractionDigits, true);

        if ( wasValid )
            this.setValueAsNumber( value );
    };

    /**
     * Sets digit grouping symbol and then re-formats value if the field is valid
     * @param newDigitGroupingSymbol
     */
    this.setDigitGroupingSymbol = function( newDigitGroupingSymbol )
    {
        var wasValid = this.isValid();
        var value;
        if ( wasValid )
            value = this.getValueAsNumber();

        digitGroupingSymbol = newDigitGroupingSymbol;
        numberFormatter = new NumberFormatter(decimalSeparator, digitGroupingSymbol, maxFractionDigits, true);

        if ( wasValid )
            this.setValueAsNumber( value );
    };

    /**
     * Sets new invalid hint
     * @param newInvalidHint new hint to set
     */
    this.setInvalidHint = function( newInvalidHint )
    {
        smartField.setInvalidHint( newInvalidHint );
    };

    // ---------- private ----------

    function charFilter(charCode)
    {
        var sym = String.fromCharCode( charCode );

        return sym >= '0' && sym <= '9'
            || sym == '-' && allowNegative
            || sym == '.'
            || sym == ','
            || sym == decimalSeparator
            || sym == digitGroupingSymbol;
    }

    function convertToNumber(text)
    {
        text = text.trim();

        // in case '-' is configured as decimal separator or digit grouping symbol,
        // we still need to treat the leading '-' as negative number sign

        var negative = false;

        if( text[0] == '-' )
        {
            text = text.substr(1);
            negative = true;
        }

        if( digitGroupingSymbol != '' )
            text = text.split( digitGroupingSymbol ).join( '' );

        text = text.replace( decimalSeparator, '.' );
        text = text.replace(',', '.');

        if( text.match( /[a-z]/i ) ) // disallow exponential and hexadecimal notation and infinity
            return NaN;

        if( negative )
            text = '-' + text;

        return Number( text );
    }

    function formatNumber(number)
    {
        return numberFormatter.format(number);
    }

    function getValidator()
    {
        return function (text) {
            var number = convertToNumber(text);
            return !isNaN(number) && (number >= 0 || allowNegative) && (number != 0 || allowZero) && (!maxValue || number < maxValue);
        };
    }

    function getFormatter()
    {
        return function (text) {
            return formatNumber( convertToNumber(text) );
        }
    }

    if (allowNegative == undefined)
        allowNegative = true;

    if (allowZero == undefined)
        allowZero = true;

    if (decimalSeparator == undefined)
        decimalSeparator = '.';

    if (digitGroupingSymbol == undefined)
        digitGroupingSymbol = '';

    var numberFormatter = new NumberFormatter(decimalSeparator, digitGroupingSymbol, maxFractionDigits, true);

    var smartField = new SmartField(fieldElement,
                                    charFilter,
                                    getValidator(),
                                    getFormatter(), allowEmpty, maxLength, invalidHint, maxValue);
}