/**
 * Renders user group management lightbox elements. Renders move to menu and delegate user and group rendering to corresponding renderers.
 * @param userGroupRenderer renderer of user groups
 * @param userRenderer renderer of users
 * @param addGroupControlRenderer renderer of add group control
 * @param userGroupContainer container for rendering user groups
 * @param userContainer container for rendering users
 * @param addGroupControlContainer container for add group control
 * @param moveToMenuDivId  dom id of element where move to menu will render
 */

function CommonRenderer(userGroupRenderer, userRenderer, addGroupControlRenderer, userGroupContainer, userContainer, addGroupControlContainer, moveToMenuDivId) {
    this.userGroupRenderer = userGroupRenderer;
    this.userRenderer = userRenderer;
    this.addGroupControlRenderer = addGroupControlRenderer;
    this.userGroupContainer = userGroupContainer;
    this.userContainer = userContainer;
    this.addGroupControlContainer = addGroupControlContainer;
    this.moveToMenuDivId = moveToMenuDivId;
}

/**
 * Render users
 * @param controller user group management controller
 * @param selectedGroup currently selected user group
 * @param usersList - list of users for generate content
 * @param isShowDisabledUsers is disabled users show in system
 * @param highlighter highlighter for user name
 */
CommonRenderer.prototype.generateUserContent = function(controller, selectedGroup, usersList, isShowDisabledUsers, highlighter) {
    this.userContainer.innerHTML = "";
    var countOfDisabledUsers = 0;
    
    var emptyDueToFilter = controller.emptyDueToFilter;
    var filtered = controller.filtered;
    var disabledUsersDivs = [];
    for (var i = 0; i < usersList.length; i++) {
        var div = this.userRenderer.generateContent(controller, usersList[i], highlighter);
        if (!usersList[i].active) {
            countOfDisabledUsers++;
            if (!isShowDisabledUsers) {
                disabledUsersDivs.push(div);
            }
        }
        this.userContainer.appendChild(div);
    }
    
    if (!filtered || (countOfDisabledUsers > 0 && countOfDisabledUsers != usersList.length) ) 
    {
        _.each(disabledUsersDivs, function (div) {div.style.display = 'none';});
    }
    else 
    {
        countOfDisabledUsers = 0;
    }
    
    if (emptyDueToFilter)
        controller.getEmptyGroupResolver().resolveEmptyUserGroupAfterFilterApplied(selectedGroup);
    else
        controller.getEmptyGroupResolver().resolveEmptyUserGroupElementState(selectedGroup, isShowDisabledUsers, countOfDisabledUsers);
};

/**
 * Render user groups
 * @param data data with user groups information
 */
CommonRenderer.prototype.generateGroupContent = function(data) {
    this.userGroupContainer.innerHTML = "";
    for (var i = 0; i < data.length; i++) {
        this.userGroupContainer.appendChild(this.userGroupRenderer.generateContent(data[i]));
    }
};

/**
 * Render move to menu
 */
CommonRenderer.prototype.generateMoveToMenu = function() {
    var button =  new Ext.Button({
                                    applyTo: this.moveToMenuDivId,
                                    cls: "at-dropdown-list-btn",
                                    ctCls: "moveToButton",
                                    menu: new Ext.menu.Menu({
                                                                items: [],
                                                                cls: "groupManagementLightBox at-scrollable-menu"
                                                            }),
                                    menuAlign: "tl-bl",
                                    text: messageResource.getMessage("group_management.move_to")
                                });
    var closeMenuCb = function(){button.hideMenu()};
    $(window).off("resize.moveToMenu").on("resize.moveToMenu", closeMenuCb);
    return button;
};

/**
 * Get menu items for move to menu
 * @param controller user group management controller
 * @param groups user groups needed for move to menu
 */
CommonRenderer.prototype.getMenuItems = function(controller, groups) {
    var menuItems = [];
    for (var i = 0; i < groups.length; i++) {
        menuItems[i] = new Ext.menu.Item({
                                             text: this.userGroupRenderer.getItemText(groups[i]),
                                             value: groups[i].id
                                         });
        menuItems[i].on({
                            'click': {
                                scope: menuItems[i],
                                fn: function () {
                                    controller.moveToMenuClicked(this.value);
                                }
                            }
                        });

    }
    return menuItems;
};

/**
 * Reinit items of move to menu
 * @param controller user group management controller
 * @param moveToMenu move to menu element
 * @param groups groups needed to view in move to menu
 */
CommonRenderer.prototype.reInitMoveToMenu = function(controller, moveToMenu, groups) {
    var menuItems = this.getMenuItems(controller, groups);
    moveToMenu.menu.removeAll();
    for (var i = 0; i < menuItems.length; i++) {
        moveToMenu.menu.addItem(menuItems[i]);
    }
    if (menuItems.length == 0) {
        moveToMenu.disable();
    }
};

/**
 * Clean all containers
 */
CommonRenderer.prototype.cleanup = function() {
    this.userGroupContainer.innerHTML = "";
    this.userContainer.innerHTML = "";
};

CommonRenderer.prototype.generateAddGroupControlContent = function() {
    this.addGroupControlContainer.innerHTML = '';
    this.addGroupControlContainer.appendChild(this.addGroupControlRenderer.generateContent());
};






