/**
 * Handles all operations with drag and drop
 * @param ddProxyClassName class name for 'users in flight' element
 * @param ddProxyRowClassName class name for one row with user name in 'users in flight' element
 * @param draggedUserDivClassName class name user row when this user is dragged now
 * @param undroppableClassName class name for place unavailable for drop
 */
function DragAndDropController(ddProxyClassName, ddProxyRowClassName, draggedUserDivClassName,
                                undroppableClassName, groupNamingKey) {
    this.ddProxyClassName = ddProxyClassName;
    this.ddProxyRowClassName = ddProxyRowClassName;
    this.draggedUserDivClassName = draggedUserDivClassName;
    this.undroppableClassName = undroppableClassName;
    this.dragGroupName = 'users';
    this.currentDraggedUsers = null;
    this.groupNamingKey = groupNamingKey;
}

/**
 * Wrap user row with specific drag and drop ext js classes. Define all event handlers.
 * @param controller user group management controller
 * @param div user row intended for wrapping
 * @param userId id of user intended to be draggable
 * @param isDDLockedNow is drag and drop locked now (due to ajax)
 */
DragAndDropController.prototype.makeDDWrapperForUser = function(controller, div, userId, isDDLockedNow) {
    var dd = new Ext.dd.DDProxy(div, this.dragGroupName, {
        isTarget  : false,
        scroll: false
    });
    dd.DDM.useCache = false;

    var onEscHandler = function(event)
    {
        if (event.keyCode == KEY_CODE_ESC)
        {
            dd.DDM.stopDrag(event);
            cancelBubble(event);
        }
    };

    var context = this;
    Ext.apply(dd, {
        onDragEnter : function(evtObj, targetElId) {
            var group = controller.getGroupController().getGroup(GroupManagementUtil.getElementId(targetElId));
            if (!controller.getGroupController().isGroupSuitableForDrop(group.id)) {
                context.dragEnterToUnavailableGroup(group, controller);
            } else {
                context.dragEnterToAvailableGroup(group);
            }
        },
        onDragOut : function(evtObj, targetElId) {
            var group = controller.getGroupController().getGroup(GroupManagementUtil.getElementId(targetElId));
            context.dragOut(group, controller);
        },
        onDragDrop : function(evtObj, targetElId) {
            var group = controller.getGroupController().getGroup(GroupManagementUtil.getElementId(targetElId));
            context.dragOut(group, controller);
            if (group.isHovered()) // if group under cursor and visible
            {
                context.dropUsersWithChecking(controller, group.id, GroupManagementUtil.makeIdArrayFromObjectArray(context.currentDraggedUsers));
            }
        },
        b4StartDrag : function(x,y) {
            controller.passiveEventHandler(function() {context.beforeStartDragAction(dd, controller, userId, x, y)}, false);
            KeyDownHandler.setOnKeyDownHandler(onEscHandler);
        },
        endDrag : function() {
            dd.getDragEl().innerHTML = "";
            document.body.style.cursor = "default";
            controller.removeDragAndDropClassNamesFromLightboxContainer();
            for (var i = 0; i < context.currentDraggedUsers.length; i++) {
                removeClass(context.currentDraggedUsers[i].div, context.draggedUserDivClassName);
                context.currentDraggedUsers[i].checkbox.disabled = "";
            }
            context.currentDraggedUsers = null;
            KeyDownHandler.restoreOnKeyDownHandler();
        },
        b4MouseDown : function() {
            controller.hideErrors();
            controller.endGroupEditing(false);
        }
    });
    if (isDDLockedNow) {
        dd.lock();
    }
    return dd;
};

/**
 * Wrap user group with drag and drop target extjs classes
 * @param div div with user group
 */
DragAndDropController.prototype.makeDDWrapperForGroup = function(div) {
    new Ext.dd.DDTarget(div, this.dragGroupName);
};

/**
 * If user enter to group, not available to drop
 * @param group GroupRow instance, unavailable for drop
 * @param controller user group management controller
 */
DragAndDropController.prototype.dragEnterToUnavailableGroup = function(group, controller) {
    var groupController = controller.getGroupController();
    addClass(group.div, this.undroppableClassName);
    if (!groupController.getEditedGroup() || groupController.getEditedGroup().id != group.id) {
        group.divOnMouseOut();
        group.disableMouseEvents();
        group.groupButtonControlOnMouseOut.call(group.editLink);
        group.groupButtonControlOnMouseOut.call(group.deleteLink);
    } else {
        group.disableEditedGroupMouseEvents();
        group.groupUndoAcceptEditButtonOnMouseOut.call(group.undoEditButton);
        group.groupUndoAcceptEditButtonOnMouseOut.call(group.acceptEditButton);
    }
};

/**
 * If user enter to group, available to drop
 * @param group GroupRow instance, available to drop
 */
DragAndDropController.prototype.dragEnterToAvailableGroup = function(group) {
    group.disableMouseEvents(true);
    group.groupButtonControlOnMouseOut.call(group.editLink);
    group.groupButtonControlOnMouseOut.call(group.deleteLink);
};

/**
 * Drag out of group row
 * @param group GroupRow instance
 * @param controller user group management controller
 */
DragAndDropController.prototype.dragOut = function(group, controller) {
    var groupController = controller.getGroupController();
    removeClass(group.div, this.undroppableClassName);
    if (!groupController.getEditedGroup() || groupController.getEditedGroup().id != group.id) {
        group.enableMouseEvents();
    } else {
        group.enableEditedGroupMouseEvents();
    }
};

/**
 * Fires before started a users drag
 * @param dd user DD wrapper
 * @param controller user group management controller
 * @param userId id of user
 */
DragAndDropController.prototype.beforeStartDragAction = function(dd, controller, userId, x,y) {
    dd.setDelta(-5, 0);
    var draggedUsers = controller.getUserController().getUsersNeededForDrag(userId, controller.isDisabledUsersShownDueToFilter());
    var ddProxy = dd.getDragEl();
    ddProxy.className = this.ddProxyClassName;
    var renderer = new DragAndDropUsersInFlightRenderer(this.ddProxyRowClassName, draggedUsers.length);
    for (var i = 0; i < draggedUsers.length; i++) {
        if (renderer.permitToRender) {
            ddProxy.appendChild(renderer.generateOneUserDiv(draggedUsers[i]));
        }
        addClass(draggedUsers[i].div, this.draggedUserDivClassName);
        draggedUsers[i].checkbox.disabled = "disabled";
    }
    dd.alignElWithMouse(ddProxy, x,y);
    document.body.style.cursor = "move";
    controller.setDragAndDropClassNamesToLightboxContainer();
    this.currentDraggedUsers = draggedUsers;
};

/**
 * fires when drop occured
 * @param controller user group management controller
 * @param groupId id of target user group
 * @param userIds ids of all dragged users
 */
DragAndDropController.prototype.onDropAction = function(controller, groupId, userIds) {
    var group = controller.getGroupController().getGroup(groupId);
    var manager = new GroupMoveRPCManager(controller.getServiceName(), controller, groupId, controller.getGroupController().getGroup(groupId).name, userIds, this.groupNamingKey);
    RpcControlManager.saveRpcControlBoundData(manager, false);
};

/**
 * Drop users with checking target user group availability for drop and if group under cursor is visible
 * @param controller user group management controller
 * @param groupId id of target user group
 * @param userIds ids of all dragged users
 */
DragAndDropController.prototype.dropUsersWithChecking = function(controller, groupId, userIds) {
    var context = this;
    var groupController = controller.getGroupController();
    if (groupController.isGroupSuitableForDrop(groupId) && userIds.length > 0) {
        controller.groupEventHandler(function() {context.onDropAction(controller, groupId, userIds)}, false);
    }
};

/**
 * Renders element displayed near the cursor when drag occurs
 * @param divClassName class name of one user row
 * @param totalCountOfUsers total count of dragged users
 */
function DragAndDropUsersInFlightRenderer(divClassName, totalCountOfUsers) {
    this.divClassName = divClassName;
    this.totalCountOfUsers = totalCountOfUsers;
    this.COUNT_OF_USERS_DISPLAYED = 4;
    this.countOfProcessedUsers = 0;
    this.permitToRender = true;
}

/**
 * Render on user row
 * @param user UserInGroupRow instance
 */
DragAndDropUsersInFlightRenderer.prototype.generateOneUserDiv = function(user) {
    var divEl = document.createElement("div");
    divEl.className = this.divClassName;
    addClass(divEl, GroupManagementUtil.textClassName);
    addClass(divEl, GroupManagementUtil.ellipsisOverflowClassName);
    if (this.countOfProcessedUsers < this.COUNT_OF_USERS_DISPLAYED) {
         divEl.innerHTML = escapeHtmlSymbols(user.name);
         this.countOfProcessedUsers++;
    } else {
        divEl.innerHTML = messageResource.getMessage("group_management.drag_and_drop.users_count", [this.totalCountOfUsers]);
        this.permitToRender = false;
    }
    return divEl;
};