/**
 * Handles state of message if selected group doesn't contain users
 * @param table dom element with message
 * @param cell cell in table where placed message
 * @param userContainer container with user rows
 */
function EmptyGroupResolver(table, cell, userContainer, config) {
    EmptyGroupResolver.WITHOUT_USERS = 1;
    EmptyGroupResolver.WITHOUT_ENABLED_USERS = 2;
    EmptyGroupResolver.MESSAGE_HIDDEN = 3;
    this.domElement = table;
    this.labelCell = cell;
    this.userContainer = userContainer;
    this.emptyNullGroupMessage = messageResource.getEscapedMessage("group_management.empty_null_group.user_group");
    this.nullGroupWithoutEnabledUsersMessage = messageResource.getEscapedMessage("group_management.null_group_without_enabled.user_group");
    this.lastState = EmptyGroupResolver.MESSAGE_HIDDEN;
    this.groupNamingKey = config.groupNamingKey;
    this.GroupNamingKey = config.GroupNamingKey;
    this.validatorGroupStrategy = config.validatorGroupStrategy;
}

/**
 * Define visibility and state of right container in the lightbox
 * @param group selected group
 * @param isShowDisabledUsers is disabled user show now
 * @param countOfDisabledUsers count of disabled users in selected group(passed directly to improve performance because this function often called after iteration over users)
 */
EmptyGroupResolver.prototype.resolveEmptyUserGroupElementState = function(group, isShowDisabledUsers, countOfDisabledUsers) {
    if (group.users.length == 0) {
        this.appendWithoutUsersElement(group);
    } else if (!isShowDisabledUsers && (countOfDisabledUsers == group.users.length)) {
        this.appendWithoutEnabledUsersElement(group);
    } else {
        this.lastState = EmptyGroupResolver.MESSAGE_HIDDEN;
        this.hideElement();
    }
};

EmptyGroupResolver.prototype.resolveEmptyUserGroupAfterFilterApplied = function(group) 
{
    this.replaceWithoutUsersLabel(group.name, true);
    this.showElement();
};

/**
 * Create dom element with empty group message
 * @param emptyUserGroupClassName class name of this element
 */
EmptyGroupResolver.prototype.createElement = function(emptyUserGroupClassName) {
    var table = document.createElement("table");
    var tbody = document.createElement("tbody");
    var tr = document.createElement("tr");
    var td = document.createElement("td");
    table.className = emptyUserGroupClassName;
    td.className = GroupManagementUtil.textClassName;
    table.appendChild(tbody);
    tbody.appendChild(tr);
    tr.appendChild(td);
    return table;
};

/**
 * Append empty user group message to the page
 * @param group selected user group
 */
EmptyGroupResolver.prototype.appendWithoutUsersElement = function(group) {
    this.lastState = EmptyGroupResolver.WITHOUT_USERS;
    if (this.validatorGroupStrategy.isNullGroup(group)) {
        this.replaceLabel(this.emptyNullGroupMessage);
    } else {
        this.replaceWithoutUsersLabel(group.name, false);
    }
    this.showElement();
};

/**
 * Append user group without enabled users message to the page
 * @param group selected user group
 */
EmptyGroupResolver.prototype.appendWithoutEnabledUsersElement = function(group) {
    this.lastState = EmptyGroupResolver.WITHOUT_ENABLED_USERS;
    if (this.validatorGroupStrategy.isNullGroup(group)){
        this.replaceLabel(this.nullGroupWithoutEnabledUsersMessage);
    } else {
        this.replaceWithoutEnabledUsersLabel(group.name);
    }
    this.showElement();
};

/**
 * Replace text in message
 * @param label new text
 */
EmptyGroupResolver.prototype.replaceLabel = function(label) {
    this.labelCell.innerHTML = label;
};

/**
 * Replace text in without enabled users message
 * @param name name of group
 */
EmptyGroupResolver.prototype.replaceWithoutEnabledUsersLabel = function(name) {
    this.replaceLabel(messageResource.getEscapedMessage("group_management.group_without_enabled.user_group", [messageResource.getMessage(this.groupNamingKey)].concat(name)));
};

/**
 * Replace text in without users message
 * @param name name of group
 * @param filtered dislays if group empty because of user by name filter
 */
EmptyGroupResolver.prototype.replaceWithoutUsersLabel = function(name, filtered) {
    var msgKey = filtered ? "group_management.empty_group.no_users_found" : "group_management.empty_group.user_group";
    this.replaceLabel(messageResource.getEscapedMessage(msgKey, [messageResource.getMessage(this.groupNamingKey), name]));
};

/**
 * Replace name of user group in message if user group is renamed. This function avoid reiniting controller in such case.
 * @param name
 */
EmptyGroupResolver.prototype.replaceUserGroupNameInLabel = function(name) {
    switch (this.lastState) {
        case EmptyGroupResolver.WITHOUT_USERS:
            this.replaceWithoutUsersLabel(name, false);
            break;
        case EmptyGroupResolver.WITHOUT_ENABLED_USERS:
            this.replaceWithoutEnabledUsersLabel(name);
            break;
    }
};

/**
 * Show empty group message and hide container with users
 */
EmptyGroupResolver.prototype.showElement = function() {
    this.domElement.style.display = '';
    this.userContainer.style.display = 'none';
};

/**
 * Hide empty group message and show container with users
 */
EmptyGroupResolver.prototype.hideElement = function() {
    this.domElement.style.display = 'none';
    this.userContainer.style.display = '';
};
