/**
 * Base entity for user group
 * @param id id
 * @param name name
 * @param nullGroup is this group bogus user group without users or not
 * @param users array of UserInGroup instances
 */
function GroupWithUsers(id, name, validatorGroupStrategy, users) {
    Group.call(this, id, name);
    this.validatorGroupStrategy = validatorGroupStrategy;
    this.users = users;
}

GroupWithUsers.prototype = new Group();

/**
 * Compute count of users with enabled access in this user group
 */
GroupWithUsers.prototype.countOfUserWithEnabledAccess = function() {
    var counter = 0;
    for (var i = 0; i < this.users.length; i++) {
        if (this.users[i].active) {
            counter++;
        }
    }
    return counter;
};

/**
 * @returns count of users in this user group
 */
GroupWithUsers.prototype.countOfUsers = function() {
    return this.users.length;
};

/**
 * Base entity for user
 * @param id id
 * @param name name
 * @param active is active or disabled
 */
function UserInGroup(id, name, active) {
    this.id = id;
    this.name = name;
    this.active = active;
}

/**
 * Wrapper for GroupWithUsers. Contains all needed dom elements of user group row
 */
function GroupRow() {}

GroupRow.prototype = new GroupWithUsers();

/**
 * @param div user group row
 * @param deleteLink user group delete button
 * @param undoDeleteButton group undo delete button
 * @param acceptDeleteButton group accept delete button
 * @param undoEditButton user group undo edit button
 * @param acceptEditButton user group accept edit button
 * @param input user group edit input field
 * @param nameCell cell with user group name
 * @param inputCell cell with edit user group input field
 * @param editGroupCell cell for edited group table
 * @param waitingAnimationCell cell with waiting ajax animation
 * @param group GroupRow to wrap
 * @param validatorGroupStrategy
 * @param controller user group management controller
 */
GroupRow.prototype.init = function(div, deleteLink, undoDeleteButton, acceptDeleteButton, undoEditButton, acceptEditButton, input, nameCell,
                  inputCell, editGroupCell, waitingAnimationCell, group, validatorGroupStrategy, controller) {
    GroupWithUsers.call(this, group.id, group.name, validatorGroupStrategy, group.users);
    this.div = div;
    this.deleteLink = deleteLink;
    this.undoEditButton = undoEditButton;
    this.acceptEditButton = acceptEditButton;
    this.undoDeleteButton = undoDeleteButton;
    this.acceptDeleteButton = acceptDeleteButton;
    this.input = input;
    this.nameCell = nameCell;
    this.nameDiv = nameCell.firstChild.firstChild;
    this.inputCell = inputCell;
    this.editGroupCell = editGroupCell;
    this.waitingAnimationCell = waitingAnimationCell;
    this.generateEventHandlers(controller);
    this.enableControls();
    this.enableRow();

    return this;
};

/**
 * Generate all needed event handlers with user group row elements
 * @param controller user group management controller
 */
GroupRow.prototype.generateEventHandlers = function(controller) {
    var context = this;

    this.normalEditGroupOnClick = function(event) {
        cancelBubble(event);
        controller.groupEventHandler(
                function()
                {
                    controller.startGroupEditing(context);
                    controller.groupSelected(context);
                },
                false);
    };

    this.normalDeleteGroupOnClick = function(event) {
        cancelBubble(event);
        controller.groupEventHandler(function() {controller.confirmGroupDelete(context)}, false);
    };

    this.noFunctionWithCancelBubble = function(event) {
        cancelBubble(event);
    };

    this.rowOnClick = function() {
        controller.groupEventHandler(function() {controller.groupSelected(context)}, false);
    };

    this.onClickOutsideEditGroupCell = function() {
        controller.hideErrors();
        controller.endGroupEditing(false);
    };

    this.inputOnClick = function() {
        cancelBubble(event);
    };

    this.undoEditButtonOnClick = function(event) {
        cancelBubble(event);
        controller.groupEventHandler(function() {}, false);
    };

    this.acceptEditButtonOnClick = function(event) {
        cancelBubble(event);
        controller.getGroupController().startNewGroupNameSaving(controller, context);
    };

    this.undoDeleteButtonOnClick = function(event) {
        cancelBubble(event);
        controller.groupEventHandler(function() {}, false);
    };

    this.acceptDeleteButtonOnClick = function(event) {
        cancelBubble(event);
        controller.groupEventHandler(function() {controller.deleteGroup(context)}, false);
    };

    this.inputOnKeyDown = function(event) {
        cancelBubble(event);
        GroupManagementUtil.createEnterPressedHandler(event, function() {controller.getGroupController().startNewGroupNameSaving(controller, context)});
        GroupManagementUtil.createEscPressedHandler(event, function() {controller.groupEventHandler(function() {}, false)});
    };

    this.divOnMouseOver = function() {
        controller.getHighlightController().highlightHoveredGroup(context);
        context.hovered = true;
    };

    this.divOnMouseOut = function() {
        controller.getHighlightController().unhighlightHoveredGroup(context);
        context.hovered = false;
    };

    this.groupButtonControlOnMouseOver = function() {
        controller.getHighlightController().highlightHoveredControl(this);
    };

    this.groupButtonControlOnMouseOut = function() {
        controller.getHighlightController().unhighlightHoveredControl(this);
    };

    this.groupUndoAcceptEditButtonOnMouseOver = function() {
        controller.getHighlightController().highlightHoveredUndoAcceptEditButton(this);
    };

    this.groupUndoAcceptEditButtonOnMouseOut = function() {
        controller.getHighlightController().unhighlightHoveredUndoAcceptEditButton(this);
    };

    this.noFunction = function() {
        return false;
    }
};

/**
 * Disable all needed controls when ajax request is processed with this row
 */
GroupRow.prototype.disableControlsDueToAjax = function() {
    if (!this.validatorGroupStrategy.isNullGroup(this)) {
        this.nameDiv.onclick = this.noFunction;
        this.deleteLink.onclick = this.noFunctionWithCancelBubble;
        this.deleteLink.onmouseover = this.noFunction;
        this.deleteLink.onmouseout = this.noFunction;
    }
};

/**
 * Enable all controls. Used to initialize event handlers
 */
GroupRow.prototype.enableControls = function() {
    this.div.onmouseover = this.divOnMouseOver;
    this.div.onmouseout = this.divOnMouseOut;
    if (!this.validatorGroupStrategy.isNullGroup(this)) {
        this.nameDiv.onclick = this.normalEditGroupOnClick;
        this.undoEditButton.onclick = this.undoEditButtonOnClick;
        this.acceptEditButton.onclick = this.acceptEditButtonOnClick;
        this.undoEditButton.onmouseover = this.groupUndoAcceptEditButtonOnMouseOver;
        this.undoEditButton.onmouseout = this.groupUndoAcceptEditButtonOnMouseOut;
        this.acceptEditButton.onmouseover = this.groupUndoAcceptEditButtonOnMouseOver;
        this.acceptEditButton.onmouseout = this.groupUndoAcceptEditButtonOnMouseOut;
        this.undoDeleteButton.onclick = this.undoDeleteButtonOnClick;
        this.acceptDeleteButton.onclick = this.acceptDeleteButtonOnClick;
        this.input.onkeydown = this.inputOnKeyDown;
    }
    if (!this.validatorGroupStrategy.isNullGroup(this) && !this.validatorGroupStrategy.isDefaultGroup(this)){
        this.deleteLink.onclick = this.normalDeleteGroupOnClick;
        this.deleteLink.onmouseover = this.groupButtonControlOnMouseOver;
        this.deleteLink.onmouseout = this.groupButtonControlOnMouseOut;
    }
};

/**
 * Disable mouse events when they are not needed
 */
GroupRow.prototype.disableMouseEvents = function(saveHover) {
    if(!saveHover)
    {
        this.div.onmouseover = this.noFunction;
        this.div.onmouseout = this.noFunction;
    }
    if (!this.validatorGroupStrategy.isNullGroup(this) && !this.validatorGroupStrategy.isDefaultGroup(this)) {
        this.deleteLink.onmouseover = this.noFunction;
        this.deleteLink.onmouseout = this.noFunction;
    }
};

/**
 * Disable mouse events on edited user group when they are not needed
 */
GroupRow.prototype.disableEditedGroupMouseEvents = function() {
    if (!this.validatorGroupStrategy.isNullGroup(this) && !this.validatorGroupStrategy.isDefaultGroup(this)) {
        this.undoEditButton.onmouseover = this.noFunction;
        this.acceptEditButton.onmouseover = this.noFunction;
        this.undoEditButton.onmouseout = this.noFunction;
        this.acceptEditButton.onmouseout = this.noFunction;
    }
};

/**
 * Enable mouse events on edited user group
 */
GroupRow.prototype.enableEditedGroupMouseEvents = function() {
    if (!this.validatorGroupStrategy.isNullGroup(this) && !this.validatorGroupStrategy.isDefaultGroup(this)) {
        this.undoEditButton.onmouseover = this.groupUndoAcceptEditButtonOnMouseOver;
        this.acceptEditButton.onmouseover = this.groupUndoAcceptEditButtonOnMouseOver;
        this.undoEditButton.onmouseout = this.groupUndoAcceptEditButtonOnMouseOut;
        this.acceptEditButton.onmouseout = this.groupUndoAcceptEditButtonOnMouseOut;
    }
};

/**
 * Enable mouse events on user group
 */
GroupRow.prototype.enableMouseEvents = function() {
    this.div.onmouseover = this.divOnMouseOver;
    this.div.onmouseout = this.divOnMouseOut;
    if (!this.validatorGroupStrategy.isNullGroup(this) && !this.validatorGroupStrategy.isDefaultGroup(this)) {
        this.deleteLink.onmouseover = this.groupButtonControlOnMouseOver;
        this.deleteLink.onmouseout = this.groupButtonControlOnMouseOut;
    }
};

/**
 * Disable mouse events on whole user group row
 */
GroupRow.prototype.disableRow = function() {
    this.div.onclick = this.noFunction;
};

/**
 * Enable mouse events on whole user group row
 */
GroupRow.prototype.enableRow = function() {
    this.div.onclick = this.rowOnClick;
    this.input.onclick = this.noFunction;
    $(document).off("mousedown.editGroupCell");
};

/**
 * Disable all previous mouse events on whole row while group editing and set handler for cancel editing
 * Set handler on active input for prevent cancel editing while click on input field and handler for cancel editing while clicking outside name input
 */
GroupRow.prototype.disableRowForEditing = function() {
    var context = this;
    var editGroupCell = $(this.editGroupCell);
    $(document).on("mousedown.editGroupCell", function(event){
        var target = $(event.target);
        if( target.parents().index(editGroupCell) == -1 )
        {
            context.onClickOutsideEditGroupCell();
        }
    });
    this.editGroupCell.onclick= this.inputOnClick;
};

/**
 * Pass user group name into text of user group edit input field
 */
GroupRow.prototype.revertInputText = function() {
    this.input.value = this.name;
};

/**
 * Defines is user group name changed or not
 */
GroupRow.prototype.isNameChanged = function() {
    return this.input.value.trim() != this.name.trim();
};

/**
 * Repaint row if ajax request is processed now onto it
 * @param additionalClassName
 */
GroupRow.prototype.markAsAjaxProcessedNow = function(additionalClassName) {
    this.waitingAnimationCell.style.display = 'table-cell';
    addClass(this.div, additionalClassName);
};

GroupRow.prototype.isHovered = function()
{
    return this.hovered;
};

GroupRow.prototype.focusAcceptDeleteButton = function()
{
    $(this.acceptDeleteButton).focus();
};

function TimeZoneGroupRow() {}

TimeZoneGroupRow.prototype = new GroupRow();

TimeZoneGroupRow.prototype.init = function(div, deleteLink, undoDeleteButton, acceptDeleteButton, undoEditButton, acceptEditButton, input, nameCell,
                  inputCell, editGroupCell, waitingAnimationCell, group, validatorGroupStrategy, controller, selectorId)
{
    GroupRow.prototype.init.call(this, div, deleteLink, undoDeleteButton, acceptDeleteButton, undoEditButton, acceptEditButton, input, nameCell,
                  inputCell, editGroupCell, waitingAnimationCell, group, validatorGroupStrategy, controller);
    this.cityPlaceId = group.cityPlaceId;
    this.cityName = group.cityName;
    this.timeZoneId = group.timeZoneId;
    this.rawOffset = group.rawOffset;
    this.currentTime = group.currentTime;
    var self = this;
    this.controller = controller;
    this.citySelector = new TimeZonesSelector().init(
            selectorId,
            {name: group.cityName, placeId: group.cityPlaceId, timeZoneId: group.timeZoneId, rawOffset: group.rawOffset},
            function() { self.cityChanged.apply(self, arguments) },
            null,
            false,
            "timeZoneGroupRowSearch",
            true
    );

    return this;
};

TimeZoneGroupRow.prototype.cityChanged = function(city)
{
    var self = this;
    this.cityName = city.name;
    this.cityPlaceId = city.placeId;
    this.timeZoneId = city.timeZoneId;
    this.rawOffset = city.rawOffset;
    this.currentTime = city.currentTime;
    this.controller.getGroupController().startNewGroupNameSaving(this.controller, self);
};

TimeZoneGroupRow.prototype.cleanup = function()
{
    this.citySelector.resetToInit();
};

/**
 * Wrapper for UserInGroup. Contains all needed dom elements of user row
 * @param div user row
 * @param checkbox checkbox on user row
 * @param ddWrapper drag and drop class for this user row
 * @param controller user group management controller
 * @param user UserInGroup instance for wrapping
 */
function UserInGroupRow(div, checkbox, ddWrapper, controller, user) {
    UserInGroup.call(this, user.id, user.name, user.active);
    this.div = div;
    this.checkbox = checkbox;
    this.ddWrapper = ddWrapper;
    this.generateEventHandlers(controller);
    this.enableControls();
}

UserInGroupRow.prototype = new UserInGroup();

/**
 * Is this user selected or not
 */
UserInGroupRow.prototype.isSelected = function() {
    return this.checkbox.checked;
};

/**
 * Hide this user
 */
UserInGroupRow.prototype.hide = function() {
    this.div.style.display = 'none';
};

/**
 * Make this user visible
 */
UserInGroupRow.prototype.open = function() {
    this.div.style.display = '';
};

/**
 * Generate all event handlers needed for this user row
 * @param controller user group management controller
 */
UserInGroupRow.prototype.generateEventHandlers = function(controller) {
    var context = this;
    this.checkBoxOnClick = function() {
        controller.passiveEventHandler(function() {controller.getUserController().selectUser(context)}, true);
    };
};

/**
 * Enable all event handlers. Calls to initialize row
 */
UserInGroupRow.prototype.enableControls = function() {
    this.checkbox.onclick = this.checkBoxOnClick;
};


