/**
 * Handle all operations with users in user group management lightbox
 * @param userDivIdPrefix dom id prefix of user row element
 * @param checkBoxIdPrefix dom id prefix of user checkbox element
 * @param selectedUserClassName class name of row with selected user
 */
function UserController(userDivIdPrefix, checkBoxIdPrefix, selectedUserClassName) {
    this.userDivIdPrefix = userDivIdPrefix;
    this.checkBoxIdPrefix = checkBoxIdPrefix;
    this.selectedUserClassName = selectedUserClassName;
    this.users = {};
}

/**
 * Init controller with users. Wrap all user data into UserInGroupRow classes
 * @param controller user group management controller
 * @param users UserInGroup array
 * @param isShowDisabledUsers is disabled users show in system
 * @param isAJAXRequestProcessedNow is ajax request processed now
 */
UserController.prototype.init = function(controller, users, isShowDisabledUsers, isAJAXRequestProcessedNow) {
    this.users = {};
    for (var i = 0; i < users.length; i++) {
        var div = document.getElementById(formatId(this.userDivIdPrefix, users[i].id));
        var checkBox = document.getElementById(formatId(this.checkBoxIdPrefix, users[i].id));
        this.users[users[i].id] = new UserInGroupRow(div, checkBox,
                                                     controller.getDDController().makeDDWrapperForUser(controller, div, users[i].id, isAJAXRequestProcessedNow),
                                                     controller, users[i]);
    }
    this.isShowDisabledUsers = isShowDisabledUsers;
};

/**
 * Select all users
 */
UserController.prototype.selectAll = function() {
    for (var i in this.users) {
        //noinspection JSUnfilteredForInLoop
        this.doSelectUser(this.users[i], true);
    }
};

/**
 * Unselect all users
 */
UserController.prototype.clearAll = function() {
    for (var i in this.users) {
        //noinspection JSUnfilteredForInLoop
        this.doSelectUser(this.users[i], false);
    }
};

/**
 * Select user
 * @param user UserInGroupRow instance
 */
UserController.prototype.selectUser = function(user) {
    this.doSelectUser(user, user.checkbox.checked);
};

/**
 * Reload disabled users
 * @param checked is show disabled users checkbox checked
 * @param filtered is filter enabled
 */
UserController.prototype.reloadDisabledUsers = function(checked, filtered) {
    var countOfDisabledUsers = 0;
    var usersToHide = [];
    var usersLength = 0;
    for (var i in this.users) {
        usersLength++;
        //noinspection JSUnfilteredForInLoop
        if (!this.users[i].active) {
            countOfDisabledUsers++;
            if (checked) {
                //noinspection JSUnfilteredForInLoop
                this.users[i].open();
            } else {
                //noinspection JSUnfilteredForInLoop
                usersToHide.push(this.users[i]);
            }
        }
    }
    
    if (!filtered || usersToHide.length > 0 && usersToHide.length != usersLength)
    {
        _.each(usersToHide, function (user) {user.hide();});
    }
    else 
    {
        countOfDisabledUsers = 0;
    }
    
    this.isShowDisabledUsers = checked;
    return countOfDisabledUsers;
};

/**
 * Private select user
 * @param user UserInGroupRow instance
 * @param checked is user checkbox checked
 */
UserController.prototype.doSelectUser = function(user, checked) {
    if (checked) {
        addClass(user.div, this.selectedUserClassName);
    } else {
        removeClass(user.div, this.selectedUserClassName);
    }
    user.checkbox.checked = checked;
};

/**
 * Get all users needed for drag in dependence on selected or not is user intended for drag.
 * If he is selected, returns all selected users, otherwise returns only this user
 * @param userId id of intended for drag user
 * @param forceShowDisabledUsers flag used when disabled users are shown due to word filter
 */
UserController.prototype.getUsersNeededForDrag = function(userId, forceShowDisabledUsers) {
    var users = [];
    var user = this.users[userId];
    if (user.isSelected()) {
        for (var i in this.users) {
            //noinspection JSUnfilteredForInLoop
            if (this._verifyUser(this.users[i], forceShowDisabledUsers)) {
                //noinspection JSUnfilteredForInLoop
                users.push(this.users[i]);
            }
        }
    } else {
        users.push(user);
    }
    users.sort(nameComparator);
    return users;
};

/**
 * Get object of all UserInGroupRow instances by id
 */
UserController.prototype.getUsers = function() {
    return this.users;
};

/**
 * Checks at least one selected visible user
 */
UserController.prototype.hasUsersSelected = function(forceShowDisabledUsers) {
    for (var i in this.users) {
        //noinspection JSUnfilteredForInLoop
        if (this._verifyUser(this.users[i], forceShowDisabledUsers)) {
            return true;
        }
    }
    return false;
};

UserController.prototype._verifyUser = function(user, forceShowDisabledUsers) {
    return user.isSelected() && (user.active || this.isShowDisabledUsers || forceShowDisabledUsers);
};

/**
 * Get all visible selected user ids
 */
UserController.prototype.getSelectedUserIds = function(forceShowDisabledUsers) {
    var selectedUserIds = [];
    for (var i in this.users) {
        //noinspection JSUnfilteredForInLoop
        if (this._verifyUser(this.users[i], forceShowDisabledUsers)) {
            //noinspection JSUnfilteredForInLoop
            selectedUserIds.push(this.users[i].id);
        }
    }
    return selectedUserIds;
};

/**
 * Restore saved users selection
 * @param userIds saved users selection
 */
UserController.prototype.selectSavedUsers = function(userIds) {
    for (var i = 0; i < userIds.length; i++) {
        var user = this.users[userIds[i]];
        if (user) {
            this.doSelectUser(user, true);
        }
    }
};