function SelectorFilter( tree, searchIndex, allActiveSet )
{
    TreeFilter.call(this);

    this.allActiveSet = allActiveSet;

    this.activeOnlySet = this.allActiveSet;
    this.selectedOnlySet = null;
    this.keywordSet = null;

    this.highlighter = new EmptyHighlighter();

    this.searchIndex = searchIndex;

    this.tree = tree;

    this.KEYWORD_EMPTY_MESSAGE_KEY = "keyword";
    this.SELECTED_ONLY_EMPTY_MESSAGE_KEY = "selectedOnly";
    this.NO_ACTIVE_NODES_EMPTY_MESSAGE_KEY = "noEnabled";
    this.NO_NODES_EMPTY_MESSAGE_KEY = "emptyTree";
}

SelectorFilter.prototype = _.create(TreeFilter.prototype);

SelectorFilter.prototype.getFilterClasses = function()
{
    var classes = [];
    if( this.keywordSet )
        classes.push( "keywordFilter" );
    if (!this.activeOnlySet)
        classes.push("showDisabled");
    else
        classes.push("hideDisabled");
    if( this.getResults().isEmpty() )
    {
        var classesMap = this.getClassesForEmptyMessages();

        for (var key in classesMap)
        {
            //noinspection JSUnfilteredForInLoop
            classes.push(classesMap[key]);
        }
    }
    return classes;
};

SelectorFilter.prototype.getClassesForEmptyMessages = function()
{
    var classesMap = {};
    if( this.keywordSet )
    {
        classesMap[this.KEYWORD_EMPTY_MESSAGE_KEY] = "showNoNodesMatchingSearch";
    }
    else if( this.selectedOnlySet )
    {
        classesMap[this.SELECTED_ONLY_EMPTY_MESSAGE_KEY] = (this.selectedOnlySet.isEmpty()) ?
                                                           "showNoSelectedNodes" : this.getClassNameForEmptyVisibleSelection();
    }
    else if (this.activeOnlySet && this.tree.getRootNode().hasChildren())
    {
        classesMap[this.NO_ACTIVE_NODES_EMPTY_MESSAGE_KEY] = "showNoActiveNodes";
    }
    else
    {
        classesMap[this.NO_NODES_EMPTY_MESSAGE_KEY] = "empty";
    }
    return classesMap;
};

SelectorFilter.prototype.getClassNameForEmptyVisibleSelection = function()
{
    return "showNoSelectedNodesHasDisabled";
};

SelectorFilter.prototype.setShowInactiveAndReFilter = function( showDisabled )
{
    this.setShowInactiveAndReFilterIfNeeded(showDisabled, true);
};

SelectorFilter.prototype.setShowInactive = function( showDisabled )
{
    this.setShowInactiveAndReFilterIfNeeded(showDisabled, false);
};

SelectorFilter.prototype.setShowInactiveAndReFilterIfNeeded = function( showDisabled, needToReFilter )
{
    if( showDisabled && this.activeOnlySet )
    {
        this.activeOnlySet = null;
        if (needToReFilter)
            this.reFilter();
    }
    else if( !showDisabled && !this.activeOnlySet )
    {
        this.activeOnlySet = this.allActiveSet;
        if (needToReFilter)
            this.reFilter();
    }
};

SelectorFilter.prototype.setShowSelectedOnlyAndReFilter = function( showSelectedOnly )
{
    if( showSelectedOnly && !this.selectedOnlySet )
    {
        this.selectedOnlySet = this.tree.getSelectionController().getSelectedNodeSet().clone();
            this.reFilter();
    }
    else if( !showSelectedOnly && this.selectedOnlySet )
    {
        this.selectedOnlySet = null;
            this.reFilter();
    }
};

// selectedOnlySet is set from initially selected nodes (may be necessary, if tree is not build yet).
// no refiltering is performed.
SelectorFilter.prototype.setSelectedOnlyFromInitiallySelectedNodeSet = function(  )
{
    this.selectedOnlySet = this.tree.getSelectionController().getInitiallySelectedNodeSet().clone();
};

SelectorFilter.prototype.updateSelectedOnlyState = function()
{
    if( this.selectedOnlySet != null )
        this.selectedOnlySet = this.tree.getSelectionController().getSelectedNodeSet().clone();
};

SelectorFilter.prototype.isShowSelectedOnly = function()
{
    return this.selectedOnlySet != null;
};

SelectorFilter.prototype.setSearchString = function( searchString )
{
    this.setSearchStringAndReFilterIfNeeded(searchString, false);
};

SelectorFilter.prototype.setSearchStringAndReFilter = function( searchString )
{
    this.setSearchStringAndReFilterIfNeeded(searchString, true);
};

SelectorFilter.prototype.setSearchStringAndReFilterIfNeeded = function( searchString, needToReFilter )
{
    var tokens = this.searchIndex.getTokenizer().getTokens( searchString.toLowerCase() );
    if( tokens.length > 0 )
    {
        var searchResults = this.searchIndex.search( tokens );
        this.keywordSet = new TreeNodeSet();
        for( var id in searchResults )
        {
            //noinspection JSUnfilteredForInLoop
            this.keywordSet.add( this.tree.getRootNode().getNodeById( id ) );
        }
        this.highlighter = new TokensHighlighter( tokens, this.searchIndex.getTokenizer().getPunctuators() );
        if (needToReFilter)
            this.reFilter();
    }
    else
    {
        if( this.keywordSet )
        {
            this.keywordSet = null;
            this.highlighter = new EmptyHighlighter();
            if (needToReFilter)
                this.reFilter();
        }
    }
};

SelectorFilter.prototype.doFilter = function()
{
    var result;
    if( this.keywordSet )
    {
        if( this.activeOnlySet )
        {
            result = this.keywordSet.intersection( this.activeOnlySet );
            if( result.isEmpty() && !this.keywordSet.isEmpty() )
                result = this.keywordSet.clone();
        }
        else
        {
            result = this.keywordSet.clone();
        }
    }
    else
    {
        if( this.activeOnlySet )
        {
            result = this.selectedOnlySet ?
                     this.selectedOnlySet.intersection( this.activeOnlySet ) :
                     this.activeOnlySet.clone();
        }
        else
        {
            result = (this.selectedOnlySet) ? this.selectedOnlySet.clone() : this.selectedOnlySet;
        }
    }

    return result;
};

SelectorFilter.prototype.reFilter = function()
{
    if (this.hasNoFilter())
        this.tree.resetFilter();
    else
        this.tree.applyFilter( this );
};

SelectorFilter.prototype.hasNoFilter = function()
{
    return !this.activeOnlySet && !this.selectedOnlySet && !this.keywordSet;
};

SelectorFilter.prototype.hasKeywordFilter = function()
{
    return this.keywordSet != null;
};

SelectorFilter.prototype.getVisibleNodeSet = function(selectedNodeSet)
{
    return this.safeIntersection(selectedNodeSet, this.activeOnlySet);
};

SelectorFilter.prototype.containsTokens = function( userNode )
{
    return this.keywordSet && this.keywordSet.contains( userNode );
};

SelectorFilter.prototype.highlightTokens = function( str )
{
    return this.highlighter.highlightTokens( str );
};

/**
 * Intersects two node sets. If second node set is not exists returns clone of first
 * @param existingSet set which is not null or undefined
 * @param secondSet set which can be null or undefined
 */
SelectorFilter.prototype.safeIntersection = function(existingSet, secondSet)
{
    if (!secondSet)
        return existingSet.clone();

    return existingSet.intersection(secondSet);
};

