function Tree( rootNode, rootVisible, collapsedNodeIds )
{
    this.rootNode = rootNode;
    this.expandController = this.createExpandController( collapsedNodeIds );
    this.nodeRenderers = [];
    this.defaultNodeRenderer = this.createDefaultNodeRenderer();
    this.activeFilter = null;
    this.previousFilterResults = null;
    this.previousFilterResultsRoots = null;
    this.treeContainer = null;
    this.rootVisible = rootVisible;
    this.uiMap = {};
}

Tree.prototype.getRootNode = function()
{
    return this.rootNode;
};

Tree.prototype.isRootVisible = function()
{
    return this.rootVisible;
};

Tree.prototype.getAllNodes = function()
{
    var nodes = [];
    function collect(node)
    {
        nodes.push(node);
        _.each(node.getChildren(), function(child)
        {
            collect(child)
        });
    }
    collect(this.rootNode);
    return nodes;
};

Tree.prototype.createExpandController = function( collapsedNodeIds )
{
    return new ExpandController(this, collapsedNodeIds);
};

Tree.prototype.getExpandController = function()
{
    return this.expandController;
};

Tree.prototype.setNodeRenderers = function( nodeRenderers )
{
    this.nodeRenderers = nodeRenderers;
};

Tree.prototype.createDefaultNodeRenderer = function()
{
    return new TreeNodeRenderer(this);
};

Tree.prototype.getRenderer = function( treeNode )
{
    var renderer = this.nodeRenderers[treeNode.getType()];
    return renderer ? renderer : this.defaultNodeRenderer;
};

Tree.prototype.updateTreeContainerClasses = function()
{
    var classes = ["treeContainer"];
    if( this.activeFilter )
    {
        classes.push("filtered");
        var filterClasses = this.activeFilter.getFilterClasses();
        for( var i = 0; i < filterClasses.length; i++ )
            classes.push( filterClasses[i] );
    }
    else
    {
        classes.push("notFiltered");
        if (!this.rootNode.hasChildren())
        {
            classes.push("empty");
        }
    }
    this.treeContainer.className = classes.join(" ");
};

// TODO: Actually adds empty message panels. There should be some messages builder or common messages logic.
Tree.prototype.decorateTreePanel = function(treePanel)
{
    //do nothing
};

Tree.prototype.createTreeContainer = function()
{
    return document.createElement("div");
};

Tree.prototype.buildDOM = function( containerEl, initialFilter )
{
    if( initialFilter )
        this.applyFilterNoUpdate( initialFilter );

    this.treeContainer = this.createTreeContainer();

    this.decorateTreePanel(this.treeContainer);
    this.updateTreeContainerClasses();

    if( this.rootVisible )
        this.buildDOMForNode( this.treeContainer, this.rootNode );
    else
        this.buildDOMForChildren( this.treeContainer, this.rootNode );

    containerEl.appendChild( this.treeContainer );

    this.customScroll = ScrollUtils.addVerticalScroll($(containerEl), {isFaded: false});
};

Tree.prototype.buildDOMForChildren = function( targetEl, treeNode )
{
    var children = treeNode.getChildren();
    var childrenContainer = ((treeNode.getParent() || this.rootVisible) && children.length > 0) ? this.createChildrenContainerDiv(targetEl) : targetEl;
    for( var i = 0; i < children.length; i++ )
        this.buildDOMForNode( childrenContainer, children[i] );
};

Tree.prototype.createChildrenContainerDiv = function(targetEl)
{
    var childrenContainerDiv = document.createElement("div");
    childrenContainerDiv.className = "childrenContainer";
    targetEl.appendChild(childrenContainerDiv);
    return childrenContainerDiv;
};

Tree.prototype.buildDOMForNode = function( targetEl, treeNode )
{
    var renderer = this.getRenderer( treeNode );

    var nodeDiv = document.createElement( "div" );
    this.getUIObjectForNode(treeNode).nodeEl = nodeDiv;

    var contentDiv = document.createElement( "div" );
    contentDiv.className = "content";

    renderer.generateContent( treeNode, contentDiv );
    renderer.updateUI( treeNode );

    nodeDiv.appendChild( contentDiv );

    this.buildDOMForChildren( nodeDiv, treeNode );

    targetEl.appendChild( nodeDiv );
};

Tree.prototype.collapseAll = function()
{
    if( this.rootVisible )
        this.expandController.collapseSubtree( this.rootNode );
    else
        this.expandController.collapseChildren( this.rootNode );
};

Tree.prototype.expandAll = function()
{
    if( this.rootVisible )
        this.expandController.expandSubtree( this.rootNode );
    else
        this.expandController.expandChildren( this.rootNode );
};

Tree.prototype.updateUIForTreeNode = function( treeNode )
{
    if ((treeNode.getParent() != null) || this.rootVisible)
    {
        this.getRenderer( treeNode ).updateUI( treeNode );
    }
    this.refreshScroller();
};

Tree.prototype.updateUIForTreeNodeSet = function( treeNodeSet )
{
    for( var level in treeNodeSet.map )
    {
        //noinspection JSUnfilteredForInLoop
        var levelObj = treeNodeSet.map[level];
        for( var id in levelObj )
        {
            //noinspection JSUnfilteredForInLoop
            this.updateUIForTreeNode(levelObj[id]);
        }
    }
};

Tree.prototype.resetFilter = function()
{
    if( this.previousFilterResults )
    {
        this.activeFilter = null;
        this.updateTreeContainerClasses();
        this.updateUIForTreeNodeSet(this.previousFilterResults.union(this.previousFilterResultsRoots));
        this.previousFilterResults = null;
        this.previousFilterResultsRoots = null;
        this.refreshScroller();
    }
};

Tree.prototype.getActiveFilter = function()
{
    return this.activeFilter;
};

Tree.prototype.hasActiveFilter = function()
{
    return this.activeFilter != null;
};

Tree.prototype.doesMatchFilterCriteria = function( treeNode )
{
    return this.previousFilterResults.contains( treeNode );
};

Tree.prototype.doesContainChildrenMatchingFilterCriteria = function( treeNode )
{
    return this.previousFilterResultsRoots.contains( treeNode );
};

Tree.prototype.applyFilterNoUpdate = function( filter )
{
    var newFilterResults = null;
    if( filter )
    {
        filter.filterNodes();
        newFilterResults = filter.getResults();
    }
    if( newFilterResults )
    {
        this.activeFilter = filter;
        if( this.previousFilterResults )
        {
            var nodesToUpdate = this.previousFilterResults;
            nodesToUpdate.addAll( newFilterResults );
            nodesToUpdate.addAll( nodesToUpdate.makePathsNodeSet() );
            this.previousFilterResults = newFilterResults;
            this.previousFilterResultsRoots = newFilterResults.makePathsNodeSet();
            return nodesToUpdate;
        }
        else
        {
            this.previousFilterResults = newFilterResults;
            this.previousFilterResultsRoots = newFilterResults.makePathsNodeSet();
            return this.previousFilterResults.union( this.previousFilterResultsRoots );
        }
    }
    else
    {
        this.resetFilter();
        return null;
    }
};

Tree.prototype.applyFilter = function( filter )
{
    var nodesToUpdate = this.applyFilterNoUpdate( filter );
    if( nodesToUpdate )
    {
        this.updateUIForTreeNodeSet(nodesToUpdate);
        this.updateTreeContainerClasses();
        this.refreshScroller();
    }
};

Tree.prototype.getUIObjectForNode = function(treeNode)
{
    var uiObject = this.uiMap[treeNode.id];
    if (!uiObject)
    {
        uiObject = {};
        this.uiMap[treeNode.id] = uiObject;
    }
    return uiObject;
};

Tree.prototype.refreshScroller = function()
{
    ScrollUtils.refreshCustomScroll(this.customScroll);
};