function TriStateSelectionController(tree)
{
    TreeSelectionController.call(this, tree);
    this.intermediateSet = new TreeNodeSet();
    this.listeners = [];
}

TriStateSelectionController.prototype = _.create(TreeSelectionController.prototype);

TriStateSelectionController.prototype.markSelected = function( treeNode )
{
    TreeSelectionController.prototype.markSelected.call( this, treeNode );
    this.intermediateSet.remove( treeNode );
};

TriStateSelectionController.prototype.markDeselected = function( treeNode )
{
    TreeSelectionController.prototype.markDeselected.call( this, treeNode );
    this.intermediateSet.remove( treeNode );
};

TriStateSelectionController.prototype.nodeSelected = function( treeNode )
{
    this.selectSubtree( treeNode, true );
    this.fireNodeSelected( treeNode );
};

TriStateSelectionController.prototype.nodeDeselected = function( treeNode )
{
    this.deselectSubtree( treeNode );
    this.fireNodeDeselected( treeNode );
};

TriStateSelectionController.prototype.selectSubtree = function( treeNode, updateUIofNodes )
{
    this.markSelected( treeNode );

    if( updateUIofNodes )
        this.tree.updateUIForTreeNode(treeNode);

    this.selectChildrenInDepth(treeNode, updateUIofNodes);

    // set intermediate selection state for all parent nodes
    var maxTopLevel = this.tree.isRootVisible() ? 0 : 1;
    var parentNode = treeNode.getParent();
    while( parentNode.getLevel() >= maxTopLevel )
    {
        this.updateParentNodeStateOnSubtreeSelected(parentNode);
        if( updateUIofNodes )
            this.tree.updateUIForTreeNode(parentNode);
        parentNode = parentNode.getParent();
    }
};

TriStateSelectionController.prototype.updateParentNodeStateOnSubtreeSelected = function (parentNode)
{
    this.intermediateSet.add( parentNode );
};

TriStateSelectionController.prototype.markDeselectedAndUpdate = function( treeNode )
{
    this.markDeselected( treeNode );
    this.tree.updateUIForTreeNode(treeNode);
};

TriStateSelectionController.prototype.deselectSubtree = function( treeNode )
{
    this.markDeselectedAndUpdate( treeNode );

    var children = treeNode.getChildren();
    for( var i = 0; i < children.length; i++ )
        this.deselectSubtree( children[i] );

    var maxTopLevel = this.tree.isRootVisible() ? 0 : 1;
    treeNode = treeNode.getParent();
    var foundIntermediate = false;
    while( treeNode.getLevel() >= maxTopLevel )
    {
        TreeSelectionController.prototype.markDeselected.call(this, treeNode);

        if( foundIntermediate || this.hasAnySelectedChild( treeNode ) )
        {
            this.intermediateSet.add( treeNode );
            foundIntermediate = true;
        }
        else
        {
            this.intermediateSet.remove( treeNode );
        }

        this.tree.updateUIForTreeNode(treeNode);
        treeNode = treeNode.getParent();
    }
};

TriStateSelectionController.prototype.hasAnySelectedChild = function( treeNode )
{
    var children = treeNode.getChildren();
    for( var i = 0; i < children.length; i++ )
    {
        var childNode = children[i];
        if( this.isSelected( childNode ) || this.intermediateSet.contains( childNode ) )
            return true;
    }
    return false;
};

TriStateSelectionController.prototype.deselectAll = function ()
{
    TreeSelectionController.prototype.deselectAll.call( this);
    var oldIntermediate = this.intermediateSet;
    this.intermediateSet = new TreeNodeSet();
    this.tree.updateUIForTreeNodeSet(oldIntermediate);
};


TriStateSelectionController.prototype.containsSelectedChildren = function ( treeNode )
{
    return this.intermediateSet.contains( treeNode );
};


TriStateSelectionController.prototype.addListener = function ( listener )
{
    this.listeners.push( listener );
};

TriStateSelectionController.prototype.fireNodeSelected = function ( treeNode )
{
    for( var i = 0; i < this.listeners.length; i++ )
        this.listeners[i].nodeSelected( treeNode );
};

TriStateSelectionController.prototype.fireNodeDeselected = function ( treeNode )
{
    for( var i = 0; i < this.listeners.length; i++ )
        this.listeners[i].nodeDeselected( treeNode );
};

TriStateSelectionController.prototype.updateUI = function()
{
    TreeSelectionController.prototype.updateUI.call(this);
    this.tree.updateUIForTreeNodeSet(this.intermediateSet);
};

TriStateSelectionController.prototype.selectChildrenInDepth = function ( treeNode, updateUIofNodes )
{
    var children = treeNode.getChildren();
    for( var i = 0; i < children.length; i++ )
        this.selectSubtree( children[i], updateUIofNodes );
};

TriStateSelectionController.prototype.resetSelectedNodes = function( nodeSet )
{
    if (!nodeSet)
    {
        nodeSet = this.initiallySelectedNodeSet;
    }
    this.nodeSet = nodeSet.clone();

    var allLevels = nodeSet.getAllLevels();

    //todo optimise (it reselects nodes that were already selected due-to parent's selection)
    for (var i=0; i < allLevels.length; i++){
        //noinspection JSUnfilteredForInLoop
        var levelMap = nodeSet.getLevelMap( allLevels[i] );
        for( var j in levelMap )
            //noinspection JSUnfilteredForInLoop
            if( this.nodeSet.contains( levelMap[j] ) )
                //noinspection JSUnfilteredForInLoop
                this.selectSubtree( levelMap[j] , false );
    }

};