/**
 * Add $.fn.searchField to wrap input field.
 * Can add previous value state to check if need update and clear button.
 * Depend on $.fn.pVal() function.
 */
(function($){

    // table used to simplify icon centring
    var defaultTemplate =
            "<table class='searchFieldWrap' cellpadding='0' cellspacing='0'>" +
                "<tr>" +
                    "<td class='fieldContainer'></td>" +
                    "<td><div class='filterIcon'></div></td>" +
                "</tr>" +
            "</table>";

    /**
     * Main plugin function. Make search field from. Wrap with clear button applies only for fist item of jQuery selection.
     * @param updateCallback [mandatory] - callback function
     * @param valueFilter - filter to input, default is $.trim
     * @param template - can be true, false, dom element, jquery element or html string. Template used if  !== false value specified.
     * If template contains "fieldContainer" class, then element will be wrapped. Also "filterIcon" class can be specified to clear field on click.
     * @returns {jQuery} - same jQuery with additional methods: updateSearchField, clearSearchField.
     * If selection contains more then one item, all of them will be updated by this methods.
     */
    $.fn.searchField = function(updateCallback, valueFilter, template)
    {
        var _updateSearchField = $.noop;
        var _clearSearchField = $.noop;

        if (this[0])
        {
            var callBack;

            if (!$.isFunction(valueFilter))
            {
                valueFilter = $.trim;
            }

            _updateSearchField = function()
            {
                // using placeholder pVal function
                var newVal = valueFilter(this.pVal());
                if (this.prevValue != newVal)
                {
                    this.prevValue = newVal;
                    callBack(newVal);
                }
            };

            _clearSearchField = function(skipUpdate)
            {
                this.val("");
                if (skipUpdate)
                    this.prevValue = "";
                else
                    this.updateSearchField();
            };

            // We can set some key event listener here

            // if we not using template, then we don`t add additional

            if (template !== false)
            {
                var wrap;
                if ($.type(template) == 'object')
                {
                    wrap = $(template);
                }
                else
                {
                    if (template === true || !template)
                        template = defaultTemplate;
                    wrap = $(template, this[0].ownerDocument);
                }

                var updateClearButton = function(newVal)
                {
                    wrap.toggleClass('hasValue', !!newVal);
                };

                // need to update button state if pVal call to change
                // But not redefine val()
                var oldPVal = this.pVal;
                this.pVal = function(newVal)
                {
                    var ret = oldPVal.call(this, newVal);
                    if (newVal !== undefined)
                        updateClearButton(oldPVal.call(this));
                    return ret;
                };

                callBack = function(newVal)
                {
                    updateClearButton(newVal);
                    updateCallback(newVal);
                };

                _clearSearchField = function(skipUpdate)
                {
                    this.val("");
                    if (skipUpdate)
                    {
                        updateClearButton(this.prevValue = "");
                    }
                    else
                    {
                        this.updateSearchField();
                    }
                };

                var fieldContainer = wrap.find('.fieldContainer');
                if (fieldContainer.length > 0)
                {
                    if (this[0].parentNode)
                    {
                        wrap.insertBefore(this[0]);
                    }
                    fieldContainer.append(this);
                }

                var self = this;
                wrap.find('.filterIcon').click(function()
                                               {
                                                   self.clearSearchField();
                                                   // focus to input field
                                                   self.focus();
                                               });

                this.focus(function()
                           {
                               wrap.addClass("inputFocused");
                           });
                this.blur(function()
                          {
                              wrap.removeClass("inputFocused");
                          });
            }
            else
            {
                callBack = updateCallback;
            }
        }

        this.prevValue = "";
        this.updateSearchField = _updateSearchField;
        this.clearSearchField = _clearSearchField;

        return this;
    }
})(jQuery);