function DataElementControl ()
{
    this.dataElement = null;
    this.textElement = null;
    this.disabledClassName = null;
    this.value = null;
    this.tooltip = null;
    this.tooltipEnabled = false;
    this.tooltipText = null;
    this.invalidDataClass = "invalid-data";
}

DataElementControl.prototype.registerElements = function( domEl,
                                                          dataElementId,
                                                          textElementPostfix,
                                                          newDataElementId,
                                                          newTextElementId,
                                                          disabledClassName,
                                                          initialValue )
{
    this.dataElement = findChildById(domEl, dataElementId);
    this.dataElement.id = newDataElementId ? newDataElementId : null;
    this.textElement = findChildById(domEl, dataElementId + textElementPostfix);
    this.textElement.id = newTextElementId ? newTextElementId : null;
    this.disabledClassName = disabledClassName ? disabledClassName : "hidden";
    this.value = initialValue ? initialValue : "";
    this.changeListeners = new Array();
}

DataElementControl.prototype.setDataElement = function ( dataElement )
{
    this.dataElement = dataElement;
}

DataElementControl.prototype.setTextElement = function ( textElement )
{
    this.textElement = textElement;
}

DataElementControl.prototype.initTooltip = function ( tooltipDivId )
{
    this.tooltip = new DhtmlTitle(tooltipDivId ? tooltipDivId : "form-field-tooltip");
}

DataElementControl.prototype.setValidator = function ( validator )
{
    this.validator = validator;
}

DataElementControl.prototype.addChangeListener = function ( handler )
{
    this.changeListeners.push(handler);
    addListener(this.dataElement, "change", handler);
};

//private
DataElementControl.prototype.notifyChangeListeners = function ()
{
    for (var i = 0; i < this.changeListeners.length; i++)
        this.changeListeners[i]();
};

DataElementControl.prototype.changeElementsState = function( elementToEnable, elementToDisable )
{
    addClass(elementToDisable, this.disabledClassName);
    removeClass(elementToEnable, this.disabledClassName)
}

DataElementControl.prototype.enableInput = function()
{
    //this.validate();
    this.changeElementsState(this.dataElement, this.textElement);
}

DataElementControl.prototype.disableInput = function()
{
    this.textElement.innerHTML = this.value;
    this.changeElementsState(this.textElement, this.dataElement);
}

/**
 * Should be overwritten by inheritors
 */
DataElementControl.prototype.block = function( block ) {}

/**
 * Should be overwritten by inheritors
 */
DataElementControl.prototype.updateData = function() {}

/**
 * Validates data has been entered. Returns true if data is valid, false if not.
 * Returns true by default.
 * Should be overwritten by inheritor.
 */
DataElementControl.prototype.validate = function()
{
    return true;
}

DataElementControl.prototype.init = function( value )
{
    this.value = value ? value : "";
    this.dataElement.onblur = makeHandler( this.validate, this );
    this.dataElement.onmouseover = makeHandler( this.dataElementOnMouseOver, this );
    this.dataElement.onmouseout = makeHandler( this.dataElementOnMouseOut, this );
    this.textElement.onmouseover = makeHandler( this.textElementOnMouseOver, this );
    this.textElement.onmouseout = makeHandler( this.textElementOnMouseOut, this );
    this.initTooltip();
}

/**
 * Shows the delayed tooltip, if tooltip is enabled.
 * @param element DOM element to show tooltip for
 */
DataElementControl.prototype.showTooltip = function( element, xOffset, yOffset )
{
    var x = xOffset ? xOffset : 0;
    var y = yOffset ? yOffset : 25;
    if ( this.tooltipEnabled && this.tooltip )
    {
        this.tooltip.showDelayedWithTimeout(this.getTooltipText(), element, x, y, 500);
    }
}

/**
 * OnMouseOver event handler of dataElement.
 * Should be overwritten by inheritors
 */
DataElementControl.prototype.dataElementOnMouseOver = function() {}

/**
 * OnMouseOver event handler of textElement.
 * Should be overwritten by inheritors
 */
DataElementControl.prototype.textElementOnMouseOver = function() {}

/**
 * Cancels showing of the delayed tooltip.
 */
DataElementControl.prototype.cancelShowTooltip = function( element )
{
    if ( this.tooltip )
        this.tooltip.cancelDelayedShow();
}

/**
 * OnMouseOut event handler of dataElement.
 * Should be overwritten by inheritors
 */
DataElementControl.prototype.dataElementOnMouseOut = function() {}

/**
 * OnMouseOut event handler of textElement.
 * Should be overwritten by inheritors
 */
DataElementControl.prototype.textElementOnMouseOut = function() {}

/**
 * Returns message to be displayed in a tooltip. Returns empty string by default.
 * Should be overwritten by inheritor.
 */
DataElementControl.prototype.getTooltipText = function()
{
    return this.tooltipText;
}

/**
 * Sets message to be displayed in a tooltip.
 * @param text text to be set
 */
DataElementControl.prototype.setTooltipText = function( text )
{
    this.tooltipText = text;
}

DataElementControl.prototype.setTooltipEnabled = function( flag )
{
    this.tooltipEnabled = flag;
}

DataElementControl.prototype.setDisplayInputError = function( display, text )
{
    this.tooltipEnabled = display;
    this.tooltipText = text;
    if ( display )
        addClass(this.dataElement, this.invalidDataClass);
    else
        removeClass(this.dataElement, this.invalidDataClass);
}

DataElementControl.prototype.focus = function()
{
    focusElement(this.dataElement);
};

//------------------

function InputControl ()
{
    DataElementControl.call(this);
}

InputControl.prototype = new DataElementControl();

InputControl.prototype.registerElements = function( domEl, dataElementId, textElementPostfix, newDataElementId,
                                                    newTextElementId, disabledClassName, initialValue )
{
    DataElementControl.prototype.registerElements.call( this, domEl, dataElementId, textElementPostfix,
                                                        newDataElementId, newTextElementId, disabledClassName,
                                                        initialValue);
    this.init();
}

InputControl.prototype.init = function( value )
{
    DataElementControl.prototype.init.call( this, value );
    this.dataElement.value = value;
    this.notifyChangeListeners();
};


InputControl.prototype.getValue = function()
{
    return this.dataElement.value;
};

InputControl.prototype.enableInput = function()
{
    this.dataElement.value = this.value;
    this.notifyChangeListeners();
    this.validate();
    DataElementControl.prototype.enableInput.call(this);
}

InputControl.prototype.disableInput = function()
{
    DataElementControl.prototype.disableInput.call(this);
    this.dataElement.value = this.value;
    this.notifyChangeListeners();    
}

InputControl.prototype.block = function( block )
{
    this.dataElement.disabled = block;
}

InputControl.prototype.updateData = function()
{
    this.validate();
    this.value =  this.dataElement.value;
}

InputControl.prototype.dataElementOnMouseOver = function()
{
    this.showTooltip( this.dataElement );
}

InputControl.prototype.dataElementOnMouseOut = function()
{
    this.cancelShowTooltip();
}

InputControl.prototype.validate = function()
{
    return !this.validator || this.validator.validate(this);
}

InputControl.prototype.setShowTextElement = function( isShow )
{
    this.textElement.style.display = (isShow) ? "" : "none";
};

//--------------------

IntegerInputControlValidator = function(minNumber)
{
    IntegerNumbersValidator.call(this, minNumber, undefined, "", false );
};

IntegerInputControlValidator.prototype = new IntegerNumbersValidator();

IntegerInputControlValidator.prototype.validate = function(inputControl)
{

    var okFunc = function()
    {
        inputControl.setDisplayInputError( false );
    };
    var errorFunc = function()
    {
        inputControl.setDisplayInputError( true, messageResource.getMessage('mail_notifications.form.input.invalid_data.positive_integer') );
    };

    var callbacks  =
    {
        onEmptyValue: okFunc,
        onInvalidString: errorFunc,
        onGreaterThanMax: errorFunc,
        onLessThanMin: errorFunc,
        onOk: okFunc
    };
    return this.doCheckField(inputControl.dataElement, true, callbacks);
};

//--------------------

function TimeInputControl ()
{
    InputControl.call(this);
    this.setValidator(new TimeInputControlValidator());
}

TimeInputControl.prototype = new InputControl();

TimeInputControl.prototype.updateData = function()
{
    this.validate();
}

TimeInputControl.prototype.init = function(value)
{
    DataElementControl.prototype.init.call(this, value);
    if (typeof value === "undefined")
        this.dataElement.value = "";
    else
        this.dataElement.value =  UserTime.formatTime(value);
    this.notifyChangeListeners();
};

TimeInputControl.prototype.enableInput = function()
{
    this.notifyChangeListeners();
    this.validate();
    DataElementControl.prototype.enableInput.call(this);
}

TimeInputControl.prototype.disableInput = function()
{
    this.dataElement.value = UserTime.formatTime(this.value);
    this.textElement.innerHTML = this.dataElement.value;
    this.changeElementsState(this.textElement, this.dataElement);
    this.notifyChangeListeners();
}

//--------------------

TimeInputControlValidator = function(minNumber)
{
};

TimeInputControlValidator.prototype.validate = function(inputControl)
{
    var stringValue = inputControl.dataElement.value,
            minutesValue = UserTime.parseTimeWithSystemSettings(stringValue);
    if(stringValue !== "" && spentTimeIsInvalid(minutesValue))
    {
        inputControl.setDisplayInputError( true, messageResource.getMessage( 'mail_notifications.form.input.invalid_data.positive_time', [SystemSettings.decimalSeparator] ) );
        inputControl.value = -1;
        return false;
    }
    else if (stringValue === "") {
        inputControl.value = "";
        inputControl.setDisplayInputError( false );
        return true;
    }
    stringValue = UserTime.formatTime(minutesValue);
    if (inputControl.dataElement.value !== stringValue)
        inputControl.dataElement.value = stringValue;

    if (inputControl.value !== minutesValue)
        inputControl.value = minutesValue;
    inputControl.setDisplayInputError( false );
    return true;
};

//-----------------------

function DropDownControl ()
{
    DataElementControl.call(this);
}

DropDownControl.prototype = new DataElementControl();

DropDownControl.prototype.enableInput = function()
{
    DataElementControl.prototype.enableInput.call(this);
}

DropDownControl.prototype.disableInput = function()
{
    DataElementControl.prototype.disableInput.call(this);
}

DropDownControl.prototype.updateData = function( value )
{
    this.value = value;
    this.textElement.innerHTML = this.value;
}

DropDownControl.prototype.textElementOnMouseOver = function()
{
    this.showTooltip( this.textElement, 7, 29 );
}

DropDownControl.prototype.textElementOnMouseOut = function()
{
    this.cancelShowTooltip( this.textElement );
}