/**
 * BaseRulesManager
 * @param rule Rule manager is created for
 */

function BaseRulesManager( rule )
{
    this.rule = rule;
}

BaseRulesManager.prototype = new RpcControlManager();

BaseRulesManager.prototype.getService = function()
{
    return jsonrpc.MailNotificationsService;
};

BaseRulesManager.prototype.getMethods = function()
{
    return [ 'MailNotificationsService.deleteRule',
             'MailNotificationsService.pauseRule',
             'MailNotificationsService.resumeRule',
             'MailNotificationsService.saveRule',
             'MailNotificationsService.saveSingleRule'];
};

BaseRulesManager.prototype.isRuleChangedException = function( exception )
{
    return exception && exception.name && ( exception.name.indexOf( 'RuleChangedException' ) >= 0 );
};

BaseRulesManager.prototype.isSingleRuleException = function( exception )
{
    return exception && exception.name && ( exception.name.indexOf( 'SingleRuleException' ) >= 0 );
};

BaseRulesManager.prototype.isRuleDeletedException = function( exception )
{
    return exception && exception.name && ( exception.name.indexOf( 'RuleDeletedException' ) >= 0 );
};

BaseRulesManager.prototype.isValidationError = function( exception )
{
    // either we started server-side validation and detected that some rule parameters are wrong,
    // or jabsorb failed to unmarshal our object - usually because numbers or dates cannot be parsed
    return exception && ( exception.name && exception.name.indexOf( 'InvalidRuleException' ) >= 0 ||
        exception.code && exception.code == JSONRpcClient.Exception.CODE_ERR_UNMARSHALL);
};

BaseRulesManager.prototype.handleSpecificError = function( exception )
{
    if( this.isRuleChangedException( exception ) )
        this.handleConcurrentChangeError( 'mail_notifications.form.js.rule_changed' );

    else if( this.isRuleDeletedException( exception ) )
        this.handleConcurrentChangeError( 'mail_notifications.form.js.rule_deleted' );

    else if( this.isValidationError( exception ) )
        this.handleValidationError( exception );

    else if ( this.isSingleRuleException ( exception ) )
        this.handleConcurrentChangeError( 'mail_notifications.form.js.single_rule' );
    else
        this.handelUnexpectedRpcError( exception );

    return true;
};

BaseRulesManager.prototype.handelUnexpectedRpcError = function( exception )
{
    var manager = new RpcErrorManager( exception );
    window.setTimeout( RpcControlManager.saveRpcControlBoundData, 500, manager );
};

BaseRulesManager.prototype.handleConcurrentChangeError = function( messageKey )
{
    alert( messageResource.getMessage( messageKey ) );
    document.location.reload();
};

BaseRulesManager.prototype.handleValidationError = function( exception )
{
    alert( messageResource.getMessage("mail_notifications.form.cannot_save.invalid_data") );
    this.rule.onSavingErrorOccurred();
};

BaseRulesManager.prototype.getRpcErrorMsg = function()
{
    return messageResource.getMessage("mail_notifications.form.error_message");
};

BaseRulesManager.prototype.getTimeoutMsg = function()
{
    return messageResource.getMessage("mail_notifications.form.saving_timeout");
};

BaseRulesManager.prototype.endSavingData = function()
{
    wait( this.rule.rpcDelay );
};

/**
 * RpcErrorManager
 */
function RpcErrorManager ( exception )
{
    RpcControlManager.call(this);
    this.exception = exception;
}

RpcErrorManager.prototype = new RpcControlManager();

RpcErrorManager.prototype.getService = function()
{
    return jsonrpc.MailNotificationsService;
};

RpcErrorManager.prototype.getMethods = function()
{
    return [ 'MailNotificationsService.processUnexpectedException' ];
};

RpcErrorManager.prototype.makeRemoteCall = function( callback )
{
    this.getService().processUnexpectedException( callback, this.getExceptionBean() );
};

RpcErrorManager.prototype.getExceptionBean = function( )
{
    var exception = {};
    exception.javaClass = exceptionBeanClass;
    exception.errorCode = this.exception.code;
    exception.errorMessage = this.exception.msg;
    exception.stackTrace = this.exception.trace;
    exception.exceptionName = this.exception.name;
    exception.hasErrorCode = this.exception.code != null;
    return exception
};

RpcErrorManager.prototype.onDataSavingComplete = function( result )
{
    document.forms[0].pageAction.value = rpcErrorRedirect;
    document.forms[0].submit();
};

/**
 * RulesSavingManager
 */
function RulesSavingManager( rule )
{
    BaseRulesManager.call(this, rule);
}

RulesSavingManager.prototype = new BaseRulesManager();

RulesSavingManager.prototype.beginSavingData = function()
{
    this.rule.saving = true;
    this.rule.onBeginSaving();
};

RulesSavingManager.prototype.endSavingData = function()
{
    BaseRulesManager.prototype.endSavingData.call( this );
    this.rule.saving = false;
    this.rule.onEndSaving();
};

RulesSavingManager.prototype.onDataSavingComplete = function( result )
{
    this.rule.id = result.id;
    this.rule.lastChangeTimestamp = Date.parseJSONTime( result.timestamp );
    this.rule.onSavingComplete();
};

RulesSavingManager.prototype.makeRemoteCall = function( callback )
{
    this.getService().saveRule(callback, this.rule.getParams());
};

/**
 * SingleRuleSavingManager
 */
function SingleRuleSavingManager( rule )
{
    RulesSavingManager.call(this, rule);
}

SingleRuleSavingManager.prototype = new RulesSavingManager();

SingleRuleSavingManager.prototype.makeRemoteCall = function( callback )
{
    this.getService().saveSingleRule(callback, this.rule.getParams());
};

/**
 * RulePauseManager
 */
function RulePauseManager( rule )
{
    BaseRulesManager.call( this, rule );
}

RulePauseManager.prototype = new BaseRulesManager();

RulePauseManager.prototype.beginSavingData = function()
{

};

RulePauseManager.prototype.endSavingData = function()
{
    BaseRulesManager.prototype.endSavingData.call( this );
    this.rule.onPauseEnd();
};

RulePauseManager.prototype.onDataSavingComplete = function()
{
    this.rule.onPauseCompleted();
};

RulePauseManager.prototype.makeRemoteCall = function( callback )
{
    this.getService().pauseRule( callback, this.rule.id, this.rule.lastChangeTimestamp );
};

/**
 * RuleResumeManager
 */
function RuleResumeManager( rule )
{
    BaseRulesManager.call( this, rule );
}

RuleResumeManager.prototype = new BaseRulesManager();

RuleResumeManager.prototype.beginSavingData = function()
{
    this.rule.onResumeBegin();
};

RuleResumeManager.prototype.endSavingData = function()
{
    BaseRulesManager.prototype.endSavingData.call( this );
    this.rule.onResumeEnd();
};

RuleResumeManager.prototype.onDataSavingComplete = function()
{
    this.rule.onResumeCompleted();
};

RuleResumeManager.prototype.makeRemoteCall = function( callback )
{
    this.getService().resumeRule( callback, this.rule.id, this.rule.lastChangeTimestamp );
};

/**
 * RuleDeleteManager
 */
function RuleDeleteManager( rule )
{
    BaseRulesManager.call( this, rule );
}

RuleDeleteManager.prototype = new BaseRulesManager();

RuleDeleteManager.prototype.beginSavingData = function()
{

};

RuleDeleteManager.prototype.endSavingData = function()
{
    BaseRulesManager.prototype.endSavingData.call( this );
    this.rule.onRemoveEnd();
};

RuleDeleteManager.prototype.onDataSavingComplete = function()
{
    this.rule.onRemoved();
};

RuleDeleteManager.prototype.makeRemoteCall = function( callback )
{
    this.getService().deleteRule( callback, this.rule.id, this.rule.lastChangeTimestamp );
};



