function UsersSelector()
{
    this.optionValue = null;
    this.button = null;
    this.dropDownControl = new DropDownControl();
    this.currentState = null;
    this.predefinedState = null;
    this.selectionHelper = null;
}

/**
 * Initiates UserSelector
 *
 * @param notificationTitle the title of notification type for which the selector has been created. If
 *              <code>notificationTitle</code> parameter is set to non-NULL value it determines that UserSelector is
 *              recipients selector and all disabled users will have a tooltip about not receiving this notification if
 *              mouse over.
 *              OPTIONAL.
 * @param labelLength maximum amount of symbols for selector's dropdown button label.
 *              OPTIONAL. set to null or leave unspecified to apply default settings.
 */

UsersSelector.prototype.init = function( selectorType, notificationTitle, labelLength, textLabelLength, selectionHelper )
{
    this.checkEmail = false;
    this.maxLabelLength = labelLength ? labelLength : 40;
    this.maxTextLabelLength = textLabelLength ? textLabelLength : 55;
    this.itemLabelLength = null;
    this.accessRight = this.getAccessRight(selectorType);
    this.noNotificationMessageParams = null;
    this.selectionHelper = selectionHelper;
    if( notificationTitle )
    {
        this.checkEmail = true;
        this.noNotificationMessageParams = this.getNoNotificationMessageParams( this.accessRight );
    }

    this.optionValue = true;

    var initConfig = {
        treeItemTooltipMessageParams: this.noNotificationMessageParams,
        showInactiveLabelKey: "mail_notifications.users_selector.show_disabled",
        checkEmail: this.checkEmail,
        itemLabelMaxLength: 62,
        collapsedCookiesKey: "mailNotifications",
        resetOnHide: false
    };

    this.menu = this.getSelectorMenu(selectorType, UsersSelector.selectorsCache, initConfig);
    this.selector = this.menu.getSelectorPanel();

    this.textSelectorHelper = new AT.ComplicateUsersAndGroupsSelectionHelper( {
        userSelectorGroupsStrategy: getUserSelectorFeatureStrategy(),
        accessRight: this.accessRight,
        maxLength: this.maxTextLabelLength,
        checkEmail: this.checkEmail
    } );

    var tooltip = new DhtmlTitle("form-field-tooltip");

    this.button = new AT.HierarchySelectorDropdownButton( {
        menu: this.menu,
        disabledClass: "disabled",
        renderTo: this.dropDownControl.dataElement,
        selectorPanelHeight: 466,
        tooltipObject: tooltip
    } );

    var context = this;

    this.button.setMenuHideCallback(function() {context.setCurrentState()});

    this.dropDownControl.init( this.button.getText() );
};

UsersSelector.prototype.defineState = function()
{
    this.menu.setCurrentParent( this.button, true );
    this.selector.loadState(this.currentState);
    this.selector.clearPreviousSelection();
};

UsersSelector.prototype.getNoNotificationMessageParams = function(accessRight)
{
    return new NoNotificationMessageParams(accessRight);
};

UsersSelector.prototype.setInitialSelection = function(initialOptionValue, selection, groupType)
{
    var showInactive = false;
    var showWithoutRight = false;

    var selectedUsers = selection.getSecondLevel();

    Ext.each( users, function ( user )
    {
        if( selectedUsers && isArrayContains( selectedUsers, user.id ) )
        {
            if( !user.isActive() )
                showInactive = true;
            if( this.accessRight && !this.accessRight.isGranted( user.accessRight ) )
                showWithoutRight = true;
        }
    }, this );

    this.predefinedState = {
        selection: selection ? selection : TwoLevelSelection.emptySelection(),
        optionValue: initialOptionValue,
        showInactive: showInactive,
        showWithoutRight: showWithoutRight,
        groupType: groupType
    };

    this.currentState = {
        selection: selection ? selection : TwoLevelSelection.emptySelection(),
        optionValue: initialOptionValue,
        showInactive: showInactive,
        showWithoutRight: showWithoutRight,
        labelMaxChars: this.maxLabelLength,
        treeItemTooltipMessageParams: this.noNotificationMessageParams,
        checkEmail: this.checkEmail,
        itemLabelMaxLength: this.itemLabelLength,
        selectionHelper: this.selectionHelper,
        groupType: groupType
    };

    this.selector.setPredefinedParameters(this.predefinedState);

    if (this.selector.isAlreadyShown()) {
        this.selector.loadState(this.currentState);
    } else {
        this.selector.loadPermanentState(this.currentState);
        this.selector.setGroupType(this.currentState.groupType);
        this.selector.markPredefinedNodesAsSelected(this.currentState.selection);
    }

    this.update(true);
};

UsersSelector.prototype.getAccessRight = function(selectorType)
{
    switch( selectorType )
    {
        case MailNotificationsUserSelectorCache.SELECTOR_USERS_WITH_ENTER_TT_RIGHT:
        case MailNotificationsUserSelectorCache.SELECTOR_USERS_WITH_ENTER_TT_RIGHT_NO_EMAIL_CHECK:
            return new SubmitTimeTrackRight();
        case MailNotificationsUserSelectorCache.SELECTOR_USERS_WITH_MANAGE_TASKS_RIGHT:
            return new ManageTasksRight();
        case MailNotificationsUserSelectorCache.SELECTOR_USERS_WITH_GENERATE_TIME_REPORTS_RIGHT:
            return new GenerateTimeReportRight();
        case MailNotificationsUserSelectorCache.SELECTOR_USERS_WITH_GENERATE_MONEY_REPORTS_RIGHT:
            return new GenerateMoneyReportRight();
    }
    return null;
};

UsersSelector.prototype.getSelectorMenu = function(selectorType, selectorsCache, initConfig)
{
    switch( selectorType )
    {
        case MailNotificationsUserSelectorCache.SELECTOR_ALL_USERS:
            return selectorsCache.getAllUsersPanel(initConfig);
        case MailNotificationsUserSelectorCache.SELECTOR_ENABLED_USERS_ONLY:
            return selectorsCache.getEnabledOnlyPanel(initConfig);
    }
    return selectorsCache.getAccessRightPanel(selectorType, initConfig, this.accessRight);
};

UsersSelector.prototype.setCurrentState = function()
{
    this.currentState.selection = this.selector.getSelection();
    this.currentState.optionValue = !this.selector.isCustomOptionSelected();
    this.currentState.showInactive = this.selector.getShowInactive();
    if (this.selector.accessRight)
    {
        this.currentState.showWithoutRight = this.selector.getShowWithoutRight();
    }
};

UsersSelector.prototype.registerElements = function ( domEl, type, textPostfix, disabledClass, initialValue )
{
    this.dropDownControl.registerElements( domEl, type, textPostfix, disabledClass, initialValue );
};

UsersSelector.prototype.enable = function()
{
    this.dropDownControl.enableInput();
    this.menu.setCurrentParent( this.button );
};

UsersSelector.prototype.disable = function()
{
    this.dropDownControl.disableInput();
};

UsersSelector.prototype.update = function(isInitial)
{
    this.menu.setCurrentParent( this.button );
    var selection = this.selector.getSelection(true);
    if (isInitial) {
        this.dropDownControl.updateData(this.selector.getLabelText(this.button.id, this.textSelectorHelper, this.maxTextLabelLength));
        var context = this;
        this.button.setGetInnerTooltipCallback(function(isForGroup) {
            return context.textSelectorHelper.getCertainEntityList(selection, isForGroup);
        });
    }
    var isTooltipEnabled = false;
    var isInnerTooltipsEnabled = false;
    var isAllSelected = !this.selector.isCustomOptionSelected();
    if (!isAllSelected) {
        isInnerTooltipsEnabled = (selection.getFirstLevel().length != 0) && (selection.getSecondLevel().length != 0);
        isTooltipEnabled = !isInnerTooltipsEnabled && !selection.isEmpty();
    }
    this.button.setInnerTooltipsEnabled(isInnerTooltipsEnabled);
    if (isInitial) {
        this.dropDownControl.setTooltipEnabled(isTooltipEnabled);
        if (isTooltipEnabled) {
            this.dropDownControl.setTooltipText( this.selector.getTooltipText() );
        }
    }
};

UsersSelector.prototype.setDisplayInputError = function( display,  message )
{
    this.button.setDisplayInputError( display, message );
};

UsersSelector.prototype.setPopupHideCallback = function( callback )
{
    this.button.setMenuHideCallback( callback );
};

UsersSelector.prototype.block = function( block )
{
   this.button.setDisabled( block );
};

UsersSelector.prototype.getDomElements = function()
{
   return this.selector.getEl() ? this.selector.getEl().dom : null;
};

//------------------

function makeUpdateDaysTextListener(control, callback)
{
    return function()
    {
        var days = trim( control.getValue() );
        if ( !isUnsignedPositiveInteger(days) )   return;
        callback.call(this, parseInt(days) == 1 );
    }
}
//------------------
function TTLimit()
{
    this.daysInput = null;
    this.timeInput = null;
    this.dayTypeSelect = null;
    this.dayTypeDropdownBtn = null;
}

TTLimit.prototype.doUpdateDaysText = function(isOneDay)
{
    this.daysInput.setShowTextElement( !isOneDay );
    this.daysText.innerHTML = messageResource.getMessage(( isOneDay ) ? "mail_notifications.form.units.day" : "mail_notifications.form.units.days");
};

TTLimit.prototype.init = function( domEl, type, ruleGlobalName )
{
    this.daysInput = new InputControl();
    this.daysInput.registerElements( domEl, String.format('notification[{0}].ttLimit.days', type), '.text');
    this.daysInput.setValidator(new IntegerInputControlValidator(1));

    this.daysText = findChildById(domEl, String.format('notification[{0}].ttLimit.daysText', type));
    this.daysInput.addChangeListener(makeHandler(makeUpdateDaysTextListener(this.daysInput, this.doUpdateDaysText), this));

    this.timeInput = new TimeInputControl();
    this.timeInput.registerElements( domEl, String.format('notification[{0}].ttLimit.time', type), '.text');


    this.dayTypeSelect = new DropDownControl();
    var selectId = String.format('{0}.ttLimit.onlyWorkingDays', ruleGlobalName);
    this.dayTypeSelect.registerElements( domEl,
                                           String.format('notification[{0}].ttLimit.onlyWorkingDays', type),
                                           '.text',
                                           selectId );

    this.dayTypeDropdownBtn = new AT.DropdownListButton({
        applyTo: selectId
    });

    this.dayTypeSelect.setDataElement( this.dayTypeDropdownBtn.getEl().dom );
    this.dayTypeSelect.init( this.dayTypeDropdownBtn.getText() );
};

TTLimit.prototype.enable = function()
{
    this.daysInput.enableInput();
    this.timeInput.enableInput();
    this.dayTypeSelect.enableInput();
}

TTLimit.prototype.disable = function()
{
    this.daysInput.disableInput();
    this.timeInput.disableInput();
    this.dayTypeSelect.disableInput();
}

TTLimit.prototype.update = function()
{
    this.daysInput.updateData();
    this.timeInput.updateData();
    this.dayTypeSelect.updateData( this.dayTypeDropdownBtn.getText() );
}

TTLimit.prototype.block = function( block )
{
    this.daysInput.block( block );
    this.timeInput.block( block );
    this.dayTypeDropdownBtn.setDisabled( block );
}

//------------------

function MoneyCondition()
{
    this.ammoutInput = null;
    this.timeRangeSelect = null;
    this.timeRangeDropdownBtn = null;
}

MoneyCondition.prototype.init = function( domEl, type, ruleGlobalName )
{
    this.amountInput = new InputControl();
    this.amountInput.registerElements( domEl, String.format('notification[{0}].money.amount', type), '.text' );
    this.amountInput.setValidator(new IntegerInputControlValidator(1));

    this.timeRangeSelect = new DropDownControl();
    var selectId = String.format('{0}.money.timeRange', ruleGlobalName)
    this.timeRangeSelect.registerElements( domEl,
                                           String.format('notification[{0}].money.timeRange', type),
                                           '.text',
                                           selectId );

    this.timeRangeDropdownBtn = new AT.DropdownListButton({
        applyTo: selectId
    });

    this.timeRangeSelect.setDataElement( this.timeRangeDropdownBtn.getEl().dom );
    this.timeRangeSelect.init( this.timeRangeDropdownBtn.getText() );
};

MoneyCondition.prototype.enable = function()
{
    this.amountInput.enableInput();
    this.timeRangeSelect.enableInput();
}

MoneyCondition.prototype.disable = function()
{
    this.amountInput.disableInput();
    this.timeRangeSelect.disableInput();
}

MoneyCondition.prototype.update = function()
{
    this.amountInput.updateData();
    this.timeRangeSelect.updateData( this.timeRangeDropdownBtn.getText() );
}

MoneyCondition.prototype.block = function( block )
{
    this.amountInput.block( block );
    this.timeRangeDropdownBtn.setDisabled( block );
}

MoneyCondition.prototype.focus = function(  )
{
    focusElement(this.amountInput);
}

//------------------

function DeadlineCondition()
{
    this.stateSelect = null;
    this.stateBtn = null;
}

DeadlineCondition.COMING_UP = 1;
DeadlineCondition.TODAY = 2;
DeadlineCondition.MISSED = 3;

DeadlineCondition.prototype.doUpdateDaysText = function(isOneDay)
{
    var daysMessageId = ( isOneDay ) ? "mail_notifications.form.units.working_day" : "mail_notifications.form.units.working_days";
    var daysAgoMessageId = ( isOneDay ) ? "mail_notifications.form.deadline.day_ago" : "mail_notifications.form.deadline.days_ago";
    this.daysText.innerHTML = '&nbsp;'+ messageResource.getMessage(daysMessageId);
    this.daysAgoText.innerHTML = messageResource.getMessage(daysAgoMessageId);
};

DeadlineCondition.prototype.init = function( domEl, type, ruleGlobalName )
{
    var selectId = String.format('{0}.deadline.state', ruleGlobalName);
    this.stateSelect = new DropDownControl();
    this.stateSelect.registerElements( domEl,
                                       String.format('notification[{0}].deadline.state', type),
                                       '.text',
                                       selectId );

    this.stateBtn = new AT.DropdownListButton({
        applyTo: selectId
    });

    this.stateBtn.on( {
        'change': {
            fn: this.onStateChanged,
            scope: this
        }
    } );

    this.stateSelect.setDataElement( this.stateBtn.getEl().dom );

    var inText = findChildById(domEl, String.format('notification[{0}].deadline.coming_up_in', type));
    this.daysText = findChildById(domEl, String.format('notification[{0}].deadline.coming_up_days', type));
    this.daysAgoText = findChildById(domEl, String.format('notification[{0}].deadline.missed_days', type));
    this.daysInput = new InputControl ();
    this.daysInput.registerElements(domEl, String.format('notification[{0}].deadline.days', type), '.text');
    this.daysInput.setValidator(new IntegerInputControlValidator(1));
    this.daysInputContainer = findChildById(domEl, String.format('notification[{0}].deadline.days', type)+'.container');

    this.daysInput.addChangeListener( makeHandler( makeUpdateDaysTextListener(this.daysInput, this.doUpdateDaysText), this ) );

    this.dependentElements = [
        {
            value: 1, // coming up
            show: [inText, this.daysInputContainer, this.daysText],
            hide: [this.daysAgoText]
        },
        {
            value: 2, // today
            show: [],
            hide: [inText, this.daysInputContainer, this.daysText, this.daysAgoText]
        },
        {
            value: 3, // missed
            show: [this.daysInputContainer, this.daysAgoText],
            hide: [inText, this.daysText]
        }
    ];
};

DeadlineCondition.prototype.onStateChanged = function()
{
    for (var i = 0; i < this.dependentElements.length; i++)
    {
        var elements = this.dependentElements[i];

        if (elements.value == this.stateBtn.getValue())
        {
            for (var j = 0; j < elements.show.length; j++)
            {
                elements.show[j].style.display = 'inline';
            }
            for (j = 0; j < elements.hide.length; j++)
            {
                elements.hide[j].style.display = 'none';
            }
        }
    }
    this.daysInput.enableInput();
};

DeadlineCondition.prototype.enable = function()
{
    this.daysInput.enableInput();
    this.stateSelect.enableInput();
}

DeadlineCondition.prototype.disable = function()
{
    this.daysInput.disableInput();
    this.stateSelect.disableInput();
}

DeadlineCondition.prototype.update = function()
{
    this.daysInput.updateData();
    this.stateSelect.updateData( this.stateBtn.getText() );
}

DeadlineCondition.prototype.block = function( block )
{
    this.daysInput.block( block );
    this.stateBtn.setDisabled( block );
}

DeadlineCondition.prototype.focus = function( )
{
    this.daysInput.focus();
};

function MouseController() {}

MouseController.prototype =
{
    getX: function()
    {
        return this.x;
    },

    getY: function()
    {
        return this.y;
    },

    getCursorWidth: function()
    {
        return 20;
    },

    getCursorHeight: function()
    {
        return 20;
    },

    update: function(event)
    {
        this.x= event.clientX + getBodyScrollLeft();
        this.y= event.clientY + getBodyScrollTop();
    }
};

/**
 *
 * RuleIconButton
 */

function IconButton ( container, elementId, onClickCallBack, tooltip, toolTipMessage )
{
    this.icon = findChildById( container, elementId );
    this.tooltip = tooltip;
    this.toolTipMessage = toolTipMessage;
    this.icon.id = null;
    this.onClickCallBack = onClickCallBack;
    this.mouse = new MouseController();
    this.icon.onclick = makeHandler (this.onClick, this);
    this.icon.onmousemove = makeHandler (this.updateMousePosition, this);
    this.icon.onmouseover = makeHandler (this.onMouseOver, this);
    this.icon.onmouseout =  makeHandler (this.onMouseOut, this);
}

IconButton.prototype.hoverCls = "hovered";

IconButton.prototype.updateMousePosition = function(event)
{
    this.mouse.update(event || window.event);
}

IconButton.prototype.onClick = function(event)
{
    this.tooltip.immediateHide();
    this.onClickCallBack(event);
}

IconButton.prototype.onMouseOver = function(event)
{
    this.normalCls = this.icon.className;
    this.icon.className = this.normalCls + "-" + this.hoverCls;
    this.updateMousePosition(event);
    this.tooltip.showDelayedWithTimeoutNearMouse( this.toolTipMessage, this.icon.firstChild, this.mouse, 600 );
}

IconButton.prototype.onMouseOut = function()
{
    this.icon.className = this.normalCls;
    this.tooltip.cancelDelayedShow();
}

IconButton.prototype.setTooltipMessage = function ( message )
{
    this.toolTipMessage = message;
}


/*
 * Common Functions
 */

function clickProhibited( evt )
{
    var event = evt ? evt : window.event;
    return event && whichButtonPressed ( event ) == 'left' &&
           (event.type.indexOf( 'click' ) != -1 || event.type.indexOf( 'mousedown' ) !=- 1);
}

function returnTrue ( )
{
    return true;
}

function returnFalse ( )
{
    return false;
}

/*
 * ScheduleButtonsControl
 */

ScheduleButtonsControl = function () {};

ScheduleButtonsControl.registerButton = function ( type, scheduledDays )
{
    scheduleControllers[type] = new SchedulePanelController(type, scheduledDays, schedulePanel);
    scheduleButtons[type] = new DropdownButton("scheduleButtons[" + type + "]",
                                                   scheduleControllers[type].getScheduledDaysString(),
                                                   schedulePanel,
                                                   false,
                                                   150,
                                                   true);

    var dropDownDiv = document.getElementById("dropDownButton_" + type);
    if ( dropDownDiv )
        dropDownDiv.innerHTML = scheduleButtons[type].getHTML();

    scheduleButtons[type].setBeforePopupShowHandler(
        function()
        {
            if ( this.dhtmlPopup.getOwner() )
            {
                if ( !this.dhtmlPopup.hide() )
                {
                    var thisButton = this;
                    var thisEvent = event;
                    this.dhtmlPopup.setDataSavingCallBack(
                        function(){
                            try
                            {
                                window.event = thisEvent;
                                thisButton.onMouseDown();
                            }
                            catch (e) //if IE
                            {
                                var el = getElementById( thisButton.getId());
                                el.fireEvent("onmousedown");
                            }
                        }
                    );
                    return "error";
                }
            }
            var id = this.id;
            this.dhtmlPopup.setPanelController( scheduleControllers[type] );
            this.dhtmlPopup.setCallerElement( this );
            this.dhtmlPopup.setPositioningStrategy( new ElementRelativePositioning( this.getId() ) );
        }
    )
    scheduleButtons[type].setAfterPopupHideHandler(
        function()
        {
            var newButtonText = this.dhtmlPopup.getPanelController().getScheduledDaysString();
            this.setCaption( newButtonText );
        }
    )
}

ScheduleButtonsControl.setIsEventProhibited = function ( fnc )
{
    this.oldIsEventProhibited = new Array();
    for( var i = 0; i < scheduleButtons.length; i++)
    {
        if ( !scheduleButtons[i] )
            continue;
        this.oldIsEventProhibited[i] = scheduleButtons[i].isEventProhibited;
        scheduleButtons[i].isEventProhibited = fnc;
    }
}

ScheduleButtonsControl.restoreIsEventProhibited = function ( fnc )
{
    for( var i = 0; i < scheduleButtons.length; i++)
    {
        if ( !scheduleButtons[i] )
            continue;
        scheduleButtons[i].isEventProhibited = this.oldIsEventProhibited[i];
    }
}



