/**
 * required files:
 *      mail_notifications.js
 *      rule_factory.js
 *      InputControl.js
 *      at.js
 *      constants_js.jsp
 *      util.js *
 *      ExtJS shall be initialized
 */

/**
 * Base object representing a single notification rule.
 *
 * @param type Integer
 * @param id Integer The unique id for this rule or null if the rule is newly created
 */
function Rule( type, id )
{
    if (!type) return;

    this.init( type, id );
}

Rule.prototype.activeCls = 'active'; // when rule is active
Rule.prototype.stoppedCls = 'stopped'; // when rule is stopped
Rule.prototype.hoveredCls = 'hovered'; // when mouse over rule
Rule.prototype.normalCls = 'normal'; // when mouse out rule
Rule.prototype.editedCls = 'edited'; // when rule is being edited
Rule.prototype.hiddenCls = 'hidden'; // when new rule is need to be hidden
Rule.prototype.errorMessageCls = 'error-message'; // when rule's UI elements display error
Rule.prototype.rpcDelay = 300; // milliseconds delay before execute rpc, to avoid blinking messages within UI

Rule.prototype.init = function( type, usersInfo )
{
    this.notificationType = type;
    this.addRuleButton = getElementById(String.format('addRule_{0}', this.notificationType));
    this.tooltip = new DhtmlTitle ( String.format('tooltip_{0}', this.notificationType), false, true );
    this.id = null;
    this.timestamp = null;
    this.recipientsSelector = new UsersSelector();
    this.domEl = null;
    this.userSelectorLabelLength = 68;
    this.usersInfo = usersInfo;

    return this;
};

/**
 * Sets initial state of new created Rule. The method sets current rule state as a saved without validation.
 */
Rule.prototype.setInitialState = function()
{
    this.ruleState.setInitialState();
};

Rule.prototype.setDomEl = function( el, ruleGlobalName )
{
    this.domEl = el;

    this.recipientsSelector.registerElements(this.domEl,
                                             String.format('recipientsSelector_{0}', this.notificationType),
                                             '.text');

    this.registerInputs( ruleGlobalName );
};

// Subclasses should override this to acquire references to notification-specific html elements.
// At the time of calling this method, this.domEl is guaranteed to be set, but not yet added to the document.
// After getting a reference to an element, subclasses shall set its id to null
// to avoid having several elements with the same id within one document
Rule.prototype.registerInputs = function( )
{
    this.removeButton = new IconButton( this.domEl,
                                        String.format('removeButton_{0}', this.notificationType),
                                        makeHandlerNoPropagation( this.remove, this ),
                                        this.tooltip,
                                        messageResource.getMessage("mail_notifications.form.tooltip.delete") );
    this.iconMessage = findChildById(this.domEl, String.format('iconMessage_{0}', this.notificationType));
    this.iconMessage.id = null;
    this.iconMessage.mouseController = new MouseController();

    this.resumeButton = new IconButton( this.domEl,
                                        String.format('resumeButton_{0}', this.notificationType),
                                        makeHandlerNoPropagation( this.switchActiveState, this ),
                                        this.tooltip,
                                        messageResource.getMessage("mail_notifications.form.tooltip.pause") );

    this.saveButton = findChildById(this.domEl, String.format('saveButton_{0}', this.notificationType));
    this.saveButton.id = null;
    this.saveButton.onclick = makeHandlerNoPropagation( this.save, this );

    this.buttonMessage = findChildById(this.domEl, String.format('buttonMessage_{0}', this.notificationType));
    this.buttonMessage.id = null;

    this.cancelButton = findChildById(this.domEl, String.format('cancelButton_{0}', this.notificationType));
    this.cancelButton.id = null;                                         
    this.cancelButton.onclick = makeHandlerNoPropagation( this.stopEdit, this );

    this.domEl.onmouseover = makeHandler( this.onMouseOver, this );
    this.domEl.onmouseout = makeHandler( this.onMouseOut, this );

    this.domEl.onclick = makeHandler( this.startEdit, this );
};

Rule.prototype.initUI = function(selectorType, maxLabelLength)
{
    this.recipientsSelector.init(selectorType, NotificationTitle[this.notificationType], maxLabelLength, undefined );
    addClass(this.domEl, this.normalCls);

    // Un comment this code if need to make some validation when UserSelector panel hides.
    // Note: need to specify the method of ruleState which have to be called
    /*var thisRule = this;
    this.recipientsSelector.setPopupHideCallback( function () { thisRule.ruleState.[some method] } );*/
};

Rule.prototype.initCPSelector = function()
{
    this.cpSelectorMenu = window.customerProjectSelectorHolder.getMenu();
    this.cpSelectorButton = new AT.HierarchySelectorDropdownButton({
                                                            renderTo: String.format(RuleFactory.CP_SELECTOR_BUTTON_ID, this.notificationType, this.index),
                                                            labelCls: "hierarchySelectorButton",
                                                            menu: this.cpSelectorMenu,
                                                            tooltipObject: new DhtmlTitle("form-field-tooltip")

    });
    this.cpSelector = this.cpSelectorMenu.getSelectorPanel();
    this.ruleState.setCheckProjectsAvailability( true );
    this.cpSelectorButton.setTooltipEnabled(false);
};

Rule.prototype.enableCPSelector = function()
{
    this.cpSelectorButton.setPlainTextMode(false);
};

Rule.prototype.disableCPSelector = function()
{
    this.cpSelectorButton.setPlainTextMode(true);
    this.cpSelectorButton.setDisplayInputError( false );
};

Rule.prototype.blockCPSelector = function( block )
{
    this.cpSelectorButton.setEnabled(!block);
    this.cpSelectorButton.setDisplayInputError( false );
};

Rule.prototype.onMouseOver = function()
{
    if( !this.domEl || this.hovered || this.edited) return;

    this.hovered = true;
    removeClass(this.domEl, this.normalCls);
    addClass(this.domEl, this.hoveredCls);
};

Rule.prototype.onMouseOut = function( )
{
    try{
        if( !this.domEl || this.edited || ( Ext.isIE && !isMouseOutElement( this.domEl, window.event )) ) return;
    }
    catch(e){ /*if error occurred switch rule to normal state*/ }

    this.hovered = false;
    removeClass(this.domEl, this.hoveredCls);
    addClass(this.domEl, this.normalCls);
};

Rule.prototype.switchActiveState = function( )
{
    if( this.stopped )
        this.resume();
    else
        this.pause();
};

Rule.prototype.resume = function()
{
    if( !this.id ||    // not saved yet
        RpcControlManager.isRpcBusy() )  // resume call cannot be executed as Rpc is busy at the moment
        return;
    if( !this.isValid())
    {
        this.startEdit();
        return;
    }
    this.registerEventHandlers();
    RpcControlManager.saveRpcControlBoundData( new RuleResumeManager( this ), false );
    this.displayMessage( "common.resuming" );
};

Rule.prototype.pause = function()
{
    if( !this.id ) return; //not saved yet

    // check if there is remote request in progress and reschedules pause call if there is
    if( RpcControlManager.isRpcBusy() && this.ruleState.setParamsError )
    {
        var thisRule = this;
        window.setTimeout(function() { thisRule.pause.call(thisRule) }, 500);
    }
    else if ( RpcControlManager.isRpcBusy() )
        return;
    else{
        this.registerEventHandlers();
        RpcControlManager.saveRpcControlBoundData( new RulePauseManager( this ), false );
    }

    this.displayMessage( "common.pausing" );
};

Rule.prototype.onPauseCompleted = function()
{
    this.stopped = true;

    if( this.domEl )
    {
        addClass( this.domEl, this.stoppedCls );
        removeClass( this.domEl, this.activeCls );
        var resumeText = messageResource.getMessage("mail_notifications.form.tooltip.resume");
        this.resumeButton.setTooltipMessage( resumeText );
        this.displayMessage( "mail_notifications.message.paused" );
        this.switchIconMessageState( true, resumeText );
        //if paused because of parameters became incomplete
        this.ruleState.setParamsError = false;
        
        PageOverlayTipsManager.getInstance().refreshDynamicTips();
    }
};

Rule.prototype.switchIconMessageState = function( setClickable, tooltipText )
{
    if( setClickable )
    {
        this.iconMessage.onclick = makeHandlerNoPropagation( this.resume, this );
        this.iconMessage.onmouseover = makeHandler (function(){
            this.tooltip.showDelayedWithTimeoutNearMouse( tooltipText, this.iconMessage, this.iconMessage.mouseController, 600 );
        }, this);
        this.iconMessage.onmouseout =  makeHandler (function(){
            this.tooltip.cancelDelayedShow();
        }, this);
        this.iconMessage.onmousemove = makeHandler (function(event){
            this.iconMessage.mouseController.update(event || window.event);
        }, this);
    }
    else
    {
        this.tooltip.immediateHide();
        this.iconMessage.onclick = Ext.emptyFn;
        this.iconMessage.onmouseover = Ext.emptyFn;
        this.iconMessage.onmousemove =  Ext.emptyFn;
    }
};

Rule.prototype.onResumeCompleted = function()
{
    this.stopped = false;

    if( this.domEl )
    {
        addClass( this.domEl, this.activeCls );
        removeClass( this.domEl, this.stoppedCls );
        this.hideIconMessage( );
        this.resumeButton.setTooltipMessage( messageResource.getMessage("mail_notifications.form.tooltip.pause") );
        this.switchIconMessageState( false, null );

        PageOverlayTipsManager.getInstance().refreshDynamicTips();
    }
};

Rule.prototype.onPauseEnd = function()
{
    this.restoreEventHandlers();
    this.hideIconMessage( );
    this.switchIconMessageState( false, null );
};

Rule.prototype.onResumeEnd = function()
{
    this.restoreEventHandlers();
    this.displayMessage( "mail_notifications.message.paused" );
    this.switchIconMessageState( true, messageResource.getMessage("mail_notifications.form.tooltip.resume") );
};

Rule.prototype.onResumeBegin = function()
{
    this.switchIconMessageState( false, null );
};


Rule.prototype.remove = function()
{
    if( RpcControlManager.isRpcBusy() ||
        !confirm( messageResource.getMessage("mail_notifications.remove_confirmation") ) )
        return;

    if( !this.id ) // not saved yet
    {
        this.onRemoved();    
    }
    else
    {
        this.registerEventHandlers();
        RpcControlManager.saveRpcControlBoundData( new RuleDeleteManager( this ), false );
        this.displayMessage( "common.removing" );
    }
};

Rule.prototype.onRemoveEnd = function()
{
    this.restoreEventHandlers();
    if ( this.stopped )
        this.displayMessage( "mail_notifications.message.paused" );
    else
        this.hideIconMessage( );
};

Rule.prototype.onRemoved = function()
{
    ruleFactory.removeRule( this );
};

Rule.prototype.displayButtons = function( display, messageKey )
{
    this.displayControls( display, [this.saveButton, this.cancelButton], this.buttonMessage, messageKey );
};

Rule.prototype.displayControls = function ( display, controls, messageElement, messageKey )
{
    var message = messageKey ? messageResource.getMessage(messageKey) : "";
    for ( var i = 0; i < controls.length; i++)
        this.displayElement( controls[i], display);
    this.displayElement( messageElement, !display);
    messageElement.innerHTML = message;
};

Rule.prototype.displayElement = function ( element, display)
{
    if( display )
        removeClass( element, this.hiddenCls );
    else
        addClass( element, this.hiddenCls );
};

Rule.prototype.displayMessage = function ( text )
{
    removeClass( this.iconMessage, this.hiddenCls );
    addClass( this.iconMessage, this.errorMessageCls );

    var message = messageResource.getMessage( text );
    this.iconMessage.innerHTML = message ? message : text;
};

Rule.prototype.hideIconMessage = function ( )
{
    this.displayElement( this.iconMessage, false );
};

Rule.prototype.startEdit = function( event )
{
    if( !this.domEl || this.edited || RpcControlManager.isRpcBusy() ) return;
    // if startEdit action has been initiated without click: new rule or after previously edited rule was saved
    if ( event )
    {
        // we should not start editing the rule, if one of the buttons was clicked
        if( isTarget(this.resumeButton, event) ) return;
        if( isTarget(this.removeButton, event) ) return;
        if( isTarget(this.saveButton, event) ) return;
        if( isTarget(this.cancelButton, event) ) return;
    }

    //if there is a rule being edited
    if ( currentEditRule && !currentEditRule.stopEdit() )
    {
        return;
    }

    this.defineStateOfAllSelectors();
    // to check if any rule's parameters become incorrect
    var verifyFields = ( (this.id != null) && (this.isValid()) );
    this.registerEventHandlers();
    // if new rule it will have disabled class
    removeClass( this.domEl, this.hiddenCls );
    this.edited = true;
    currentEditRule = this;
    this.enableInputs();
    removeClass(this.domEl, this.hoveredCls);
    removeClass(this.domEl, this.normalCls);
    addClass(this.domEl, this.editedCls);
};

/**
 * Finishes Rule editing process.
 * @return boolean true if Rule has been stopped successfully, false if not for instance user decided to keep editing rule.
 */
Rule.prototype.stopEdit = function()
{
    if( !this.domEl || this.saving ) return false;
    var discardConfirmationMessage = this.id ? "mail_notifications.discard_changes_confirmation"
                                             : "mail_notifications.cancel_rule_creation_confirmation";
    if( this.isChanged() && !confirm( messageResource.getMessage( discardConfirmationMessage ) ) )
    {
        //window.scrollTo( 0 , findPosY( this.domEl ) );
        this.scrollWindowIfNeeded();
        return false;
    }
    if( !this.id ) this.onRemoved(); //not saved yet

    this.restoreEventHandlers();
    this.edited = false;
    currentEditRule = null;

    removeClass(this.addRuleButton, "disabled-link");
    removeClass(this.domEl, this.editedCls);
    addClass(this.domEl, this.normalCls);
    this.ruleState.reset();
    this.disableInputs();
    return true;
};

/**
 * Saves rule and calls back given function on given context if saving procedure competed successfully
 * @param callBackFunction function to be called back as rule is saved
 * @param callBackContext context to call <code>callBackFunction</code> on
 */
Rule.prototype.saveAndCallBack = function( callBackFunction,  callBackContext )
{
    this.callBackFunction = callBackFunction;
    this.callBackContext = callBackContext;
    this.save();
};

/**
 * Private.
 * Executes call back
 */
Rule.prototype.callBack = function()
{
    if ( this.callBackFunction )
    {
        this.callBackFunction.call ( this.callBackContext );
    }
    this.clearCallBack();
};

/**
 * Private.
 * Clears call back
 */
Rule.prototype.clearCallBack = function()
{
    this.callBackFunction = null;
    this.callBackContext = null;
};

/**
 * Determines whether Rule is being edited at the moment
 */
Rule.prototype.isEdited = function()
{
    return this.edited;
};

Rule.prototype.scrollWindowIfNeeded = function()
{
    return scrollToElementIfNeeded( this.domEl );
};

/**
 * Private
 * Should be overwritten by inheritors
 */
Rule.prototype.disableInputs = function()
{
    this.recipientsSelector.disable();
};

/**
 * Private
 * Should be overwritten by inheritors
 */
Rule.prototype.enableInputs = function()
{
    this.recipientsSelector.enable();
};

Rule.prototype.defineStateOfAllSelectors = function()
{
    this.recipientsSelector.defineState();
};

Rule.prototype.defineStateOfCustomerProjectSelector = function()
{
    this.cpSelectorMenu.setCurrentParent(this.cpSelectorButton, true);
    this.ruleState.setCPSelectorState( this.cpSelector, this.ruleState.cpSelectorKey );
};

Rule.prototype.focusFirstErroneousInput = function()
{
    this.ruleState.focusFirstErroneousInput();
};

/**
 * Initiates rule saving.
 * Returns true if saving process has been initiated, false if not.
 */

Rule.prototype.save = function( )
{
    //Check if selected parameters are valid
    if ( !this.isValid() )
    {
        alert( messageResource.getMessage("mail_notifications.form.cannot_save.invalid_data") );
        this.scrollWindowIfNeeded();
        this.clearCallBack();
        this.focusFirstErroneousInput();
        return false;
    }
    // Check if there are possible issues with notification delivery caused by recipients and projects selection
    // (no email, no access right, no projects assigned)
    var recipientsCheck = this.ruleState.checkRecipients();
    if( recipientsCheck )
    {
        alert( trim( messageResource.getMessage("mail_notifications.warning") + "\n" + recipientsCheck ) );
    }
    var savingMgr = this.savingManager ? this.savingManager : new RulesSavingManager( this );
    RpcControlManager.saveRpcControlBoundData( savingMgr, false );
    return true;
};

/**
 * Determines whether Rule's state has been modified.
 * Returns true if Rule's state has been modified, false if not
 */
Rule.prototype.isChanged = function()
{
    this.updateState();
    return this.ruleState.isChanged();
};

/**
 * Determines whether current Rule's state is valid.
 * Returns true if Rule's state is valid, false if not
 */
Rule.prototype.isValid = function()
{
    this.updateState();
    return this.ruleState.isValid()
};

Rule.prototype.updateState = function()
{
    this.updateInputs();
    this.ruleState.getCurrentState();
};

/**
 * @return params Rule's parameters values. For more information on <code>params</code> object format please refer to corresponding
 * RuleState implementation.
 */
Rule.prototype.getParams = function()
{
    var params = this.ruleState.getCurrentState();
    params.javaClass = RuleJavaClassNames[this.notificationType];
    params.active = !this.stopped;
    params.id = this.id || null;
    params.lastChangeTimestamp = this.lastChangeTimestamp || null;
    return params;
};

/**
 * Sets Rule's params values. For more information on <code>params</code> object format please refer to corresponding
 * RuleState implementation.
 * @param params
 */
Rule.prototype.setParams = function( params )
{
    this.ruleState.initState( params );
    this.active = params.active;
    this.id = params.id;
    this.lastChangeTimestamp = params.lastChangeTimestamp;
    if ( !this.active )
        this.onPauseCompleted();
    else if( this.ruleState.setParamsError )
    {
        this.pause();
    }
};

Rule.prototype.setBlockInputs = function( block )
{
    this.recipientsSelector.block( block );
};

Rule.prototype.onBeginSaving = function()
{
    this.displayButtons(false, "common.saving");
    this.updateInputs();
    this.setBlockInputs( true );
};

Rule.prototype.onEndSaving = function()
{
    this.setBlockInputs( false );
    this.displayButtons( true );
};

Rule.prototype.onSavingComplete = function()
{
    this.ruleState.update();
    this.stopEdit();
    this.callBack();
};

Rule.prototype.onSavingErrorOccurred = function()
{
    this.displayButtons(true);
};

/**
 * Private
 */
Rule.prototype.updateInputs = function()
{
    this.recipientsSelector.update();
};

/**
 * Registers event handlers
 */
Rule.prototype.registerEventHandlers = function()
{
    var thisRule = this;
    ScheduleButtonsControl.setIsEventProhibited( makeHandler( clickProhibited, this ) );

    KeyDownHandler.setOnKeyDownHandler(function(event)
    {
        // Entered data is discarded if 'Esc' key is pressed
        handleEscape(function(){thisRule.stopEdit()}, event);
        // Data has been entered is saved if 'Enter' key is pressed
        handleEnter(function(){thisRule.save()}, event);
        // Switching between elements by pressing 'tab' key is blocked
        handleTab(function(){window.event.cancelBubble = true;}, event);
    });

    // Register "onmousedown" handler
    this.oldDocumentMousedown = document.onmousedown;
    document.onmousedown = function ()
    {        
        var target = getEventTarget( event );
        if ( !thisRule.isStopEditEvent( event ) || isTarget( thisRule.domEl, event )){
            if ( isTarget( thisRule.domEl, event ) && thisRule.saving )
                stopEventPropagation( event );
            return;
        }
        if ( thisRule.edited )
        {
            var now = new Date().getTime();
            var rulePosition = findPosY( thisRule.domEl );
            if (!thisRule.stopEdit())
                stopEventPropagation( event );
            else if ( now + 100 < new Date().getTime() ||
                      (!thisRule.id && rulePosition < calculateCoords( event ).y) ) // need to fire event if there was:
                      // 1. delay because of required user's decision on changes have been made to the edited rule
                      // 2. if new rule creation has been canceled and mouse event occurred on the element below new rule
                      // For more information on case #2 please refer to Bug 32913: http://pmxbugdb.actimind.com/show_bug.cgi?id=32913
            {
                fireMouseEvent( event,
                                "click",
                                 false );
            }
        }
    };

    this.initDescendantEventsListener($(thisRule.domEl), {
        "dropdown.menuHide": function() { thisRule.menuShown = false; },
        "dropdown.menuShow": function() { thisRule.menuShown = true; }
    });
};

Rule.prototype.isRuleTarget = function ( event )
{
    return hasParentWithClassName( getEventTarget( event ), "rule" );
};

/**
 * Restores event handlers
 */
Rule.prototype.restoreEventHandlers = function()
{
    KeyDownHandler.restoreOnKeyDownHandler();
    ScheduleButtonsControl.restoreIsEventProhibited();
    document.onmousedown = this.oldDocumentMousedown;
};

Rule.prototype.isStopEditEvent = function ( evt )
{
    var event = evt ? evt : window.event;
    var classes = [ "x-menu", "disabled-link" ];
    var i;
    var target = getEventTarget( event );

    if ( whichButtonPressed ( event ) != "left" ||
         target.nodeName.toLowerCase() == "html" ) // check if scroll bar has been clicked
        return false;

    if ( RpcControlManager.isRpcBusy() )
        return true;

    for( i = 0; i < classes.length; i++ )
    {
        if ( hasParentWithClassName( target, classes[i] ) )
            return false;
    }

    if (this.menuShown)
        return false;

    return true;
};

_.extend(Rule.prototype, DescendantEventsListenerMixin);

//-----------------------------------------------

function PersonalTTRule() {}

PersonalTTRule.prototype = _.create(Rule.prototype);

PersonalTTRule.prototype.init = function ( usersInfo )
{
    Rule.prototype.init.call( this, NotificationType.PERSONAL_TT, usersInfo );
    this.ttLimit = new TTLimit();
    this.ruleState = new PersonalTTRuleState( this );

    return this;
};

PersonalTTRule.prototype.initUI = function ()
{
    Rule.prototype.initUI.call( this, MailNotificationsUserSelectorCache.SELECTOR_USERS_WITH_ENTER_TT_RIGHT, this.userSelectorLabelLength );
};

PersonalTTRule.prototype.registerInputs = function ( ruleGlobalName )
{
    Rule.prototype.registerInputs.call( this );

    this.ttLimit.init( this.domEl, NotificationType.PERSONAL_TT, ruleGlobalName );
};

PersonalTTRule.prototype.enableInputs = function()
{
    Rule.prototype.enableInputs.call(this);
    this.ttLimit.enable();
};

PersonalTTRule.prototype.disableInputs = function()
{
    Rule.prototype.disableInputs.call(this);
    this.ttLimit.disable();
};

PersonalTTRule.prototype.updateInputs = function()
{
    Rule.prototype.updateInputs.call(this);
    this.ttLimit.update();
};

PersonalTTRule.prototype.setBlockInputs = function( block )
{
    Rule.prototype.setBlockInputs.call( this, block );
    this.ttLimit.block( block );
};

//-----------------------------------------------

function UnsubmittedTTRule(){}

UnsubmittedTTRule.prototype = _.create(Rule.prototype)

UnsubmittedTTRule.prototype.init = function ( usersInfo )
{
    Rule.prototype.init.call( this, NotificationType.UNSUBMITTED_TT, usersInfo );
    this.ttUsersSelector = new UsersSelector();
    this.ttLimit = new TTLimit();
    this.ruleState = new UnsubmittedTTRuleState( this );
    
    return this;
};

UnsubmittedTTRule.prototype.registerInputs = function ( ruleGlobalName )
{
    Rule.prototype.registerInputs.call( this );

    this.ttLimit.init( this.domEl, NotificationType.UNSUBMITTED_TT, ruleGlobalName );

    this.ttUsersSelector.registerElements(this.domEl,
                                          String.format('ttUsersSelector_{0}', NotificationType.UNSUBMITTED_TT),
                                          '.text');

};

UnsubmittedTTRule.prototype.initUI = function ()
{
    Rule.prototype.initUI.call( this, MailNotificationsUserSelectorCache.SELECTOR_USERS_WITH_GENERATE_TIME_REPORTS_RIGHT );

    this.ttUsersSelector.init( MailNotificationsUserSelectorCache.SELECTOR_USERS_WITH_ENTER_TT_RIGHT_NO_EMAIL_CHECK, null, this.userSelectorLabelLength );
};

UnsubmittedTTRule.prototype.enableInputs = function()
{
    Rule.prototype.enableInputs.call(this);
    this.ttLimit.enable();
    this.ttUsersSelector.enable();
};

UnsubmittedTTRule.prototype.defineStateOfAllSelectors = function()
{
    Rule.prototype.defineStateOfAllSelectors.call(this);
    this.ttUsersSelector.defineState();
};

UnsubmittedTTRule.prototype.disableInputs = function()
{
    Rule.prototype.disableInputs.call(this);
    this.ttLimit.disable();
    this.ttUsersSelector.disable();
};

UnsubmittedTTRule.prototype.updateInputs = function()
{
    Rule.prototype.updateInputs.call(this);
    this.ttLimit.update();
};

UnsubmittedTTRule.prototype.setBlockInputs = function( block )
{
    Rule.prototype.setBlockInputs.call(this, block);
    this.ttLimit.block( block );
    this.ttUsersSelector.block( block );
};

//-----------------------------------------------

function DeadlineRule(){}

DeadlineRule.prototype = _.create(Rule.prototype)

DeadlineRule.prototype.init = function ( usersInfo )
{
    Rule.prototype.init.call( this, NotificationType.DEADLINE, usersInfo );
    this.deadlineCondition = new DeadlineCondition();
    this.ruleState = new DeadlineRuleState( this );
    
    return this;
};

DeadlineRule.prototype.initUI = function ()
{
    Rule.prototype.initUI.call( this, MailNotificationsUserSelectorCache.SELECTOR_USERS_WITH_MANAGE_TASKS_RIGHT );
};

DeadlineRule.prototype.registerInputs = function ( ruleGlobalName )
{
    Rule.prototype.registerInputs.call( this );

    this.initCPSelector();
    this.deadlineCondition.init( this.domEl, this.notificationType, ruleGlobalName );
};

DeadlineRule.prototype.enableInputs = function()
{
    Rule.prototype.enableCPSelector.call(this);
    Rule.prototype.enableInputs.call(this);
    this.deadlineCondition.enable();
};

DeadlineRule.prototype.disableInputs = function()
{
    Rule.prototype.disableCPSelector.call(this);
    Rule.prototype.disableInputs.call(this);
    this.deadlineCondition.disable();
};

DeadlineRule.prototype.defineStateOfAllSelectors = function()
{
    Rule.prototype.defineStateOfAllSelectors.call(this);
    this.defineStateOfCustomerProjectSelector();
};

DeadlineRule.prototype.updateInputs = function()
{
    Rule.prototype.updateInputs.call(this);
    this.deadlineCondition.update();
};

DeadlineRule.prototype.setBlockInputs = function( block )
{
    Rule.prototype.setBlockInputs.call(this, block);
    this.blockCPSelector( block );
    this.deadlineCondition.block( block );
};

//-----------------------------------------------

function EstimateRule(){}

EstimateRule.prototype = _.create(Rule.prototype)

EstimateRule.prototype.init = function ( usersInfo )
{
    Rule.prototype.init.call( this, NotificationType.ESTIMATE, usersInfo );
    this.ruleState = new EstimateRuleState( this );
    this.additionalTextSelectedClassname = "no-input-static-text unselectable additionalTextSelected";
    this.additionalTextClassname = "no-input-static-text unselectable additionalText";
    
    return this;
};

EstimateRule.prototype.initUI = function ()
{
    Rule.prototype.initUI.call( this, MailNotificationsUserSelectorCache.SELECTOR_USERS_WITH_GENERATE_TIME_REPORTS_RIGHT );
};

EstimateRule.prototype.registerInputs = function ( ruleGlobalName )
{
    Rule.prototype.registerInputs.call( this );

    this.initCPSelector();

    this.estimateInput = new InputControl();
    var dataElementId = String.format('notification[{0}].estimatePercent', this.notificationType);
    this.estimateInput.registerElements( this.domEl,
                                         dataElementId,
                                         '.text' );
    this.estimateInput.setValidator(new IntegerInputControlValidator(1));
    this.additionalText = findChildById(this.domEl, dataElementId + ".additionalText");
};

EstimateRule.prototype.enableInputs = function()
{
    Rule.prototype.enableCPSelector.call(this);
    Rule.prototype.enableInputs.call(this);
    this.estimateInput.enableInput();
    this.additionalText.className = this.additionalTextSelectedClassname;
};

EstimateRule.prototype.disableInputs = function()
{
    Rule.prototype.disableCPSelector.call(this);
    Rule.prototype.disableInputs.call(this);
    this.estimateInput.disableInput();
    this.additionalText.className = this.additionalTextClassname;
};

EstimateRule.prototype.defineStateOfAllSelectors = function()
{
    Rule.prototype.defineStateOfAllSelectors.call(this);
    this.defineStateOfCustomerProjectSelector();
};

EstimateRule.prototype.updateInputs = function()
{
    Rule.prototype.updateInputs.call(this);
    this.estimateInput.updateData();
};

EstimateRule.prototype.setBlockInputs = function( block )
{
    Rule.prototype.setBlockInputs.call(this, block);
    this.blockCPSelector( block );
    this.estimateInput.block( block );
};
//-----------------------------------------------


function ProjectBudgetRule(){}

ProjectBudgetRule.prototype = _.create(Rule.prototype)

ProjectBudgetRule.prototype.init = function ( usersInfo )
{
    Rule.prototype.init.call( this, NotificationType.PROJECT_BUDGET, usersInfo );
    this.moneyCondition = new MoneyCondition();
    this.ruleState = new ProjectBudgetRuleState( this );
    
    return this;
};

ProjectBudgetRule.prototype.initUI = function ()
{
    Rule.prototype.initUI.call( this, MailNotificationsUserSelectorCache.SELECTOR_USERS_WITH_GENERATE_MONEY_REPORTS_RIGHT );
};

ProjectBudgetRule.prototype.registerInputs = function ( ruleGlobalName )
{
    Rule.prototype.registerInputs.call( this );

    this.initCPSelector();
    this.moneyCondition.init( this.domEl,  this.notificationType, ruleGlobalName);
};

ProjectBudgetRule.prototype.enableInputs = function()
{
    Rule.prototype.enableCPSelector.call(this);
    Rule.prototype.enableInputs.call(this);
    this.moneyCondition.enable();
};

ProjectBudgetRule.prototype.disableInputs = function()
{
    Rule.prototype.disableCPSelector.call(this);
    Rule.prototype.disableInputs.call(this);
    this.moneyCondition.disable();
};

ProjectBudgetRule.prototype.defineStateOfAllSelectors = function()
{
    Rule.prototype.defineStateOfAllSelectors.call(this);
    this.defineStateOfCustomerProjectSelector();
};

ProjectBudgetRule.prototype.updateInputs = function()
{
    Rule.prototype.updateInputs.call(this);
    this.moneyCondition.update();
};

ProjectBudgetRule.prototype.setBlockInputs = function( block )
{
    Rule.prototype.setBlockInputs.call(this, block);
    this.blockCPSelector( block );
    this.moneyCondition.block( block );
};
//-----------------------------------------------

function PersonalEarningsRule() {}

PersonalEarningsRule.prototype = _.create(Rule.prototype)

PersonalEarningsRule.prototype.init = function ( usersInfo )
{
    Rule.prototype.init.call( this, NotificationType.PERSONAL_EARNINGS, usersInfo );
    this.moneyCondition = new MoneyCondition();
    this.ttUsersSelector = new UsersSelector();
    this.ruleState = new PersonalEarningsRuleState( this );
    
    return this;
};

PersonalEarningsRule.prototype.registerInputs = function ( ruleGlobalName )
{
    Rule.prototype.registerInputs.call( this );

    this.moneyCondition.init( this.domEl, this.notificationType, ruleGlobalName );

    this.ttUsersSelector.registerElements(this.domEl,
                                          String.format('ttUsersSelector_{0}', NotificationType.PERSONAL_EARNINGS),
                                          '.text');
};

PersonalEarningsRule.prototype.initUI = function ()
{
    Rule.prototype.initUI.call( this, MailNotificationsUserSelectorCache.SELECTOR_USERS_WITH_GENERATE_MONEY_REPORTS_RIGHT );

    this.ttUsersSelector.init( MailNotificationsUserSelectorCache.SELECTOR_ALL_USERS, null, this.userSelectorLabelLength, null,
                               new AT.ComplicateUsersAndGroupsSelectionHelper( {
                                                                                   userSelectorGroupsStrategy: getUserSelectorFeatureStrategy(),
                                                                                   accessRight: null,
                                                                                   maxLength: this.userSelectorLabelLength,
                                                                                   checkEmail: false
                                                                               } ) );
};

PersonalEarningsRule.prototype.enableInputs = function()
{
    Rule.prototype.enableInputs.call(this);
    this.moneyCondition.enable();
    this.ttUsersSelector.enable();
};

PersonalEarningsRule.prototype.defineStateOfAllSelectors = function()
{
    Rule.prototype.defineStateOfAllSelectors.call(this);
    this.ttUsersSelector.defineState();
};

PersonalEarningsRule.prototype.disableInputs = function()
{
    Rule.prototype.disableInputs.call(this);
    this.moneyCondition.disable();
    this.ttUsersSelector.disable();
};

PersonalEarningsRule.prototype.updateInputs = function()
{
    Rule.prototype.updateInputs.call(this);
    this.moneyCondition.update();
};

PersonalEarningsRule.prototype.setBlockInputs = function( block )
{
    Rule.prototype.setBlockInputs.call(this, block);
    this.moneyCondition.block( block );
    this.ttUsersSelector.block( block );
};
