 /**
 * Required files:
 * mail_notifications_rules.js
 */


/**
 * RuleFactory is responsible for producing a new Rule object and creating HTML tbody element for it
 * based on specified notification type.
 * The factory also holds counters for created rule objects of each type, which are used
 * to give unique ids to input fields
 */
function RuleFactory( users, userGroups, timeZoneGroups )
{
    this.PROTOTYPE_ID = 'rulePrototype_{0}';
    this.INSERT_MARKER_ID = 'addRule_{0}';
    this.MAIN_TABLE_ID = 'rulesTable';
    RuleFactory.CP_SELECTOR_BUTTON_ID = 'cpSelectorButton_{0}_{1}';

    var usersInfo = {
        users: users,
        userGroups: userGroups,
        timeZoneGroups: timeZoneGroups
    };
 
    this.creators = [];

    this.creators[NotificationType.PERSONAL_TT] = new PersonalTTRuleCreator().init(usersInfo);
    this.creators[NotificationType.UNSUBMITTED_TT] = new UnsubmittedTTRuleCreator().init(usersInfo);
    this.creators[NotificationType.DEADLINE] = new DeadlineRuleCreator().init(usersInfo);
    this.creators[NotificationType.ESTIMATE] = new EstimateRuleCreator().init(usersInfo);
    this.creators[NotificationType.PROJECT_BUDGET] = new ProjectBudgetRuleCreator().init(usersInfo);
    this.creators[NotificationType.PERSONAL_EARNINGS] = new PersonalEarningsRuleCreator().init(usersInfo);

    this.rules = {};
    for (var type in NotificationType)
    {
        this.rules[NotificationType[type]] = [];
    }
}

RuleFactory.prototype.isEdited = function()
{
    for(var i in this.rules)
        for (var j = 0; j < this.rules[i].length; j++)
            if (this.rules[i][j] != null && this.rules[i][j].isEdited())
                return true;
    return false;
};

RuleFactory.prototype.isEmpty = function()
{
    for(var i in this.rules)
        for (var j = 0; j < this.rules[i].length; j++)
            if (this.rules[i][j] != null)
                return false;
    return true;
};

RuleFactory.prototype.getAddingRule = function(notificationType)
{
    var ruleset = this.rules[notificationType];
    for (var i = 0; i < ruleset.length; i++)
    {
        var rule = ruleset[i];
        if (rule && !rule.id)
            return rule;
    }
    return null;
};

RuleFactory.prototype.addRule = function( notificationType, params )
{
    //a workaround for blinking rule section if click 'Add Rule' when RPC is in action (FF3 and IE)
    if( rulesLoadCompleted && RpcControlManager.isRpcBusy() )
        return;
    var addingRule = this.getAddingRule(notificationType);
    if (addingRule != null) return;

    var rule = this.createRule( notificationType );

    if ( !rule ) return;

    this.rules[notificationType].push( rule );

    rule.index = this.rules[notificationType].length - 1;

    var ruleEl = this.createRuleRow( notificationType, rule.index );
    ruleEl.id = String.format('ruleRow_{0}_{1}', notificationType, rule.index);

    rule.setDomEl( ruleEl, String.format('ruleFactory.rules[{0}][{1}]', notificationType, rule.index) );
    rule.initUI();

    if( params )
    {
        rule.setParams( params );
    }
    else
    {
        addClass(document.getElementById( String.format( this.INSERT_MARKER_ID, notificationType ) ), "disabled-link");
        addClass (ruleEl, rule.hiddenCls);
        rule.setInitialState();
        rule.startEdit();
    }

    PageOverlayTipsManager.getInstance().refreshDynamicTips();

    return rule;
};

// private
RuleFactory.prototype.createRule = function( notificationType )
{
    return this.creators[notificationType].createRule();
};

// private
RuleFactory.prototype.createRuleRow = function( notificationType, index )
{
    var prototype = document.getElementById( String.format( this.PROTOTYPE_ID, notificationType ) );
    var insertMarker = document.getElementById( String.format( this.INSERT_MARKER_ID, notificationType ) );
    var table = document.getElementById( this.MAIN_TABLE_ID );
    var cpSelectorContainer = $(prototype).find(".cpSelectorButton")[0];
    if (cpSelectorContainer)
    {
        cpSelectorContainer.id = String.format(RuleFactory.CP_SELECTOR_BUTTON_ID, notificationType, index);
    }
    var ruleEl = prototype.cloneNode(true);
    ruleEl.id = null;
    this.creators[notificationType].beforeInsertRow( ruleEl );
    table.insertBefore(ruleEl, insertMarker);
    return ruleEl;
};

RuleFactory.prototype.removeRule = function( rule )
{
    // TODO: figure out why timeout is needed here
    if (rule.id)
        setTimeout(_.bind(this.removeRuleDOM, this, rule), 350);
    else
        this.removeRuleDOM(rule);
    this.rules[rule.notificationType][rule.index] = null;
    this.creators[rule.notificationType].removeRule();
};

RuleFactory.prototype.removeRuleDOM = function(rule)
{
    rule.domEl.parentNode.removeChild(rule.domEl);
    PageOverlayTipsManager.getInstance().refreshDynamicTips();
};

/* --------------------------------------------- */

function RuleCreator()
{
    this.counter = 0;
}

RuleCreator.prototype.init = function ( usersInfo ) 
{
    this.usersInfo = usersInfo;
    return this;
};
 
RuleCreator.prototype.getTypeId = function ()
{
    return -1; // subclasses should override this
};

RuleCreator.prototype.doCreateRule = function ()
{
    return null; //subclasses should override this
};

RuleCreator.prototype.createRule = function ()
{
    var rule = this.doCreateRule();

    this.counter++;
    return rule;
};

RuleCreator.prototype.removeRule = function ()
{
    this.counter--;
};

RuleCreator.prototype.beforeInsertRow = function ( domEl )
{
    // subclasses should override this to modify created DOM elements before they are added to the document
};


/*  Specific rule creators  */

function PersonalTTRuleCreator() { }

PersonalTTRuleCreator.prototype = new RuleCreator();

PersonalTTRuleCreator.prototype.getTypeId = function()
{
    return NotificationType.PERSONAL_TT;
};

PersonalTTRuleCreator.prototype.doCreateRule = function()
{
    return new PersonalTTRule().init(this.usersInfo);
};


function UnsubmittedTTRuleCreator() { }

UnsubmittedTTRuleCreator.prototype = new RuleCreator();

UnsubmittedTTRuleCreator.prototype.getTypeId = function()
{
    return NotificationType.UNSUBMITTED_TT;
};

UnsubmittedTTRuleCreator.prototype.doCreateRule = function()
{
    return new UnsubmittedTTRule().init(this.usersInfo);
};


function DeadlineRuleCreator() { }

DeadlineRuleCreator.prototype = new RuleCreator();

DeadlineRuleCreator.prototype.getTypeId = function()
{
    return NotificationType.DEADLINE;
};

DeadlineRuleCreator.prototype.doCreateRule = function()
{
    return new DeadlineRule().init(this.usersInfo);
};


function EstimateRuleCreator() { }

EstimateRuleCreator.prototype = new RuleCreator();

EstimateRuleCreator.prototype.getTypeId = function()
{
    return NotificationType.ESTIMATE;
};

EstimateRuleCreator.prototype.doCreateRule = function()
{
    return new EstimateRule().init(this.usersInfo);
};


function ProjectBudgetRuleCreator() { }

ProjectBudgetRuleCreator.prototype = new RuleCreator();

ProjectBudgetRuleCreator.prototype.getTypeId = function()
{
    return NotificationType.PROJECT_BUDGET;
};

ProjectBudgetRuleCreator.prototype.doCreateRule = function()
{
    return new ProjectBudgetRule().init(this.usersInfo);
};


function PersonalEarningsRuleCreator() { }

PersonalEarningsRuleCreator.prototype = new RuleCreator();

PersonalEarningsRuleCreator.prototype.getTypeId = function()
{
    return NotificationType.PERSONAL_EARNINGS;
};

PersonalEarningsRuleCreator.prototype.doCreateRule = function()
{
    return new PersonalEarningsRule().init(this.usersInfo);
};
