/**
 * Formats, parses, validates number fields and highlightes if there are any errors.
 * The code below is copied from numbers_validotor.js with some modifications:
 * as opposed to numbers_validator.js, this class supports digit grouping;
 * Decimal separator and digit grouping separator are passed to constructor;
 * '.' or ',' symbol is treated as invalid unless it is associated with one of the separators.
 *
 * requires:
 * inited client_side_errors.js,
 * at.js,
 * messageResource object with imported corresponding namespace.
 */

/**
 *  Initializes numbers validator instance.
 *
 * @param decimalSeparator (Required) is character to separate decimal part of number. Not undefined, not empty.
 * @param digitalGrouping (Required) is character to delimit digits in the number. Can be undefined.
 * @param maxNumber is maximum number value (applicable to non-percent values only).
 * @param allowPercent true to allow entering percentage
 * @param maxFractionalDigits (Optional) max number of digits in fractional part. Defaults to 2
 * @param trimTrailingNulls (Optional) Whether to remove trailing null digits from fractional part. Defaults to false
 */
function NumbersValidator(decimalSeparator, digitalGrouping, maxNumber, allowPercent, maxFractionalDigits, trimTrailingNulls)
{
    this.decimalSeparator = decimalSeparator || ".";
    this.digitalGrouping = digitalGrouping || "";
    this.maxNumber = maxNumber;
    this.allowPercent = allowPercent;
    this.maxFractionalDigits = maxFractionalDigits || 2;
    this.trimTrailingNulls = trimTrailingNulls || false;

    /**
     * Returns decimal separator that using in the validator.
     */
    this.getDecimalSeparator = function ()
    {
        return decimalSeparator;
    };

    /**
    * Parses string with number using specified delimiters.
    * If the value is not valid, returns null.
    * If the given value is percentage and it is allowed, then percent sign is ignored.
    * Use {@link #isPercentage} to check if the entered value is percentage.
    * @param value is string to parse, not null, not empty, must be validated with {@link #isNumberStringValid}
    * @return {@link Float} number.
    */
    this.parseNumber = function (value)
    {
        var str = new String(this.removeDigitSeparator(value));
        
        if (!this.isNumberStringValid(value))
            return null;

        str = str.replace(this.decimalSeparator, '.');

        if (str == '.') return 0;

        return parseFloat(str);
    };

    /**
     * Verifies if the given value is percentage.
     * If the value is not valid, returns false.
     * If percentage is not allowed, returns false.
     * @param value is string to parse, not null, not empty, must be validated with {@link #isNumberStringValid}
     * @return boolean
     */
    this.isPercentage = function (value)
    {
        if (!this.isNumberStringValid(value))
            return false;

        return value.substring(value.length - 1) == "%";
    };

    /**
     * Validates string with double number equal or greater than zero
     * and optional percent sign (if allowed)
     * @param str string with number.
     */
    this.isNumberStringValid = function(str)
    {
        var value = this.removeDigitSeparator(str);
        var suffix = this.allowPercent ? "%?" : "";

        var regexFractional = '^\\d*[' + this.decimalSeparator + ']\\d*' + suffix + '$';
        var regexInteger = '^\\d*'+ suffix + '$';

        return (value.match( regexFractional ) != null) || (value.match( regexInteger ) != null);
    };

    /**
     * Validates string with double number equal or greater than zero, digit grouping symbols
     * and optional percent sign (if allowed)
     * @param value string with number.
     */
    this.isNumberWithDigitSeparatorValid = function(value)
    {
        if (!this.digitalGrouping || this.digitalGrouping.length == 0) {
            return this.isNumberStringValid(value);
        }

        var suffix = this.allowPercent ? "%?" : "";

        var regexFractional = '^((\\d|\\d\\d|\\d\\d\\d))?([' + this.digitalGrouping
                + ']\\d{3})*[' + this.decimalSeparator + ']\\d*' + suffix + '$';
        var regexInteger = '^((\\d|\\d\\d|\\d\\d\\d))?([' + this.digitalGrouping
                + ']\\d{3})*'+ suffix + '$';

        return (value.match( regexFractional ) != null) || (value.match( regexInteger ) != null);
    };

    /**
    * Formats number. Format is ##.##, where '.' is specified decimal separator.
    * @param value is string with number to format.
    * @return formated string or "" if value is null, empty or invalid.
    */
    this.formatNumber = function (value)
    {
        if ((value == null) || (trim(value) === ""))
            return "";

        var num = this.round(value);

        if (isNaN(num)) return "";

        var str = new String(num);

        if (this.trimTrailingNulls) {
            var firstNull;
            for (firstNull = str.length - 1; firstNull > str.indexOf("."); firstNull --) {
                if (str.charAt(firstNull) != '0') {
                    break;
                }
            }

            if (str.charAt(firstNull) == ".") firstNull--;

            str = str.slice(0, firstNull + 1);
        }

        if (this.decimalSeparator != undefined)
            str = str.replace('.', this.decimalSeparator);

        return str;
    };

    /**
     * Rounds the given value to display a fixed number of fractional digits specified by maxFractionalDigits.
     * @param value Number or String with well-formed number
     * @return rounded value
     */
    this.round = function (value) {
        var roundBase = Math.pow(10, this.maxFractionalDigits);
        var rounded = new Number(Math.floor(value*roundBase + 0.5)/roundBase);
        return rounded.toFixed(this.maxFractionalDigits);
    };

    /**
    * Formats number. Format is ##_###.##, where '.' is specified decimal separator and '_' is digit grouping symbol.
    * @param value is string with number to format.
    * @return formated string or "" if value is null, empty or invalid.
    */
    this.formatNumberWithDigitSeparator = function(value, isPercent)
    {
        var res = this.formatNumber(value, isPercent);
        if (res === "") return res;

        return this.addDigitSeparator(res);
    };

    /**
     * Adds digit grouping symbol to a number. Leading/trailing spaces are removed during the operation.
     * No error checking is performed for input parameter, so number should be checked before calling this method.
     * @param str String that represents valid number (float or integer)
     * @return formatted value
     */
    this.addDigitSeparator = function (str)
    {
        var value = this.removeDigitSeparator(str);
        var valueNum = this.parseNumber(value);

        if (this.decimalSeparator == "" || isNaN(valueNum) || valueNum < 1000) return value;

        var res = "";
        var decimalSeparatorIdx = value.indexOf(this.decimalSeparator);
        var intPart = decimalSeparatorIdx > 0 ? value.slice(0, decimalSeparatorIdx) : value;

        var n = intPart.length;
        while (n > 3)
        {
            res = this.digitalGrouping + intPart.substr(n - 3, 3) + res;
            n -= 3;
        }

        if (n > 0)
            res = intPart.substr(0, n) + res;

        if (decimalSeparatorIdx > 0) {
            res = res + value.substr(decimalSeparatorIdx);
        }

        return res;
    };

    this.removeDigitSeparator = function (value)
    {
        // digit grouping symbol is supposed to differ from decimal separator and any of digits
        var res = trim(value);

        if (!this.digitalGrouping || this.digitalGrouping.length == 0) return res;
        
        res = res.replace(new RegExp('[' + this.digitalGrouping + ']', "gi"), "");
        return res;
    };

    /**
    *  Checks number for format, equal or greater than zero,
    *  less than maxNumber. Marks field with errors, trims and formats input.
    *  @param field is html input text element.
    *  @param invalidNumberError - message key to show when number format is invalid.
    *  @param maxNumberError - message key to show when number is greater than maxNumber.
    *  @param emptyFieldIsValid - true if empty field must be processed as valid
    *  @param oldValue - is old value for the field.
     * @param doNotMarkAsValid - true not to mark the field as valid even if number format is right
    *  @returns true if field valid, else false.
    */
    this.checkNumber = function (field, invalidNumberError, maxNumberError, emptyFieldIsValid, oldValue, doNotMarkAsValid)
    {
        var value = trim(new String(field.value));

        if (value != field.value)
            field.value = value;

        value = this.removeDigitSeparator(value);

        if (value == "")
        {
            if (oldValue != "")
            {
                ClientSideErrors.markTextFieldAsValid(field);
            }

            return emptyFieldIsValid;
        }

        if ( !this.isNumberStringValid(value) )
        {
            if (this.decimalSeparator != undefined)
                ClientSideErrors.markTextFieldAsInvalid(field,
                        messageResource.getMessage(invalidNumberError, [this.decimalSeparator]));
            else
                ClientSideErrors.markTextFieldAsInvalid(field, messageResource.getMessage(invalidNumberError));

            return false;
        }
        else
        {
            var floatValue = this.parseNumber(value);
            var isPercent = this.isPercentage(value);

            if ((!this.maxNumber) || (floatValue < this.maxNumber))
            {
                if (value != "")
                    field.value = this.formatNumberWithDigitSeparator(floatValue, isPercent) + (isPercent ? "%" : "");

                if (!doNotMarkAsValid) {
                    ClientSideErrors.markTextFieldAsValid(field);
                }

                return true;
            }
            else
            {
                ClientSideErrors.markTextFieldAsInvalid(field, messageResource.getMessage(maxNumberError,
                        [this.formatNumberWithDigitSeparator(new String(this.maxNumber))]));

                return false;
            }
        }
    };
}