var ReportChartUtils = {

    getSortCategoryFunctionForChart: function(groupingLevel)
    {
        switch (groupingLevel)
        {
            case ReportGroupingLevel.USER:
                return function(user1, user2)
                {
                    var res = stringComparatorCaseSensitive(user1.lastName, user2.lastName);
                    return res == 0 ? stringComparatorCaseSensitive(user1.firstName, user2.firstName) : res;
                };
            case ReportGroupingLevel.USER_GROUP:
            {
                return function(item1, item2)
                {
                    if (item1.id == item2.id) return 0;
                    if (item1.id == SystemSettings.usersWithoutGroupId) return -1;
                    if (item2.id == SystemSettings.usersWithoutGroupId) return 1;

                    return stringComparatorCaseSensitive(item1.name, item2.name);
                };
            }
            case ReportGroupingLevel.TIME_ZONE_GROUP:
                return function(item1, item2)
                {
                    if (item1.id == item2.id) return 0;
                    if (item1.id == DEFAULT_TIME_ZONE_GROUP_ID) return -1;
                    if (item2.id == DEFAULT_TIME_ZONE_GROUP_ID) return 1;

                    return stringComparatorCaseSensitive(item1.name, item2.name);
                };
            case ReportGroupingLevel.CUSTOMER:
            case ReportGroupingLevel.PROJECT:
            case ReportGroupingLevel.BILLING_TYPE:
                return function(item1, item2)
                {
                    return stringComparatorCaseSensitive(item1.name, item2.name);
                };
            case ReportGroupingLevel.DATE:
                return function(date1, date2)
                {
                    var jsDate1 = Date.parseJSONDate(date1);
                    var jsDate2 = Date.parseJSONDate(date2);
                    return jsDate1.getTime() > jsDate2.getTime() ? 1 : -1;
                };
            case ReportGroupingLevel.WEEK:
                return function(week1, week2)
                {
                    var jsDate1 = Date.parseJSONDate(week1.from);
                    var jsDate2 = Date.parseJSONDate(week2.from);
                    return jsDate1.getTime() > jsDate2.getTime() ? 1 : -1;
                };
            case ReportGroupingLevel.MONTH:
                return function(month1, month2)
                {
                    var jsDate1 = Date.parseJSONDate(month1);
                    var jsDate2 = Date.parseJSONDate(month2);
                    return jsDate1.getTime() > jsDate2.getTime() ? 1 : -1;
                };
        }
    },

    getFormatCategoryFunctionForChart: function(selectedCategoryType, dateFormat, escape, partsLimit, recursively)
    {
        if (!partsLimit)
            partsLimit = 17;

        switch (selectedCategoryType)
        {
            case ReportGroupingLevel.USER:
                return function(user)
                {
                    if ($.isEmptyObject(user)) return "";

                    var formattedName = UserNameFormat.formatLFM(user.firstName, user.lastName, user.middleName);
                    if (formattedName.length < partsLimit)
                        return escape ? escapeHtmlSymbols(formattedName) : formattedName;
                    var res = "";
                    if (user.lastName.length > partsLimit)
                        res = (escape ? escapeHtmlSymbols(user.lastName.substr(0, partsLimit - 2)) : user.lastName.substr(0, partsLimit - 2)) + '...,<br>';
                    else
                        res = (escape ? escapeHtmlSymbols(user.lastName) : user.lastName) + ",<br>";
                    var firstWithMiddleName = user.firstName + ( user.middleName ? " " + user.middleName + "." : "" );
                    if (firstWithMiddleName.length > 18)
                        res += (escape ? escapeHtmlSymbols(user.firstName.substr(0, partsLimit - 2)) : user.firstName.substr(0, partsLimit - 2)) + '...';
                    else
                        res += escape ? escapeHtmlSymbols(firstWithMiddleName) : firstWithMiddleName;
                    return res;
                };
            case ReportGroupingLevel.USER_GROUP:
            {
                return function(item)
                {
                    if (item.id == SystemSettings.usersWithoutGroupId)
                    {
                        item.name = messageResource.getMessage("common.users_without_group");
                    }
                    return !$.isEmptyObject(item) ? StringUtils.insertBrakes(item.name, partsLimit, ' ', partsLimit, "...", escape, recursively) : "";
                };
            }
            case ReportGroupingLevel.TIME_ZONE_GROUP:
            {
                return function(item)
                {
                    var name = _.result(item, "name", "");
                    if (item.id == DEFAULT_TIME_ZONE_GROUP_ID)
                    {
                        name += " " + messageResource.getMessage("common.default_group");
                    }
                    return !$.isEmptyObject(item) ? StringUtils.insertBrakes(name, partsLimit, ' ', partsLimit, "...", escape, recursively) : "";
                };
            }
            case ReportGroupingLevel.CUSTOMER:
            case ReportGroupingLevel.BILLING_TYPE:
                return function(item)
                {
                    return !$.isEmptyObject(item) ? StringUtils.insertBrakes(item.name, partsLimit, ' ', partsLimit, "...", escape, recursively) : "";
                };
            case ReportGroupingLevel.PROJECT:
                return function(item)
                {
                    return !$.isEmptyObject(item) ? StringUtils.insertBrakes(item.name + " (" + item.customerInfo.name + ")", partsLimit, ' ', partsLimit,
                                                                             "...", escape, recursively) : "";
                };
            case ReportGroupingLevel.DATE:
                return function(date)
                {
                    if ($.isEmptyObject(date)) return "";
                    return new DateFormat(dateFormat).formatDate(date.parseJSONDate());
                };
            case ReportGroupingLevel.WEEK:
                return function(week)
                {
                    if ($.isEmptyObject(week)) return "";
                    var from = week.from.parseJSONDate();
                    var to = week.to.parseJSONDate();
                    var firstPart = week.from == week.to ? "" : new DateFormat('MMM dd').formatDate(from) + " - ";
                    return firstPart + new DateFormat('MMM dd, yyyy').formatDate(to);
                };
            case ReportGroupingLevel.MONTH:
                return function(month)
                {
                    if ($.isEmptyObject(month)) return "";
                    return new DateFormat('MMMM , yyyy').formatDate(month.parseJSONDate());
                };
        }
    }
};