function ReportPrintableHeadersBuilder() {
    this.constructor = ReportPrintableHeadersBuilder;
}

ReportPrintableHeadersBuilder.prototype.init = function ( data ) {
    this.isFilteredByAssignments = data.isFilteredByAssignments;

    this.paramStringBuilder = new ReportParamsStringBuilder().init( {
        users: data.users,
        userGroupsFeatureEnabled: EnabledFeatures.UGR,
        userGroups: data.userGroups,
        timeZoneGroups: data.timeZoneGroups,
        cpList: data.cpList,
        leaveTypes: data.leaveTypes
    } );
    return this;
};

ReportPrintableHeadersBuilder.prototype._customerFormatter = {
    format: function(customer) {
        return customer.name;
    }
};

ReportPrintableHeadersBuilder.prototype._projectFormatter = {
    format: function(project) {
        return project.name;
    }
};

ReportPrintableHeadersBuilder.prototype._userFormatter = {
    format: function (user) {
        return UserNameFormat.formatLFM( user.firstName, user.lastName, user.middleName );
    }
};

ReportPrintableHeadersBuilder.prototype._userGroupFormatter = {
    format: function(group) {
        return group.name;
    }
};

ReportPrintableHeadersBuilder.prototype._leaveTypesFormatter = {
    format: function ( leaveType ) {
        return leaveType.name;
    }
};

ReportPrintableHeadersBuilder.prototype._getCPKeySuffix = function () {
    return (this.isFilteredByAssignments) ? '_filtered' : '';
};

ReportPrintableHeadersBuilder.prototype._toEmphasis = function( str ) {
    return '<span class="param">' + str + '</span>';
};

ReportPrintableHeadersBuilder.prototype._getEscapedNames = function ( objects, formatter, delimiter ) {
    return escapeHtmlSymbols( this._getNames(objects, formatter, delimiter) );
};

ReportPrintableHeadersBuilder.prototype._getNames = function( objects, formatter, delimiter ) {
    if( !delimiter ) delimiter = ',';
    var names = '';
    for (var i = 0; i < objects.length - 1; i++)
    {
        names += formatter.format(objects[i]) + delimiter + ' ';
    }
    if (objects.length)
    {
        names += formatter.format(objects[objects.length - 1]);
    }
    return names;
};

ReportPrintableHeadersBuilder.prototype.addCPSelectionParamsStringToArray = function( cpSelection, array ) {
    var self = this;
    var keySuffix = this._getCPKeySuffix();

    if (cpSelection.qualifier == CustomersProjectsSelectionQualifier.ALL_ACTIVE
        || cpSelection.qualifier == CustomersProjectsSelectionQualifier.ALL)
    {
        var activeOnly = cpSelection.qualifier == CustomersProjectsSelectionQualifier.ALL_ACTIVE;
        var label = messageResource.getEscapedMessage( 'projects_customers_selector.customers_and_projects' ) + ':';
        var value;
        if ( activeOnly )
            value = messageResource.getEscapedMessage( "projects_customers_selector.all_active_projects_customers" + keySuffix);
        else
            value = messageResource.getEscapedMessage( "projects_customers_selector.all_projects_customers" + keySuffix);
        array.push( new PrintableHeader().init( label, value ) );
    }
    else
    {
        var customers = self.paramStringBuilder.getFilteredCustomers(cpSelection.customers);
        var projects = self.paramStringBuilder.getFilteredProjects(cpSelection.projects);
        var label, names;

        if ( customers.length ) {
            var customerKey = ( customers.length == 1 ) ? "common.Customer" : "common.Customers";
            label = messageResource.getEscapedMessage(customerKey) + ':';

            var postfix = "projects_customers_selector.with_all_projects" + keySuffix;

            postfix = this._toEmphasis( messageResource.getEscapedMessage(postfix) );
            names = self._getEscapedNames( customers, self._customerFormatter ) + ' ' + postfix;
            array.push( new PrintableHeader().init( label, names ) );
        }
        names = '';
        if ( projects.length ) {
            var projectKey = (projects.length == 1) ? "common.Project" : "common.Projects";
            label = messageResource.getEscapedMessage(projectKey) + ':';

            names = self._getEscapedNames( projects, self._projectFormatter );
            array.push( new PrintableHeader().init( label, names ) );
        }
        else if (customers.length == 0)
        {
            array.push( new PrintableHeader().init(
                    messageResource.getEscapedMessage('common.Projects') + ':',
                    messageResource.getEscapedMessage('report.no_projects')
            ) );
        }
    }
};

ReportPrintableHeadersBuilder.prototype.addStaffSelectionParamsToArray = function ( userSelection, array ) {
    var label = this._getStaffSelectionLabelToArray();

    if (userSelection.allUsers)
        array.push( new PrintableHeader().init( label, messageResource.getMessage("report.staffoutput.all_staff") ) );
    else
        array.push( new PrintableHeader().init( label, this._getSelectedStaff( userSelection ) ) );
};

ReportPrintableHeadersBuilder.prototype._getStaffSelectionLabelToArray = function() {
    return messageResource.getMessage("report.staff_label");
};

ReportPrintableHeadersBuilder.prototype._getSelectedStaff = function( userSelection ) {
    var USERS_WITHOUT_GROUP_ID = SystemSettings.usersWithoutGroupId;
    var self = this;

    var res = '';
    var users = this.paramStringBuilder.getFilteredUsers(userSelection.selectedUsers);
    var groups = [];
    var isUsersWithoutUserGroupSelected = false;
    if (EnabledFeatures.UGR && userSelection.groupType == GroupType.USER_GROUP)
    {
        groups = this.paramStringBuilder.getFilteredGroups( userSelection.selectedGroups, USERS_WITHOUT_GROUP_ID );
        isUsersWithoutUserGroupSelected = userSelection.selectedGroups.length - groups.length == 1;
    }
    else if (EnabledFeatures.TZG && userSelection.groupType == GroupType.TIME_ZONE_GROUP)
    {
        groups = this.paramStringBuilder.getFilteredTimeZoneGroups(userSelection.selectedGroups);
    }
    users = this.paramStringBuilder._convertSelectedGroupsToSelectedUsersIfNecessary( users, userSelection );
    var andText = messageResource.getMessage( "report.common.and_staff_delimiter" );

    if (users.length)
    {
        res = self._getEscapedNames( users, self._userFormatter, ';' );
    }
    if (groups.length)
    {
        var allUsersFromGroupsText = userSelection.groupType == GroupType.USER_GROUP ?
                                     messageResource.getEscapedMessage("report.staff.all_users_from_user_group" + ((groups.length == 1) ? '' : 's')) :
                                     messageResource.getEscapedMessage("report.staff.all_users_from");
        if (res)
        {
            var allUsersPrefix = ( isUsersWithoutUserGroupSelected ) ? ', ' : ' ' + andText;
            res += allUsersPrefix + ' ' + self._toEmphasis(allUsersFromGroupsText.toLowerCase());
        }
        else
        {
            res = self._toEmphasis( allUsersFromGroupsText );
        }
        res += ' ' + self._getEscapedNames( groups, self._userGroupFormatter );
    }
    if (isUsersWithoutUserGroupSelected)
    {
        var allUsersWithoutUserGroupText = messageResource.getMessage(
                "report.staff.all_users_without_user_groups",
                [ messageResource.getEscapedMessage( "common.users_without_group" ).toLowerCase() ]
        );
        allUsersWithoutUserGroupText = self._toEmphasis(allUsersWithoutUserGroupText);

        if (res)
        {
            res += ' ' + self._toEmphasis( andText ) + ' ' + allUsersWithoutUserGroupText.toLowerCase();
        }
        else
        {
            res = allUsersWithoutUserGroupText;
        }
    }
    return res;
};

ReportPrintableHeadersBuilder.prototype.addTimeApprovalStatusToArray = function ( timeApprovalStatus, array ) {
    if ( EnabledFeatures.TTA || timeApprovalStatus != TimeApprovalStatus.BOTH )
    {
        var label = messageResource.getMessage( "report.time_track_status" ) + ":";
        var value = '';

        switch( timeApprovalStatus )
        {
            case TimeApprovalStatus.BOTH:
            {
                value = messageResource.getMessage( "report.form.show_both_approved_and_not" );
                break;
            }
            case TimeApprovalStatus.APPROVED:
            {
                value = messageResource.getMessage( "report.form.show_only_approved" );
                break;
            }
            case TimeApprovalStatus.NOT_APPROVED:
            {
                value = messageResource.getMessage( "report.form.show_only_not_approved" );
                break;
            }
        }
        array.push( new PrintableHeader().init( label, value ) );
    }
};

ReportPrintableHeadersBuilder.prototype.addDateRangeHeaderString = function( dateRangeSelection ) {
    return messageResource.getMessage( "range.custom.pattern", [
        new DateFormat( SystemSettings.dateFormat ).formatDate( dateRangeSelection.fromDate.parseJSONDate() ),
        new DateFormat( SystemSettings.dateFormat ).formatDate( dateRangeSelection.toDate.parseJSONDate() ) ] );
};

ReportPrintableHeadersBuilder.prototype.addDateRangeHeaderStringParsed = function( dateRangeSelection ) {
    return messageResource.getMessage( "range.custom.pattern", [
        new DateFormat( SystemSettings.dateFormat ).formatDate( dateRangeSelection.fromDate ),
        new DateFormat( SystemSettings.dateFormat ).formatDate( dateRangeSelection.toDate ) ] );
};

ReportPrintableHeadersBuilder.prototype.addEvaDateHeaderString = function( dateSelection ) {
    return messageResource.getMessage("report.budgeted_vs_actual.view.printable_date_range_prefix")
           + ' ' + new DateFormat( SystemSettings.dateFormat ).formatDate( dateSelection.date.parseJSONDate() );
};

ReportPrintableHeadersBuilder.prototype.addEvaDateHeaderStringParsed = function( dateSelection ) {
    return messageResource.getMessage("report.budgeted_vs_actual.view.printable_date_range_prefix")
           + ' ' + new DateFormat( SystemSettings.dateFormat ).formatDate( dateSelection.date);
};

ReportPrintableHeadersBuilder.prototype.addLeaveTypesParamsToArray = function ( leaveTypeSelection, columnsConfig, array ) {
    if (columnsConfig == LeaveTypeColumnConfig.BALANCES_ONLY)
        return;

    var self = this;
    var label = messageResource.getMessage( "report.leave_types.label" );
    var value = '';

    switch ( leaveTypeSelection.qualifier )
    {
        case LeaveTypeSelectionQualifier.ALL:
        {
            value =  messageResource.getMessage( "report.leave_types.all" );
            break;
        }
        case LeaveTypeSelectionQualifier.SELECTED:
        {
            var leaveTypes = self.paramStringBuilder.getFilteredLeaveTypes( leaveTypeSelection.leaveTypes );
            value = self._getEscapedNames( leaveTypes, self._leaveTypesFormatter );
            break;
        }
    }
    array.push( new PrintableHeader().init( label, value ) );
};

ReportPrintableHeadersBuilder.prototype.addTaskSelectionParamsStringToArray = function( tasksConfig, includeNotEstimated, includeNotStarted, dateSelectionJSON, array ) {
    var label = messageResource.getEscapedMessage( "common.Tasks" ) + ':';
    var value = escapeHtmlSymbols(new EVAReport.ParametersFormatter().init({}).formatTaskSelectionParamsString(tasksConfig, includeNotEstimated, includeNotStarted,
                                                                                                               {dateSelection: dateSelectionJSON, isParsed: true}));
    
    array.push( new PrintableHeader().init( label, value ) );
};

ReportPrintableHeadersBuilder.prototype.addBillableTaskSelectionParamsStringToArray = function( reportInfo, array ) {
    var messageKey = '';
    switch (reportInfo) {
        case BillingSummaryReportInfo.TIME_ON_NON_BILLABLE_TASKS:
            messageKey = "report.billing_summary.non_billable_only";
            break;
        case BillingSummaryReportInfo.TIME_ON_BOTH_BILLABLE_AND_NON_BILLABLE_TASKS:
            messageKey = "report.billing_summary.billable_nonbillable";
            break;
        default:
            messageKey = "report.billing_summary.billable_only";
    }
    array.push( new PrintableHeader().init( messageResource.getEscapedMessage( "common.Tasks" ) + ":", messageResource.getEscapedMessage( messageKey ) ) );
};

ReportPrintableHeadersBuilder.prototype.addBillableReportTypeParamsStringToArray = function( reportType, array ) {
    var label = messageResource.getMessage( "report.billing_summary.report_type_label" );
    var messageKey = '';
    switch (reportType)
    {
        case "ALL_TIME_WITHIN_DATE_RANGE":
            messageKey = "report.billing_summary.time_for_date_range";
            break;
        case "ONLY_FOR_COMPLETED_TASKS":
            messageKey = "report.billing_summary.time_task_completed_for_date_range";
            break;
    }
    array.push( new PrintableHeader().init( label, messageResource.getEscapedMessage( messageKey ) ) );
};

function PrintableHeader()
{
    this.constructor = PrintableHeader;
}

PrintableHeader.prototype.init = function ( label, value )
{
    this.label = label;
    this.value = value;
    return this;
};