function ChartPDFDefinitionBuilder(){}

ChartPDFDefinitionBuilder.prototype.init = function(logoPicture, chartType, paperSize, paperOrientation, grandTotalValue, headerTexts, categoryFormatter, valueFormatter) {
    this.chartType = chartType;

    this.paperSize = paperSize;
    this.paperOrientation = paperOrientation;

    this.allParams = {};
    $.extend(true, this.allParams, ChartPDF.Parameters); // need to prevent changes parameters by pdfmake
    this.orientationParams = this._getOrientationParams(this.allParams, paperOrientation);
    this.paperSizeParams = this._getPaperSizeParams(this.orientationParams, paperSize);
    this.chartTypeParams = this._getChartTypeParams(this.paperSizeParams, chartType);

    this.chartTypeName = headerTexts.chartTypeName;
    this.filterTexts = headerTexts.filterTexts;
    this.dateRangeDescription = headerTexts.dateRangeDescription;
    this.grandTotalDescription = headerTexts.grandTotalDescription;

    this.categoryFormatter = categoryFormatter;
    this.valueFormatter = valueFormatter;

    this.grandTotalValue = grandTotalValue;

    this.logoPicture = logoPicture;
    
    this.templateCtrs = ChartPDF.TemplateConstructors;

    return this;
};

/**
 * Crete new doc definition with current page parameters and with footer
 * @returns {{}}
 */
ChartPDFDefinitionBuilder.prototype.createNewDocDefinitionWithFooter = function(){

    var docDefinition = {};

    docDefinition.pageSize = this.paperSizeParams.pageSize;
    docDefinition.pageOrientation = this.orientationParams.pageOrientation;
    docDefinition.pageMargins = this.orientationParams.pageMargins;

    docDefinition.content = [];
    docDefinition.footer = ChartPDF.Footers.simpleFooter();
    docDefinition.styles = ChartPDF.Styles;

    return docDefinition;
};

/**
 * Method add header to pdf page doc definition
 * @param docDefinition - current doc definition
 * @param showLogo - true if need add logo to page
 */
ChartPDFDefinitionBuilder.prototype.addHeader = function( docDefinition, showLogo) {
    var placeCtrs = ChartPDF.PlaceholderConstructors;

    var filterTextMaxLen = this.paperSizeParams.filterViewTextLen;
    var filterViewElement = this.templateCtrs.filterView(this.orientationParams.filterViewParameters, this._cutLongLabels(this.filterTexts.firstRow, filterTextMaxLen), this._cutLongLabels(this.filterTexts.secondRow, filterTextMaxLen));
    var logoElement = showLogo ? this.templateCtrs.logoPicture(this.logoPicture.imageBase64, this._calculatePDFLogoDimensions(this.logoPicture)) : '';

    var chartInfoParameters = showLogo ? this.paperSizeParams.chartInfoParameters.withLogo : this.paperSizeParams.chartInfoParameters.withoutLogo;

    var chartInfoElement = this.makeChartInformationHeader(chartInfoParameters, logoElement);
    var headerElement;
    var appender = _.bind(this._appendContentElement, this, docDefinition);
    
    if (this.paperOrientation == PaperOrientation.PORTRAIT ) {
        headerElement = placeCtrs.headerPortrait(chartInfoElement);
        appender(headerElement);
        appender(filterViewElement);
    } else {
        headerElement = placeCtrs.headerLandscape(chartInfoElement, filterViewElement);
        appender(headerElement);
    }
};

ChartPDFDefinitionBuilder.prototype.makeChartInformationHeader = function(chartInfoParameters, logoElement)
{
    return this.templateCtrs.chartInformation(chartInfoParameters, logoElement, this.chartTypeName, this.dateRangeDescription,
                                              this.grandTotalDescription, this.valueFormatter(this.grandTotalValue));
};

/**
 * method adds image and data on page depends page parameters
 * @param docDefinition - doc definition of pdf
 * @param imageBase64 - image
 * @param pageDataList - list of data for one page
 * @param showDataList - if need to show data
 * @param showPercentsList - if need show percent values
 * @param fitDataToOnePage - if data list on one page
 */
ChartPDFDefinitionBuilder.prototype.addImageAndData = function(docDefinition, imageBase64, pageDataList, showDataList, showPercentsList, fitDataToOnePage) {
    var placeCtrs = ChartPDF.PlaceholderConstructors;

    var isHorBarAndLand = this.chartType == ChartType.HORIZONTAL_BAR && this.paperOrientation == PaperOrientation.LANDSCAPE; // in this case there are unique parameters for draw image and table in pdf
    var maxValuesOnPage = fitDataToOnePage ? this.chartTypeParams.maxValuesOnFitPage : this.chartTypeParams.maxValuesOnPage;
    var maxRowsInColumn = isHorBarAndLand ? maxValuesOnPage : Math.round(maxValuesOnPage / 2);

    var imageParams = this.chartTypeParams.imageParams.singleImage;
    var tableMargins = this.chartTypeParams.dataTableMargins;
    var tableStyles =  fitDataToOnePage ? this.allParams.dataTableCommonStyles.small : this.allParams.dataTableCommonStyles.big;
    var rowDataMaxLen = fitDataToOnePage ? this.chartTypeParams.rowDataMaxLen.fit  : this.chartTypeParams.rowDataMaxLen.normal;

    var image = this.templateCtrs.imageWithSizes(imageParams.sizes, imageBase64);

    var rows = this._getDataRows(pageDataList, showDataList, showPercentsList, maxRowsInColumn, tableStyles.tableValueStyles, rowDataMaxLen);

    if (isHorBarAndLand){
        this._appendContentElement(docDefinition, placeCtrs.horizontalBarsChartImageAndData(image, rows, imageParams.margins, tableMargins, tableStyles.tableStyle));
    } else {
        var table = placeCtrs.chartDoubleColumnsTable(rows, tableMargins, tableStyles.tableStyle);
        this._appendContentElement(docDefinition, placeCtrs.singleImage(image, imageParams.margins));
        this._appendContentElement(docDefinition, table);
    }
};

/**
 * Method add double image on one page depends of parameters
 * @param docDefinition - current doc definition
 * @param firstImageBase64 - firstImage
 * @param secondImageBase64 - secondImage
 */
ChartPDFDefinitionBuilder.prototype.addDoubleImages = function(docDefinition, firstImageBase64, secondImageBase64) {
    var imageParams = this.chartTypeParams.imageParams.doubleImage;
    var placeCtrs = ChartPDF.PlaceholderConstructors;

    var imageFirstElement = this.templateCtrs.imageWithSizes(imageParams.sizes, firstImageBase64);
    var imageSecondElement;
    if (secondImageBase64){
        imageSecondElement = this.templateCtrs.imageWithSizes(imageParams.sizes, secondImageBase64);
    } else {
        imageSecondElement = this.templateCtrs.fakeImage();
    }

    if(this.chartType == ChartType.HORIZONTAL_BAR && this.paperOrientation == PaperOrientation.LANDSCAPE){
        this._appendContentElement(docDefinition, placeCtrs.horizontalBarsChartDoubleImage(imageFirstElement, imageSecondElement, imageParams.margins));
    } else {
        this._appendContentElement(docDefinition, placeCtrs.singleImage(imageFirstElement, imageParams.margins));
        this._appendContentElement(docDefinition, placeCtrs.singleImage(imageSecondElement, imageParams.margins));
    }
};

/**
 * Add only table for page
 * @param docDefinition - current doc definition
 * @param pageDataList - list of data for current page
 * @param showDataList - if need to show data
 * @param showPercentsList - if need to show percents
 * @param showHeaderOnEachPage - nValues depends on header
 */
ChartPDFDefinitionBuilder.prototype.addOnlyTableToPage = function(docDefinition, pageDataList, showDataList, showPercentsList, showHeaderOnEachPage) {
    var maxValuesOnPage = showHeaderOnEachPage ?
        this.chartTypeParams.maxValuesOnPageWithoutImage.withHeader : this.chartTypeParams.maxValuesOnPageWithoutImage.withoutHeader;
    var maxRowsInColumn = Math.round(maxValuesOnPage / 2);

    var tableStyles = this.allParams.dataTableCommonStyles.big; // always big table
    var rowDataMaxLens = this.chartTypeParams.rowDataMaxLen.normal; // always big table (normal rows)

    var rows = this._getDataRows(pageDataList, showDataList, showPercentsList, maxRowsInColumn, tableStyles.tableValueStyles, rowDataMaxLens);

    var table = ChartPDF.PlaceholderConstructors.chartDoubleColumnsTable(rows, this.chartTypeParams.dataTableMargins, tableStyles.tableStyle);
    this._appendContentElement(docDefinition, table);
};

/**
 * Add only image to page
 * @param docDefinition - current doc definition
 * @param imageBase64 - image in base64
 */
ChartPDFDefinitionBuilder.prototype.addOnlyImageToPage = function(docDefinition, imageBase64) {
    var imageParams = this.chartTypeParams.imageParams.singleImage;
    var image = this.templateCtrs.imageWithSizes(imageParams.sizes, imageBase64);
    this._appendContentElement(docDefinition, ChartPDF.PlaceholderConstructors.singleImage(image, imageParams.margins));
};

/**
 * Method create rows for table.
 * @param pageDataList - list of data for one page
 * @param showDataList - if need show data
 * @param showPercentsList  - if need show percents
 * @param maxRowsInColumn - max rows in one column
 * @param tableValueStyles - styles for table values
 * @param rowDataMaxLens - max len of data in row of each cell
 * @returns {Array} - array of rows elements
 * @private
 */
ChartPDFDefinitionBuilder.prototype._getDataRows = function(pageDataList, showDataList, showPercentsList, maxRowsInColumn, tableValueStyles, rowDataMaxLens){
    var rows = [];
    for ( var i = 0 ; i < maxRowsInColumn && i < pageDataList.length ; i++ ) {
        var isSecond = i  % 2 != 0; //!counter starts with 0
        if (! $.isEmptyObject(pageDataList[i].category)) {

            var leftCell = this._getDataCell(pageDataList[i], showDataList, showPercentsList, tableValueStyles, rowDataMaxLens);

            if (pageDataList[i + maxRowsInColumn] && !$.isEmptyObject(pageDataList[i + maxRowsInColumn].category)){
                var rightCell = this._getDataCell(pageDataList[i + maxRowsInColumn], showDataList, showPercentsList, tableValueStyles, rowDataMaxLens);
                rows.push(this.templateCtrs.doubleColumnsRow(leftCell, rightCell, isSecond));
            } else {
                rows.push(this.templateCtrs.singleColumnRow(leftCell, isSecond));
            }
        }
    }
    return rows;
};

/**
 * Create data cell element depends parameters or chartType
 * If chartType is pie, than cell will contains color rectangle
 * @param cellValue - object of value for one cell
 * @param showDataList - if need show values
 * @param showPercentsList - if need show percents
 * @param tableValueStyles - styles for table cell
 * @param rowDataMaxLens - max len in chars of value string (name + value)
 * @returns {*}
 * @private
 */
ChartPDFDefinitionBuilder.prototype._getDataCell = function(cellValue, showDataList, showPercentsList, tableValueStyles, rowDataMaxLens) {
    var isValueAndPercentsCell = showDataList && showPercentsList;
    var maxDataRowLen = isValueAndPercentsCell ? rowDataMaxLens.valueAndPercents : rowDataMaxLens.valueOrPercents; // get max len for current parameters

    var formattedValue = this.valueFormatter(cellValue.value);
    var formattedPercent = this._getPercentValue(cellValue.value);

    maxDataRowLen -= showDataList ? (formattedValue.length - 3) : formattedPercent.length; // substrate len of value without 3 chars (:dd or .dd) or percent length
    var formattedCategory = this.categoryFormatter(cellValue.category, maxDataRowLen);

    if (isValueAndPercentsCell) {
        return this.chartType == ChartType.PIE_CHART ?
               this.templateCtrs.colorValuePercentCell(cellValue.color ? cellValue.color : ' ', formattedCategory, formattedValue, formattedPercent, tableValueStyles) :
               this.templateCtrs.valuePercentCell( formattedCategory, formattedValue, formattedPercent, tableValueStyles) ;
    } else {
        var cellData = showDataList ? formattedValue : formattedPercent; // percent or value
        return this.chartType == ChartType.PIE_CHART ?
               this.templateCtrs.colorValueCell(cellValue.color ? cellValue.color : ' ', formattedCategory, cellData, tableValueStyles) :
               this.templateCtrs.valueCell( formattedCategory, cellData, tableValueStyles);
    }
};

ChartPDFDefinitionBuilder.prototype._getViewTTTableHeaderRow = function(cellValue, tableValueStyles) 
{
    return this.templateCtrs.headerCell(cellValue, tableValueStyles);
};
/**
 * Add to doc definition page breaker
 * @param docDefinition - current doc definition
 */
ChartPDFDefinitionBuilder.prototype.addPageBreaker = function(docDefinition){
    this._appendContentElement(docDefinition, ChartPDF.PlaceholderConstructors.pageBreaker());
};

/**
 * Append element to content list in doc definition
 * @param dd - doc def
 * @param element - element fo add
 * @private
 */
ChartPDFDefinitionBuilder.prototype._appendContentElement = function(dd, element) {
    dd.content.push(element);
};

/**
 * returns params depends on page orientation
 * @param allParameters - bundle of all parameters
 * @param paperOrientation - current paper orientation
 * @returns {*}
 * @private
 */
ChartPDFDefinitionBuilder.prototype._getOrientationParams = function(allParameters, paperOrientation) {
    return paperOrientation == PaperOrientation.PORTRAIT ? allParameters.Portrait : allParameters.Landscape;
};
/**
 * returns params depends on paper size
 * @param orientationParams - bundle of params depends on page orientation
 * @param paperSize - current paper size
 * @returns {*}
 * @private
 */
ChartPDFDefinitionBuilder.prototype._getPaperSizeParams = function(orientationParams, paperSize) {
    return paperSize == PaperSize.A4 ? orientationParams.A4 : orientationParams.Letter;
};

/**
 * returns params depends on chart type
 * @param paperSizeParams - bundle of params depends on paper size
 * @param chartType - current chart type
 * @returns {*}
 * @private
 */
ChartPDFDefinitionBuilder.prototype._getChartTypeParams = function(paperSizeParams, chartType) {
    if (chartType == ChartType.VERTICAL_BAR) {
        return paperSizeParams.verticalBar;
    } else if( chartType == ChartType.HORIZONTAL_BAR ){
        return paperSizeParams.horizontalBar;
    } else {
        return paperSizeParams.pie;
    }
};

/**
 * Method returns percent of grand total value
 * @param value - current value
 * @return string with percents
 * @private
 */
ChartPDFDefinitionBuilder.prototype._getPercentValue = function(value){
    var percents = this.grandTotalValue == 0 ? 0 : (100 * value / this.grandTotalValue) ;
    percents = percents.toFixed(2);
    percents = percents.replace('.', SystemSettings.decimalSeparator);
    return percents + " %";
};

ChartPDFDefinitionBuilder.prototype._cutLongLabels = function(label, maxLen) {
    if (label.length > maxLen){
        return label.substr(0, maxLen - 3) + '...';
    }
    return label;
};

/**
 * method calculate logo image dimensions for fit it to placeholder sizes and save ration
 * @param logoPicture logo picture (logo + sizes in pixels)
 * @returns {{w: number, h: number, mL: number, mT: number}}
 * @private
 */
ChartPDFDefinitionBuilder.prototype._calculatePDFLogoDimensions = function(logoPicture) {

    var maxW = ChartPDF.Parameters.logoPlaceholderSizes.width;
    var maxH = ChartPDF.Parameters.logoPlaceholderSizes.height;

    var widthInPix = logoPicture.width;
    var heightInPix = logoPicture.height;

    var ratioForPlace = maxW/maxH;
    var rationForPixSizes = widthInPix/ heightInPix;

    var coeff = 0; // coeff for calculate pdf logo dimensions
    // if ration for placeholder(w/h) less than for image, then need to adjust max height of image or width otherwise
    if (ratioForPlace > rationForPixSizes){
        coeff = heightInPix / maxH;
    } else {
        coeff = widthInPix / maxW;
    }
    var wInPDF = widthInPix / coeff;
    var hInPDF = heightInPix / coeff;
    return {
        w: wInPDF,
        h: hInPDF,
        mL: (maxW - wInPDF) / 2,
        mT: 0
    }
};