function ChartPDFGenerator(){}

ChartPDFGenerator.prototype.init = function(onGenerateCb)
{
    this.PDFCustomLogoURL = PDFCustomLogoURL;
    this.onGeneratePdfCallback = onGenerateCb;
    
    this.downloadedLogo = { // logo downloaded with first generation
        isLoaded: false,
        imageBase64: null,
        width: 0,
        height: 0
    };

    this.lastGeneratedPDF = { // save last generated pdf for prevent new generate on download
        encodedPDF: null,
        fileName: 'file.pdf'
    };

    this.pdfFormatter = new PDFFormatterCreator();
    this.onGenerateBreaker = {needBreak: false};
    
    return this;
};

/**
 * We cant interrupt pdf generated process, but we can interrupt on generate process
 */
ChartPDFGenerator.prototype.interruptGeneration = function()
{
    this.onGenerateBreaker.needBreak = true;
};

ChartPDFGenerator.prototype.downloadPDF = function()
{
    new PDFGeneratorUtil().downloadPDF(this.lastGeneratedPDF.encodedPDF, this.lastGeneratedPDF.fileName);
};

/**
 * Get chart type dependent strategy for create doc definition builder
 * @param chartType - pie, vertical/horizontal bars
 * @returns {*} strategy
 * @private
 */
ChartPDFGenerator.prototype._getCreatePDFDefinitionStrategy = function(chartType)
{
    if (ChartType.PIE_CHART == chartType)
    {
        return new PieChartPDFCreateDefinitionObjectStrategy();
    }
    else
    {
        return new HorVertBarsChartPDFCreateDefinitionObjectStrategy();
    }
};

ChartPDFGenerator.prototype._getParametersFormattingStrategy = function(chartTypeValue, selectorsData, dateFormat){
    var dateFormatForFileName = new DateFormat('yyyy-MM-dd');
    switch (chartTypeValue){
        case ReportChartType.TOTAL_WORKING_HOURS:
        case ReportChartType.APPROVED_WORKING_HOURS:
        case ReportChartType.COST_OF_WORK:
        case ReportChartType.BILLABLE_AMOUNTS:
            return (new ChartPDFWorkingHoursCostBillingParametersFormattingStrategy()).init(selectorsData, dateFormat, dateFormatForFileName);
        case ReportChartType.LEAVE_HOURS:
            return (new ChartPDFLeaveParametersFormattingStrategy()).init(selectorsData, dateFormat, dateFormatForFileName);
        case ReportChartType.PTO_BALANCE:
        case ReportChartType.SICK_DAYS_BALANCE:
            return (new ChartPDFBalanceParametersFormattingStrategy()).init(selectorsData, dateFormat, dateFormatForFileName);
        case ReportChartType.OVERTIME_HOURS:
            return (new ChartPDFOvertimeParametersFormattingStrategy()).init(selectorsData, dateFormat, dateFormatForFileName);
    }
};

ChartPDFGenerator.prototype.getNeededChartDimensionsInPixels = function (charType, pdfParameters) {
    var needToShowTable = pdfParameters.showListOfDataValues || pdfParameters.showListOfPercentValues;
    var isDoubleImages = charType != ChartType.PIE_CHART && !needToShowTable;
    var pdfImageSizes =  ChartPDF.getNeededImageDimensions(
        pdfParameters.paperOrientation,
        pdfParameters.paperSize,
        charType,
        isDoubleImages
    );

    return {
        width: pdfImageSizes.w * 2,
        height: pdfImageSizes.h * 2
    }
};

/**
 * Calculate total number of pages in dependence on number of values, chart type, paper orientation, paper size, selected number of bars on page
 * if data fit to one page than return 1
 * @param lengthOfDataList - number of values
 * @param chartType - type of char: pie, hor. bars, vert. bars
 * @param pdfPrintParameters - paper size, paper orientation, bars on page, show header on each page, show data list
 * @returns {number}
 */
ChartPDFGenerator.prototype.calculateTotalNumberOfPages = function(lengthOfDataList, chartType, pdfPrintParameters) {
    var nPages = 0;
    if(pdfPrintParameters.fitDataToOnePage){
        return 1;
    }
    var nValues = lengthOfDataList;
    var needToShowTable = pdfPrintParameters.showListOfDataValues || pdfPrintParameters.showListOfPercentValues;

    if(ChartType.PIE_CHART == chartType){
        if (needToShowTable){
            var maxValuesOnPage = ChartPDF.getMaxValuesOnPage(PaperOrientation.PORTRAIT, pdfPrintParameters.paperSize, ChartType.PIE_CHART);
            var maxValuesOnPageWithoutImage = ChartPDF.getMaxValuesOnPageWithoutImage(PaperOrientation.PORTRAIT, pdfPrintParameters.paperSize, pdfPrintParameters.showHeaderOnEachPage, ChartType.PIE_CHART);
            if(nValues >= maxValuesOnPage){
                nPages++;
                nValues-=maxValuesOnPage;
            }
            if (nValues > 0){
                nPages+= Math.ceil(nValues/maxValuesOnPageWithoutImage);
            }
        } else {
            nPages = 1;
        }
    } else {
        nPages = needToShowTable ?
            Math.ceil(nValues/pdfPrintParameters.barsOnPage) : Math.ceil(nValues/(2 * pdfPrintParameters.barsOnPage));

    }
    return nPages;
};

/**
 * download logo from server to browser only once (lazy download)
 * @param logoUrl - url for logo
 * @param onLoadImage - callback onload image
 * @private
 */
ChartPDFGenerator.prototype._downloadLogoBase64 = function(logoUrl, onLoadImage)
{
    var image = new Image();
    image.onload = function()
    {
        var canvas = document.createElement('canvas');
        canvas.width = this.naturalWidth; // or 'width' if you want a special/scaled size
        canvas.height = this.naturalHeight; // or 'height' if you want a special/scaled size

        var context = canvas.getContext('2d');
        context.drawImage(this, 0, 0);
        // code below needs for set white background for transparent images
        context.globalCompositeOperation = "destination-over";
        context.fillStyle = "#FFFFFF";
        context.fillRect(0, 0, this.naturalWidth, this.naturalHeight);

        onLoadImage(canvas.toDataURL('image/jpeg'), this.naturalWidth, this.naturalHeight);
    };

    image.src = logoUrl;
};

ChartPDFGenerator.prototype.generatePDF = function(config)
{
    if (this.downloadedLogo.isLoaded)
    {
        config.logo = this.downloadedLogo;
        this._generatePDF(config)
    }
    else
    {
        // if logo not loaded wait while images needed to pdf will be loaded
        var self = this;
        this._downloadLogoBase64(
                self.PDFCustomLogoURL,
                function(logoBase64, width, height)
                {
                    self.downloadedLogo = {
                        isLoaded: true,
                        imageBase64: logoBase64,
                        width: width,
                        height: height
                    };

                    config.logo = self.downloadedLogo;
                    self._generatePDF(config);
                }
        )
    }
};

ChartPDFGenerator.prototype.createPDFBuilder = function(config, paramFormattingStrategy)
{
    var selectedParameters = config.selectedParameters;

    return new ChartPDFDefinitionBuilder().init(
            config.logo,
            config.chartType,
            config.pdfPrintParameters.paperSize,
            config.pdfPrintParameters.paperOrientation,
            config.totalValue,
            {
                chartTypeName: paramFormattingStrategy.formatChartPDFHeaderTitle(selectedParameters),
                dateRangeDescription: paramFormattingStrategy.formatChartPDFHeaderDateInformation(selectedParameters),
                filterTexts: paramFormattingStrategy.formatChartPDFFilterInformation(selectedParameters),
                grandTotalDescription: paramFormattingStrategy.formatGrandTotalDescription(selectedParameters)
            },
            this.pdfFormatter.getCategoryToStringFunction(selectedParameters.groupingLevelValue, config.chartType),
            this.pdfFormatter.getValueFormattedFunction(selectedParameters.chartTypeValue)
    );
};

ChartPDFGenerator.prototype._generatePDF = function(config)
{
    var newBreaker = {
        needBreak: false
    };
    this.onGenerateBreaker = newBreaker;

    var selectedParameters = config.selectedParameters;
    var paramFormattingStrategy = this._getParametersFormattingStrategy(selectedParameters.chartTypeValue, config.selectorsData, config.dateFormat);
    var ddBuilder = this.createPDFBuilder(config, paramFormattingStrategy);
    var createDDStrategy = this._getCreatePDFDefinitionStrategy(config.chartType);

    pdfMake.fonts = PDFFonts;
    var doc = pdfMake.createPdf(createDDStrategy.createDocDefinitionObject(ddBuilder, config.pagesData, config.pdfPrintParameters));

    var currentFileName = paramFormattingStrategy.formatChartPDFFileName(selectedParameters);
    var self = this;
    try
    {
        doc._createDoc(
                ChartPDF.Options,
                function(result)
                {
                    if (newBreaker.needBreak)
                    {
                        return;
                    }

                    self.lastGeneratedPDF = { // save last generated pdf for prevent new generate on download
                        encodedPDF: result,
                        fileName: currentFileName
                    };

                    if (self.onGeneratePdfCallback)
                    {
                        self.onGeneratePdfCallback(result);
                    }
                }
        );

    }
    catch (e)
    {
        throw e;
    }
};
