/**
 * Validator of support form.
 * @param messageManager support form message manager.
 */
function SupportFormValidator(messageManager)
{
    this.fields = [];  // All registered fields
    this.emailField = null;     // Registered email field
    this.textArea = null;       // Registered description text area
    this.messageManager = messageManager;   // Support form message manager
}

SupportFormValidator.prototype = _.create(Object.prototype, {
    maxTextAreaLength: 32000,

    /**
     * Adds field to this validator object
     * @param field field DOM object
     */
    addField: function(field)
    {
        this.fields.push(field);
    },

    /**
     * Adds email field to this validator object
     * @param emailField email field DOM object
     */
    addEmailField: function(emailField)
    {
        this.emailField = emailField;
        this.addField(emailField);
    },

    /**
     * Adds text area to this validator object
     * @param textArea text area DOM object
     */
    addTextArea: function(textArea)
    {

        this.textArea = textArea;
        this.addField(textArea);
    },

    /**
     * Adds error indicator to field. Currently highlights its title with error color.
     * @param field field containing error
     */
    addErrorIndicator: function(field)
    {
        // Each field has title id of which equals to field id + "Title" string
        $(field).addClass("error");
    },

    /**
     * Removes error indicator from field.
     * @param field to remove error indicator from.
     */
    removeErrorIndicator: function(field)
    {
        // Each field has title id of which equals to field id + "Title" string
        $(field).removeClass("error");
    },

    /**
     * Creates error count object
     * @param count number of errors to initialize object with
     * @return error count object initialized with specified number of errors
     */
    createErrorCountObject: function(count)
    {
        // Create object and initialize it with parameter value
        var errCount = new Object();
        errCount.value = count;
        errCount.getErrorCount = function() {return this.value;}
        // Add methods to increment and decrement error count value
        errCount.incrementErrorCount = function() {this.value++};
        errCount.decrementErrorCount = function() {this.value--};

        return errCount;
    },

    /**
     * Mark field as containing error. Adds error indicator to it also
     * adds value change handler to field. The handler is intended to remove error highlighting from the field
     * and also to remove error message in case there are no any other fields with the same error.
     *
     * @param field field to add handler to
     * @param errCount number of fields having the same error
     */
    markErrorField: function(field, errCount)
    {
        // Add error indicator to field
        this.addErrorIndicator(field);
        var messageManager = this.messageManager; // Dereference message manager

        // Define value change event handler
        field.valueChangeHandler = function()
        {

            errCount.decrementErrorCount(); // Decrement error count

            // If there are no field with empty value then hide error message
            if (errCount.getErrorCount() <= 0)
            {

                messageManager.hideErrorMessage();
            }

            // Remove error indicator and this handler from this field
            SupportFormValidator.prototype.removeErrorIndicator(field);
            SupportFormValidator.prototype.removeValueChangeHandler(field, field.valueChangeHandler);
            field.valueChangeHandler = null;    // Also remove this method from object
        };

        // Add value change handler to field
        this.addValueChangeHandler(field, field.valueChangeHandler);
    },

    /**
     * Validates that all fields contain non-empty values. If there are such fields then corresponding
     * field title is highlighted with error color and field is assigned to "value change" handler that
     * will remove highliting once field's value is changed later
     *
     * @return true if all form fields contain non-empty value, false if at least one of the fields has empty value
     */
    validateNotEmpty: function()
    {
        // Create object that will hold total number of error fields
        var errCount = this.createErrorCountObject(0);

        // For each field
        for (var i = 0; i < this.fields.length; i++)
        {

            var field = this.fields[i];
            var value;

            if (field == this.textArea) // Text area has no placeholder
            {
                value = trim(field.value);
                field.value = value;
            }
            else
            {
                value = trim($(field).pVal());
                $(field).pVal(value);
            }

            // If value is empty string
            if (value == "")
            {

                errCount.incrementErrorCount(); // Increment error field count
                this.markErrorField(field, errCount); // Mark field as containing error
            }
        }

        // If there are no error true value is returned, false otherwise
        return errCount.getErrorCount() == 0;
    },

    /**
     *  Validates email address. If email address is not valid adds error indicator to email address field and
     *  also adds handler that will remove the error indicator and error message once value of email value is changed
     *  @return true if email address has valid value, false otherwise
     */
    validateEmailAddress: function()
    {
        if (this.emailField == null)
        {
            return true;
        }

        // If email field is not valid
        if (!isEmailValid($(this.emailField).pVal()))
        {

            // Mark field as containing error and return false to indicate error
            this.markErrorField(this.emailField, this.createErrorCountObject(1));
            return false;
        }

        return true; // Return true if email address is valid
    },

    /**
     * Validates description text length
     * @return true if text lentgh does not exceed maximum length value, false otherwise
     */
    validateTextLength: function()
    {
        // Throw exception if text are is not registered with form validator
        if (this.textArea == null)
        {
            throw "Description text area is not registered!";
        }

        // Get text length
        var length = trim(getStringWithUnifiedNewLineSymbol(this.textArea.value)).length;

        // If text exceeds maximum allowed text length
        if (length > this.maxTextAreaLength)
        {

            // Mark field as containing error
            this.markErrorField(this.textArea, this.createErrorCountObject(1));
            return false;  // Return false to indicate error
        }

        return true; // Return true if text length is below or equal to maximum allowed value
    },

    /**
     * Resets form validator
     */
    resetValidator: function()
    {

        // Hide client side error message
        this.messageManager.hideErrorMessage();

        // For each field
        for (var i = 0; i < this.fields.length; i++)
        {

            var field = this.fields[i];

            this.removeErrorIndicator(field);   // Remove error indicator

            // If value change handler has not been removed remove it
            if (field.valueChangeHandler)
            {

                // Remove value change handler from field
                this.removeValueChangeHandler(field, field.valueChangeHandler);
                field.valueChangeHandler = null;    // Also remove this method from object
            }
        }
    },

    /**
     * Validates form
     * @return true if form does not contain error, false otherwise
     */
    validateForm: function()
    {

        // Reset validator object
        this.resetValidator();

        // Validate that form does not contain empty values for registered fields
        if (!this.validateNotEmpty())
        {

            this.messageManager.showEmptyFieldMsg()
            return false;
        }

        // Validate that form contains valid email address
        else if (!this.validateEmailAddress())
        {

            this.messageManager.showInvalidEmailMsg();
            return false;
        }

        // Valid that form contains description text of allowed length
        else if (!this.validateTextLength())
        {

            this.messageManager.showTooLongDescMsg(this.maxTextAreaLength);
            return false;
        }

        return true; // Return true if there are no errors
    },

    /**
     * Adds value change evenet handler to field
     * @field field to add handler to
     * @handler handler to add
     */
    addValueChangeHandler: function(field, handler)
    {
        // If FireFox
        if (field.addEventListener)
        {
            // Because of bug in Mozilla (Bugzilla Bug# 195696) can not assign directly must use addEventListener
            field.addEventListener("input", handler, false);
        }
        else
        {
            // Otherwise IE
            field.onpropertychange = function()
            {
                // Check that "value" property changed
                if (window.event.propertyName == 'value')
                {
                    handler();
                }
            }
        }
    },

    /**
     * Removes value change event handler from field
     * @param field to remove handler from
     * @param handler handler to remove
     */
    removeValueChangeHandler: function(field, handler)
    {
        // If FireFox
        if (field.removeEventListener)
        {
            field.removeEventListener("input", handler, false);
        }
        else
        {
            // Otherwise IE
            field.onpropertychange = null;
        }
    }
});
