function Table(tHeadEl, containerEl)
{
    this.containerEl = containerEl;
    this.rows = [];
    this.tableEl = document.createElement("table");
    this.tableEl.cellPadding = 0;
    this.tableEl.cellSpacing = 0;
    this.tableEl.border = 0;
    this.tbody = document.createElement("tBody");

    // TODO: too much stuff happening in the constructor
    
    if (tHeadEl)
        this.tableEl.appendChild(tHeadEl);
    this.tableEl.appendChild(this.tbody);

    //noinspection SpellCheckingInspection
    var thisDeref = this;

    this.selectionController = this.createSelectionController();
    if (this.selectionController != null)
        this.selectionController.addSelectionChangeListener(function(selectionChangeEvent, mouseEvent)
                                                            {
                                                                thisDeref.selectionChanged(selectionChangeEvent, mouseEvent);
                                                            });

    this.hoverController = this.createHoverController();
    if (this.hoverController != null)
        this.hoverController.addHoverChangeListener(function(hoverChangeEvent, mouseEvent)
                                                    {
                                                        thisDeref.hoverChanged(hoverChangeEvent, mouseEvent);
                                                    });

    this.mouseEventsHandler = this.createMouseEventsHandler();
}

Table.prototype = {
    // abstract
    createSelectionController: function()
    {
        return new SelectionController();
    },

    // abstract
    createHoverController: function()
    {
        return new HoverController();
    },

    // abstract
    createMouseEventsHandler: function()
    {
        return new TableMouseEventsHandler(this, this.getSelectionController(), this.getHoverController());
    },

    getTBody: function()
    {
        return this.tbody;
    },


    getSelectionController: function()
    {
        return this.selectionController;
    },

    getHoverController: function()
    {
        return this.hoverController;
    },

    getMouseEventsHandler: function()
    {
        return this.mouseEventsHandler;
    },

    setupRow: function(rowIdx, row)
    {
        row.setRowIdx(rowIdx);
        row.setTable(this);
    },

    addRow: function(row)
    {
        this.setupRow(this.rows.length, row);
        this.rows.push(row);
    },

    replaceRowAndUpdateDOM: function(rowIdx, newRow, updateRowBelow)
    {
        var oldRow = this.rows[rowIdx];

        this.setupRow(rowIdx, newRow);
        this.rows[rowIdx] = newRow;

        var tmpTableEl = document.createElement("table");
        var tmpTBodyEl = document.createElement("tbody");
        tmpTableEl.appendChild(tmpTBodyEl);

        newRow.buildDOM(tmpTBodyEl);
        newRow.writeCellsContent();
        newRow.getRowEl().className = this.getRowElClasses(rowIdx).join(" ");

        oldRow.getRowEl().parentNode.replaceChild(newRow.getRowEl(), oldRow.getRowEl());

        if (updateRowBelow)
        {
            var belowIdx = rowIdx + 1;
            if (belowIdx <= this.getLastRow().getRowIdx())
            {
                var belowRow = this.getRow(belowIdx);
                this.updateCellRegionUI(new TableRegion(belowIdx, 0, belowIdx, belowRow.getCells().length - 1));
            }
        }
    },

    getRows: function(row)
    {
        return this.rows;
    },

    getRow: function(rowIdx)
    {
        return this.rows[rowIdx];
    },

    getLastRow: function()
    {
        return this.rows[this.rows.length - 1];
    },

    getFirstRow: function()
    {
        return this.rows[0];
    },

    getCell: function(rowIdx, colIdx)
    {
        return this.rows[rowIdx].getCell(colIdx);
    },

    updateCell: function(cell)
    {
        cell.updateCell();
    },

    buildModelRowsDOM: function(containerEl)
    {
        var aRow = null;
        for (var i = 0; i < this.rows.length; i++)
        {
            aRow = this.rows[i];
            aRow.buildDOM(containerEl);
            aRow.writeCellsContent();
            aRow.getRowEl().className = this.getRowElClasses(i).join(" ");
        }
    },

    deleteAllRowsFromModelAndDom: function()
    {
        for (var i = 0; i < this.rows.length; i++)
            this.tbody.removeChild(this.rows[i].getRowEl());
        this.rows = [];
    },

    removeRow: function(rowIdx)
    {
        var row = this.rows[rowIdx];
        this.tbody.removeChild(row.getRowEl());
        this.rows.splice(rowIdx, 1);
        for (var i = rowIdx; i < this.rows.length; i++)
            this.setupRow(i, this.rows[i]);
    },


    getTableEl: function()
    {
        return this.tableEl;
    },

    buildDOM: function(containerEl)
    {
        this.buildModelRowsDOM(this.tbody);

        this.disableTextSelection();

        containerEl.appendChild(this.tableEl);
    },

    disableTextSelection: function()
    {
        disableTextSelection(this.tableEl);
    },

    getBaseRowClasses: function(rowIdx)
    {
        return [];
    },

    getRowElClasses: function(rowIdx)
    {
        var rowClasses = this.getBaseRowClasses(rowIdx);
        this.appendSelectedRowClassIfNecessary(rowIdx, rowClasses);
        return rowClasses;
    },

    appendSelectedRowClassIfNecessary: function(rowIdx, rowClasses)
    {
        if (this.getSelectionController().getSelection().containsRow(rowIdx))
            rowClasses.push("hasSelection");
    },

    updateCellRegionUI: function(region)
    {
        // update region
        var row = null;
        for (var rowIdx = region.rowMin; rowIdx <= region.rowMax; rowIdx++)
        {
            row = this.rows[rowIdx];
            row.getRowEl().className = this.getRowElClasses(rowIdx).join(" ");
            for (var colIdx = region.colMin; colIdx <= region.colMax; colIdx++)
                row.updateCellUI(colIdx);
        }
    },

    computeChangedRegion: function(regionChangeEvent)
    {
        return regionChangeEvent.oldValue.union(regionChangeEvent.newValue);
    },

    selectionChanged: function(selectionChangeEvent)
    {
        var updateRegion = this.computeChangedRegion(selectionChangeEvent);
        this.updateSelectionChangeRegionUI(updateRegion);
    },

    updateSelectionChangeRegionUI: function(updateRegion)
    {
        this.updateCellRegionUI(
                // +1 col and +1 row to update right and bottom borders of the selection region
                new TableRegion(
                        updateRegion.rowMin,
                        updateRegion.colMin,
                        Math.min(updateRegion.rowMax + 1, this.getLastRow().getRowIdx()),
                        Math.min(updateRegion.colMax + 1, this.getRow(updateRegion.rowMax).getCells().length - 1)
                )
        );
    },


    hoverChanged: function(hoverChangeEvent)
    {
        var updateRegion = this.computeChangedRegion(hoverChangeEvent);
        if (updateRegion != null)
            this.updateCellRegionUI(updateRegion);
    },

    clearSelection: function()
    {
        this.getSelectionController().clearSelection();
    },

    clearHover: function()
    {
        this.getHoverController().unhover();
    }
};
