/**
 * Adds page-specific tips, refreshes them when needed, enables/disables them.
 * @constructor
 */
function PageOverlayTipsManager()
{
    this._dynamicTips = [];
    this._deferredUntilEnabled = [];
    this._refreshDynamicTipsDeferred = false;
    this._enabledOnce = false;
    this._tipsKit = new OverlayTipsKit();
}

/**
 * Gets instance of overlay tips manager for current page.
 * @returns {PageOverlayTipsManager}
 */
PageOverlayTipsManager.getInstance = function()
{
    if (!PageOverlayTipsManager.instance) PageOverlayTipsManager.setInstance(new PageOverlayTipsManager());
    return PageOverlayTipsManager.instance;
};

/**
 * Sets instance of overlay tips manager for current page.
 * @param instance New instance
 */
PageOverlayTipsManager.setInstance = function(instance)
{
    if (PageOverlayTipsManager.instance !== instance)
    {
        var oldManager = PageOverlayTipsManager.instance;
        if (oldManager)
        {
            instance.enabled = oldManager.enabled;
            if (oldManager.enabled)
            {
                oldManager._tipsKit.hideAll();
                instance.enable();
            }
            else
            {
                instance._scheduleQueueProcessing();
            }
        }

        PageOverlayTipsManager.instance = instance;
    }
};

/**
 * Forgets tips enabled status from cookies.
 */
PageOverlayTipsManager.disableCookie = function()
{
    $.cookie("at-tips-enabled", null, {path: "/", expires: 365});
};

/**
 * Remembers tips enabled status in cookies.
 */
PageOverlayTipsManager.enableCookie = function()
{
    $.cookie("at-tips-enabled", true, {path: "/", expires: 365});
};

/**
 * Remembers tips first time status in cookies.
 */
PageOverlayTipsManager.enableFirstTimeCookie = function()
{
    $.cookie("at-tips-first-time", true, {path: "/", expires: 365});
};

/**
 * Remembers tips first time status in cookies.
 */
PageOverlayTipsManager.disableFirstTimeCookie = function()
{
    $.cookie("at-tips-first-time", null, {path: "/", expires: 365});
};

/**
 * Remembers a state, which prevents tips from showing immediately.
 */
PageOverlayTipsManager.enableShowLater = function()
{
    PageOverlayTipsManager._showLater = true;
};

PageOverlayTipsManager.showFirstTimeTips = function()
{
    PageOverlayTipsManager.getInstance()._tipsKit.showAll();
};

PageOverlayTipsManager.disableMenuIcon = function()
{
    $(".popup_menu_button_tips").children(".popup_menu_icon").removeClass("tips_icon_light_on").addClass("tips_icon");
};

PageOverlayTipsManager.enableMenuIcon = function()
{
    $(".popup_menu_button_tips").children('.popup_menu_icon').removeClass("tips_icon").addClass("tips_icon_light_on");
};

PageOverlayTipsManager.prototype = _.create(Object.prototype, {
    /**
     * Initializes tips manager, auto-enabling them if tips are pre-enabled in cookies.
     */
    init: function()
    {
        if ($.cookie("at-tips-enabled"))
            this.enable();
        return this;
    },

    /**
     * Base function, which adds initial tips, usually on page load.
     */
    addBaseTips: function()
    {
        if ($.cookie("at-tips-first-time"))
        {
            this._firstTimeTip = this.add($(".popup_menu_button_tips"), {
                messageKey: "tips.toggle_tips",
                direction: OverlayTips.DIRECTION_BOTTOM_LEFT,
                offsetX: -20,
                offsetY: 10,
                extraClass: "toggleTipsTipZindex"
            });

            if (!PageOverlayTipsManager._showLater)
                PageOverlayTipsManager.disableFirstTimeCookie();
        }
    },

    /**
     * Toggles tips enabled status.
     */
    toggle: function()
    {
        if (this.enabled)
            this.disable();
        else
            this.enable();
    },

    /**
     * Disables all tips on the page, hiding them,
     * and updates navigation tip button status.
     */
    disable: function()
    {
        this.enabled = false;
        PageOverlayTipsManager.disableCookie();

        if (this._firstTimeTip)
        {
            this._tipsKit.remove(this._firstTimeTip.id);
            this._firstTimeTip = null;
        }

        this._tipsKit.hideAll();
        PageOverlayTipsManager.disableMenuIcon();
    },

    /**
     * Enables all tips on the page, showing them,
     * and updates navigation tip button status.
     */
    enable: function()
    {
        this.enabled = true;
        PageOverlayTipsManager.enableCookie();

        if (!this._enabledOnce)
        {
            this.addBaseTips();
            for (var i = 0; i < this._deferredUntilEnabled.length; i++)
                this._deferredUntilEnabled[i].call(this);
            this._enabledOnce = true;
        }
        else
        {
            this._tipsKit.showAll();
            this._scheduleQueueProcessing();
        }

        if (this._refreshDynamicTipsDeferred)
            this.refreshDynamicTips();

        PageOverlayTipsManager.enableMenuIcon();
    },

    _scheduleQueueProcessing: function()
    {
        if(!this._tipsKit.isQueueProcessed())
            this._tipsKit.scheduleQueueProcessing();
    },

    /**
     * Add static or dynamic tip to specified element.
     * @param relative Relative jQuery element or a function returning jQuery element.
     *                 If a function is specified, added tip will be dynamic.
     * @param config OverlayTips config.
     */
    add: function(relative, config)
    {
        if (_.isFunction(relative))
        {
            // If relative element is a function, this is a dynamic tip.
            // Such tips are re-calculated each time refreshDynamicTips() is called.
            var relativeCalculated = relative();
            var configCalculated = (relativeCalculated.config) ? relativeCalculated.config : config;
            var elemCalculated = (relativeCalculated.element) ? relativeCalculated.element : relativeCalculated;
            this._dynamicTips.push({
                                       element: elemCalculated,
                                       relativeFunction: relative,
                                       config: configCalculated
                                   });
            relative = elemCalculated;
            config = configCalculated;
        }

        return this._add(relative, config);
    },

    /**
     * Re-evaluates every dynamic tip, deleting them or moving to new relatives if necessary.
     */
    refreshDynamicTips: function(animated)
    {
        if (_.isUndefined(animated))
            animated = true;

        if (!this.enabled)
        {
            this._refreshDynamicTipsDeferred = true;
            return;
        }

        var self = this;
        _.each(this._dynamicTips, function(dt)
        {
            // Find dynamic tip element in DOM
            var tip = document.getElementById("overlayTip" + dt.config.id);
            var newRelative = dt.relativeFunction();
            var newElementCalculated = (newRelative.element) ? newRelative.element : newRelative;
            var newConfigCalculated = (newRelative.config) ? newRelative.config : dt.config;
            var oldElementCalculated = dt.element;
            var addAgain = false;
            if (tip)
            {
                // Tip element exists, check if its relative is still valid
                if (oldElementCalculated[0] != newElementCalculated[0] || !_.isMatch(dt.config, newConfigCalculated))
                {
                    // Old relative is no longer valid, so we remove the tip and ask to re-add it
                    self._tipsKit.remove(dt.config.id);
                    addAgain = true;
                }
            }
            else
            {
                // Tip element is no longer in DOM, so we ask to re-add it
                addAgain = true;
            }

            // Attempt to add the tip again. If newRelative is empty, nothing will be added
            if (addAgain)
            {
                dt.element = newElementCalculated;
                // If we don't clone the object, then we can get few instances of one object in queue
                dt.config = _.clone(newConfigCalculated);
                dt.config.refreshAnimated = animated;
                self._add(dt.element, dt.config);
            }
        });
    },

    deferUntilEnabled: function(f)
    {
        if (this.enabled)
            f.call(this);
        else
            this._deferredUntilEnabled.push(f);
    },

    /**
     * Adds new tip with specified config to the element.
     * @param relative Relative jQuery element.
     * @param config OverlayTips config.
     * @private
     */
    _add: function(relative, config)
    {
        // config.show is a dynamic parameter
        config.show = this.enabled && !PageOverlayTipsManager._showLater;

        var $el = relative.first();
        if ($el.length == 0)
            return;

        config.relativeTo = $el;
        return this._tipsKit.enqueueTip(config);
    },

    repositionOutsideTips: function()
    {
        this._tipsKit.repositionOutsideTips();
    }
});