/**
 * @author kostenko
 *
 * A controller for task list and search field attached to it.
 *
 * Requires:
 *   jQuery
 *   jQuery.scrollintoview
 *   /js/util/jquery_rpc.js
 *   /js/constants_js.jsp
 *   /js/at.js
 *   /js/message_resources.js (using 'submit_tt' and 'js_escaped_macros.submit_tt' prefixes)
 *   /js/spin.min.js
 *
 * Note: the params below which accept DOM elements may also accept jQuery objects or selectors.
 *
 * @param container - task list container element, which contains all the needed markup
 *                    (for example of markup see 'taskList' element in task_search_control.jsp or task_select_control.jsp)
 * @param searchField - search text field element
 * @param initialRequestSpinnerPlaceholder - element in which to render spinner when doing an initial task list request
 *                                           (as opposed to the 'Show more' request)
 * @param selectedUserId - id of the user whose time-track is being viewed/edited now
 * @param selectedUserName - display name of the selected user
 * @param isOwnTimeTrack - whether it is logged in user's own time-track page
 * @param isRecentTasksMode - in this mode:
 *                            - only those tasks are listed which have time tracked by the specified user,
 *                              ordered by last tracking date, most recent first
 *                            - tasks that are already added are not listed
 *                            - 'Show more tasks' link is not shown
 *                            - desiredPortionSize has no effect, only maxResultsPerPortion matters
 * @param isImportTasksMode - in this mode all tasks (open and completed) are shown
 * @param isActive - whether to show tasks in active projects or in archived
 * @param alreadyAddedTaskIds - array of ids of task which are already added to TT
 * @param desiredPortionSize - desired number of tasks in a portion, must be > 0.
 * @param maxResultsPerPortion - maximum number of tasks in a portion, must be >= desiredPortionSize.
 * @param isMultiSelect - whether multiple selection is allowed (with checkboxes)
 * @param isEnableKeyboardNavigation - whether to move cursor with Up/Down keys and simulate click with Enter key
 * @param isHideOnEmptyQuery - whether the list should be hidden when no search query is entered
 * @param onRequestProcessingComplete - function executed when an RPC request is completed and its processing finished
 * @param onRequestError - function(jqXHR, textStatus, errorThrown) executed on RPC error.
 *                         Must return true if suppression of built-in error handler is needed.
 *                         See jquery_rpc.js for details.
 * @param onTaskSelect - executed when the user selects a task in single selection mode
 * @param onSelectionChange - executed after the set of selected tasks changes,
 *                            or after the list is cleared or re-built (regardless of the selection state before that).
 *                            Makes sense only in multi-selection mode.
 *                            Use getSelectedTaskIds method to obtain the actual list of selected tasks.
 * @param updateSearchFieldUICallback - function with new search query as argument, executed before make new search or apply empty query
 * @constructor
 *
 * The following methods are declared:
 *   hide
 *   clearQuery
 *   makeInitialRequest
 *   getSelectedTaskIds
 *   clear
 * See their declaration in the constructor body for comments.
 *
 * The following entities are defined in at.js:
 * @name messageResource
 * @name messageResource.getMessageWithEscapedMacros
 * @name KEY_CODE_UP
 * @name KEY_CODE_DOWN
 * @name KEY_CODE_ESC
 * @name KEY_CODE_ENTER
 * @name BoolTimer
 * @name BoolTimer.switchValue
 * @name BoolTimer.isTrue
 */
function TaskListController( container, searchField, initialRequestSpinnerPlaceholder,
                             selectedUserId, selectedUserName, isOwnTimeTrack,
                             isRecentTasksMode, isImportTasksMode,
                             alreadyAddedTaskIds,
                             desiredPortionSize, maxResultsPerPortion,
                             isMultiSelect, isEnableKeyboardNavigation, isHideOnEmptyQuery,
                             onRequestProcessingComplete, onRequestError, onTaskSelect, onSelectionChange, needClearButton)
{
    var self = this;
    var hoverEnabled = new BoolTimer( true, 100 );
    var spinner = new Spinner({ length: 3, width: 2, radius: 3, lines: 8, color: '#aaaaaa', trail: 60 });
    var searchDelayTimer;
    var query = "";
    var customerId = 0;
    var projectId = 0;
    var rpcRequest;
    var showMoreDisabled = false;
    var isInActiveProjects = true;
    var selectAllRows = false;
    var findTaskService = isImportTasksMode ? "TaskListControllerService.findTasksToImport" :
                                              "TaskListControllerService.findTasksToAdd";

    container = $(container);
    searchField = $(searchField).searchField(processSearchQuery, $.trim, !!needClearButton);
    
    initialRequestSpinnerPlaceholder = $(initialRequestSpinnerPlaceholder);

    if( !onRequestProcessingComplete ) onRequestProcessingComplete = function() {};
    if( !onRequestError ) onRequestError = function() {};
    if( !onTaskSelect ) onTaskSelect = function() {};
    if( !onSelectionChange ) onSelectionChange = function() {};


    /**
     * Hides the task list, aborting pending request.
     */
    this.hide = function()
    {
        abortRequestAndStopTimer();
        container.hide();
        query = '';   // clear remembered query, but not the text field,
                      // to allow reloading results by pressing enter in the field
    };

    /**
     * Clears the search text box and does the necessary processing.
     */
    this.clearQuery = function(skipUpdate)
    {
        searchField.clearSearchField(skipUpdate);
        if(skipUpdate) query = searchField.val();
    };

    /**
     * Starts the initial RPC request (as opposed to 'show more' request) using the given customer or project id for
     * filtering. Note: if project id is given, then customer id is ignored by the server.
     * @param newCustomerId
     * @param newProjectId
     */
    this.makeInitialRequest = function( newCustomerId, newProjectId )
    {
        customerId = newCustomerId;
        projectId = newProjectId;
        selectAllRows = false;

        makeInitialRequest_internal();
    };

    /**
     * Returns the ids of the selected tasks (only makes sense in multi-select mode).
     * @return {Array}
     */
    this.getSelectedTaskIds = function()
    {
        var ids = [];

        container.children(".row.task").each( function()
        {
            if( $(this).find("input:checked").length > 0 )
                ids.push( getTaskId(this) );
        });

        return ids;
    };

    /**
     * Clears the current results, aborting pending request.
     */
    this.clear = function()
    {
        abortRequestAndStopTimer();
        deleteAllRows();
        hideNoMatchesMessage();
    };

    this.isShown = function()
    {
        return container.is(":visible");
    };

    this.selectAll = function()
    {
        container.children(".row.task" ).each( function()
        {
            $(this ).find("input" ).prop("checked", true ).prop("disabled", true);
            $(this ).addClass("selected");
        });
        selectAllRows = true;
        fireOnSelectionChange();
    };

    this.unselectAll = function()
    {
        container.children(".row.task" ).each( function()
        {
            $(this ).find("input" ).prop("checked", false ).prop("disabled", false);
            $(this ).removeClass("selected");
        });
        selectAllRows = false;
        fireOnSelectionChange();
    };

    this.isSelectAllRows = function()
    {
        return selectAllRows;
    };

    this.getCustomerId = function()
    {
        return customerId;
    };

    this.getProjectId = function()
    {
        return projectId;
    };

    this.setInActiveProjects = function( inActive )
    {
        isInActiveProjects = inActive;
    };

    function moveCursorUp()
    {
        moveCursorUpOrDown( true );
    }

    function moveCursorDown()
    {
        moveCursorUpOrDown( false );
    }

    function moveCursorUpOrDown( isUp )
    {
        var activeRow = getActiveRow();
        var switchTo = isUp ? getPrevRow( activeRow ) : getNextRow( activeRow );

        if( switchTo.size() == 1 )
        {
            setActiveRow( switchTo );
            hoverEnabled.switchValue(); // don't react to mouse move while changing active row the other way
            switchTo.scrollintoview( {duration: 0} );
        }
    }

    function getActiveRow()
    {
        return container.children(".row.active");
    }

    function setActiveRow( row )
    {
        getActiveRow().removeClass("active");
        $(row).addClass("active");
    }

    function getNextRow( row )
    {
        return $(row).next(".row");
    }

    function getPrevRow( row )
    {
        return $(row).prev(".row");
    }

    function deleteAllRows()
    {
        container.children(".row").remove();
        fireOnSelectionChange();
    }

    function getShownTasksCount()
    {
        return container.children(".row.task").length;
    }

    function scrollToTop()
    {
        container.scrollTop(0);
    }

    /**
     * @param rpcResultData
     * @config tasks
     * @config resultsLeft
     */
    function createRows( rpcResultData )
    {
        var shownTasksCount = getShownTasksCount();
        var taskRow;
        var firstNewRow = null;

        /*
         * @param this
         * @config id
         * @config name
         * @config nameMatchMap
         * @config parentTaskName
         * @config parentTaskNameMatchMap
         * @config projectName
         * @config customerName
         */
        $.each(rpcResultData.tasks.list, function(i)
        {
            var alreadyAdded = $.inArray( this.id, alreadyAddedTaskIds ) != -1;

            if( isRecentTasksMode && alreadyAdded )
                return;

            taskRow = container.find(".templates .row.task").clone().appendTo(container);
            taskRow.find(".taskId").text(this.id);
            taskRow.find(".taskName").html( getNamePresentationWithMatches(this.name, this.nameMatchMap.list) );

            if( this.parentTaskName )
                taskRow.find(".taskName").prepend(
                        getNamePresentationWithMatches(this.parentTaskName, this.parentTaskNameMatchMap.list),
                        " / " );

            if( projectId == 0 )
                taskRow.find(".details").text( (customerId == 0 ? this.customerName + " / " : "") + this.projectName );
            else
                taskRow.find(".details").text("");

            if( alreadyAdded )
                taskRow.find(".alreadyAdded").show();

            if( i == rpcResultData.tasks.list.length - 1 && rpcResultData.resultsLeft == 0 )
                taskRow.addClass("last");

            taskRow.mousemove(onRowMouseMove);
            taskRow.click(onRowClick);

            if( isMultiSelect )
                taskRow.find("input").click(onRowCheckboxClick).change(onRowCheckboxChange);

            shownTasksCount++;

            if( firstNewRow == null )
                firstNewRow = taskRow;

            if( selectAllRows )
            {
                taskRow.find("input" ).prop("checked", true ).prop("disabled", true);
                taskRow.addClass("selected");
            }
        });

        container.children(".showMore").remove();

        if( !isRecentTasksMode && rpcResultData.resultsLeft > 0 )
        {
            var showMoreRow = container.find(".templates .row.showMore").clone().appendTo(container);
            showMoreRow.find(".resultsContainDiv").html(
                    messageResource.getMessageWithEscapedMacros( "js_escaped_macros.submit_tt.task_search.results_contain_tasks",
                                               [shownTasksCount, shownTasksCount + rpcResultData.resultsLeft]) );

            showMoreRow.mousemove(onRowMouseMove);
            showMoreRow.click(onRowClick);
        }

        if( firstNewRow != null )
        {
            if( isEnableKeyboardNavigation  )
                setActiveRow( firstNewRow );

            container.animate( { scrollTop: container.scrollTop() + firstNewRow.position().top } );
        }

        showMoreDisabled = false;
    }

    function spinInShowMoreRow()
    {
        var showMoreRow = container.children(".showMore");
        showMoreRow.find('.panelDiv').css('visibility', 'hidden');
        spinner.spin(showMoreRow[0]);
    }

    function spinForInitialRequest()
    {
        spinner.spin( initialRequestSpinnerPlaceholder[0] );
    }

    function stopSpin()
    {
        spinner.stop();
        container.children(".showMore").find(".panelDiv").css('visibility', 'visible'); // restore the 'show more' row
        // this will only have effect if 'show more' row was actually present
        // and this is needed only in case of 'show more' request error
        // (in case of success the 'show more' row will be replaced by more search results)
    }

    function showNoMatchesMessage( text )
    {
        container.find(".noMatches").text(text).show();
    }

    function hideNoMatchesMessage()
    {
        container.find(".noMatches").hide();
    }

    function makeShowMoreRequest()
    {
        if( showMoreDisabled )
            return;

        showMoreDisabled = true; // disable 'show more' to prevent inconsistent results because of double-clicks

        rpcRequest = makeRpcCall( findTaskService,
                                  [query, customerId, projectId, selectedUserId,
                                   getShownTasksCount(), desiredPortionSize, maxResultsPerPortion,
                                   isInActiveProjects, !isInActiveProjects],
                                  onShowMoreRequestComplete, onRequestError_internal );
        spinInShowMoreRow();
    }

    function getTaskId( row )
    {
        return Number( $(row).find(".taskId").text() );
    }

    function abortRequestAndStopTimer()
    {
        clearTimeout( searchDelayTimer );

        if( rpcRequest )
            rpcRequest.abort();
    }


    var updateSearchFieldBinded = _.bind(searchField.updateSearchField, searchField);
    function prepareSearchQueryProcessing( delay )
    {
        abortRequestAndStopTimer();
        searchDelayTimer = setTimeout(updateSearchFieldBinded, delay );
    }

    function makeInitialRequest_internal()
    {
        abortRequestAndStopTimer();

        showMoreDisabled = true; // disable 'show more' to prevent inconsistent results

        if( isRecentTasksMode )
            rpcRequest = makeRpcCall( "TaskListControllerService.findRecentTasksToAdd",
                                      [selectedUserId, maxResultsPerPortion],
                                      onInitialRequestComplete, onRequestError_internal );
        else
            rpcRequest = makeRpcCall( findTaskService,
                                      [query, customerId, projectId, selectedUserId,
                                       0, desiredPortionSize, maxResultsPerPortion,
                                       isInActiveProjects, !isInActiveProjects],
                                      onInitialRequestComplete, onRequestError_internal );
        spinForInitialRequest();
    }

    function processSearchQuery(newQuery)
    {
        query = newQuery;

        var tokenizer = new StringTokenizer(StringTokenizer.PUNCTUATORS_FOR_SEARCH);
        query = query.replace(tokenizer.replacePunctuatorsPattern, " ");


        if( query == '' && isHideOnEmptyQuery )
        {
            self.hide();
        }
        else
        {
            makeInitialRequest_internal();
        }
    }

    function fireOnSelectionChange()
    {
        setTimeout( onSelectionChange, 0 );
    }

    /**
     * TODO: this is a candidate for moving to some common file
     */
    function getNamePresentationWithMatches( name, matchMap )
    {
        var result = $('<span/>');

        // by using 'text' method below we ensure that the strings will be properly escaped

        result.append( $('<span/>').text( name.substr(0, matchMap[0]) ) );

        for( var i = 0; i < matchMap.length; i+=2 )
        {
            result.append( $('<span class="highlightToken"/>').text( name.substring(matchMap[i], matchMap[i+1]) ) );
            result.append( $('<span/>').text( name.substring(matchMap[i+1], matchMap[i+2]) ) );
        }

        return result;
    }


    function onRowMouseMove()
    {
        if( hoverEnabled.isTrue() )
            setActiveRow(this);
    }

    function onMouseLeave()
    {
        if( !isEnableKeyboardNavigation )
            setActiveRow( null );
    }

    function onScroll()
    {
        hoverEnabled.switchValue(); // don't react to mouse move while scrolling
    }

    /**
     * @param data
     * @config assignmentsFeatureEnabled
     * @config viewerHasAccessToAllProjects
     */
    function onInitialRequestComplete( data )
    {
        stopSpin();

        container.show();
        scrollToTop();
        deleteAllRows();
        hideNoMatchesMessage();

        createRows( data );

        if( getShownTasksCount() == 0 )
        {
            var key = (isImportTasksMode ? 'task_selector' : 'submit_tt') + '.task_list.nothing_found';

            if( projectId != 0 )
                key += '.project_selected';
            else if( customerId != 0 )
                key += '.customer_selected';

            if( query != '' )
                key += '.name_given';

            if( isRecentTasksMode )
                key += '.recent';

            if( projectId == 0 && !isRecentTasksMode )
            {
                if( !data.viewerHasAccessToAllProjects )
                    key += '.your';
                if( !isOwnTimeTrack && data.assignmentsFeatureEnabled )
                    key += '.his';
            }

            showNoMatchesMessage( messageResource.getMessage(key, [selectedUserName]) );
        }

        onRequestProcessingComplete();
    }

    function onShowMoreRequestComplete( data )
    {
        stopSpin();
        createRows( data );

        onRequestProcessingComplete();
    }

    function onRequestError_internal( jqXHR, textStatus, errorThrown )
    {
        stopSpin();
        showMoreDisabled = false;

        if( onRequestError( jqXHR, textStatus, errorThrown ) )
            return true;

        if( errorThrown.exception == 'com.actimind.actitime.services.NoRightsForRemoteCallException' )
        {
            alert( messageResource.getMessage( "common.no_rights_for_operation_anymore" ) );
            document.location.reload();
            return true;
        }

        if( errorThrown.exception == 'com.actimind.actitime.administration.NoSuchUserException' )
        {
            document.location.reload();
            return true;
        }

        return false;
    }

    function onRowClick()
    {
        var taskId = getTaskId( this );

        if( taskId > 0 )
        {
            if( isMultiSelect )
            {
                $(this).find("input").click();
            }
            else
            {
                abortRequestAndStopTimer();
                self.hide();
                onTaskSelect( taskId );
            }
        }
        else
        {
            makeShowMoreRequest();
        }
    }

    function onRowCheckboxClick( event )
    {
        fireOnSelectionChange();
        event.stopPropagation(); // if event propagates to parent element (row) we'll stuck in a loop
    }

    function onRowCheckboxChange()
    {
        if( $(this).prop('checked') )
            $(this).parents('.row.task').addClass('selected');
        else
            $(this).parents('.row.task').removeClass('selected');
    }

    function onSearchFieldKeyDown( event )
    {
        if( event.keyCode == KEY_CODE_UP && isEnableKeyboardNavigation )
        {
            moveCursorUp();
            event.preventDefault();
        }
        else if( event.keyCode == KEY_CODE_DOWN && isEnableKeyboardNavigation )
        {
            moveCursorDown();
            event.preventDefault();
        }
        else if( event.keyCode == KEY_CODE_ENTER && isEnableKeyboardNavigation )
        {
            if( self.isShown() )
                getActiveRow().click();
            else
                searchField.updateSEarchField();

            event.preventDefault();
        }
        else if( event.keyCode == KEY_CODE_ESC )
        {
            self.clearQuery();
            event.preventDefault();
        }
        else
        {
            prepareSearchQueryProcessing( 200 );
        }
    }


    container.scroll(onScroll);
    container.mouseleave(onMouseLeave);
    searchField.keydown(onSearchFieldKeyDown);
}
