/**
 * @author kostenko
 *
 * Requires:
 *  cp_selector_controller.js
 *  task_list_controller.js
 *
 * Markup: task_select_control.jsp
 * Styles: add_tasks.css
 *
 * @name setMouseDownOutsideHandler (defined in at.js)
 * @name smoothlyScrollDownToElement (defined in at.js)
 *
 * @param id {string} id of the HTML element that contains the control
 * @param outerButtonElement {string} HTML element of outer control button
 * @param selectedUserId {number} selected user id
 * @param selectedUserName {string} display name of the selected user
 * @param isOwnTimeTrack {boolean} whether it is logged in user's own time-track page
 * @param isImportTasksMode {boolean} if true - shown all tasks (open and completed)
 * @param alreadyAddedTaskIds {array} ids of task which are already added to TT
 * @param onTasksChosen {function(taskIds, needToLoadDescriptions, customerId, projectId, isInActiveProjects)} that will be called when user submits tasks to add
 * @param scrollableContainerElement {string} HTML element of scrollable container in which outer control button is located (need to hide panel on scroll)
 */
function initTaskSelectControl( id, outerButtonElement, selectedUserId, selectedUserName, isOwnTimeTrack,
                                isImportTasksMode, alreadyAddedTaskIds, onTasksChosen, scrollableContainerElement )
{
    var controlContainer = $("#" + id);
    var outerButton = $(outerButtonElement);
    var panel = controlContainer.find(".addTasksPanel");
    var innerButton = panel.find( isImportTasksMode ? ".importSelectedButton" : ".addSelectedButton");
    var importArchivedCheckbox = panel.find("#importArchivedCheckbox");
    if (isImportTasksMode){
        panel.find(".importArchivedLabel" ).text(truncateStringByLength(messageResource.getMessage("task_selector.import_archived"),35,35 ));
        panel.find(".buttonImportTitle" ).text(truncateStringByLength(messageResource.getMessage("task_selector.import_selected"),30,30 ));
    }
    var importDescriptionsCheckbox = panel.find("#importDescriptionsCheckbox");
    var rpc = null;
    var desiredPortionSize = 100;
    var maxResultsPerPortion = 150;
    var cpSelectorControllerActive = new CPSelectorController( panel.find(".customerProjectSelector.activeProjects"), onCustomerOrProjectSelect );
    var cpSelectorControllerArchive = new CPSelectorController( panel.find(".customerProjectSelector.archivedProjects"), onCustomerOrProjectSelect );
    var cpSelectorController = importArchivedCheckbox.prop("checked") ? cpSelectorControllerArchive : cpSelectorControllerActive;
    var taskListController = new TaskListController( panel.find(".taskList"),
                                                     panel.find(".filterField"),
                                                     panel.find(".spinnerPlaceholder"),
                                                     selectedUserId, selectedUserName, isOwnTimeTrack, false, isImportTasksMode,
                                                     alreadyAddedTaskIds, desiredPortionSize, maxResultsPerPortion,
                                                     true, false, false,
                                                     null, null, null, onSelectionChange );


    function showCpSelectorController( needReinit )
    {
        var cpSelectorToShow = importArchivedCheckbox.prop("checked") ? cpSelectorControllerArchive : cpSelectorControllerActive;
        var cpSelectorToHide = importArchivedCheckbox.prop("checked") ? cpSelectorControllerActive : cpSelectorControllerArchive;

        taskListController.setInActiveProjects( !importArchivedCheckbox.prop("checked"));
        cpSelectorToHide.hideSelector();
        cpSelectorToShow.showSelector();
        cpSelectorController = cpSelectorToShow;
        if( !cpSelectorToShow.isInitialized() && rpc == null || needReinit )
        {
            cpSelectorToShow.spin();
            rpc = makeRpcCall( "RetrieveCustomersProjectsService.getCustomersProjectsForTaskSelectControl",
                               [selectedUserId],
                               onCustomersProjectsRequestComplete, onCustomersProjectsRequestError );
        }
    }

    function showPanel()
    {
        if( panel.hasClass( 'hidden' ) )
        {
            panel.removeClass( 'hidden' );
            if( isImportTasksMode )
                controlContainer.position( {of: outerButton, at: "left bottom", my: "left top", collision: "none"} );
            outerButton.addClass( 'active' );

            showCpSelectorController( true );
            KeyDownHandler.setOnKeyDownHandler( onKeyDown );

            smoothlyScrollDownToElement( panel, 5 );

            if( scrollableContainerElement )
                $( scrollableContainerElement ).one( "scroll", hidePanel );

        }
    }

    function hidePanel()
    {
        if( !panel.hasClass( 'hidden' ) )
        {
            panel.addClass( 'hidden' );
            outerButton.removeClass( 'active' );
            KeyDownHandler.restoreOnKeyDownHandler();
            cpSelectorController.closeDropdown();
            // clear task list
            onCustomerOrProjectSelect( 0, 0 );
        }
    }

    function openFilter()
    {
        panel.find(".filterPanel").show();
        panel.find(".filterField").focus();
    }

    function closeFilter(skipUpdate)
    {
        panel.find(".filterPanel").hide();
        taskListController.clearQuery(skipUpdate);
    }

    function isFilterOpen()
    {
        return panel.find(".filterPanel:visible").length == 1;
    }


    function onOuterButtonClick()
    {
        if( panel.is(':visible') )
            hidePanel();
        else
            showPanel();
    }

    function onInnerButtonClick()
    {
        if( !innerButton.hasClass( "disabled" ) )
        {
            var taskIds = undefined;
            var customerId = 0;
            var projectId = 0;
            if( taskListController.isSelectAllRows() )
            {
                customerId = taskListController.getCustomerId();
                projectId = taskListController.getProjectId();
            }
            else
            {
                taskIds = taskListController.getSelectedTaskIds();
            }
            hidePanel();
            cpSelectorController.hideSelector();
            onTasksChosen( taskIds, importDescriptionsCheckbox.prop( "checked" ), customerId, projectId, !importArchivedCheckbox.prop( "checked" ) );
        }
    }

    function onMouseDownOutside( event )
    {
        if( $(event.target).closest( outerButton ).length == 0 )
            hidePanel();
    }

    /**
     * @param data
     * @config customers
     * @config projects
     */
    function onCustomersProjectsRequestComplete( data )
    {
        cpSelectorControllerActive.init( data, true );
        cpSelectorControllerArchive.init( data, false );
        cpSelectorController.stopSpin();
    }

    function onCustomersProjectsRequestError( jqXHR, textStatus, errorThrown )
    {
        cpSelectorController.stopSpin();
        hidePanel();
        rpc = null;

        if( errorThrown.exception == 'com.actimind.actitime.administration.NoSuchUserException' )
        {
            document.location.reload();
            return true;
        }

        return false;
    }

    function onCustomerOrProjectSelect( customerId, projectId )
    {
        closeFilter(true);

        if( customerId != 0 || projectId != 0 )
        {
            panel.find(".pleaseSelectCustomerOrProject").hide();
            panel.find(".selectorFilterButton").prop('disabled', false);
            taskListController.makeInitialRequest( customerId, projectId );
        }
        else
        {
            taskListController.clear();
            panel.find(".pleaseSelectCustomerOrProject").show();
            panel.find(".selectorFilterButton").prop('disabled', true);
        }
    }

    function onFilterButtonClick()
    {
        openFilter();
    }

    function onFilterCloseButtonClick()
    {
        closeFilter();
    }

    function onSelectAllButtonClick()
    {
        taskListController.selectAll();
    }

    function onUnselectAllButtonClick()
    {
        taskListController.unselectAll();
    }

    function onKeyDown( event )
    {
        event = event || window.event;
        if( event.keyCode == KEY_CODE_ESC )
        {
            if( isFilterOpen() )
                closeFilter();
            else
                hidePanel();
        }
    }

    function onSelectionChange()
    {
        innerButton.toggleClass( "disabled", taskListController.getSelectedTaskIds().length == 0 );
        innerButton.prop( "disabled", taskListController.getSelectedTaskIds().length == 0 );
    }

    function onImportArchivedCheckboxClick()
    {
        showCpSelectorController();
    }

    outerButton.click( onOuterButtonClick );
    innerButton.click( onInnerButtonClick );
    importArchivedCheckbox.click( onImportArchivedCheckboxClick );
    setMouseDownOutsideHandler( controlContainer, onMouseDownOutside );
    panel.find(".selectorFilterButton").click( onFilterButtonClick );
    panel.find(".filterPanel .closeButton").click( onFilterCloseButtonClick );
    panel.find(".selectAllButton").click( onSelectAllButtonClick );
    panel.find(".unselectAllButton").click( onUnselectAllButtonClick );
}