/**
 * Cross domain controller contains basic functionality to perform cross domain requests (page load/form submit)
 */
function CrossDomainController(){

    this.timer = null;      // Timer for request operation
    this.requestId = null;  // Holds request id received during request
    this.state = new CrossDomainControllerState_Inactive(this); // Controller state, initially incative
}

// Server call types, used to report  what call was received while controller was in particular state
CrossDomainController.prototype.NO_CALL = 0;
CrossDomainController.prototype.REQUEST_ID_RECEIVED = 1;
CrossDomainController.prototype.REQUEST_CONFIRMED = 2;
CrossDomainController.prototype.REQUEST_VERIFIED = 3;

CrossDomainController.prototype.serviceURL = CrossDomainControllerConstants.serviceURL;
CrossDomainController.prototype.workerFrameSrc = CrossDomainControllerConstants.workerFrameSrc;

/**
 * @return String URL for obtaining request id
 */
CrossDomainController.prototype.getRequestIdActionURL = function(){

    return this.serviceURL + "?action=1";
};

/**
 * @return String URL for request confirmation action
 */
CrossDomainController.prototype.getConfirmationActionURL = function(){

    return this.serviceURL + "?action=2&requestId=" + this.requestId;
};

/**
 * @return String URL for request verification action
 */
CrossDomainController.prototype.getVerificationActionURL = function(){

    return this.serviceURL + "?action=3&requestId=" + this.requestId;
};

/**
 * This function is called when request id has been received
 */
CrossDomainController.prototype.requestIdReceived = function(requestId){

    // Set request id and report that request id has been received
    this.requestId = requestId;                      
    this.state.callReceived(this.REQUEST_ID_RECEIVED);
};

/**
 * The function is called when request has been successfully verified
 */
CrossDomainController.prototype.requestVerified = function(){

    // Report that request has been verified
    this.state.callReceived(this.REQUEST_VERIFIED);
};

/**
 * @return request id
 */
CrossDomainController.prototype.getRequestId = function(){

    return this.requestId;
};

/**
 * @return service URL for current controller state
 *          (if controller is in incative state empty string is returned)
 */
CrossDomainController.prototype.getServiceURL = function(){

    return this.state.getServiceURL();
};

/**
 * Starts frame loading
 */
CrossDomainController.prototype.startLoading = function(){

    return this.state.startLoading();
};

/**
 * Called when frame loading finished
 */
CrossDomainController.prototype.loadingFinished = function(){

    var self = this;

    // timeout is needed for Chrome & Safari, to let the script loaded into iframe execute before this event is fired  
    window.setTimeout(function () {
        self.state.loadingFinished()
    }, 100);
};

/**
 * Switches controller to next state and starts frame loading
 *
 * @param newState new state for the controller
 */
CrossDomainController.prototype.switchToNextState = function(newState){

    // Set controller state and start loading
    this.setState(newState);
    this.startLoading();
};

/**
 * Sets controller state
 *
 * @param newState new state for the controller
 */
CrossDomainController.prototype.setState = function(newState){

    this.state = newState;
};

/**
 * Starts execution of this controller
 */
CrossDomainController.prototype.startExecution = function(){

    // Start timeout timer and start loading frame
    this.startTimer();
    this.startLoading();
};

/**
 * Starts timeout timer
 */
CrossDomainController.prototype.startTimer = function(){

    var controller = this;
    var callback = function(){ controller.requestFailed();};
    this.timer = window.setTimeout(callback, 120000);
};

/**
 * Stops timeout timer
 */
CrossDomainController.prototype.stopTimer = function(){

    if(this.timer != null){

        clearTimeout(this.timer);
        this.timer = null;
    }
};

/**
 * Performs cross domain request
 */
CrossDomainController.prototype.doRequest = function(){

    // Do nothing, sublcass overrides this method to provide specific implementation
};

/**
 * This method is called when cross domain request successfully accomplished
 */
CrossDomainController.prototype.requestSucceeded = function(){

    this.stopTimer();   // Stop timeout timer
};

/**
 * This method is called when cross domain request fails
 */
CrossDomainController.prototype.requestFailed = function(){

    this.stopTimer();
    this.state = new CrossDomainControllerState_Failure(this);
};

/**
 * Create call back function that is invoked in order to indicate that functional lifetime of
 * particular step (state) exhausted
 */
CrossDomainController.prototype.createStepCompletionHandler = function(){

     // Create call back function
    var controller = this;
    return function () {controller.loadingFinished();};
};


/**
 *  Base class for cross domain controller state
 */
function CrossDomainControllerState(controller){

    this.controller = controller;       // Reference to context
    this.serverCall = CrossDomainController.prototype.NO_CALL;  // Stores type of call that was received by the
                        // context from server side generated script while it was in this state
}

/**
 * The method is called to notify that controller received particular call from
 * server side generated script while it was in this state
 * @param serverCall call identificator
 */
CrossDomainControllerState.prototype.callReceived = function(serverCall){

    this.serverCall = serverCall;
};

/**
 * @return type of call received by the controller from server side generated script
 */
CrossDomainControllerState.prototype.getReceivedCall = function(){

    return this.serverCall;
};

/**
 * Initiates request step corresponding to this state
 */
CrossDomainControllerState.prototype.startLoading = function(){

    // Do nothing, sublcass overrides this method to provide specific implementation
};

/**
 * Called when request step corresponding to this state finished
 */
CrossDomainControllerState.prototype.loadingFinished = function(){

    // Do nothing, sublcass overrides this method to provide specific implementation
};

/**
 * Returns service URL corresponding to this state
 */
CrossDomainControllerState.prototype.getServiceURL = function(){

    return "";
};

/**
 * Creates work frame with specified id
 * @param frameId id value
 */
CrossDomainControllerState.prototype.createWorkFrame = function(frameId){

    // Create work frame
    createWorkFrame(frameId, this.controller.workerFrameSrc ,
        this.controller.createStepCompletionHandler());
};


/**
 * Inactive state
 */
function CrossDomainControllerState_Inactive(controller){

    this.parent = CrossDomainControllerState;
    this.parent(controller);
}

CrossDomainControllerState_Inactive.prototype = new CrossDomainControllerState();


/**
 * State in which controller is obtaining request id
 */
function CrossDomainControllerState_QueryRequestId(controller){

    this.parent = CrossDomainControllerState;
    this.parent(controller);
}

CrossDomainControllerState_QueryRequestId.prototype = new CrossDomainControllerState();

CrossDomainControllerState_QueryRequestId.prototype.startLoading = function(){

    this.createWorkFrame("queryRequestIdFrame");
};

CrossDomainControllerState_QueryRequestId.prototype.loadingFinished = function(){

    // If request id has been received continue by switching to next state
    if(this.getReceivedCall() == this.controller.REQUEST_ID_RECEIVED){

        this.controller.switchToNextState(new CrossDomainControllerState_DoRequest(this.controller));
    }
    else{
        // Otherwise report the controller that request failed
        this.controller.requestFailed();
    }
};

CrossDomainControllerState_QueryRequestId.prototype.getServiceURL = function(){

    return this.controller.getRequestIdActionURL();
};


/**
 * State in which actual request is performed
 */
function CrossDomainControllerState_DoRequest(controller){

    this.parent = CrossDomainControllerState;
    this.parent(controller);
}

CrossDomainControllerState_DoRequest.prototype = new CrossDomainControllerState();

CrossDomainControllerState_DoRequest.prototype.startLoading = function(){

    this.controller.doRequest();
};

CrossDomainControllerState_DoRequest.prototype.loadingFinished = function(){

    // In this we do not check that request has been confirmed since currently server side does not generate script
    this.controller.switchToNextState(new CrossDomainControllerState_VerifyRequest(this.controller));
};


/**
 * State in which request is verified
 */
function CrossDomainControllerState_VerifyRequest(controller){

    this.parent = CrossDomainControllerState;
    this.parent(controller);
}

CrossDomainControllerState_VerifyRequest.prototype = new CrossDomainControllerState();

CrossDomainControllerState_VerifyRequest.prototype.startLoading = function(){

    this.createWorkFrame("verifyRequestFrame");
};

CrossDomainControllerState_VerifyRequest.prototype.loadingFinished = function(){

    // If request has been verified report that request succeeded and switch to incative state
    if(this.getReceivedCall() == this.controller.REQUEST_VERIFIED){

        this.controller.requestSucceeded();
        this.controller.switchToNextState(new CrossDomainControllerState_Inactive(this.controller));
    }
    else{
        // Otherwise report the controller that request failed
        this.controller.requestFailed();
    }
};

CrossDomainControllerState_VerifyRequest.prototype.getServiceURL = function(){

    return this.controller.getVerificationActionURL();
};


/**
 * Failure state
 */
function CrossDomainControllerState_Failure(controller){

    this.parent = CrossDomainControllerState;
    this.parent(controller);
}

CrossDomainControllerState_Failure.prototype = new CrossDomainControllerState();

