/**
 * Program registrator
 * @param callBack method to execute after program is registered
 */
function ProgramRegistrator(callBack){

    // Current registration id, should be escaped before being used in JavaScript
    this.registrationId = ProgramRegistratorConstants.registrationId;
    this.callBack = callBack;   // Call back method
}

// Registration service URL, frame to perform registration in, frame id
ProgramRegistrator.prototype.registrationServiceURL = ProgramRegistratorConstants.registrationServiceURL;
ProgramRegistrator.prototype.workerFrameSrc = ProgramRegistratorConstants.workerFrameSrc;
ProgramRegistrator.prototype.workerFrameId = "registrationFrame";

/**
 * @return URL of the service that generates registration id
 */
ProgramRegistrator.prototype.getRegistrationServiceURL = function(){

    return this.registrationServiceURL;
};

/**
 * Sets registration id
 */
ProgramRegistrator.prototype.setRegistrationId = function(registrationId){

    this.registrationId = registrationId;
};

/**
 * @return registration id
 */
ProgramRegistrator.prototype.getRegistrationId = function(){

    return this.registrationId;
};

/**
 * Resets registration id value
 */
ProgramRegistrator.prototype.resetRegistrationId = function(){

    this.setRegistrationId('');
};

/**
 * @return true if program already has registration, false otherwise
 */
ProgramRegistrator.prototype.isRegistrationIdValid = function(){

    return this.getRegistrationId() != null && this.getRegistrationId() != '';
};

/**
 * Initiates query for registration id
 */
ProgramRegistrator.prototype.queryRegistrationId = function(){

    // If program is already registered make call back and exit
    if(this.isRegistrationIdValid()){

        this.makeCallBack();
        return;
    }

    // Create method for IFRAME "onload" event
    var reg = this;
    var loadingFinished = function(){reg.loadingFinished();};

    // Create work frame
    createWorkFrame(this.workerFrameId, this.workerFrameSrc, loadingFinished);
};

/**
 * The method is called "on load" of registrator IFRAME
 */
ProgramRegistrator.prototype.loadingFinished = function(){

    // Save registration id
    this.saveRegistrationId();
};

/**
 * Makes call back if provided
 */
ProgramRegistrator.prototype.makeCallBack = function(){

    // Make call back if provided
    if(this.callBack){

        this.callBack();
    }
};

/**
 * Saves registration id
 */
ProgramRegistrator.prototype.saveRegistrationId = function(){

    try{

        // Do not save registration id if it has invalid value
        if(!this.isRegistrationIdValid()){

            //noinspection ExceptionCaughtLocallyJS
            throw "Invalid registration id returned from PWS";
        }

        // If rpc is not ready
        if(!isRpcReady( ['RegistrationInfoService.saveRegistrationId'] )){

            var thisRegistrator = this;
            var thisMethod = function(){thisRegistrator.saveRegistrationId();};
            window.setTimeout(thisMethod, 100);
            return;
        }

        // If service is not registered throw exception
        if(!jsonrpc.RegistrationInfoService){
            //noinspection ExceptionCaughtLocallyJS
            throw "RegistrationInfoService is not registered";
        }

        // Try to save registration id and assign returned value to "registration id" field
        this.registrationId = jsonrpc.RegistrationInfoService.saveRegistrationId(this.registrationId);
    }
    catch(e){
        // If error occured reset registration id value
        this.resetRegistrationId();
    }

    // Finally make callback
    this.makeCallBack();
};