/**
 * Support page helper class
 *
 * @param pageURL URL of the page to be loaded
 * @param pageContainerId id of an element that includes all other elements to be displayed when page is loaded
 * @param documentContainerId id of an element where page is loaded
 * @param progressScreenId id of an element that contains progress screen
 * @param failScreenId id of an element that contains fail screen
 */
function SupportPageLoader(pageURL, pageContainerId, documentContainerId, progressScreenId, failScreenId){

    this.supportController();

    this.pageURL = pageURL;
    this.pageContainerId = pageContainerId;
    this.documentContainerId = documentContainerId;
    this.progressScreenId = progressScreenId;
    this.failScreenId = failScreenId;

    this.pageOnload = null; // Method to invoke after page has been successfully loaded
}

SupportPageLoader.prototype = new SupportController();
SupportPageLoader.prototype.supportController = SupportController;

/**
 * @return HTMLElement document container element
 */
SupportPageLoader.prototype.getDocumentContainerElm = function(){

    return document.getElementById(this.documentContainerId);
};

/**
 * Loads support page
 */
SupportPageLoader.prototype.loadPage = function(){

    this.hideElement(this.pageContainerId); // Hide page container
    this.showElement(this.progressScreenId);// Show progress screen
    this.startExecution();                  // Triggers execution of page load process
};

/**
 * @return String request URL
 */
SupportPageLoader.prototype.getRequestURL = function(){

    // Add request id and application URL parameters to page URL
    return this.pageURL + "?requestId=" + this.getRequestId() + "&appURL=" + SupportPageLoaderConstants.appURL +
        "&programEdition=" + SupportPageLoaderConstants.programEdition;
};

/**
 * Perform page load request
 */
SupportPageLoader.prototype.doRequest = function(){

    var iframeId = "contentFrame";
    var iframe = document.getElementById(iframeId);

    // If iframe with this id already exists remove it
    if(iframe != null){
        document.body.removeChild(iframe);
    }

    iframe = document.createElement("IFRAME");  // Create iframe element
    iframe.id = iframeId;   // Id and name have identical values
    iframe.name = iframeId;
    iframe.frameBorder = 0;     // No border and margins
    iframe.marginWidth = 0;
    iframe.marginHeight = 0;
    iframe.width = "100%"; // Takes all allowed space
    iframe.height = "100%";
    // Construct iframe source: page URL + parameters
    iframe.src = this.getRequestURL();

    // Append "IFRAME" element to document container
    this.getDocumentContainerElm().appendChild(iframe);

    // Create onload handler and attach it to iframe
    var onloadHandler = this.createStepCompletionHandler();
    addOnloadHandler(iframe, onloadHandler);
};

/**
 * This method is called when page load request succeeds
 */
SupportPageLoader.prototype.requestSucceeded = function(){

    this.supportController.prototype.requestSucceeded.call(this);  // Call parent method
    this.hideElement(this.progressScreenId);    // Hide progress screen
    this.showElement(this.pageContainerId);     // Show page container

    // If there is onload handler call it
    if(this.pageOnload){
        this.pageOnload();
    }
};

/**
 * This method is called when page load request fails
 */
SupportPageLoader.prototype.requestFailed = function(){

    this.supportController.prototype.requestFailed.call(this);     // Call parent method
    this.hideElement(this.progressScreenId);    // Hide progress screen
    this.showElement(this.failScreenId);        // Show fail screen
};

/**
 * Sets page's "onload" handler
 * @param handler handler to invoke after support page has been successfully loaded
 */
SupportPageLoader.prototype.setPageOnload = function(handler){

    this.pageOnload = handler;
};